%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function [pred, dummy] = gen_paths(PAR)
% pred = gen_paths(PAR)
  
%sg_fn = which('sg')
%unix(sprintf('ldd %s', sg_fn))

% dummy return argument
dummy=[] ;

%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;%GLOBAL_blocks;
if isempty(GLOBAL_cPAR),
  L = load(PAR.cPAR_fname, 'cPAR') ;
  GLOBAL_cPAR = L ;
  %blocks = load_struct(L.cPAR.blocks_fname) ; 
  %GLOBAL_blocks = blocks ;
else
  L = GLOBAL_cPAR ;
  %blocks = GLOBAL_blocks ;
end ;

%block = blocks(PAR.block_id) ;
block = PAR.block ;

% fix some field ordering
L.cPAR.model = fix_model(L.cPAR.model) ;
L.cPAR.parameters = reorder_fields(L.cPAR.parameters, L.cPAR.parameters_fields) ;
block = fix_blocks(block, L.cPAR.model) ;

model = L.cPAR.model;
parameters = L.cPAR.parameters ;

%%%%%%%%%%%%%%%%%

if ~isfield(PAR, 'max_len')
  PAR.max_len=[] ;
end ;
if ~isfield(PAR, 'prediction')
  PAR.prediction=0 ;
end ;
block.seq(find(block.seq~='A'&block.seq~='C'&block.seq~='G'&block.seq~='T'))='A';


% subset the positions to approximate the full dynamic program
if isfield(PAR, 'approximate') && isequal(PAR.approximate, 1),

  %if keyboard_allowed(),
  %  keyboard
  %end ;

  % make sure all previously predicted positions, true positions and strong positions are in the list
  used_positions = [] ;
  for i=1:length(block.pred),
	  used_positions = [used_positions block.pred(i).pos] ;
  end ;
  true_positions = [] ;
  for i=1:length(block.truth),
    true_positions = [true_positions block.truth.pos] ;
  end ;
  strong_positions_acc = block.Signals.acc.pos(find(block.Signals.acc.Conf_cum>0.75)) ;
  strong_positions_don = block.Signals.don.pos(find(block.Signals.don.Conf_cum>0.75)) ;
  strong_positions = union(strong_positions_acc, strong_positions_don) ;
  good_positions = union(union(used_positions, strong_positions), true_positions) ;

  [tmp,idx_pos]=intersect(block.all_pos, good_positions) ;

  block.all_pos = block.all_pos(idx_pos) ;
  block.pos = block.pos(idx_pos) ;

  if iscell(block.features)
    for i=1:length(block.features)
      block.features{i} = block.features{i}(:, idx_pos) ;
    end ;
  else
    block.features = block.features(:, idx_pos, :) ;
  end ;
  block.content_pred = block.content_pred(:, idx_pos) ;

  % update seg_path in truth (this is all_pos based)
  for t=1:length(block.truth)
    block.truth(t) = gen_loss_matrix(block.truth(t), block.all_pos, block.gene_status, model) ;
  end ;
else
  idx_pos = 1:length(block.all_pos) ;
end ;

% generate full feature matrix, if still in sparse format
if iscell(block.features)
  block.features = full_features(block.features);
end

[penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model, block.split, PAR.FN, parameters, PAR.max_len) ;

loss = model.loss.segments ;
if PAR.prediction,
  loss(:)=0 ;
  seg_path = zeros(2,length(block.all_pos)) ;
else
  %block.truth(1) = gen_loss_matrix(block.truth(1), block.pos, ...
  %                                 block.gene_confirmed, model) ;
  
  seg_path = block.truth(1).seg_path;
  % if the alternatives disagree, then don't incur a loss
  for i=2:length(block.truth)
    idx = find(block.truth(i).seg_path(1,:) ~= block.truth(1).seg_path(1,:)) ;
    seg_path(2,idx) = 0 ;
  end ;
end ;

if 0,
  if keyboard_allowed(), keyboard ; end ;
  old_model = model ;
  old_a_trans = a_trans ;
  old_block = block ;
  old_state_signals = state_signals ;
  
  [model, a_trans, block, state_signals] = duplicate_states4transcript_loss(model, a_trans, block, state_signals) ;
end ;

% SHOGUN interface
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  use_orf = 1;
  num_svms = 8;
  sg_set_plif(penalty_array)
  sg('init_dyn_prog', num_svms)
  if isfield(block, 'content_pred')
    block.content_pred(end+1:num_svms,:) = deal(0);
    sg('set_lin_feat', block.seq, int32(block.all_pos-1), block.content_pred);
  else
    sg('precompute_content_svms',block.seq,int32(block.all_pos-1), content_weights');
  end
  sg('set_model', model.transition_pointers, use_orf, int32(model.mod_words), int32(state_signals),int32(model.orf_info))
  sg('set_feature_matrix', block.features)
  if isfield(model, 'tiling')
    sg('precompute_tiling_features',double(block.intensities), int32(block.probe_local_pos-1), int32(model.tiling_plif_ids-1))
  end
  if isfield(model, 'rna_seq')
    if isfield(block, 'coverage')
      sg('precompute_tiling_features',double(block.coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1))
    elseif ~isfield(model, 'rna_seq_intron_track')% hack for backward compatibility
    sg('precompute_tiling_features',double(block.intron_coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1)) 
    end
  end
  if isfield(model, 'rna_seq_intron_track')
    sg('precompute_tiling_features',double(block.intron_coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_intron_track_plif_ids-1))
  end
  if length(PAR.viterbi_nbest)<2,
    PAR.viterbi_nbest(2) = 0 ;
  end ;
  [path_scores, path, ppos]= sg('best_path_trans', model.p', model.q', int32(PAR.viterbi_nbest), seg_path, a_trans, loss);


% very important check to assert that feature generation and veterbi decoding are equivalent
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~(isfield(PAR, 'skip_checks') && isequal(PAR.skip_checks,1)), 
  path_ = path(1,ppos(1,:)~=-1);
  ppos_ = ppos(1,ppos(1,:)~=-1);
  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', model.p', model.q', seg_path, a_trans, loss, int32(path_), int32(ppos_));
  %%% test
  A = model.A ;
  A(~isinf(A)) = parameters.transitions ;
  A(isinf(A)) = 0 ;
  score = sum(sum(a_deriv.*A))+sum(losses);
  for i=1:size(penalty_deriv,1)
    score = score + sum(penalty_deriv(i,:).*penalty_array{i}.penalties);
  end ;
  assert(all(losses>=0))
  if ~(abs(score-path_scores(1))<1e-4)
    fprintf('unexpected score differences: %1.3f %1.3f (diff=%1.3f)\n', ...
            score, path_scores, score-path_scores) ;
    if keyboard_allowed(),
      keyboard ;
    end ;
  end ;
end ;

pred = []; loss=[] ;
for i=1:sum(PAR.viterbi_nbest)
  pred(i).loss = nan ;
  pred(i).path = path(i,ppos(i,:)~=-1)+1;
  pred(i).pos_idx = ppos(i,ppos(i,:)~=-1)+1 ;
  pred(i).pos = block.all_pos(pred(i).pos_idx);
  [pred(i).segments, pred(i).genes] = path2segmentation(pred(i).path, pred(i).pos_idx, block, model);
  if ~PAR.prediction
    [pred(i).weights, pred(i).loss, pred_x(i).score, pred_x(i).losses, pred_x(i).scores, pred_x(i).ok] = compute_weights(pred(i), block, model, PAR.FN, parameters) ;   
    if isfield(PAR, 'approximate') && isequal(PAR.approximate, 1),
      if keyboard_allowed(),
        %keyboard
      end ;
    end ;
  end ;
  % undo position subseting in index to positions (done for approximation)
  pred(i).pos_idx = idx_pos(pred(i).pos_idx) ;
end
pred = orderfields(pred) ;

%disp(['saving pred to ' PAR.fn_save]) ;
cPAR = L.cPAR;
save(PAR.fn_save, 'pred', 'PAR', '-v7')

return



return

function features = full_features(sp)
% features = full_features(sp)

assert(length(sp)==2)
assert(iscell(sp))
assert(isequal(size(sp{1}),size(sp{2})))

features = full(sp{1});
features(:,:,2) = full(sp{2});

features(features==0) = -inf;
features = features-1;

return

function [penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, FN,  parameters, max_len) ;
% [penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, parameters, max_len) ;
  
if nargin<4 || isempty(parameters)
  use_param = 0 ;
else
  use_param = 1 ;
end ;

content_names = fieldnames(model.contents) ;
%idx=sort_numeric_fields(model.contents) ;
%content_names = content_names(idx) ;

length_names = fieldnames(model.lengths) ;
%idx=sort_numeric_fields(model.lengths) ;
%length_names = length_names(idx) ;

signal_names = fieldnames(model.signals) ;
%idx=sort_numeric_fields(model.signals) ;
%signal_names = signal_names(idx) ;

if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling);
end
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq);
end
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate penalty_array. set plif penalties to zero 

if nargin<5 || isempty(max_len)
  max_len = 0 ;
  for s = 1:model.cnt_lengths,
    len = getfield(model.lengths_range, length_names{s}) ;
    if len(2)>max_len,
      max_len = len(2) ;
    end ;
  end ;
end ;

penalty_array = {} ;
content_weights = [] ;
num_svms = 0 ;
for s = 1:model.cnt_contents,
  pen.id        = getfield(model.contents, content_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['contents_' content_names{s}] ;
  pen.limits    = getfield(model.boundaries.contents, content_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  % keyboard
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  num_svms = num_svms + 1 ;
  pen.use_svm   = num_svms ;
  if strcmp(content_names{s},'dummy')
    content_weights(num_svms, :) = deal(0);
  else
    %fn_content_all_models = [FN.output_cont.(content_names{s}).fn_SVMs '_partition=' num2str(cont_subset_id) '.mat'];
    if ~isempty(FN) && isfield(FN, 'output_cont') && isfield(FN.output_cont, content_names{s})
      fn_content_all_models = FN.output_cont.(content_names{s}).fn_SVMs;
    else
      fn_content_all_models = '';
    end
    if ~fexist(fn_content_all_models) || exist(fn_content_all_models, 'dir') 
      %fprintf('file %s with content svms not found, use zeros as weights\n', fn_content_all_models);
    elseif ~ismember('w', who_file(fn_content_all_models)), 
      %fprintf('no weight vector found in file %s, use zeros as weights\n', fn_content_all_models);
    else
      l = load(fn_content_all_models, 'w');
      content_weights(num_svms, :) = l.w;
    end
    clear l
  end
  penalty_array{end+1} = pen ;
end ;

if size(content_weights,1)<8
  % current shogun requirement
  content_weights(end+1:8,:)=0 ;
end ;

for s = 1:model.cnt_lengths,
  pen.id        = getfield(model.lengths, length_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['lengths_' length_names{s}] ;
  pen.limits    = getfield(model.boundaries.lengths, length_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = 'log(+1)' ;
  len=getfield(model.lengths_range, length_names{s}) ;
  pen.min_value   = len(1) ;
  pen.max_value   = min(len(2), max_len) ;
  pen.use_cache = 1 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;

for s = 1:model.cnt_signals,
  pen.id        = getfield(model.signals, signal_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['signals_' signal_names{s}] ;
  pen.limits    = getfield(model.boundaries.signals, signal_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  pen.penalties = zeros(1,length(pen.limits)) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;

if isfield(model,'tiling')
  for s = 1:model.cnt_tiling,
    pen.id        = getfield(model.tiling, tiling_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['tiling_' tiling_names{s}] ;
    pen.limits    = getfield(model.boundaries.tiling, tiling_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end
if isfield(model,'rna_seq')
  for s = 1:model.cnt_rna_seq,
    pen.id        = getfield(model.rna_seq, rna_seq_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['rna_seq_' rna_seq_names{s}] ;
    pen.limits    = getfield(model.boundaries.rna_seq, rna_seq_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end
if isfield(model,'rna_seq_intron_track')
  for s = 1:model.cnt_rna_seq_intron_track,
    pen.id        = getfield(model.rna_seq_intron_track, rna_seq_intron_track_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['rna_seq_intron_track_' rna_seq_intron_track_names{s}] ;
    pen.limits    = getfield(model.boundaries.rna_seq_intron_track, rna_seq_intron_track_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num_signals=0;
for i=1:model.cnt_states
  num_signals=max([num_signals length(model.states(i).signal)]);
end

state_signals=zeros(model.cnt_states, num_signals) ;
for i=1:model.cnt_states,
  state_signals(i,1:length(model.states(i).signal))=model.states(i).signal ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


A = model.A ;
if use_param
  A(~isinf(A)) = parameters.transitions ;
  model.A = A;
end

a_trans = model.a_trans ;
k=0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  assert(all(a_trans(1,k+1:k+length(idx))==i-1)) ;
  assert(all(a_trans(2,k+1:k+length(idx))==idx-1)) ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  k=k+length(idx) ;
end ;
a_trans=a_trans' ;
[tmp,idx]=sort(a_trans(:,2)) ;
a_trans = a_trans(idx,:) ;


function idx=sort_numeric_fields(s)
% idx=sort_numeric_fields(s)

names=fieldnames(s) ;
values=[] ;
for i=1:length(names),
  values(i) = s.(names{i}) ;
end ;
[tmp,idx]=sort(values) ;

return

function [segments,genes,ok] = path2segmentation(my_path,my_pos_idx,block, model)
% [segmentation,ok] = path2segmentation(path,pos_idx,block, model)
%pos = [0     length(block.all_pos)-1] ;
%path = [model.state_ids.seq_start    model.state_ids.seq_end]-1 ;
  
segment_names = fieldnames(model.segments) ;
segments = [] ;
ok = 1 ;
genes = {} ;

my_pos_idx(my_path==getfield(model.state_ids,'intergenic_long'))=[];
my_path(my_path==getfield(model.state_ids,'intergenic_long'))=[];
% same for intron long
if isfield(model.state_ids,'intron_long')
  for i=getfield(model.state_ids,'intron_long')
    del_idx=my_path==i;
    my_pos_idx(del_idx)=[];
    my_path(del_idx)=[];
  end
end
for i=1:length(my_path)-1
  lengths_idx = model.transition_pointers(my_path(i+1), my_path(i)) ;
  if my_path(i+1) == model.state_ids.tis(2),
    if ~(my_path(i) == model.state_ids.don(1)) ;
      error('invalid path') ;
      %warning('invalid path') ;
      ok=0 ;
      return ;
    end ;
    % handle the agatg-case properly 
    segment_type = model.seg_links(find(model.seg_links(:,1)==lengths_idx),2) ;
    segments = [segments; [block.all_pos(my_pos_idx(i)) block.all_pos(my_pos_idx(i+1)), model.segments.intron ]] ;
    continue ;
  end ;
  
  if lengths_idx~=0
    segment_type = model.seg_links(find(model.seg_links(:,1)==lengths_idx),2) ;
    % hack
    if isfield(model.segments,'intercistronic')&&segment_type==model.segments.intercistronic & ...
          block.all_pos(my_pos_idx(i+1)) - block.all_pos(my_pos_idx(i)) > model.lengths_range.intercistronic(2),
      segment_type = model.segments.intergenic ;
    end 
  else
    segment_type = 0 ;
    warning('unable to decode path') ;
    ok = 0 ;
  end ;
  segments = [segments; [block.all_pos(my_pos_idx(i)) block.all_pos(my_pos_idx(i+1)),segment_type ]] ;
end

segments(1,1) = 1;
segments(end,2) = length(block.seq);
idx =  find(segments(:,3) ==getfield(model.segments,'intergenic'));
if isfield(model.segments,'intercistronic')
  idx = [idx find(segments(:,3) ==getfield(model.segments,'intercistronic'))];
end
n_genes = length(idx)-1 ;
genes{1} = [];
for i=1:n_genes
  genes{i} = segments(idx(i)+1:idx(i+1)-1,:);
end

return

function [weights, loss, score, losses, scores, ok] = compute_weights(SEG, block, model, FN, parameters, check_result) ;
% weights=compute_weights(segmentation, block, model) ;

if nargin<5
  parameters = [];
end
if nargin<6
  check_result = 1 ;
end
ok=true ;
[penalty_array, content_weights, state_signals, a_trans] = gen_penalty_array(model, block.split, FN, parameters) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%for i=1:length(segmentation.genes)
%  segmentation.genes{i}(1,3)=model.segments.utr5exon ;
%  segmentation.genes{i}(end,3)=model.segments.utr3exon ;
%end ;

% simple state sequence -> generate this from segmentation
% if ~isfield(SEG,'path') | isfield(SEG,'pos_idx') 
%   [my_path,pos_idx,ok]=segmentation2path(SEG.segmentation, block, model);
% else
%   my_path = SEG.path ;
%   pos_idx = SEG.pos_idx
% end
my_path = SEG.path ;
pos_idx = SEG.pos_idx ;

% keyboard
% sg('send_command', 'loglevel WARNING') ;
% sg('send_command', 'loglevel ALL') ;
%model.loss.segments(:)=1 ;
%model.loss.segments(:,9:16)=2 ;
%keyboard

%[my_path,pos_idx,pos,ok]=segmentation2path(SEG, block, model) ;

seg_path = block.truth(1).seg_path;
% if the alternatives disagree, then don't incur a loss
for i=2:length(block.truth)
  if isempty(block.truth(i).seg_path), continue ; end ;
  idx = find(block.truth(i).seg_path(1,:) ~= block.truth(1).seg_path(1,:)) ;
  seg_path(2,idx) = 0 ;
end ;
if iscell(block.features)
  features = full_features(block.features);
else 
  features = block.features;
end

if isfield(model, 'use_conservation') && model.use_conservation
  error('not yet implemented')
else
  use_orf=1;
  num_svms=8;
  sg_set_plif(penalty_array)
  sg('init_dyn_prog', num_svms)
  if isfield(block, 'content_pred')
    block.content_pred(end+1:num_svms,:) = deal(0);
    sg('set_lin_feat', block.seq, int32(block.all_pos-1), block.content_pred);
  else
    sg('precompute_content_svms',block.seq,int32(block.all_pos-1), content_weights');
  end
  sg('set_model', model.transition_pointers, use_orf, int32(model.mod_words), int32(state_signals),int32(model.orf_info))
  sg('set_feature_matrix',features)
  if isfield(model, 'tiling')
    sg('precompute_tiling_features',block.intensities, int32(block.probe_local_pos-1), int32(model.tiling_plif_ids-1))
  end
  if isfield(model, 'rna_seq')
    sg('precompute_tiling_features',block.coverage, int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1))
  end
  if isfield(model, 'rna_seq_intron_track')
    sg('precompute_tiling_features',block.intron_coverage, int32(0:length(block.seq)-1), int32(model.rna_seq_intron_track_plif_ids-1))
  end

  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', model.p', model.q', seg_path, a_trans, model.loss.segments, int32(my_path-1), int32(pos_idx-1));
%  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', my_path-1, pos_idx-1, model.p', ...
%        model.q',a_trans, block.features, block.all_pos-1, lower(block.seq)',model.transition_pointers, state_signals, ...
%	penalty_array, content_weights', model.mod_words, model.loss.segments, seg_path) ;
%
end ;

loss = sum(losses) ;
score = sum(scores) ;

if isempty(parameters) && check_result
  if ~sum(scores)==0
    idx = find(scores)
    pos_idx(idx(1)+[-1:1]) 
    block.all_pos(pos_idx(idx(1)+[-1:1]))
    my_path(idx(1)+[-1:1])
    block.id
    error('invalid path: sum(scores) ~= 0')
    %weights = [];
    %ok=false ;
    %return
  end
else
  if check_result,
    assert(~isinf(sum(scores))) ;
    if isinf(sum(scores)), 
      ok=false; 
    end ;
  end ;
end

weights = init_weights(model) ;
for i=1:size(penalty_deriv,1)
  pen = penalty_array{i} ;
  weights = setfield(weights, pen.name, penalty_deriv(pen.id,:)) ;
end ;

%%% transitions
weights.transitions(:) = a_deriv(model.active_transitions) ;



%%%% for debugging
if 0 
  dbup
  dbstack
  
  idx = find(isinf(scores))
  
  SEG.path(idx:idx+1)
  
  %%%check state

  block.seq(SEG.path(idx),SEG.pos_idx(idx))
  %%%check transition 
  
  trans_lengths = SEG.pos(idx+1)-SEG.pos(idx:idx+1)
  idx2 = find(ismember(SEG.segments(:,1:2),SEG.pos(idx:idx+1),'rows'));
  seg_type = SEG.segments(idx2,3)
  
  range = model.lengths_range
end

return

function sg_set_plif(penalty_array)
% sg_set_plif(penalty_array)

for j=1:length(penalty_array)
  all_ids(j)		= penalty_array{j}.id;
  all_names{j} 		= penalty_array{j}.name;
  all_limits(:,j) 	= penalty_array{j}.limits;
  all_penalties(:,j) 	= penalty_array{j}.penalties;
  if isempty(penalty_array{j}.transform)
    all_transform{j} 	= 'linear';
  else
    all_transform{j} 	= penalty_array{j}.transform;
  end
  all_min_values(j) 	= penalty_array{j}.min_value;
  all_max_values(j) 	= penalty_array{j}.max_value;
  all_use_cache(j) 	= penalty_array{j}.use_cache;
  all_use_svm(j) 	= penalty_array{j}.use_svm;
  all_do_calc(j) 	= 1;%penalty_array{j}.do_calc;
end

sg('set_plif_struct',int32(all_ids)-1,all_names, all_limits, all_penalties, all_transform, all_min_values, all_max_values, int32(all_use_cache), int32(all_use_svm), int32(all_do_calc)); 
[ret_ids,ret_names, ret_limits, ret_penalties, ret_transform, ret_min_values, ret_max_values, ret_use_cache, ret_use_svm, ret_do_calc]= sg('get_plif_struct');

[engine, env]=determine_engine() ;

assert(isequal(ret_do_calc, all_do_calc))
assert(isequal(ret_use_svm, all_use_svm))
assert(isequal(ret_use_cache, all_use_cache))
assert(isequal(ret_max_values, all_max_values))
assert(isequal(ret_min_values, all_min_values))
if ~isequal(engine, 'octave'),
  assert(isequal(ret_transform', all_transform))
end ;
assert(isequal(ret_penalties, all_penalties))
assert(isequal(ret_limits, all_limits))
if ~isequal(engine, 'octave'),
  assert(isequal(ret_names', all_names))
end ;
assert(isequal(ret_ids+1, all_ids))


return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function model=fix_model(model) ;
% model = fix_model(model) ;

model.contents = reorder_fields(model.contents, model.contents_fields) ;
model.lengths = reorder_fields(model.lengths, model.lengths_fields) ;
model.signals = reorder_fields(model.signals, model.signals_fields) ;
model.segments = reorder_fields(model.segments, model.segments_fields) ;

return

function n = reorder_fields(s, fields)
% n = reorder_fields(s, fields)

  if isempty(s),
    n=s ;
    return ;
  end ;

  n=struct ;
  for i=1:length(fields)
    for j=1:length(s) ;
      n(j).(fields{i}) = s(j).(fields{i}) ;
    end ;
  end ;

  remaining = setdiff(fieldnames(s), fields) ;

  if ~isempty(remaining)
    remaining
    error('these fields are unassigned');
  end ;


return

function blocks = fix_blocks(blocks, model) ;
% blocks = fix_blocks(blocks, model) ;

assert(isequal(model, fix_model(model))) ;
w_fields = fieldnames(init_weights(model)) ;

for i=1:length(blocks),
	if isfield(blocks, 'truth')
		for j=1:length(blocks(i).truth),
			blocks(i).truth(j).weights = reorder_fields(blocks(i).truth(j).weights, w_fields) ;
        	end ;
	end
	if isfield(blocks, 'pred')
		for j=1:length(blocks(i).pred),
			blocks(i).pred(j).weights = reorder_fields(blocks(i).pred(j).weights, w_fields) ;
        	end ;
	end
end ;

return

function truth = gen_loss_matrix(truth, all_pos, gene_confirmed, model)
% truth = gen_loss_matrix(truth, all_pos, gene_confirmed, model)
% gene_confirmed: -1: predicted
%                  0: partially confirmed
%                  1: confirmed

% path = truth.path;
% pos = truth.pos;
% seg_path = ones(2,length(all_pos));
% [Pos,idx1,idx2]=intersect(all_pos,pos);
% st=1;
% for i = 2:length(idx1)
%   ii = find(model.a_trans(1,:)==(path(st));
%   ii2 = find(model.a_trans(2,ii)==(path(i));
%   seg_path(st:idx(i)) = model.a_trans(3,ii(ii2));
%   ii = find(all_pos>= Pos(i)- model.loss.zero_range(path(st),path(i))...
%              &all_pos<= Pos(i)+ model.loss.zero_range(path(st),path(i));
%   seg_path(2,idx1(i)) = 1; 
% end

seg = truth.segments;
%seg_path = ones(2,length(all_pos));
seg_path = zeros(2,max(all_pos));

for i=1:size(seg,1)
  seg_path(1, seg(i,1):seg(i,2)-1) = seg(i,3) ;
end ;

% confirmed genes are worth more than unconfirmed ones
seg_path(2,:) = model.loss.unconfirmed ;

gene_idx = setdiff(unique(truth.segments(:,4)),0);
for j=1:length(gene_idx)
  idx = find(truth.segments(:,4)==gene_idx(j));
  genes{j} = truth.segments(idx,1:3) ;
end

assert(length(genes)==length(gene_confirmed)) ;
for i=1:length(genes)
  if gene_confirmed(i)==1%confirmed
    start = genes{i}(1,1) ;
    stop  = genes{i}(end,2) ;
    seg_path(2, max(1,start-model.loss.confirmed_tol) : min(size(seg_path,2), stop+model.loss.confirmed_tol) ) = model.loss.confirmed ;
  elseif gene_confirmed(i)==0%partially confirmed 
    start = genes{i}(1,1) ;
    stop  = genes{i}(end,2) ;
    seg_path(2, max(1,start-model.loss.confirmed_tol) : min(size(seg_path,2), stop+model.loss.confirmed_tol) ) = (model.loss.confirmed+model.loss.unconfirmed)/2 ;
  end ;
end ;

pos = truth.pos;
[Pos,idx1,idx2]=intersect(all_pos, pos);
assert(length(Pos)==length(pos)) ;
signal_names = fieldnames(model.signals) ;
for i = 1:length(idx1)
  signal = model.states(truth.path(idx2(i))).signal ;
  if length(signal)~=1, continue ; end ;
  found = 0 ;
  for j=1:model.cnt_signals,
    if getfield(model.signals, signal_names{j})==signal
      signal = j ;
      found = 1 ;
      break ;
    end ;
  end ;
  assert(found~=0) ;
  tol = getfield(model.loss.zero_range, signal_names{signal}) ;
  if tol==0, continue ; end ;
  idx = max(1,(Pos(i) - tol)):min((Pos(i)+ tol),size(seg_path,2)) ;
  seg_path(2,idx) = 0 ; 
end

seg_path = seg_path(:, all_pos) ;

truth.seg_path = seg_path ;

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function user_name = whoami
% user_name = whoami
  
global whoami_user_name  
  
if isempty(whoami_user_name)
  
  [ret, whoami_user_name]=unix('whoami') ; 
  if ret~=0,
    if keyboard_allowed(),
      keyboard ;
    end ;
    whoami_user_name='unknown' ;
  end ;
end ;

user_name= whoami_user_name ;
idx=find(user_name<30, 1, 'first') ;
user_name = user_name(1:idx-1) ;

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function list=who_file(fname) ;
% list=who_file(fname) ;
  
engine = determine_engine() ;
if isequal(engine, 'matlab')
  list=who('-file', fname) ;
else
  L=load(fname) ;
  list=fieldnames(L) ;
end ;

return

