%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function x = load_and_train(P, Train_Data)

% load_and_train(P)
%
% INPUT: P with fields:   
%         - fn_set,             file name of sets to be loaded 
%         - SETs.which_svm,     specifies the sets on which training is
%                               performed (e.g [1 2 3])
%         - SETs.train_subsample_neg  number between 0-1: for subsampling of negative examples, 
%                                0: setting label of examples labeled with 0 to -1   
%                               -1: removing examples labeled with 0 
%                               default(1) no subsampling 
%         - SETs.train_subsample_pos  number between 0-1: for subsampling of positive examples,                             
%                               default(1) no subsampling 
%         - par,                window around consensus to be used  
%         - use_cons            specifies if conservation is to be used
%         - fn_SVMs             file nmame to save results  
%  
%   
% OUTPUT: Train_Data with fields:
%  (XT,LT,SC,pos,reg,str,P)
%  saved to file P.fn_SVMs
  
%if( ~P.method.submit_batch )
%  paths;
%end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SETs = P.SETs;
Signal = P.Signal;
fn_filter_settings = P.fn_filter_settings;
fn_examples = P.fn_examples;

which_train = SETs.which_train;
train_fct = Signal.method.train_fct;

if ~isfield(SETs,'train_subsample_pos') && isfield(SETs,'subsample_pos')
  subsample_pos = SETs.subsample_pos ;
  error('warning: please rename field from subsample_pos to train_subsample_pos') ;
else
  subsample_pos = SETs.train_subsample_pos ;
end

if ~isfield(SETs,'train_subsample_neg') && isfield(SETs,'subsample_neg')
  subsample_neg = subsample_neg ;
  error('warning: please rename field from subsample_neg to train_subsample_neg') ;
else
  subsample_neg = SETs.train_subsample_neg ;
end

if ~exist('Train_Data')
  Train_Data = feval(Signal.load_fct,Signal,fn_filter_settings,fn_examples, which_train, 'train',subsample_neg, subsample_pos);
end
Train_Data.P = P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% TRAIN 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t = cputime;
feval(train_fct,Train_Data);
fprintf('\ntraining took %.2f min\n',(cputime-t)/60);
% if PAR.method.name=='SVM'
%   tssTrain(Train_Data)
% elseif PARn.method.name=='PWM'
%   hmm_train(Train_Data)
% else 
%   error('unknown method');
% end

% dummy return argument
x=0;

return

function [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(PAR, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% method = PAR.method;
% Signal = PAR.Signals.(PAR.Signal_name);
%  FN = PAR.FN;
% input_sig = FN.input_sig.(Signal_name);
% fn_filter_settings = input_sig.fn_filter_settings;
% fn_examples = input_sig.fn_examples;

method = Signal.method;

XT = []; LT = [] ; POS.region_id = []; POS.pos=[]; POS_tmp = []; 
SC = []; use_sc = [];
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = Signal.method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT');
  L.XT = load_matrix(filename, 'XT');
  P = load(filename, 'POS');
  %if isfield(signal,'center')
  %  par.center = signal.center;
  %else
  %  par.center = size(L.XT,1)./2;
  %end
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;
  have_sc = isfield(L, 'SC_') ;
  if ~have_sc, L.SC_=[] ; end ;  

  [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  if ~have_sc, L=rmfield(L, 'SC_') ; end ;
  
  if ~isfield(method,'feature_type') || isempty(method.feature_type) ||isequal(method.feature_type, 'char')
    L.XT_char = char(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = char(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'uint8')
    L.XT_char = uint8(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = uint8(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'double')
    XT = [XT L.XT];
  else
    error('unknown feature type; should be "char" or "double"')
  end
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  if (isfield(L,'SC_')&& isfield(L.use_sc)) || use_cons
    fprintf('using consensus information\n');
    SC = [SC L.SC_] ;
    use_sc = [use_sc L.use_sc];
  end

  POS.region_id =[POS.region_id P.POS.region_id];
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
% check whether everything is ok (matlab bug successfully avoided?)
for i=1:size(XT,1)
  if all(XT(i,:)==0)
    % i
    % keyboard
  end
  if  ~isfield(method,'feature_type') || isequal(method.feature_type, 'char')
    assert(all(XT(i,:)~=0)) ;
  end
end ;

clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end

if ~isempty(SC)
  XT(:,~use_sc) = [] ;
  POS(~use_sc) =[];
  LT(~use_sc) = [];
  SC(:,~use_sc) = [];
  if ~isempty(LT)
    fprintf('%i positive, %i negative examples (%.2f %% positives) \n',sum(LT==1),sum(LT==-1),sum(LT==1)/sum(LT==1|LT==-1)*100);
  end
end

Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS
%this assert is very mem consuming
%assert(~any(XT(:)~='A' & XT(:)~='G' & XT(:)~='C' & XT(:)~='T')) ;
% XT = to_upper_case(char(XT)) ;
if ~isempty(SC)
  Data.SC = char(SC) ;
  assert(~any(SC(:)~='A' & SC(:)~='G' & SC(:)~='C' & SC(:)~='T')) ;
  
  % SC = to_upper_case(char(SC)) ;
end


assert(size(Data.XT,2)==length(Data.LT))
assert(length(Data.LT)==length(Data.Pos.pos))
assert(length(Data.LT)==length(Data.Pos.region_id))

return

function init_svm_and_kernels(method,pred_from_seq)
% init_svm_and_kernels(method,pred_from_seq)
  
  
  conservWdsWeighting =0;
  %------------------------------------------------------------------------------%
  % init svm
  %------------------------------------------------------------------------------%
  
  % === cleaning
  sg( 'clean_features', 'TRAIN' );
  sg( 'clean_preproc' );
  sg( 'clean_kernel');
  %clear sg;
  
  %sg( 'loglevel', 'ALL');   % DEBUG
  if ~isfield(method, 'svm_name')
    warning( 'no svm specified, set to default: LIGHT')
    method.svm_name='LIGHT'
  end
  
  % === SVM
  sg( 'new_svm', method.svm_name);
  %sg( 'svm_epsilon', 1e-4 );
  %%%sg( 'svm_epsilon', PAR.svmEps);
  sg('svm_epsilon', 1e-4);
  %%%sg( 'send_command', sprintf( 'use_linadd %d', PAR.useLinadd ) );
  sg('use_linadd', true);
  %%%sg( 'svm_qpsize', PAR.qpSize );
  if ~isfield(method, 'qpsize')
    warning( 'qpsize not specified, set to default: 50')
    method.qpsize=50;
  end
  sg('svm_qpsize', method.qpsize) ; 
  if( conservWdsWeighting > 0 )
    sg('use_batch_computation', false);
  else
    sg('use_batch_computation', true);
  end
  
  % === parallel training?
  if isfield(method,'threads')
    sg('threads', method.threads);
  else
    fprintf('number of threads not specified, set to default: 16')
    sg('threads', 16);
  end;
  
  
  %------------------------------------------------------------------------------%
  % init kernels
  %------------------------------------------------------------------------------%
  
  nofKernels = length(method.kernels);
  if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
    % for content sensors; several orders are trained simulatuously.
    % but this is done by concatination of the feature vector directly.
    nofKernels = 1;
  end
  
  assert( nofKernels >= 1 );
  if( nofKernels == 1 )
    addKernel = {'set_kernel'};
  else
    addKernel = {'add_kernel', 1};
    sg( 'set_kernel', 'COMBINED', 100);
  end;
  
  for k = 1:nofKernels
    kernel = method.kernels{k};
    % for compatibility with older versions
    if ~isfield(kernel,'kernel_name') && isfield(kernel, 'name')
      kernel.kernel_name = kernel.name;
    end
    fprintf(1, '%s: %s\n', sprintf('%s %1.1f', addKernel{:}), kernel.kernel_name)
    
    switch kernel.kernel_name
     case 'WD'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      sg( addKernel{:}, 'WEIGHTEDDEGREE', 'CHAR', 10, kernel.order, kernel.mismatch, true, 1) ;
      
     case 'WDS'
      if ~isfield(kernel, 'mismatch')
        kernel.mismatch=0;
      end
      if ~isfield(kernel, 'shift')
        kernel.shift=0;
      end
      if ~isfield(kernel, 'shift_const')
        kernel.shift_const=0;
      end
      assert(length(kernel.order)==1)
      assert(length(kernel.mismatch)==1)
      len = kernel.rwin-kernel.lwin+1;
      shifts = ceil(kernel.shift * abs( (kernel.lwin:kernel.rwin)+1 ));
      shifts = shifts + kernel.shift_const;
      %shifts = sprintf( '%i ', shifts );
      sg( addKernel{:}, 'WEIGHTEDDEGREEPOS3', 'CHAR', 10, kernel.order, kernel.mismatch, len, 1, int32(shifts)) ;
      clear shifts len
      
     case 'SP'
      use_sign = false;
      % use_sign and prediction from position lists does not work
      %assert(~use_sign|~PAR.tasks.pred_from_seq)
      
      %addKernel_ = {'add_kernel', 0};
      
      sg( addKernel{:}, 'COMMSTRING', 'WORD', 10, use_sign);
      if pred_from_seq
        %sg('loglevel','ALL')
        sg( 'use_diagonal_speedup',true)
      end
     case 'RBF'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      assert(length(kernel.sigma)==1)
      sg( addKernel{:}, 'GAUSSIAN', 'REAL', kernel.cache, kernel.sigma) ;
      
     case 'POLY'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end 
      if ~isfield(kernel, 'norm')
        kernel.type = 'R';
      end
      assert(length(kernel.degree)==1)
      sg( addKernel{:}, 'POLY', Kernel.type, kernel.cache, kernel.degree, kernel.norm) ;
      
     case 'LINEAR'
      if ~isfield(kernel, 'cache')
        kernel.cache = 40;
      end
      if ~isfield(kernel, 'type')
        kernel.type = 'REAL';
      end
      sg( addKernel{:}, 'LINEAR', kernel.type, kernel.cache) ;
      
     otherwise
      fprintf(1, 'not yet implemented: kernel_name: %s\n', kernel.kernel_name)
    end;
    
  end;%loop over kernels
  
  if isfield(method, 'subkernel_weights') 
    assert(length(method.subkernel_weights)==nofKernels)
    sg('set_subkernel_weights', method.subkernel_weights);
  end
  
  % for compatibility with older versions
  if ~isfield(method.par_ms,'C') && isfield(method.kernels{1}, 'C')
    method.par_ms.C = method.kernels{1}.C;
  end
  sg('c', method.par_ms.C) ;
  
  disp('done with kernel setup');

return

function setFeatures(method, Data, svmMode, pred_from_seq, use_preproc);
% setFeatures(method,Data, svmMode,pred_from_seq,use_preproc);


% method = PAR.method;
% tasks = PAR.tasks;
% pred_from_seq = tasks.pred_from_seq;

  
%------------------------------------------------------------------------------%
% check
%------------------------------------------------------------------------------%

assert( strcmp(svmMode,'TRAIN') || strcmp(svmMode,'TEST') );

%assert( isscalar(Data.pos0) );
%assert( size(XT,2) > 0 );
%assert( ischar( XT ) );

if nargin<5
  use_preproc=1;
end
if ~isfield(method,'conservWdsWeighting')
  conservWdsWeighting=0;
else
  conservWdsWeighting= method.conservWdsWeighting;
end
%------------------------------------------------------------------------------%
% pass data
%------------------------------------------------------------------------------%

%sg('loglevel', 'ALL');
if pred_from_seq
  seq = Data.seq;
  idx = find(seq~='A'&seq~='C'&seq~='G'&seq~='T');
  if ~isempty(idx)
    warning('non AGCT-Symbols found; changed to As')
    seq(idx)='A';
  end
  pos = Data.pos;
else
  XT = Data.XT;
  %pos0=Data.pos0;
  if isfield(Data, 'LT')
    LT = Data.LT;
  end
  if isfield(Data, 'W')
    W = Data.W;
  end
end

%==============================================================================%
% set labels
%==============================================================================%
if strcmp(svmMode,'TRAIN')
  sg( 'set_labels', 'TRAIN', LT ); 
end
%==============================================================================%
% set features
%==============================================================================%



nofKernels = length(method.kernels);
if nofKernels>1 && ~isfield(method.kernels{2},'name') && ~isfield(method.kernels{2}, 'kernel_name')
  % for content sensors; several orders are trained simulatuously.
  % but this is done by concatination of the feature vector directly.
  nofKernels = 1;
end
assert( nofKernels >= 1 );
if( nofKernels == 1 )
  addFeats = 'set_features';
else
  addFeats = 'add_features';
end;


for k = 1:nofKernels
  kernel = method.kernels{k};
  % for compatibility with older versions
  if ~isfield(kernel, 'kernel_name') && isfield(kernel,'name')
    kernel.kernel_name = kernel.name;
  end
  if ~pred_from_seq   
    if ~isfield(method,'center') || isempty(method.center)
      select= [1:size(XT,1)];
    else
      if ~isfield(kernel,'lwin') 
        warning('no lwin specified for kernel')
        kernel.lwin = -method.center;
      end
      if ~isfield(kernel,'rwin') 
        warning('no rwin specified for kernel')
        kernel.rwin = -method.center+size(XT,1)-1;
      end
      select = (kernel.lwin:kernel.rwin)+method.center+1;
    end
    if ~isequal(select,[1:size(XT,1)])
      XT_temp = XT(select,:);
    else
      XT_temp = XT;
    end
    %keyboard;
  else
    %positiones should be a 0-based index
    pos_temp=pos+kernel.lwin-1;
    winsize_temp=kernel.rwin-kernel.lwin+1;
    assert(length(pos_temp)>0)
    assert(length(seq)>max(pos_temp)+winsize_temp)
    assert(winsize_temp>0)

    % work-around for from_position_list bug
    if length(pos_temp)==1,
      pos_temp = [pos_temp pos_temp] ;
    end ;
  end
  %------------------------------------------------------------------------------%
  % kernel: WD or WDS
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'WD') || strcmp(kernel.kernel_name,'WDS')
    if ~pred_from_seq
      fprintf( '--> XT[%d:%d], num_examples=%i,  order=%d\n', min(select), max(select),size(XT,2), kernel.order );
      sg( addFeats, svmMode, XT_temp, 'DNA' );
    else
      sg( addFeats, svmMode,{seq}, 'DNA' );
      sg('from_position_list','TEST', winsize_temp, int32(pos_temp));
      % NEW
      %sg( addFeats, svmMode, {seq}, 'DNA', 'from_position_list', winsize_temp, int32(pos_temp), 1);
    end
  end;
  

  %------------------------------------------------------------------------------%
  % kernel: Spectrum
  %------------------------------------------------------------------------------%

  if strcmp(kernel.kernel_name,'SP' )
    if~pred_from_seq 
      fprintf( '--> XT[%d:%d], wordlen=%d\n', min(select), max(select), kernel.wordlen );
      sg( addFeats, svmMode, XT_temp, 'DNA');
    
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1 ) ;
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, kernel.wordlen-1, kernel.gaps );
      end;
      
      if( strcmp(svmMode,'TRAIN') && use_preproc )
        fprintf('--> clean_preproc\n')
        sg( 'clean_preproc' );
        fprintf('--> add_preproc SORTWORDSTRING\n')
        sg( 'add_preproc', 'SORTWORDSTRING' );
        fprintf('--> attach_preproc %s\n', svmMode)
        sg( 'attach_preproc', svmMode );
      end;
    else
      % from_position_list doesnot work for training for spectrum kernel  
      assert(strcmp(svmMode,'TEST')) ;
      sg( addFeats, svmMode, {seq}, 'DNA' );
      % window is cut out from sequence if vonvertion is made after calling 'from_position_list
      % if convertion is done at this place the window size must be reduced by -ordnung-1;
      % (not yet tested) this woud save a lot of time and space. 
      % 
      if( ~isfield(kernel,'gaps') )
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0);
      else
        fprintf('--> convert\n')
        sg( 'convert', svmMode, 'STRING', 'CHAR', 'STRING', 'WORD', kernel.wordlen, 0, kernel.gaps );
      end;
      sg('from_position_list','TEST', winsize_temp- kernel.wordlen+1, int32(pos_temp+kernel.wordlen-1));
    end
  end;
 
  %------------------------------------------------------------------------------%
  % kernel:  RBF 
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'RBF')
    fprintf( '--> XT[%d:%d], num_examples=%i,  sigma=%d\n', min(select), max(select),size(XT,2), kernel.sigma );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  POLYNOMIAL
  %------------------------------------------------------------------------------%
 
  if strcmp(kernel.kernel_name,'POLY')
    fprintf( '--> XT[%d:%d], num_examples=%i,  degree=%d\n', min(select), max(select),size(XT,2), kernel.degree );
    sg( addFeats, svmMode, XT_temp);    
  end;
  
  %------------------------------------------------------------------------------%
  % kernel:  LINEAR
  %------------------------------------------------------------------------------%
  
  if strcmp(kernel.kernel_name,'LINEAR')
    fprintf( '--> XT[%d:%d], num_examples=%i,\n', min(select), max(select),size(XT,2) );
    sg( addFeats, svmMode, XT_temp);    
  end;
end%loop over kernels
%------------------------------------------------------------------------------%
% done
%------------------------------------------------------------------------------%

if( use_preproc & strcmp(svmMode,'TEST')  & ~pred_from_seq )
   fprintf('--> attach_preproc %s', svmMode)
  sg( 'attach_preproc', svmMode );
end;

clear XT;
sg( 'init_kernel', svmMode );

% --- set feature weights
if( conservWdsWeighting > 0 )
  W1 = W( select1, : );
  if( conservWdsWeightNorming )
    fprintf( '--> conserv(WDS): using normalized feature weights!\n' );
    meanW = mean( W1, 1 );
    W1 = W1 .* repmat( 1./meanW, size(W1,1), 1 );
  else
    fprintf( '--> conserv(WDS): using raw feature weights!\n' );
    W1 = 3/size(W1,1) * W1;
  end;
  sg( 'set_subkernel_weights_combined', W1, 1 );
  clear W1;
end;



return

function [ alphas, b, betas, PAR1 ] = svm_train(Data);

P = Data.P;

method  = P.Signal.method;
fn_SVMs = P.fn_SVMs;
pred_from_seq = P.pred_from_seq;
use_preproc = P.use_preproc;


%------------------------------------------------------------------------------%
% SVM already trained?
%------------------------------------------------------------------------------%

%if( ~isfield(PAR,'conservWeighting') )
method.conservWeighting = 0;
%end;
if( ~exist('testDataPar','var') )
  testDataPar = [];
end;


%------------------------------------------------------------------------------%
% train SVM
%------------------------------------------------------------------------------%


if( method.conservWeighting == 0 )
  Data.W = [];
else
  Data.W = res.W;
  assert( ~isempty(W) );
end;

% keyboard
init_svm_and_kernels(method,pred_from_seq);


if isequal(method.feature_type,'double') && method.normalize_features
  idx_rm = find(std(Data.XT')<1e-8);
  Data.XT(idx_rm,:)  = [];
  mean_tr = mean(Data.XT');
  Data.XT = Data.XT - repmat(mean_tr,size(Data.XT,2),1)';
  std_tr = std(Data.XT');
  Data.XT = Data.XT./repmat(std_tr,size(Data.XT,2),1)';
end

setFeatures(method, Data,'TRAIN',pred_from_seq,use_preproc);

m = length( Data.LT );
% PAR1 = P;
% PAR1.POS = Data.Pos;
% PAR1.lenTrain = size( Data.XT, 1 );

sg_ver=sg('get_version');

% === train
nofKernels = length(method.kernels);
t0 = cputime;
sg('set_kernel_optimization_type', 'FASTBUTMEMHUNGRY' );
%sg( 'set_kernel_optimization_type', 'SLOWBUTMEMEFFICIENT' );
if( method.use_mkl )
  sg('use_mkl', true );
  sg('mkl_parameters', 1e-3, 0 );
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
end;
sg('svm_train' );
fprintf( '\n' );
t = cputime - t0;

%------------------------------------------------------------------------------%
% get alphas and betas
%------------------------------------------------------------------------------%

% === alphas
[ b, alphas ] = sg( 'get_svm' );


% === betas (kernel weights)
if(method.use_mkl)
  betas = sg( 'get_subkernel_weights' );
  assert( size(betas) == [nofKernels,1] );
  betas
else
  betas = ones( nofKernels, 1 );
end;


%------------------------------------------------------------------------------%
% save SVM
%------------------------------------------------------------------------------%
Train.alphas = alphas;
Train.b = b;
Train.PAR = P;
Train.POS = Data.Pos;
sup_vec_idx = alphas(:,2)+1;
Train.XT = Data.XT(:,sup_vec_idx);
Train.LT = Data.LT(sup_vec_idx);

w = zeros(size(Train.XT,1),1);
for a=1:length(Train.LT)
  w = w+Train.alphas(a,1).*Train.XT(:,a);
end
Train.w = w;

Train.POS.pos = Data.Pos.pos(sup_vec_idx);
Train.POS.region_id = Data.Pos.region_id(sup_vec_idx);

if isequal(method.feature_type,'double') && method.normalize_features
  Train.idx_rm = idx_rm
  Train.mean = mean_tr;
  Train.std = std_tr;
end

clear P Data
sg( 'clean_features', 'TRAIN' );
sg( 'clean_features', 'TEST' );

Train.time=t;
Train.sg_ver = sg_ver ; 
fprintf('save: %s',fn_SVMs)
save(fn_SVMs, '-V7', 'Train')






return

function [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)

% [fn, exists]=gen_example_filename(Signal,fn_filter_settings,fn_examples,which,split)
% [fn, exists]=gen_example_filename(PAR,which,split)
% which is 'train','eval','test'

% sig = PAR.Signal_name;
% Signal = PAR.Signals.(sig); 
% fn_filter_settings =  PAR.FN.input_sig.(sig).input_sig;
% fn_examples = PAR.FN.input_sig.(sig).fn_examples;

whichs = fieldnames(Signal.filter_label);


if isnumeric(which)
  assert(which<=length(whichs))
  which = whichs{which};
else
  assert(ismember(which,whichs))
end
clear whichs

current_settings = Signal.filter_label.(which);

if fexist(fn_filter_settings)
  load(fn_filter_settings, 'filter_settings')
  idx = find([filter_settings.split]==split);
else
  filter_settings = [];
  idx = [];
end
idx_setting = [];is_equal=0;
for i=idx
  temp = rmfield(filter_settings(i),'split');
  temp = rmfield(temp,'filename');
  if isequal(temp, current_settings)
    idx_setting = [idx_setting i];
    is_equal=1;
  else
    assert(length(fieldnames(temp))==length(fieldnames(current_settings))&&...
           all(strcmp(sort(fieldnames(temp)),sort(fieldnames(current_settings)))))
  end
end
assert(length(idx_setting)<=1);
if isempty(idx_setting)
  temp = struct;
end
assert(length(setdiff(fieldnames(temp),fieldnames(current_settings)))==0)

if ~isempty(idx_setting)
  fn = filter_settings(idx_setting).filename;
  if fexist(fn)
    exists = 1;
  else
    exists = 0;
  end
else 
  settings_names = fieldnames(current_settings);
  s = length(filter_settings)+1;
  for j = 1:length(settings_names)
    filter_settings(s).(settings_names{j}) = current_settings.(settings_names{j});
  end
  filter_settings(s).split = split;
  filter_settings(s).filename = sprintf('%s_settings=%i_split=%i.mat',fn_examples, s, split);
  fn = filter_settings(s).filename;
  if fexist(fn)
    error('I do not know where that file comes from!!!')
  else
    exists = 0;
  end 
  save(fn_filter_settings, 'filter_settings')
end



       
%fn = sprintf('%s_%s_pc=%i_pa=%i_nc=%i_na=%i_split=%i.mat',...
%	input_sig.fn_examples,PAR.Signal_name,pos_conf,...
%	pos_anno,neg_conf,neg_anno, s);

return

function [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_content_data(Content,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)


method = Content.method;

XT = []; LT = [] ;  POS.region_id = []; POS.pos=[]; POS_tmp = []; 

if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Content,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT','XT','POS');
  %  L.XT = load_matrix(filename, 'XT');
  % [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if isfield(L,'LT')
    if size(L.LT,1)>1,
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  POS.region_id =[POS.region_id L.POS.region_id];
  POS.pos = [POS.pos L.POS.pos];
  
  % generate feature vectors
  %----------------------------------------------------------------
  dim = 0;
  for o=1:length(method.kernels)
    dim = dim + 4^method.kernels{o}.wordlen;
  end
  xt = zeros(dim,length(L.XT));
  for j=1:length(L.XT) 
    dim = 0;
    for o=1:length(method.kernels)
      [mask hist] = compute_mers(L.XT{j}, method.kernels{o}.wordlen, method.kernels{o}.stepping, method.kernels{o}.offset);
      xt(dim+[1:4^method.kernels{o}.wordlen],j) = hist';
      dim = dim+ 4^method.kernels{o}.wordlen;
    end
  end
  clear L
  XT=[XT xt];
  
end ;

if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end


Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS

assert(size(Data.XT,2)==length(Data.LT))
% assert(length(Data.LT)==length(Data.Pos.pos))
% assert(length(Data.LT)==length(Data.Pos.region_id))







return

function [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)
% [XT,LT,POS,SC] = subsample(XT, LT, POS, SC, subsample_neg, subsample_pos)

if subsample_neg<1 & subsample_neg>0 & subsample_pos<=1 & subsample_pos>0 % & ~isempty(LT) 

  fprintf('subsampling %i %% of negative examples \n',subsample_neg*100);
  fprintf('subsampling %i %% of positive examples \n',subsample_pos*100);
  posidx = find(LT==1);
  negidx = find(LT==-1);
  rand('seed', pi) ;
  idx2 = randperm(length(negidx));
  idx3 = randperm(length(posidx));
  num_neg = round(subsample_neg*sum(LT==-1));
  num_pos = round(subsample_pos*sum(LT==1));
  
  XT = [XT(:,posidx(idx3(1:num_pos))) XT(:,negidx(idx2(1:num_neg)))] ;
  
  if ~isempty(SC)
    SC = [SC(:,posidx(idx3(1:num_pos))) SC(:,negidx(idx2(1:num_neg)))] ;
  end
  LT = [LT(posidx(idx3(1:num_pos))) LT(negidx(idx2(1:num_neg)))] ; 
  POS.pos = [POS.pos(posidx(idx3(1:num_pos))) POS.pos(negidx(idx2(1:num_neg)))] ; 
  POS.region_id = [POS.region_id(posidx(idx3(1:num_pos))) POS.region_id(negidx(idx2(1:num_neg)))] ; 
  fprintf('%i examples remaining (%.2f %% positives)\n',length(LT),sum(LT==1)/length(LT)*100);

end

return

function var=load_matrix(filename, varname)
% var=load_matrix(filename, varname)

load(filename, 'meta')
if ~exist('meta')
  load(filename, varname);
  eval(sprintf('var = %s;',varname))
  return
end

var = [] ;
col = 1;
for j=1:meta.num_splits
  %if col>1
  %  assert(~all(var(:,col-1)==-1* ones(meta.size(1),1)))
  %end
  %assert(all(var(:,col)==-1*ones(meta.size(1),1)))
   
  load(filename, sprintf('%s_split_%i',varname,j));
  eval(sprintf('temp = %s_split_%i;',varname,j));

  if j==1,
    if isnumeric(temp)
      var = zeros(meta.size(1), meta.size(2), class(temp));
    else
      var = zeros(meta.size(1), meta.size(2),'uint8');
      var = char(var);
    end
  end ;

  var(:,col:col + size(temp, 2)-1)=temp;
  col = col + size(temp, 2);
end
%assert(~all(var(:,end)==-1* zeros(meta.size(1),1)))

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function [mask hist] = compute_mers(string, order, stepping, offset)
%function [mask hist] = compute_mers(string, order, stepping)

mask    = zeros(1,4^order) ;
hist    = zeros(1,4^order) ;

%if nargin<3,
%  stepping = 1;
%end;
%
%if nargin<4,
%  offset = 0;
%end;
%
%if isempty(string), 
%  return; 
%end;
string = lower(char(string)) ;

if ~isempty(find(~ismember(string,'acgt')))
  string(find(~ismember(string,'acgt'))) = 'a';
end;

assert(order<=8);

if length(string)<order,
  return ;
end
t = sg('translate_string', double(string), order, order-1) ;
if length(t)<order
  return ;
end ;

% transform t to matlab coordinates
t = t(1:end-order+1)+1 ;

% throw away kmers if stepping > 1
assert(stepping>0);
%t = t(1:stepping:end);
t = t(end-offset:-stepping:1);

% reduce t to the information whether or not a certain kmer occurs 
u = unique(t) ;

% get histogram counts
t = sort(t);
[dummy loc] = ismember(u, t);
occ = loc - [0 loc(1:end-1)];

% return histogram
hist(u) = occ/sum(occ);

% return "binary histogram" 
% telling whether or not a certain kmer is present
mask(u) = 1/sqrt(length(u)) ;

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

