function mgene_predict(fasta_fname, fasta_dir, train_fname, train_dir, fn_out, output_dir, run_locally, use_old_files)
%  mGenePredict(fasta_fname, fasta_dir, train_fname, train_dir, fn_out, output_dir, run_locally, use_old_files)
%    

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started mgene_predict at %s\n', timedate) ;

fprintf('Results are written to: %s\n', fn_out) ;

if ~exist('use_old_files','var')
  use_old_files = 0;
end

% find internal files
[train_dir,train_fname] = galaxy_find_internal_files(train_dir,train_fname) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
config_dir = sprintf('%s/genome_dir',output_dir) ;
info_name = sprintf('%s/genome_info',output_dir) ;

unix(sprintf('rm -rf %s', config_dir)) ;

nice_mkdir(config_dir)
tic; galaxy_genometool(fasta_fname, fasta_dir, info_name, config_dir) ;toc;

load(sprintf('%s/mgene_predictor.mat', train_dir), 'train_dirs', 'train_files');

genome_config = [config_dir '/genome.config'] ;
genome_info = init_genome(genome_config);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNALS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if ~exist('run_locally', 'var')
  run_locally = -1 ;
%end ;

[engine,env]=determine_engine;
if isequal(env, 'internal'),
  bdir = sprintf('/fml/ag-raetsch/home/%s/svn/projects/genefinding/',whoami);
  
  
  jobinfo = rproc_empty(0) ;
  options.start_dir = [bdir 'galaxy'];
  options.addpaths = {bdir, [bdir 'data_processing_sensors'],[bdir 'sensors'],[bdir 'contents'],[bdir 'data_processing_lsl'],[bdir '/lsl_paper'],[bdir 'galaxy'],...
                      [bdir 'utils'],[bdir 'template_files'],[bdir 'template_files/Contents'],[bdir 'template_files/Organisms'],...
                      [bdir 'template_files/Methods'],[bdir 'template_files/Signals'],[bdir 'template_files/lsl/'], [bdir 'template_files/exp']...
                      [bdir '../splicing/splicegraphs/'], [bdir '../splicing/splicegraphs/detect_altsplice/'], [bdir '../../tools/utils'],[bdir '../../tools/genomes']...
                      [bdir '../../tools/prob_plif'],...
                      '/fml/ag-raetsch/home/raetsch/tmp/src'};
  % '/fml/ag-raetsch/home/raetsch/svn/projects/genefinding/shogun'};
  %'/fml/ag-raetsch/home/jonas/shogun/trunk/src/matlab/'};
  
  options.envstr = 'export LD_LIBRARY_PATH=~/mgene_galaxy/shogun.matlab_new/trunk/src/libshogun:~/mgene_galaxy/shogun.matlab_new/trunk/src/libshogunui';
  
  % options.identifier = sprintf('%spred%s_%s',upper(PAR.organism.name(1:4)),PAR.Signal_name,PAR.method.name) ;
else
  options = [] ;
  options.force_octave = 1 ;
  options.envstr = 'export LD_LIBRARY_PATH=/home/galaxy/shogun.octave_new/trunk/src/libshogun/:/home/galaxy/shogun.octave_new/trunk/src/libshogunui';
end ;

%[train_dirs.tss, tis_dir, train_dirs.acc, train_dirs.don, train_dirs.cdsStop, train_dirs.cleave] = ...
%    sort_signals(train_dirs.tss, tis_dir, train_dirs.acc, train_dirs.don, train_dirs.cdsStop, train_dirs.cleave) ;

signal_names = {'acc', 'don', 'tis', 'cdsStop', 'tss', 'cleave'} ;

P.rproc_options = options;
P.config_dir = config_dir; 
P.train_on = 0;
P.use_old_files = use_old_files;
P.run_locally = run_locally;

num_jobs = 0;
if 1
for signal_idx = 1:length(signal_names)
  num_jobs = num_jobs + 1;
  P.signal_name = sprintf('signal_%i', signal_idx) ;%signal_names{signal_idx} ;
  
  % training files
  P.tsp_train_dir = train_dirs.(signal_names{signal_idx}) ;%P.signal_name);
  
  % prediction
  P.spf_pred_fname = sprintf('%s/signal_%s_predictions.spf',output_dir, P.signal_name) ;
  P.spf_pred_dir = sprintf('%s/signal_%s_predictions_files',output_dir, P.signal_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/signal_%s_eval.txt',output_dir, P.signal_name) ;
 
  if run_locally==-1,
    run_locally_signal_wrapper = rproc_policy('signal_wrapper', genome_info) ;
  else
    run_locally_signal_wrapper = run_locally ;
  end ;

  if ~run_locally_signal_wrapper,
    
    [mem_req, time_req, opts] = rproc_memtime_policy(['signal_wrapper:' signal_names{signal_idx}], [], options) ;

    jobinfo(num_jobs) = rproc('signal_wrapper', P, mem_req, opts, time_req);
  else
    signal_wrapper(P);
  end
  pred_dirs.(signal_names{signal_idx}) =  P.spf_pred_dir;
  pred_files.(signal_names{signal_idx}) = P.spf_pred_fname;
end
end
clear P 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL ALL CONTENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
P.config_dir = config_dir; 
P.use_old_files = use_old_files;
P.run_locally = run_locally;

content_names = {'frame0','intron', 'cds_exon', 'utr3exon', 'utr5exon', 'intergenic'} ;
P.train_on = 0;
for content_idx = 1:length(content_names) ;
  P.content_name = sprintf('content_%i', content_idx) ;%content_names{content_idx} ;
  
  % training
  try
    P.tcp_train_dir = train_dirs.(content_names{content_idx});
  catch
    warning(sprintf('content %s skipped', content_names{content_idx}));
    continue
  end
  num_jobs = num_jobs + 1;
  
  % prediction
  P.cpf_pred_fname = sprintf('%s/content_%s_predictions.spf',output_dir, P.content_name) ;
  P.cpf_pred_dir = sprintf('%s/content_%s_predictions_files',output_dir, P.content_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/content_%s_eval.txt',output_dir, P.content_name) ;

  if run_locally==-1,
    run_locally_content_wrapper = rproc_policy('content_wrapper', genome_info) ;
  else
    run_locally_content_wrapper = run_locally ;
  end ;
  
  if ~run_locally_content_wrapper,
    P.rproc_options = options;
    [mem_req, time_req, opts] = rproc_memtime_policy(['content_wrapper:' content_names{content_idx}], [], options) ;
    
    jobinfo(num_jobs) = rproc('content_wrapper', P, mem_req, opts, time_req);
  else
    content_wrapper(P);
  end
  pred_dirs.(content_names{content_idx}) =  P.cpf_pred_dir;
  pred_files.(content_names{content_idx}) =  P.cpf_pred_fname;
end

save([output_dir '/prediction_files.mat'], 'pred_dirs', '-v7');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL Gene Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(jobinfo),
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
end


logfile = [output_dir '/log_train'];

% output file of Gene_predict
% fn_predictions_gff3 = [output_dir '/gene_predictions.gff3'];


galaxy_gene_predict(	config_dir, train_files.gene_predictor, train_dirs.gene_predictor, logfile, ...
	 	pred_files.tss, pred_dirs.tss, pred_files.tis, pred_dirs.tis, pred_files.acc, pred_dirs.acc, ...
            	pred_files.don, pred_dirs.don, pred_files.cdsStop, pred_dirs.cdsStop, pred_files.cleave, pred_dirs.cleave, ...
		pred_files.intergenic, pred_dirs.intergenic, pred_files.utr5exon, pred_dirs.utr5exon, ...
		pred_files.cds_exon, pred_dirs.cds_exon, pred_files.intron, pred_dirs.intron, ...
           	pred_files.utr3exon, pred_dirs.utr3exon, fn_out, output_dir, run_locally)






[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('finished mgene_predict at %s\n', timedate) ;

return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

