function [jobinfo]=rproc(ProcName, P1, Mem, options, time)
% [jobinfo]=rproc(ProcName, P1, Mem, options, time)
%
% time in minutes
% mem in mb

% currently only has adversal effects, change this when a new
% gridengine version has been installed
time = 10000 ; 

global MATLAB_BIN_PATH OCTAVE_BIN_PATH RPROC_PATH SGE_BIN_PATH USE_SGE SGE_TMP_PATH RPROC_LD_LIBRARY_PATH RPROC_PATH_LIST

if isempty(USE_SGE), 
  USE_SGE = 1 ;
end ;
assert(USE_SGE>0) ;

[engine, environment] = determine_engine() ;

if isempty(RPROC_PATH),
  [RPROC_PATH]=fileparts(which('rproc')) ;
  warning('RPROC_PATH configuration variable not set. Initializing with %s\n', RPROC_PATH) ;
end ;
if isempty(SGE_TMP_PATH)
  SGE_TMP_PATH=sprintf('%s/tmp/sge/', getenv('HOME')) ;
  warning('SGE_TMP_PATH configuration variable not set. Initializing with %s\n', SGE_TMP_PATH) ;
end ;
if isequal(engine, 'matlab') && isempty(MATLAB_BIN_PATH),
  [ret, MATLAB_BIN_PATH] = system('which matlab') ;
  if ret==0,
    MATLAB_BIN_PATH(MATLAB_BIN_PATH==sprintf('\n')) = [] ;
    warning('MATLAB_BIN_PATH configuration variable not set. Initializing with %s\n', MATLAB_BIN_PATH) ;
  else
    warning('MATLAB_BIN_PATH configuration variable not set. Cannot find Matlab executable\n') ;
  end ;
end ;
if isequal(engine, 'octave') && isempty(OCTAVE_BIN_PATH),
  [ret, OCTAVE_BIN_PATH] = system('which octave') ;
  if ret==0,
    OCTAVE_BIN_PATH(OCTAVE_BIN_PATH==sprintf('\n')) = [] ;
    warning('OCTAVE_BIN_PATH configuration variable not set. Initializing with %s\n', OCTAVE_BIN_PATH) ;
  else
    warning('OCTAVE_BIN_PATH configuration variable not set. Cannot find Matlab executable\n') ;
  end ;
end ;

if ~isfield(options, 'force_octave'),
  options.force_octave = 0 ;
end ;
if ~isfield(options, 'force_matlab'),
  options.force_matlab = 0 ;
end ;

if (isequal(engine, 'matlab') && options.force_octave==0) || options.force_matlab==1,
  use_engine='matlab' ;
else
  use_engine='octave' ;
end ;


if nargin<3, Mem=300 ; end ;
if isequal(use_engine, 'matlab'),
  if Mem<200, warning('You specified to allocate less than 200Mb for your job. Matlab might not be able to start.') ; end ;
else
  if Mem<300, warning('You specified to allocate less than 300Mb for your job. Octave might not be able to start.') ; end ;
end ;

if nargin<4, options=[] ; end ;
if nargin<5, time=100*24*60 ; end ;
if Mem<100, Mem=100 ; end ;
if ~isfield(options,'verbosity')
  options.verbosity = 1 ;
end ;
if ~isfield(options,'maxjobs')
  options.maxjobs=2000; %hardcoded limit by sebastian
end
if ~isfield(options,'waitonfull')
  options.waitonfull=0 ;
end ;
if ~isfield(options,'immediately')
  options.immediately=0 ;
end ;
if ~isfield(options,'immediately_bg')
  options.immediately_bg=0 ;
end ;
if ~isfield(options,'submit_now')
  options.submit_now=1 ;
end ;
if ~isfield(options,'nicetohave')
  options.nicetohave=0 ;
end ;
if ~isfield(options, 'start_dir')
  dirctry=cwd ;
else
  dirctry=options.start_dir ;
end ;
if ~isfield(options, 'resubmit'),
  options.resubmit = 0 ;
  options.time_req_resubmit = [] ;
  options.mem_req_resubmit = [] ;
end ;
if ~isfield(options, 'data_size'),
  options.data_size = [] ;
end ;

jobinfo=rproc_empty ;

jobinfo.ProcName = ProcName ;
jobinfo.P1 = P1 ;
jobinfo.Mem = Mem ;
jobinfo.options = options ;
jobinfo.time = time ;
jobinfo.created = 1 ;
jobinfo.resubmit = options.resubmit ;
jobinfo.mem_req_resubmit = options.mem_req_resubmit ;
jobinfo.time_req_resubmit = options.time_req_resubmit ;
jobinfo.data_size = options.data_size ;

if ~exist(SGE_TMP_PATH)
  unix(sprintf('mkdir -p %s', SGE_TMP_PATH)) ;
end ;

option_str = [] ;
if isfield(options,'hold')
  if options.hold==1,
	option_str=[option_str ' -h u '] ;
  end
end ;
if isfield(options,'queue') 
  option_str=[option_str sprintf(' -q "%s" ', options.queue)] ;
end ;
if isfield(options,'priority') 
  option_str=[option_str sprintf(' -p %i ', options.priority)] ;
end ;
if isfield(options,'express') && isequal(options.express,1)
  option_str=[option_str ' -l express '] ;
end ;
if ~isfield(options, 'addpaths'),
  options.addpaths = {} ;
end ;
if ~isempty(RPROC_PATH_LIST)
  % appending global path list to local one
  for i=1:length(RPROC_PATH_LIST)
    options.addpaths{end+1} = RPROC_PATH_LIST{i} ;
  end ;
end ;
options.RPROC_PATH_LIST = RPROC_PATH_LIST ;
options.RPROC_PATH = RPROC_PATH ;
options.MATLAB_BIN_PATH = MATLAB_BIN_PATH ;
options.OCTAVE_BIN_PATH = OCTAVE_BIN_PATH ;
options.SGE_BIN_PATH = SGE_BIN_PATH ;
options.SGE_TMP_PATH = SGE_TMP_PATH ;
options.RPROC_LD_LIBRARY_PATH = RPROC_LD_LIBRARY_PATH ;

if isequal(use_engine, 'matlab')
  bin_str = MATLAB_BIN_PATH  ;
elseif isequal(use_engine, 'octave'),
  bin_str = OCTAVE_BIN_PATH ;
else
  error('unknown engine') ;
end ;


identifier='' ;
if isfield(options, 'identifier'),
  identifier = options.identifier ;
end ;
if isempty(identifier)
  identifier = 'J' ; % cannot be empty
end ;

cc=round(rand(1)*100000) ;
prefix=sprintf('%s%i-%1.10f', identifier, cc(1),now) ;
mat_fname=sprintf('%s/%s.mat', SGE_TMP_PATH, prefix) ;
result_fname=sprintf('%s/%s_result.mat', SGE_TMP_PATH, prefix) ;
m_fname=sprintf('%s/%s.m', SGE_TMP_PATH, prefix) ;
while fexist(mat_fname) | fexist(result_fname) | fexist(m_fname),
  cc=round(rand(1)*100000) ;
  prefix=sprintf('%s%i-%1.10f', identifier,cc(1), now) ;
  mat_fname=sprintf('%s/%s.mat', SGE_TMP_PATH, prefix) ;
  result_fname=sprintf('%s/%s_result.mat', SGE_TMP_PATH, prefix) ;
  m_fname=sprintf('%s/%s.m', SGE_TMP_PATH, prefix) ;
end ;


clo = clock ;
log_fname = sprintf('%s/%s_%s_%i_%i.rproc', SGE_TMP_PATH, prefix, date,	clo(4), clo(5)) ;
qsublog_fname = sprintf('%s.qsubout',log_fname) ;

jobinfo.prefix = prefix ;
jobinfo.mat_fname = mat_fname ;
jobinfo.result_fname = result_fname ;
jobinfo.m_fname = m_fname ;
jobinfo.log_fname = log_fname ;
jobinfo.qsublog_fname = qsublog_fname ;

% save the call information
save(mat_fname, 'ProcName', 'P1', 'dirctry', 'options', '-v7') ;

evalstring_=sprintf('addpath(''%s'') ;', RPROC_PATH) ;
evalstring=['start_proc(''' mat_fname ''')'] ;
evalstring=[evalstring_ 'cd ' dirctry '; ' evalstring '; exit'] ;
fd=fopen(m_fname,'w+') ;
fprintf(fd,'%s\n',evalstring) ;
fclose(fd) ;

cl=clock ;
clstr=sprintf('%i:%i:%i--%i.%i.%i', round(cl([6 5 4 3 2 1]))) ;
mem=sprintf('%iMb',Mem) ;

if isfield(options, 'envstr'),
  envstr=options.envstr ;
  if ~isempty(envstr),
    envstr(end+1)=';' ;
  end ;
else
  envstr = '' ; 
end ;
if ~isempty(RPROC_LD_LIBRARY_PATH)
  envstr = [envstr 'export LD_LIBRARY_PATH=' RPROC_LD_LIBRARY_PATH ';'] ;
end ;

if options.immediately,
  str=[RPROC_PATH '/starter.sh ' engine ' ' bin_str ' ' m_fname ' ' log_fname ' "' envstr '"'] ;
elseif options.immediately_bg,
  str=[RPROC_PATH '/starter.sh ' engine ' ' bin_str ' ' m_fname ' ' log_fname ' "' envstr '"&'] ;
else
  str=['echo ''' RPROC_PATH '/starter.sh "' engine '" "' bin_str '" "' m_fname '" "' log_fname '" "' envstr '"'' | qsub -o "' qsublog_fname '" -j y -r y ' option_str ' -N ' prefix ' 2>&1 >> ' log_fname] ;
end ;

if options.submit_now,
  if options.verbosity>1
    disp(str)
  end ;
end ;

% wait until we are allowed to submit again, i.e. #jobs < maxjobs
if ~options.immediately && ~options.immediately_bg && isequal(options.waitonfull,1)
  while 1,
    [s,num_queued]=unix('qstat -u `whoami`  2> /dev/null | grep `whoami` | wc -l | tr -d " "');
    if (s~=0)
      warning('could not determine how many jobs are scheduled');
      break;
    end
    num_queued=str2double(num_queued);
    
    %keep 50 spare jobs if multiple rprocs are scheduling...
    if (num_queued < options.maxjobs)
      break;
    else
      if options.verbosity>=1
        fprintf('queue full, sleeping 60 seconds (%i/%i)\n', ...
                num_queued, options.maxjobs);
      end ;
      system('sleep 60');
    end
  end
end ;


if options.submit_now,
  if options.immediately && options.verbosity>0,
    disp('immediatedly starting job on local machine') ;
  end ;
  if options.immediately_bg && options.verbosity>0,
    disp('immediatedly starting job on local machine in background') ;
  end ;
  if options.immediately_bg,
    while(1)
      [tmp,str_]=unix('uptime');
      idx=strfind(str_, 'average:') ;
      assert(~isempty(idx)) ;
      b=separate(str_(idx+8:end), ',') ; 
      cpu_load = str2num(b{1}) ;
      if cpu_load>10,
        if options.verbosity>=1
          fprintf('load too high: %1.2f\n', cpu_load) ;
        end ;
        pause(10) ;
      else
        break ;
      end ;
    end
    pause(2)
  end ;

  unix(str) ;
  jobinfo.submission_time = now ;

  if ~options.immediately && ~options.immediately_bg,
    fd = fopen(log_fname,'r') ;
    jobinfo.jobid = -1 ;
    if fd>=0,
      s=fgetl(fd) ;
      items=separate(s,' ');
      if ~(isequal(items{1}, 'Your') && isequal(items{2}, 'job'))
        error('error: submission failed: %s',s);
      end
      jobinfo.jobid = str2num(items{3}) ;
      fclose(fd) ;

      rproc_register('submit', jobinfo) ;
    end ;
  else
    jobinfo.jobid = 0 ;
  end ;
else
  jobinfo.jobid = 0 ;
end ;
