%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% Written (W) 2005-2009 Gunnar Raetsch, Gabriele Schweikert, Jonas Behr, Soeren Sonnenburg, Alexander Zien, Georg Zeller, Andre Noll, Cheng Soon Ong, Petra Philips
% Copyright (C) 2005-2009 Max Planck Society and Fraunhofer Institute FIRST 
%
function mgene_train_workflow(fasta_fname, gff_fname, version, seqid, source, level, converters, mGene_predictor_dir, train_options)
% mgene_train_workflow(fasta_fname, gff_fname, version, seqid, source, level, converters, mGene_predictor_dir, train_options)

global MGENE_TMP_PATH

if nargin~=8 && nargin~=9 && nargin~=4 && nargin~=5,
  first_arg=[] ; 
  if nargin>=1, first_arg=fasta_fname; end ;

  if show_usage('mgene_train_workflow', {'<fasta-file>, <GFF3-file>, ''wormbase''|''flybase''|''tair8''|''ensembl'', <mGene-predictor-dir>[, <training-options>]',
  '<fasta-file>, <GFF3-file>, ''other'', <seqids>, <sources>, <level-info>, <converters>, <mGene-predictor-dir>[, <training-options>]'}, ...
     nargin, first_arg),
     return
  end ;
end ;

% rename input parameters
if nargin==5
   train_options = source ;
end ;		 
if nargin<=5
  mGene_predictor_dir = seqid ;
  seqid = '' ;
  source = '' ;
  level = '' ;
  converters = '' ;
end ;		 
if ~exist('train_options', 'var')
  train_options='' ;
end ;

fn_mGene_predictor = tempname(MGENE_TMP_PATH) ;

if isdir(fasta_fname)
  fasta_dir = fasta_fname ;
  fasta_fname = '' ;
else
  fasta_dir = '' ;
end ;

mgene_train(fasta_fname, fasta_dir, gff_fname, version, seqid, source, ...
            level, fn_mGene_predictor, mGene_predictor_dir, -1, 0, train_options, converters)

fprintf('\n\n') ;
system(sprintf('cat %s && rm %s', fn_mGene_predictor, fn_mGene_predictor)) ;


return

function mgene_train(fasta_fname, fasta_dir, gff_fname, version, seqid, source, level, fn_mGene_predictor, mGene_predictor_dir, run_locally_par, use_old_files, train_options, converters)
%  mgene_train(fasta_fname, fasta_dir, gff_fname, fn_mGene_predictor, mGene_predictor_dir, run_locally_par, version, seqid, source, level,  use_old_files, train_options, converters)

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started mgene_train at %s\n', timedate) ;

fprintf('Results are written to: %s\n', fn_mGene_predictor) ;

if ~exist('run_locally_par', 'var')
  run_locally_par = -1 ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output_dir = mGene_predictor_dir ;

config_dir = sprintf('%s/genome_dir',output_dir) ;
info_name = sprintf('%s/genome_info',output_dir) ;

annotation_fname = sprintf('%s/anno_info',output_dir);
annotation_dir = sprintf('%s/',output_dir);

ignore_missing_referenced_entries = 1 ;
coding_flag = 1 ;
nice_mkdir(config_dir);
fd=fopen(fn_mGene_predictor, 'w+') ; 
fprintf(fd,'------------- \n')
fprintf(fd,'mGeneTrain started %s\n', timedate) ;
fprintf(fd,'------------- \n\n')

fprintf(fd, 'Training Based on FASTA File : %s\n',fasta_fname) ; 
fprintf(fd, '                    GFF File : %s\n',gff_fname) ; 
fclose(fd) ;

genometool(fasta_fname, fasta_dir, info_name, config_dir) ;
unix(sprintf('cat %s >> %s', info_name, fn_mGene_predictor)) ;
fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;

genome_config = [config_dir '/genome.config'] ;
genome_info = init_genome(genome_config);

if run_locally_par==-1,
  run_locally_gff2anno = rproc_policy('gff2anno', genome_info)
else
  run_locally_gff2anno=run_locally_par ;
end ;
gff2anno(config_dir, gff_fname, annotation_fname, annotation_dir, version, seqid, source, level, ignore_missing_referenced_entries, run_locally_gff2anno, converters) ;

unix(sprintf('cat %s >> %s',annotation_fname, fn_mGene_predictor)) ;
fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fd=fopen(fn_mGene_predictor, 'a+') ; 

fprintf(fd, 'mGeneTrain (started %s)\n',  timedate)
fprintf(fd, '----------------------------\n\n') ;

genes = load_struct(sprintf('%s/genes.mat', annotation_dir), 'genes') ;
fprintf(fd, 'Read %i from GFF files\n', length(genes)) ;
clear genes ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNALS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
options = [] ;
MEMREQ = 3500;
time = 60*10;  

signal_names = {'acc', 'don', 'tis', 'cdsStop', 'tss', 'cleave'} ;

P.covered_genic_positions_only = 1 ;
P.config_dir = config_dir ; 
P.annotation_fname = annotation_fname ;
P.annotation_dir = annotation_dir ;
P.train_on = 1 ;

genome_config = [config_dir '/genome.config'] ;
genome_info = init_genome(genome_config);
num_contigs = length(genome_info.contig_names); 

if exist('use_old_files')
  P.use_old_files = use_old_files;
else
  P.use_old_files = 0;
end
P.rproc_options = options;
P.run_locally = run_locally_par ;

num_jobs = 0;

fprintf(fd,'------------- \n')
fprintf(fd,'start signal and content training and prediction\n') ;
fprintf(fd,'--- this may take a while ---  \n') ;
fprintf(fd,'------------- \n\n')
fclose(fd) ;


for signal_idx = 1 :length(signal_names)
  num_jobs = num_jobs + 1;
  P.signal_name = signal_names{signal_idx} ;
  
  % label generation
  P.spf_label_fname = sprintf('%s/signal_%s_label.spf',output_dir, P.signal_name) ;
  P.spf_label_dir = sprintf('%s/signal_%s_label_files',output_dir, P.signal_name) ;

  % training
  P.tsp_train_info = sprintf('%s/signal_%s_classifier_info',output_dir, P.signal_name) ;
  P.tsp_train_dir = sprintf('%s/signal_%s_classifier_files',output_dir, P.signal_name) ;
  
  % prediction
  P.spf_pred_fname = sprintf('%s/signal_%s_predictions.spf',output_dir, P.signal_name) ;
  P.spf_pred_dir = sprintf('%s/signal_%s_predictions_files',output_dir, P.signal_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/signal_%s_eval.txt',output_dir, P.signal_name) ;

  if run_locally_par==-1,
    run_locally_signal_wrapper = rproc_policy('signal_wrapper', genome_info) ;
  else
    run_locally_signal_wrapper = run_locally_par ;
  end ;

  if ~run_locally_signal_wrapper,
    [mem_req, time_req, opts] = rproc_memtime_policy(['signal_wrapper:' signal_names{signal_idx}], [], options) ;

    jobinfo(num_jobs) = rproc('signal_wrapper', P, mem_req, opts, time_req);
  else
    signal_wrapper(P);
  end
  train_dirs.(P.signal_name) =  P.tsp_train_dir;
  train_files.(P.signal_name) = P.tsp_train_info;
  pred_dirs.(P.signal_name) =  P.spf_pred_dir;
  pred_files.(P.signal_name) = P.spf_pred_fname;
end
clear P 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL ALL CONTENTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% rmpath ~raetsch/svn/projects/genefinding/shogun
% addpath /fml/ag-raetsch/home/jonas/shogun/trunk/src/matlab

P.config_dir = config_dir; 
P.annotation_fname = annotation_fname;
P.annotation_dir = annotation_dir;
P.vs_content_names = [];
P.intergenic_dist_to_genes = [];
P.covered_genic_positions_only = 1 ;

if exist('use_old_files')
  P.use_old_files = use_old_files;
else
  P.use_old_files = 0;
end

P.run_locally = run_locally_par ;
P.rproc_options = options;

content_names = {'frame0', 'intron', 'cds_exon', 'utr3exon', 'utr5exon', 'intergenic'} ;

P.train_on = 1;
P.use_old_files = 0 ;
for content_idx = 1:length(content_names) ;
  num_jobs = num_jobs + 1;
  P.content_name = content_names{content_idx} ;
  
  % label generation 
  P.cpf_label_fname = sprintf('%s/content_%s_label.cpf',output_dir, P.content_name) ;
  P.cpf_label_dir = sprintf('%s/content_%s_label_files',output_dir, P.content_name) ;
  
  % training
  P.tcp_train_info = sprintf('%s/content_%s_classifier_info',output_dir, P.content_name) ;
  P.tcp_train_dir = sprintf('%s/content_%s_classifier_files',output_dir, P.content_name) ;
  
  % prediction
  P.cpf_pred_fname = sprintf('%s/content_%s_predictions.spf',output_dir, P.content_name) ;
  P.cpf_pred_dir = sprintf('%s/content_%s_predictions_files',output_dir, P.content_name) ;
  
  % evaluation
  P.eval_fname = sprintf('%s/content_%s_eval.txt',output_dir, P.content_name) ;
  
  if run_locally_par==-1,
    run_locally_content_wrapper = rproc_policy('content_wrapper', genome_info) ;
  else
    run_locally_content_wrapper = run_locally_par ;
  end ;

  if ~run_locally_content_wrapper,
    [mem_req, time_req, opts] = rproc_memtime_policy(['content_wrapper:' content_names{content_idx}], [], options) ;

    jobinfo(num_jobs) = rproc('content_wrapper', P, mem_req, opts,time_req);
  else
    content_wrapper(P);
  end
  train_dirs.(P.content_name) =  P.tcp_train_dir;
  train_files.(P.content_name) = P.tcp_train_info;
  pred_dirs.(P.content_name) =  P.cpf_pred_dir;
  pred_files.(P.content_name) = P.cpf_pred_fname;
end

if ~isempty(jobinfo),
  %rproc_kill(jobinfo) ;
  [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1);
  if num_crashed>0,
    error('%i jobs crashed. Cannot continue\n', num_crashed) ;
  end ;
end

save([output_dir '/prediction_files.mat'],'pred_dirs', '-v7');

fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\nUnderlying signal predictors:\n-----------------------------\n\n') ; fclose(fd) ;
for signal_idx = 1:length(signal_names)
  unix(sprintf('cat %s >> %s', train_files.(signal_names{signal_idx}), fn_mGene_predictor)) ;
  fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;
end ;

fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\nUnderlying content predictors:\n------------------------------\n\n') ; fclose(fd) ;
for content_idx = 1:length(content_names) ;
  unix(sprintf('cat %s >> %s', train_files.(content_names{content_idx}), fn_mGene_predictor)) ;
  fd=fopen(fn_mGene_predictor, 'a+') ; fprintf(fd, '\n\n') ; fclose(fd) ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% TRAIN, PREDICT AND EVAL Gene Structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


logfile = [output_dir '/log_train'];
pred_dirs.gene_predictor = sprintf('%s/gene_predictor/',output_dir);
fn_gene_predictor = sprintf('%s/gene_predictor.dat',output_dir);

save([output_dir '/trained_model.mat'],'train_dirs', '-v7');

signal_fns = ''; % spf-files if no matlab files 
content_fns = ''; % cpf-files if no matlab files 

%save('workspace_mgenetrain', '-v7')
gene_train(config_dir, annotation_dir, logfile, signal_fns, ...
           pred_dirs.tss, signal_fns, pred_dirs.tis, signal_fns, ...
           pred_dirs.acc, signal_fns, pred_dirs.don, signal_fns, ...
           pred_dirs.cdsStop, signal_fns, pred_dirs.cleave, content_fns, ...
           pred_dirs.intergenic, content_fns, pred_dirs.utr5exon, ...
           content_fns, pred_dirs.cds_exon, content_fns, pred_dirs.intron, ...
           content_fns, pred_dirs.utr3exon, fn_gene_predictor, ...
           pred_dirs.gene_predictor, train_options);


make_mgene_predictor(fn_gene_predictor, pred_dirs.gene_predictor, ...
                     train_files.tss, train_dirs.tss, train_files.tis, ...
                     train_dirs.tis, train_files.acc, train_dirs.acc, ...
                     train_files.don, train_dirs.don, train_files.cdsStop, ...
                     train_dirs.cdsStop, train_files.cleave, ...
                     train_dirs.cleave, train_files.intergenic, ...
                     train_dirs.intergenic, train_files.utr5exon, ...
                     train_dirs.utr5exon, train_files.cds_exon, ...
                     train_dirs.cds_exon, train_files.intron, ...
                     train_dirs.intron, train_files.utr3exon, ...
                     train_dirs.utr3exon, fn_mGene_predictor, ...
                     mGene_predictor_dir);


[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('finished mgene_train at %s\n', timedate) ;


fd=fopen(fn_mGene_predictor, 'a+') ; 
fprintf(fd, '\n mGene.web Train finished : %s\n', timedate) ;  
fclose(fd) ;

return

function genometool(fasta_fname, fasta_dir, information_file, base_dir) ;
% genometool(fasta_fname, fasta_dir, information_file, genome_config_dir) ;

global MGENE_SRC_PATH 

[ret, timedate] = unix('date') ;
assert(ret==0) ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started genometool at %s\n', timedate) ;


mkdir(base_dir) ;

if exist('information_file') && ~isempty(information_file)
  fprintf('Results are written to: %s\n', information_file) ;

  fd = fopen(information_file, 'w+') ;
else
  fd=1 ;
end ;
fprintf(fd,'---------------------------------------------------- \n');
fprintf(fd,'GenomeTool started %s\n', timedate) ;
fprintf(fd,'---------------------------------------------------- \n\n');
if fd~=1,
  fclose(fd)   
end

genome_config = [base_dir '/genome.config'] ;

fprintf('Using fasta file from %s \n\n',fasta_fname) ;

[ret,home]=unix('echo $HOME');
assert(ret==0) ;
python_script=sprintf('%s/make_gio.py',MGENE_SRC_PATH);
if exist('information_file')  && ~isempty(information_file),
  python_call = sprintf('python %s %s %s %s', python_script, fasta_fname, base_dir, information_file);
else
  tmpname = tempname 
  python_call = sprintf('python %s %s %s %s && cat %s && rm %s', python_script, fasta_fname, base_dir, tmpname, tmpname, tmpname)
end ;
ret=system(python_call);
assert(ret==0) ;

if exist('information_file')  && ~isempty(information_file),
  fd = fopen(information_file, 'a+') ;
else
  fd = 1 ;
end ;

[ret, timedate] = unix('date') ;
assert(ret==0) ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf(fd,'\n---------------------------------------------------- \n');
fprintf(fd,'finished GenomeTool at %s\n', timedate) ;
fprintf(fd,'---------------------------------------------------- \n');

if fd~=1,
  fclose(fd) ;
end ;

return

function gff2anno(genome_dir, gff_fname, annotation_fname, annotation_dir, version, seqid, source, level, ignore_missing_referenced_entries, run_locally, converter) ;
%        gff2anno(genome_dir, gff_fname, annotation_fname, annotation_dir, version, seqid, source, level, ignore_missing_referenced_entries, run_locally, converter) ;


[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
nice_mkdir(annotation_dir)
fd=fopen(annotation_fname, 'w+') ;

fprintf(fd,'-------------------------------------------------- \n') ;
fprintf(fd,'GFF2Anno started %s\n', timedate) ;
fprintf(fd,'-------------------------------------------------- \n\n') ;
fprintf('-------------------------------------------------- \n') ;
fprintf('GFF2Anno started %s\n', timedate) ;
fprintf('-------------------------------------------------- \n\n') ;

if ~exist('converter', 'var'),
  converter='';
end ;

if ~exist('run_locally')
  run_locally =-1;
end

if isequal(source, '-'),
  source='' ;
end ;
if isequal(seqid, '-'),
  seqid='' ;
end;
if isequal(level, '-'),
  level='' ;
end;
if ~exist('ignore_missing_referenced_entries'),
  ignore_missing_referenced_entries = 1 ;
end ;


fprintf(fd, 'Reading GFF File : %s\n\n',gff_fname) ; 
fprintf('Reading GFF File : %s\n\n',gff_fname) ; 

do_correct_tis_stop = 0 ;
if isempty(converter) && isequal(version, 'ensembl-gtf')
  converter = 'gtf-to-gff3' ;
  do_correct_tis_stop = 1 ; % ensembl GTF files do not often show
                            % the consensus sites for TIS and STOP
end ;

if ~isempty(converter),
  if any(converter == ',')
    converter = separate(converter, ','); 
  else
    converter = {converter} ;
  end ;

  if ismember('gtf-to-gff3', converter)
    fprintf(fd, 'Running GTF-to-GFF3 converter before parsing the file\n') ;
    fprintf('Running GTF-to-GFF3 converter before parsing the file\n') ;

    tmpname = sprintf('%s/gtf.gff3', annotation_dir) ;
    fprintf('Writing GFF3 file to %s\n', tmpname) ;

    gtf2gff(gff_fname, tmpname) ;

    gff_fname = tmpname ;
  end ;

  if ismember('correct-tis-stop', converter),
    do_correct_tis_stop = 1 ;
  end ;
end ;

disp('-------------------------------------') ;
disp('Step 1: Loading genome information...') ;
disp('-------------------------------------') ;

genome_config = [genome_dir '/genome.config'] ;
genome_info = init_genome(genome_config);



disp('------------------------------------') ;
disp('Step 2: Setting up datastructures...') ;
disp('------------------------------------') ;

fprintf('Input parameters:\n') ;
fprintf('version: "%s"\n', version) ;
fprintf('source : "%s"\n', source) ;
fprintf('seqid  : "%s"\n', seqid) ;
fprintf('level  : "%s"\n', level) ;

if isequal(version, 'generic')
  seqid_names = {};
  source_names = {} ;
  level1_names = {'gene'} ;
  level2_names = {'mRNA'} ;
  level3_names = {'CDS','five_prime_UTR','three_prime_UTR'} ;
  level3_coding_names = {'CDS'} ;
elseif isequal(version, 'wormbase')
  seqid_names = {};
  source_names = {'Coding_transcript'} ;
  level1_names = {'gene'} ;
  level2_names = {'mRNA'} ;
  level3_names = {'CDS','five_prime_UTR','three_prime_UTR'} ;
  level3_coding_names = {'CDS'} ;
elseif isequal(version, 'flybase')
  seqid_names = {};
  source_names = {'FlyBase','flybase'} ;
  level1_names = {'gene'} ;
  level2_names = {'mRNA'} ;
  level3_names = {'CDS','five_prime_UTR','three_prime_UTR'} ;
  level3_coding_names = {'CDS'} ;
elseif isequal(version, 'tair')
  seqid_names = {};
  source_names = {'TAIR8'} ;
  level1_names = {'gene'} ;
  level2_names = {'mRNA'} ;
  level3_names = {'CDS','five_prime_UTR','three_prime_UTR'} ;
  level3_coding_names = {'CDS'} ;
elseif isequal(version, 'ensembl-gtf')
  seqid_names = {};
  source_names = {'protein_coding'} ;
  level1_names = {'gene'} ;
  level2_names = {'mRNA'} ;
  level3_names = {'CDS','exon','five_prime_UTR','three_prime_UTR'} ;
  level3_coding_names = {'CDS'} ;
elseif isequal(version, 'other') 
  seqid_names = {};
  if ~isempty(seqid) && ismember(',',seqid)
    seqid_names = separate(seqid,','); 
  elseif ~isempty(seqid)
    seqid_names{1} = seqid;
  end ;
  
  source_names = {};
  if ~isempty(source) &&ismember(',', source)
    source_names = separate(source,','); 
  elseif ~isempty(source)
  source_names{1} = source;
  end
  
  levels = separate(level,':') ; 
  assert(isequal(levels{1},'level'))
  level1_names = {};
  if ~isempty(levels{2})&&ismember(',',levels{2})
    level1_names = separate(levels{2},','); 
  elseif ~isempty(levels{2})
    level1_names = levels(2);
  end
  
  level2_names = {};
  if ~isempty(levels{3})&&ismember(',',levels{3})
    level2_names = separate(levels{3},','); 
  elseif ~isempty(levels{3})
    level2_names = levels(3);
  end
  level3_names = {};
  level3_coding_names = {};
  if ~isempty(levels{4})&&ismember(',',levels{4})
    level3_names = separate(levels{4},','); 
    level3_coding_names = separate(levels{4},','); ;
  elseif ~isempty(levels{4})
    level3_names = levels(4);
    level3_coding_names = levels(4);
  end
  if isequal(levels{5}, 'None'),
    levels{5}='' ;
  end ;
  if ~isempty(levels{5}) && ismember(',',levels{5})
    names = separate(levels{5},','); 
    level3_names = {level3_names{:}, names{:}};
  elseif ~isempty(levels{5})
    level3_names = {level3_names{:}, levels{5}};
  end
else
  error('unknown version %s', version)
end

if ~isempty(seqid_names)
  fprintf(fd, 'Parsing following sequids: \n') ; 
  for i=1:length(seqid_names)
	  fprintf(fd, '\t %s\n',seqid_names{i}) ; 
  end
end ;

if ~isempty(source_names)
  fprintf(fd, 'Parsing following sources: \n') ; 
  for i=1:length(source_names)
  	fprintf(fd, '\t %s\n',source_names{i}) ; 
  end
end

fprintf(fd, 'Parsing following types (level 1): \n') ; 
for i=1:length(level1_names)
	fprintf(fd, '\t %s\n',level1_names{i}) ; 
end
fprintf(fd, 'Parsing following types (level 2): \n') ; 
for i=1:length(level2_names)
	fprintf(fd, '\t %s\n',level2_names{i}) ; 
end
fprintf(fd, 'Parsing following types (level 3): \n') ; 
for i=1:length(level3_names)
	fprintf(fd, '\t %s\n',level3_names{i}) ; 
end
fprintf(fd, 'Following types are considered coding exons: \n') ; 
for i=1:length(level3_coding_names)
	fprintf(fd, '\t %s\n\n',level3_coding_names{i}) ; 
end
fclose(fd) ;

TYPE.short_names = {level1_names{:} level2_names{:} level3_names{:}} ;
TYPE.IDs = TYPE.short_names ;
TYPE.hierachies = [ones(1,length(level1_names)) 2*ones(1,length(level2_names)) 3*ones(1,length(level3_names))] ;

fprintf('Done.\n\n') ;

nofgenes=0 ;
for i=1:length(level1_names),
	[ret, nofgenes_] = unix(sprintf('grep %s %s | wc -l', level1_names{i}, gff_fname)) ;
  assert(ret==0)
  nofgenes_ = str2num(nofgenes_);
  nofgenes = nofgenes + nofgenes_ ;
end ;

fprintf('Found %i level1 objects (genes, etc.) \n', nofgenes) ;

genes = init_genes(max(nofgenes,1));
[ret, seq_ids] = unix(sprintf('sed -e "1d" %s | grep -v "^#" | grep -v "^$" | cut -f 1 | sort -u', gff_fname)); 
assert(ret==0) ;

if isempty(seqid_names)
  contig_names = separate(seq_ids, sprintf('\n')) ;
  if ~isempty(contig_names) && isempty(contig_names{end})
    contig_names(end)=[] ;
  end ;
else
  contig_names = seqid_names;
end

% what is this useful for?
% in any case this is extremely slow for bigger files
if 0,
  N=0;
  tmp_fname = tempname;
  for contig=1:length(contig_names)
    cname = contig_names{contig};
    unix(sprintf('grep "^%s\t" %s > %s', cname,gff_fname,tmp_fname));
    [tmp,num_seqs] = unix(sprintf('sed -e "1d" %s | cut -f 1 | wc -l',tmp_fname));
    N = N+str2num( num_seqs);
  end
  if N==0
    error('GFF file does not contain specifed seqids')
  end
end ;

if run_locally ==-1
  run_locally = rproc_policy('gff2anno',genome_info,[nofgenes length(contig_names)]) ;
end

Data.source = source_names;
Data.gff_fname = gff_fname;
Data.TYPE = TYPE;
Data.genome_config = genome_config;
Data.level3_coding_names= level3_coding_names;
Data.do_correct_tis_stop = do_correct_tis_stop ;

%run_locally = 1 ; 

% split by contig
if ~run_locally
  fprintf('parsing by contig\n')
  for contig=1:length(contig_names)
    Data.cname = contig_names(contig);
    Data.save_fname = sprintf('%s/%s_', annotation_dir, Data.cname{:});
    Data.log_fname  = sprintf('%s/%s_log', annotation_dir, Data.cname{:});
    options.addpaths = {'~/svn/projects/genefinding/data_processing_sensors'} ;

    [mem_req, time_req, opts] = rproc_memtime_policy('mgene_gff2anno_helper',  0, options) ;

    jobinfo(contig)=rproc('mgene_gff2anno_helper', Data, mem_req, opts, time_req);
  end
  jobinfo = rproc_wait(jobinfo, 30, 1, -1);
  len = 0 ;
  for contig=1:length(contig_names)
    save_fname = sprintf('%s/%s_',annotation_dir,contig_names{contig});
    l = load([save_fname 'genes.mat'],'genes'); 
    genes(len+1:len+length(l.genes))=l.genes;
    len = len + length(l.genes);
    l = load([save_fname 'FAULTS1.mat'],'FAULTS1'); 
    FAULTS1(contig) = l.FAULTS1;
    l = load([save_fname 'FAULTS2.mat'],'FAULTS2'); 
    FAULTS2(contig) = l.FAULTS2;
    clear l
    log_fname  = sprintf('%s/%s_log', annotation_dir, Data.cname{:});
    unix(sprintf('cat %s >> %s', log_fname, annotation_fname)) ;
    unix(sprintf('rm %sgenes.mat',save_fname));
    unix(sprintf('rm %sFAULTS1.mat',save_fname));
    unix(sprintf('rm %sFAULTS2.mat',save_fname));
    genes = genes(1:len) ;
  end
else
  Data.cname = contig_names;
  Data.save_fname = [annotation_dir '/all_'];
  Data.log_fname  = [annotation_dir '/all_log'];

  mgene_gff2anno_helper(Data) ;

  load([Data.save_fname 'genes.mat'],'genes'); 
  load([Data.save_fname 'FAULTS1.mat'],'FAULTS1'); 
  load([Data.save_fname 'FAULTS2.mat'],'FAULTS2'); 

  unix(sprintf('cat %s >> %s', Data.log_fname, annotation_fname )) ;
  unix(sprintf('rm %s', Data.log_fname));
end
for i=1:length(genes)
  genes(i).id = i;
end

fprintf('Done.\n\n') ;


fd=fopen(annotation_fname, 'a') ;
fprintf(fd, '\nAnnotation with %i genes ', length(genes)) ;
% add some more information here

disp('------------------------------------') ;
disp('Step 7: Saving genome annotation ...') ;
disp('------------------------------------') ;

% unix(sprintf('mkdir -p %s', annotation_dir)) ;
save_struct(sprintf('%s/genes.mat', annotation_dir), genes, 'genes') ;
save_struct(sprintf('%s/FAULTS1.mat', annotation_dir),FAULTS1, 'FAULTS1') ;
save_struct(sprintf('%s/FAULTS2.mat', annotation_dir),FAULTS2, 'FAULTS2') ;

% save(sprintf('%s/info.mat', annotation_dir), 'genome_info', 'genome_config') ;
fprintf(fd, '(saved to %s/genes.mat)\n', annotation_dir) ;

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf(fd,'\n-------------------------------------------------- \n') ;
fprintf(fd,'GFF2Anno finished at %s\n', timedate) ;
fprintf(fd,'-------------------------------------------------- \n') ;
fprintf('\n-------------------------------------------------- \n') ;
fprintf('GFF2Anno finished at %s\n', timedate) ; 
fprintf('--------------------------------------------------- \n') ;

fclose(fd) ;

disp('Done.') ;



return

function  genes = curate_anno(genes, genome_info, log_fname)
  % genes = curate_anno(genes, genome_info, log_fname)

if exist('log_fname')
  fd=fopen(log_fname, 'a') ;
end

all_trans_comp = 0;  
some_trans_comp = 0; 
add_utr3 = zeros(1,length(genes));
add_utr5 = zeros(1,length(genes));
add_cds = zeros(1,length(genes));

add_5utr_length = 10;
add_3utr_length = 10;

for i=1:length(genes),
  gene = genes(i);
  gene_contig_len = contig_len(gene.chr, genome_info) ;

  if ~gene.is_valid
    continue
  end
  idx = find(gene.transcript_complete) ;
  if all(gene.transcript_complete)
    all_trans_comp = all_trans_comp + 1;
  elseif ~isempty(idx) 
    % remove incomplete transcripts
    some_trans_comp = some_trans_comp + 1;
    gene.exons = gene.exons(idx) ;
    gene.exons_confirmed = gene.exons_confirmed(idx);
    gene.cds_exons = gene.cds_exons(idx) ;
    gene.transcripts = gene.transcripts(idx) ;
    gene.transcript_valid = gene.transcript_valid(idx) ;
    gene.transcript_complete = gene.transcript_complete(idx) ;
    
    if ~isempty(gene.transcript_status)
      gene.transcript_status = gene.transcript_status(idx) ;
    end  
    gene.utr5_exons = gene.utr5_exons(idx) ;
    gene.utr3_exons = gene.utr3_exons(idx) ;
    if ~isempty(gene.tis),
      gene.tis = gene.tis(idx) ;
    end ;
    if ~isempty(gene.cdsStop),
      gene.cdsStop = gene.cdsStop(idx) ;
    end ;
    gene.tss = gene.tss(idx) ;
    gene.cleave = gene.cleave(idx) ;
    try,
      gene.polya = gene.polya(idx) ;
    catch
      % warning('polya elements missing') ;
    end ;
    if isempty(idx),
      gene.is_valid = 0 ;
    end ;
  else
    % fill in missing entries for all transcripts
    for k=1:length(gene.transcripts)
      
      % exons must be annotated
      if length(gene.exons)<k||isempty(gene.exons{k})
        assert(gene.is_valid==0)
        continue
      end
      
      % check if cds_exons are annotated
      if length(gene.cds_exons)>=k||~isempty(gene.cds_exons{k})
        
        % cds exons and exons have identical start  and no utrs
        if isequal(gene.cds_exons{k}(1,1), gene.exons{k}(1,1))
          if gene.strand =='+' 
            % no 5' utrs annotated 
            if length(gene.utr5_exons)<k || isempty(gene.utr5_exons{k})
              if gene.exons{k}(1,1)>add_5utr_length,
                add_utr5(i) = add_utr5(i)+1; 
                gene.utr5_exons{k} = [gene.exons{k}(1,1) - add_5utr_length, gene.exons{k}(1,1)];
                gene.exons{k}(1,1) = gene.exons{k}(1,1) - add_5utr_length; 
                assert(gene.tis{k} == gene.cds_exons{k}(1,1));
                gene.tss{k} = gene.exons{k}(1,1);
              else
                gene.utr5_exons{k} = [gene.exons{k}(1,1), gene.exons{k}(1,1)];
                gene.transcript_valid(k) = 0 ;
              end ;
            else
              warning('cds exons and exons identical, utrs annoted, check parser')
            end
          else %gene.strand =='-'
               % no 3' utrs annotated 
            if length(gene.utr3_exons)<k || isempty(gene.utr3_exons{k})
  	      if gene.exons{k}(1,1)>add_3utr_length,
                add_utr3(i) = add_utr3(i)+1; 
                gene.utr3_exons{k} = [gene.exons{k}(1,1) - add_3utr_length gene.exons{k}(1,1)];
                gene.exons{k}(1,1) = gene.exons{k}(1,1) - add_3utr_length; 
                assert(gene.cdsStop{k} == gene.cds_exons{k}(1,1)+3);
                gene.cleave{k} = gene.exons{k}(1,1);
	      else
		gene.utr3_exons{k} = [gene.exons{k}(1,1) gene.exons{k}(1,1)];
                gene.transcript_valid(k) = 0 ;
              end ;
            else % cds_exons and exons are identical, but annotated utrs
              warning('cds exons and exons identical, utrs annoted, check parser')
            end
          end
          
        end
        
        % cds exons and exons have identical end and no utrs
        if isequal(gene.cds_exons{k}(end,2), gene.exons{k}(end,2))
          if gene.strand =='+'
            % no 3'utrs annotated
            if length(gene.utr3_exons)<k||isempty(gene.utr3_exons{k})
              if gene.exons{k}(end,2) + add_3utr_length<gene_contig_len
                add_utr3(i) = add_utr3(i)+1; 
                gene.utr3_exons{k} = [gene.exons{k}(end,2) gene.exons{k}(end,2) + add_3utr_length]; 
                gene.exons{k}(end,2) = gene.exons{k}(end,2) + add_3utr_length; 
                assert(gene.cdsStop{k} == gene.cds_exons{k}(end,2)-3);
                gene.cleave{k} = gene.exons{k}(end,2);
              else
		gene.utr3_exons{k} = [gene.exons{k}(end,2) gene.exons{k}(end,2)];
                gene.transcript_valid(k) = 0 ;
              end ;
            else
              warning('cds exons and exons identical, utrs annoted, check parser')
            end
          else %gene.strand =='-'
               % no 5' utrs annotated 
            if length(gene.utr5_exons)<k || isempty(gene.utr5_exons{k})
              if gene.exons{k}(end,2) + add_5utr_length<gene_contig_len,
                add_utr5(i) = add_utr5(i)+1; 
                gene.utr5_exons{k} = [gene.exons{k}(end,2) gene.exons{k}(end,2) + add_5utr_length]; 
                gene.exons{k}(end,2) = gene.exons{k}(end,2) + add_5utr_length; 
                assert(gene.tis{k} == gene.cds_exons{k}(end,2));
                gene.tss{k} = gene.exons{k}(end,2);
              else
                gene.utr5_exons{k} = [gene.exons{k}(end,2), gene.exons{k}(end,2)];
                gene.transcript_valid(k) = 0 ;
              end ;
            else % cds_exons and exons are identical, but annotated utrs
              warning('cds exons and exons identical, utrs annoted, check parser')
            end
          end %sif strand          
        end
        
        if isempty(gene.tss) || isempty(gene.tss{k}),
          fprintf('warning: adding missing tss site\n') ;
          if gene.strand == '+',
            gene.tss{k} = gene.exons{k}(1,1) ;
          else
            gene.tss{k} = gene.exons{k}(end,2) ;
          end ;
        end ;
        if isempty(gene.cleave) || isempty(gene.cleave{k}),
          fprintf('warning: adding missing cleavage site\n') ;
          if gene.strand == '+',
            gene.cleave{k} = gene.exons{k}(end,2)
          else
            gene.cleave{k} = gene.exons{k}(1,1) ;
          end ;
        end ;
        
        if gene.transcript_valid(k),
          try
            %  cds_exons and exons are not identical, transcript should be complete 
            if gene.strand =='+'
              assert(gene.tis{k} == gene.cds_exons{k}(1,1));
              assert(gene.cdsStop{k} == gene.cds_exons{k}(end,2)-3);
              assert(gene.tss{k} == gene.exons{k}(1,1));
              assert(gene.cleave{k} == gene.exons{k}(end,2));
              assert(gene.utr5_exons{k}(1,1)==gene.tss{k});
              assert(gene.utr5_exons{k}(end,2)==gene.tis{k});
              assert(gene.utr3_exons{k}(1,1)==gene.cdsStop{k}+3);
              assert(gene.utr3_exons{k}(end,2)==gene.cleave{k});
            else
              assert(gene.tis{k} == gene.cds_exons{k}(end,2));
              assert(gene.cdsStop{k} == gene.cds_exons{k}(1,1)+3);
              assert(gene.tss{k} == gene.exons{k}(end,2));
              assert(gene.cleave{k} == gene.exons{k}(1,1));
              assert(gene.utr5_exons{k}(1,1)==gene.tis{k});
              assert(gene.utr5_exons{k}(end,2)==gene.tss{k});
              assert(gene.utr3_exons{k}(1,1)==gene.cleave{k});
              assert(gene.utr3_exons{k}(end,2)==gene.cdsStop{k}-3);
            end
          catch
            warning('one of the final checks failed, invalidating transcript %s', gene.name) ;
            gene.transcript_valid(k)=0 ;
          end;
        end ;
      elseif length(gene.cds_exons)<k||isempty(gene.cds_exons{k})
        warning('find orf')
        add_cds(i) = add_cds(i)+1;
      end
    end % loop over trancripts     
    if gene.strand=='+'
      gene.start = min([gene.tss{:}]);
      gene.stop = max([gene.cleave{:}]);
    else
      gene.start = min([gene.cleave{:}]);
      gene.stop = max([gene.tss{:}]);
    end
    genes(i) = gene;
  end % if all transcripts are incomplete
  
end %%loop over genes

take_map=ones(1,length(genes)) ;
num_trans_removed = 0 ;
for i=1:length(genes),
  idx = find(genes(i).transcript_valid) ;
  if isempty(idx), take_map(i)=0 ; end ;
  num_trans_removed = num_trans_removed + length(genes(i).transcript_valid)-length(idx) ;
  
  genes(i).exons = genes(i).exons(idx) ;
  genes(i).exons_confirmed = genes(i).exons_confirmed(idx);
  genes(i).cds_exons = genes(i).cds_exons(idx) ;
  genes(i).transcripts = genes(i).transcripts(idx) ;
  genes(i).transcript_valid = genes(i).transcript_valid(idx) ;
  genes(i).transcript_complete = genes(i).transcript_complete(idx) ;
end ;
genes=genes(take_map==1) ;
num_gene_removed = sum(take_map==0) ;

fprintf(1, ' %i genes with all transcripts complete.\n', sum(all_trans_comp)) ;
fprintf(1, ' %i genes with at least one transcript complete.\n', sum(some_trans_comp)) ;
fprintf(1, ' %i 5UTRs added to transcripts of %i genes.\n',sum(add_utr5), sum(add_utr5>0) ) ;
fprintf(1, ' %i 3UTRs added to transcripts of %i genes.\n',sum(add_utr3), sum(add_utr3>0) ) ;
fprintf(1, ' %i max ORF determined for transcripts of %i genes.\n', sum(add_cds), sum(add_cds>0)) ;
fprintf(1, ' %i transcripts and %i genes removed at contig boundary\n', num_trans_removed, num_gene_removed) ;

if exist('fd')
fprintf(fd, '\n')
fprintf(fd, ' %i genes with all transcripts complete.\n', sum(all_trans_comp)) ;
fprintf(fd, ' %i genes with at least one transcript complete.\n', sum(some_trans_comp)) ;
fprintf(fd, ' %i 5UTRs added to transcripts of %i genes.\n',sum(add_utr5), sum(add_utr5>0) ) ;
fprintf(fd, ' %i 3UTRs added to transcripts of %i genes.\n',sum(add_utr3), sum(add_utr3>0) ) ;
fprintf(fd, ' %i max ORF determined for transcripts of %i genes.\n', sum(add_cds), sum(add_cds>0)) ;
fprintf(fd, ' %i transcripts and %i genes removed at contig boundary\n', num_trans_removed, num_gene_removed) ;
fclose(fd);
end

return

function genes = filter_invalid_genes(genes, genome_info,log_fname)
%  genes = filter_invalid_genes(genes, genome_info,log_fname)
  
if exist('log_fname')
  fd=fopen(log_fname, 'a') ;
end

ignore_missing_referenced_entries = 1;  
ignore_genes_without_transcripts = 1;
ignore_genes_without_exons = 1;
ignore_genes_without_chr = 1;

take_map.is_valid = ones(1,length(genes)) ;
take_map.no_parents = ones(1,length(genes)) ;
take_map.no_transcripts = zeros(1,length(genes)) ;
take_map.no_exons = ones(1,length(genes)) ;
take_map.no_chr = ones(1,length(genes)) ;

%--------------

if length(genes)==1 && isequal(genes.id, []) && isequal(genes.transcripts, [])
  genes(1)=[] ;
end ;
assert(length([genes.is_valid]) == length(genes)) ;
take_map.is_valid(find(~[genes.is_valid]))=0 ;

assert(length([genes.is_correctly_gff3_referenced]) == length(genes)) ;
take_map.no_parents(find(~[genes.is_correctly_gff3_referenced]))=0 ;

num_invalid_transcripts = 0 ;

for i=1:length(genes),
  % find genes with no transcript
  take_map.no_transcripts(i) = ~isempty(genes(i).transcripts) ;
  % find genes with no exons 
  empty = 1;
  for j=1:length(genes(i).exons),
    if ~isempty(genes(i).exons{j})
      empty=0 ;
      break
    end ;
  end ;
  if empty,
    take_map.no_exons(i) = 0 ;
  end ;
  % find genes of unknown contig
  chr_num = find(ismember(lower(genome_info.contig_names), lower(genes(i).chr))) ;
  take_map.no_chr(i) = length(chr_num)==1 ;
  
  % remove invalid transcripts
  num_invalid_transcripts = num_invalid_transcripts + sum(~genes(i).transcript_valid) ;
  idx = find(genes(i).transcript_valid) ;
  genes(i).exons = genes(i).exons(idx) ;
  genes(i).exons_confirmed = genes(i).exons_confirmed(idx);
  genes(i).cds_exons = genes(i).cds_exons(idx) ;
  genes(i).transcripts = genes(i).transcripts(idx) ;
  genes(i).transcript_valid = genes(i).transcript_valid(idx) ;
  try
    genes(i).transcript_complete = genes(i).transcript_complete(idx) ; 
  catch
  end
  if ~isempty(genes(i).transcript_status)
    genes(i).transcript_status = genes(i).transcript_status(idx) ;
  end  
  genes(i).utr5_exons = genes(i).utr5_exons(idx) ;
  genes(i).utr3_exons = genes(i).utr3_exons(idx) ;
  if ~isempty(genes(i).tis),
    genes(i).tis = genes(i).tis(idx) ;
  end ;
  if ~isempty(genes(i).cdsStop),
    genes(i).cdsStop = genes(i).cdsStop(idx) ;
  end ;
  try,
    genes(i).tss = genes(i).tss(idx) ;
  catch
    % warning('tss elements missing') ;
  end ;
  try,
    genes(i).cleave = genes(i).cleave(idx) ;
  catch
    % warning('cleave elements missing') ;
  end ;
  try,
    genes(i).polya = genes(i).polya(idx) ;
  catch
    % warning('polya elements missing') ;
  end ;
end ;

fprintf(1, ' %i valid genes.\n', sum(take_map.is_valid==1)) ;
fprintf(1, ' %i invalid genes.\n', sum(take_map.is_valid==0)) ;
fprintf(1, '      %i genes with no exon segments.\n', sum(take_map.no_exons==0)) ;
fprintf(1, '      %i genes with no transcripts.\n', sum(take_map.no_transcripts==0)) ;
fprintf(1, '      %i genes incorrectly referenced (parent gene missing).\n', sum(take_map.no_parents==0)) ;
fprintf(1, '      %i genes on unknown contigs/chromosomes.\n', sum(take_map.no_chr==0)) ;
fprintf(1, '      %i invalid transcripts found.\n',num_invalid_transcripts);
if exist('fd')
fprintf(fd,'\n')
fprintf(fd, ' %i valid genes.\n', sum(take_map.is_valid==1)) ;
fprintf(fd, ' %i invalid genes.\n', sum(take_map.is_valid==0)) ;
fprintf(fd, '      %i genes with no exon segments.\n', sum(take_map.no_exons==0)) ;
fprintf(fd, '      %i genes with no transcripts.\n', sum(take_map.no_transcripts==0)) ;
fprintf(fd, '      %i genes incorrectly referenced (parent gene missing).\n', sum(take_map.no_parents==0)) ;
fprintf(fd, '      %i genes on unknown contigs/chromosomes.\n', sum(take_map.no_chr==0)) ;
fprintf(fd, '      %i invalid transcripts found.\n',num_invalid_transcripts);
fclose(fd);
end

take_idx = ones(1,length(genes));
take_idx = take_idx&take_map.is_valid;

if ignore_missing_referenced_entries,
  take_idx = take_idx&take_map.no_parents;
end ;
if ignore_genes_without_transcripts
  take_idx = take_idx&take_map.no_transcripts;
end ;
if ignore_genes_without_exons
  take_idx = take_idx&take_map.no_exons;
end ;
if ignore_genes_without_chr
  take_idx = take_idx&take_map.no_chr;
end ;
  
genes = genes(take_idx) ;
  
      
fprintf('Done.\n\n') ;


return

function gene_train(	genome_info_dir, anno_dir, logfile, tss_fn, tss_dir, tis_fn, tis_dir, ...
			acc_fn, acc_dir, don_fn, don_dir, stop_fn, stop_dir, cleave_fn, cleave_dir,...
			intergenic_fn, intergenic_dir, utr5_fn, utr5_dir, exon_fn, exon_dir, intron_fn, intron_dir, ...
			utr3_fn, utr3_dir, fn_out, output_dir, train_options)
% gene_train(	genome_info_dir, anno_dir, logfile, tss_fn, tss_dir, tis_fn, tis_dir, ...
%		acc_fn, acc_dir, don_fn, don_dir, stop_fn, stop_dir, cleave_fn, cleave_dir,...
%		intergenic_fn, intergenic_dir, utr5_fn, utr5_dir, exon_fn, exon_dir, intron_fn, intron_dir, ...
%               utr3_fn, utr3_dir, fn_out, output_dir, maxNumIter)

global SHOGUN_PATH USE_SGE

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;
fprintf('started gene_train at %s\n', timedate) ;

fprintf('Results are written to: %s\n', fn_out) ;

fd_out = fopen(fn_out, 'w+') ;
fprintf(fd_out, 'Trained gene predictor (training started %s)\n\n', timedate) ;

disp('------------------------------------') ;
disp('START GENE TRAIN');
disp('------------------------------------') ;

extra_path = SHOGUN_PATH ;
shogun_envstr = ['export LD_LIBRARY_PATH=' SHOGUN_PATH] ;
addpath(extra_path)

%fid = fopen(logfile, 'w+');
fid=1 ;

if ~exist('train_options', 'var'),
  train_options = '' ;
end ;

%defaults
C=0.02 ;
maxNumIter  = 200 ;
maxNumBlock = inf ;
block_design = 'merge' ;
block_design_merge_num = 3 ;
block_design_merge_sep = 1000 ;
block_design_regionauto_num = 3 ;
block_design_regionlist = '' ;

if ~isempty(train_options)
  train_option_items = separate(train_options, ';') ;
else
  train_option_items=[] ;
end ;
for i=1:length(train_option_items),
  if isempty(train_option_items{i}), 
    continue ;
  end ;
  items = separate(train_option_items{i},'=') ;
  if isequal(items{1}, 'block_design_merge_sep')
    block_design_merge_sep = str2num(items{2}) ;
  elseif isequal(items{1}, 'maxNumIter')
    maxNumIter = str2num(items{2}) ;
  elseif isequal(items{1}, 'maxNumBlock')
    if ~isequal(items{2},'-'),
      maxNumBlock = str2num(items{2}) ;
      if maxNumBlock==0,
        maxNumBlock=inf ;
      end ;
    end ;
  elseif isequal(items{1}, 'C')
    C = str2num(items{2}) ;
  elseif isequal(items{1}, 'block_design') ;
    block_design = items{2} ;
  elseif isequal(items{1}, 'block_design_merge_num') ;
    block_design_merge_num = str2num(items{2}) ;
  elseif isequal(items{1}, 'block_design_merge_sep') ;
    block_design_merge_sep = str2num(items{2}) ;
  elseif isequal(items{1}, 'block_design_regionauto_num') ;
    block_design_regionauto_num = str2num(items{2}) ;
  elseif isequal(items{1}, 'block_design_regionlist')
    block_design_regionlist = items{2},
  elseif isempty(items{1})
  else
    train_option_items{i}
    error('unknown option') ;
  end ;
end ;

for fd=[1 fd_out],
  fprintf(fd, '\nThe following options are used:\n') ;
  fprintf(fd, '\tMaximal number of iterations:\t\t %i\n', maxNumIter) ;
  fprintf(fd, '\tRegularization parameter C:\t\t %1.2f\n', C) ;
  fprintf(fd, '\tMaximal number of training blocks:\t %1.2f\n', maxNumBlock) ;
  fprintf(fd, '\tBlock design\t\t\t\t %s\n', block_design) ;
  if isequal(block_design, 'merge')
    fprintf(fd, '\t\tMerge %i genes into one block\n', block_design_merge_num) ;
    fprintf(fd, '\t\tSeparation between genes\t %ibp\n', block_design_merge_sep) ;
  elseif isequal(block_design, 'regionauto')
    fprintf(fd, '\t\tMerge %i genes into one block\n', block_design_regionauto_num) ;
  elseif isequal(block_design, 'regionlist')
    fprintf(fd, '\t\tRegion list:\t\t%s\n', block_design_regionlist) ;
  end ;
  fprintf(fd, '\n') ;
end ;

if isequal(block_design, 'regionauto') || isequal(block_design, 'regionlist')
  error('This feature is not implemented yet. Please come back later.') ;
end ;

anno_log_fname 	= [anno_dir '/mgene_anno.txt'];
gff_fname 	= [anno_dir '/anno.gff'];
fn_genes 	= [anno_dir '/genes.mat'];
%if ~fexist(fn_genes)
%  disp('------------------------------------') ;
%  disp('generate gene structure from gff file');
%  disp('------------------------------------') ;

%  fprintf(fid, 'parse gff-annotation file...');
%  gff2anno(gff_fname, anno_log_fname, anno_dir, 1, 1) ;
%  fprintf(fid, 'done\n');
%end

disp('------------------------------------') ;
disp('Obtain parameter structure');
disp('------------------------------------') ;


fn_config =  [genome_info_dir '/genome.config'] ;

galaxy_PAR.dir = output_dir ;
galaxy_PAR.genome_config = fn_config;

PAR = create_PAR('exp_galaxy', galaxy_PAR);
PAR.tasks.run_locally=1 ;

create_directories(PAR.FN, PAR.organism) ;

PAR.FN.input.fn_genome_config = fn_config ;
PAR.FN.genome.fn_genome_config = fn_config ;
organism = PAR.organism.name;

%struct_levels_to_print(2)
%PAR.LSL.method.par_ms.C_regul

PAR.LSL.method.max_num_iterations = maxNumIter;
PAR.LSL.method.par_ms.C_regul.contents = C ;
PAR.LSL.method.par_ms.C_regul.signals = C ;
PAR.LSL.method.par_ms.C_regul.lengths = C ;
PAR.LSL.method.par_ms.C_regul.tiling = C ;
PAR.LSL.method.par_ms.C_regul.rna_seq = C ;
PAR.LSL.method.par_ms.C_regul.rna_seq_intron_track = C ;
PAR.LSL.method.par_ms.C_regul.transitions = C ;
PAR.LSL.method.par_ms.C_regul.transitions_sq = C ;
PAR.LSL.method.par_ms.C_regul.plif_ys_sq = C ;
PAR.LSL.method.par_ms.C_regul.smoothness_sq = C ;

[tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir] = ...
    sort_signals(tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir) ;

ok = check_signal(tss_dir, 'tss') ; assert(ok) ;
ok = check_signal(tis_dir, 'tis') ; assert(ok) ;
ok = check_signal(acc_dir, 'acc') ; assert(ok) ;
ok = check_signal(don_dir, 'don') ; assert(ok) ;
ok = check_signal(stop_dir, 'cdsStop') ; assert(ok) ;
ok = check_signal(cleave_dir, 'cleave') ; assert(ok) ;

PAR.FN.output_sig.tss.fn_pred = [tss_dir '/pred/'] ;
PAR.FN.output_sig.tis.fn_pred = [tis_dir '/pred/'] ;
PAR.FN.output_sig.acc.fn_pred = [acc_dir '/pred/'] ;
PAR.FN.output_sig.don.fn_pred = [don_dir '/pred/'] ;
PAR.FN.output_sig.cdsStop.fn_pred = [stop_dir '/pred/'] ;
if isfield(PAR.FN.output_sig, 'polya') ;
  PAR.FN.output_sig=rmfield(PAR.FN.output_sig, 'polya') ;
end 
PAR.FN.output_sig.cleave.fn_pred = [cleave_dir '/pred/'] ;

if isfield(PAR.model.signals, 'polya')
  PAR.model.signals = rmfield(  PAR.model.signals, 'polya') ;
end ;


disp('------------------------------------') ;
disp('Loading genes and fixing boundaries/splicegraphs');
disp('------------------------------------') ;

fprintf(fid, 'Start preprocessing of genes for training:\n');
load(fn_genes, 'genes') ;
genome_info = init_genome(fn_config);
genes = filter_invalid_genes(genes, genome_info);

%genes = genes(~[genes.is_alt]) ;
%genes = genes(1:3) ;
for i=1:length(genes),
  for j=1:length(genes(i).transcripts)
    genes(i).start = min(genes(i).start, min(min(genes(i).exons{j}(:,1:2)))) ;
    genes(i).stop  = max(genes(i).stop, max(max(genes(i).exons{j}(:,1:2)))) ;
  end ;
  genes(i).id = i ;
end ;
genes = update_splicegraph(genes, PAR.tasks.run_locally) ;
fprintf(fid, 'Done.\n');

fprintf('Generated %i usable gene structures from genome annotation\n\n', length(genes)) ;
fprintf(fd_out, 'Generated %i usable gene structures from genome annotation\n\n', length(genes)) ;

disp('------------------------------------') ;
disp('generate blocks from genes');
disp('------------------------------------') ;

for i=1:length(genes),
  genes(i).transcript_status = ones(1,length(genes(i).transcripts)) ;
end ;

fprintf(fid, 'Check consistency of training examples:\n')
blocks = make_blocks_from_genes(genes, fn_config, block_design_merge_sep, 0) ;
for i=1:length(blocks), blocks(i).split = [1] ; end ;

fprintf(1,'Check genes, generate segmentation and do some filter\n')
[blocks, genes, split] = gen_lsl_blocks(genes, blocks, PAR.FN.genome.fn_genome_config, PAR.model, '', PAR.tasks.run_locally);
fprintf(fid, 'done\n');

fprintf('Saving genes\n') ;
save(strrep(PAR.FN.input_lsl.fn_training_blocks, 'blocks', 'genes'), 'genes', 'PAR', '-v7') ;
clear genes ;

% reduce number of blocks
%------------------------------------------------------------- 
train_test_cnt = subsample_policy(length(blocks), 'gene');
train_test_cnt(1) = min(train_test_cnt(1), maxNumBlock) ;
maxval = sum(train_test_cnt);
merge_num = block_design_merge_num; 
if length(blocks)/merge_num>maxval*1.1
  fprintf(1,'Reduce total number of blocks to some reasonable number (from %i to ', length(blocks));
  blocks = blocks(1:floor(maxval*merge_num*1.1)) ;
  fprintf('%i)', length(blocks)) ;
end

% add signals to blocks 
%------------------------------------------------------------- 
fprintf(fid, 'Load signal predictions:\n');
blocks = make_prediction_blocks(PAR, blocks, PAR.model, 0);
%fprintf(fid, 'done\n');

% add content predictions to blocks
%------------------------------------------------------------- 
fprintf(fid, 'Load predictions of content sensors:\n');
[exon_dir, intron_dir, intergenic_dir, utr5_dir, utr3_dir] = ...
    sort_contents(exon_dir, intron_dir, intergenic_dir, utr5_dir, utr3_dir) ;
cont_pred_dirs = {exon_dir, intron_dir, intergenic_dir, utr5_dir, utr3_dir}; 
blocks = add_content_predictions_to_blocks(blocks, cont_pred_dirs);
%fprintf(fid, 'done\n'); 

%%%%%%%%%%%%%%%%%%%%
% merge blocks for lsl training
%%%%%%%%%%%%%%%%%%%%

fprintf(1, 'Merge single-gene blocks \n')
%blocks_orig = blocks ;
blocks = merge_blocks(blocks, PAR.model, merge_num);
for i=1:length(blocks), blocks(i).split = [1] ; end ;

%%%%%%%%%%%%%%%%%%%%
% obtain signal predictions for lsl training
%%%%%%%%%%%%%%%%%%%%
fprintf(fid, 'Create feature matrix for dynamic programs:\n');
%fprintf(1,'generate feature matrix for blocks \n')
blocks = gen_features(blocks, PAR.model);
%fprintf(fid, 'done\n');


%%%%%%%%%%%%%%%%%%%%
% subsample to block positions
%%%%%%%%%%%%%%%%%%%%
for i=1:length(blocks)
  blocks(i).content_pred = blocks(i).content_pred(:, blocks(i).all_pos) ;
end ;

% move tss and cleave signal positions in true segmentation to nearest 
% position with tss or cleave prediction
%-------------------------------------------------------------
fprintf(1, 'Find nearest predicted tss and cleave position for the annotated ones and move them\n')
rm_idx = [];
for b = 1:length(blocks)
  for t=1:length(blocks(b).truth)
    new_truth = correct_cleave_and_tss_pos(PAR, blocks(b).truth(t), blocks(b), PAR.model);
    if isempty(new_truth), 
      rm_idx = [rm_idx b];
      continue ;
    end
    blocks(b).truth(t) = new_truth ;
  end
end
fprintf(1, 'Remove %i of %i blocks because no tss or cleave position was found\n', length(rm_idx), length(blocks));
blocks(rm_idx) = [];


%%%%%%%%%%%%%%%%%%%%%%%%
% create paths from segmentation
%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(fid, 'Create label sequence for dynamic programm:\n');
%fprintf(1,'create paths from segmentations\n')
take_map = logical(ones(1,length(blocks))) ;

for b = 1:length(blocks)
  blocks(b).genestr=blocks(b).seq ;
  blocks(b).pos=blocks(b).all_pos ;

  for t=1:length(blocks(b).truth)
    [blocks(b).truth(t).path,blocks(b).truth(t).pos_idx,blocks(b).truth(t).pos]=...
         segmentation2path(blocks(b).truth(t),blocks(b),PAR.model);
    % check that the inverse function produces again the original segmentation
    [segments,tmp,ok] = path2segmentation(blocks(b).truth(t).path, blocks(b).truth(t).pos_idx, blocks(b), PAR.model);
    if ok,
      assert(isequal(segments,blocks(b).truth(t).segments(:,1:3)))
    else
      take_map(b)=false ;
    end ;
    clear segments
  end
end
blocks = blocks(take_map) ;
%fprintf(fid, 'done\n'); 

% determine length distributions for segment types
%-------------------------------------------------------------
PAR.model = prepare_model(PAR,blocks) ;

f = fieldnames(PAR.model.boundaries.lengths); 
for j=1:length(f), 
  %if strcmp(f{j},'intergenic_long'), continue, end
  if PAR.model.lengths_range.(f{j})(2) < exp(PAR.model.boundaries.lengths.(f{j})(end-1))+1
    PAR.model.lengths_range.(f{j})(2) = exp(PAR.model.boundaries.lengths.(f{j})(end-1))+1; 
  end
end


% check consistency of 'true' path with model
%-------------------------------------------------------------
fprintf(fid, 'Check consistency of label sequence with gene model: ')
ok_idx = check_block_model_consistency(blocks,PAR.model);
if length(ok_idx)<length(blocks)*90/100
  fprintf(fid, '\n%i out of %i blocks have label sequences that are not consistent with our gene model\n', length(blocks)-length(ok_idx),length(blocks));
end
blocks = blocks(ok_idx);

for i=1:length(blocks),
  blocks(i).id = i ;
end ;
fprintf(fid, 'Done.\n');


% compute weights for the true paths
%-------------------------------------------------------------

fprintf(fid, 'Obtain features for the label sequences:\n');
blocks = initialize_blocks(blocks, PAR.model, 1, PAR.FN);
%fprintf(fid, 'done\n');



% split blocks in training and validation examples
%-------------------------------------------------------------------------------- 

fprintf('Generated %i usable blocks (each with 3 genes) from genome annotation\n\n', length(blocks)) ;
fprintf(fd_out, 'Generated %i usable blocks (each with 3 genes) from genome annotation\n\n', length(blocks)) ;

train_test_cnt = subsample_policy(length(blocks), 'gene');
num_train = min(train_test_cnt(1), maxNumBlock) ;
num_test = train_test_cnt(2);

fprintf('\n\n-------------------------------------------------------------------------------------\n') ;
blocks_train = blocks(1:num_train) ;
if num_test>0
  fprintf('performing validation on %i blocks (%i training blocks)\n', num_test, num_train) ;
  fprintf(fd_out, 'Performing validation on %i blocks (%i training blocks).\n\n', num_test, num_train) ;
  blocks_val = blocks(num_train+1:num_train+num_test) ;
else
  fprintf('performing validation on %i training blocks (not enough examples)\n', length(blocks)) ;
  fprintf(fd_out, 'Performing validation on %i training blocks (not enough examples).\n\n', length(blocks)) ;
  blocks_val = blocks;
end
fprintf('-------------------------------------------------------------------------------------\n\n') ;

fprintf('saving blocks') ;
save_append(PAR.FN.input_lsl.fn_training_blocks, 0, 'blocks', blocks_train) ;
save_append(PAR.FN.input_lsl.fn_val_blocks, 0, 'blocks', blocks_val) ;

blocks_split{1} = 1:length(blocks_train) ;
save(PAR.FN.input_lsl.fn_training_split, 'blocks_split', '-v7') ;

PAR_predict = PAR ;
PAR_predict.fn_pred = [output_dir '/predict/'] ;
unix(sprintf('rm -rf %s; mkdir -p %s', PAR_predict.fn_pred, PAR_predict.fn_pred)) ;
PAR_predict.fn_blocks = [PAR_predict.fn_pred '/blocks.mat'] ;
save_append(PAR_predict.fn_blocks, 0, 'blocks', blocks_val) ;


PAR.LSL.method.add_lin_feat = 0 ;
PAR.LSL.method.exm_per_solve = 500 ;
%PAR.RPROC.options.immediately_bg = 1 ;
if length(blocks_train)>500,
  PAR.RPROC.exm_per_batch = 5 ;
else
  PAR.RPROC.exm_per_batch = 1 ;
end ;

PAR.RPROC.options.addpaths = { fileparts(which('sg'))};
PAR.RPROC.options.envstr = shogun_envstr ;

clear blocks blocks_val blocks_train ;

if USE_SGE,
  PAR.submit = 1;
else
  PAR.submit = 0;
end ;

PAR.debug_mode = 0 ;
fprintf(fid, 'start training of the gene predictor (num_iterations=%i)...', maxNumIter) ;
[fn_predictor, iter] = train_path(PAR) ;
unix(sprintf('cp %s %s/predictor.mat', fn_predictor, output_dir));
fprintf(fid, 'Done.\n\n');

fprintf(fd_out, 'Optimized mGene QP for %i/%i iterations\n\n', iter, maxNumIter) ;
fprintf('Optimized mGene QP for %i/%i iterations\n\n', iter, maxNumIter) ;

% predict and evaluate on training set

PAR_predict.submit = 1;
PAR_predict.RPROC.options.addpaths = { fileparts(which('sg'))};
PAR_predict.RPROC.options.envstr = shogun_envstr ;
PAR_predict.evaluate=1 ;
[tmp,res] = predict(PAR_predict);
clear tmp ;

fprintf('Performance on validation/training set\n') ;
fprintf('  Exon level accuracy:\n')
print_struct(1, res.cds_exons) ;
fprintf('  Transcript level accuracy:\n')
print_struct(1, res.cds_transcripts) ;

fprintf(fd_out, 'Performance on validation/training set\n') ;
fprintf(fd_out, '  Exon level accuracy:\n')
print_struct(fd_out,  res.cds_exons) ;
fprintf(fd_out, '  Transcript level accuracy:\n')
print_struct(fd_out,  res.cds_transcripts) ;

[ret, timedate] = unix('date') ;
timedate(timedate==sprintf('\n')) = [] ;

fprintf(fd_out, '\n\nTraining finished %s.\n', timedate) ;

fclose(fd_out) ;

%fclose(fid);

rmpath(extra_path)

fprintf('finished gene_train at %s', timedate) ;

return

function blocks = add_content_predictions_to_blocks(blocks, cont_pred_dirs)
% blocks = add_content_predictions_to_blocks(blocks, cont_pred_dirs)

contigs = unique({blocks.chr}) ;

for i=1:length(cont_pred_dirs),
  fnames{i}=  sprintf('%s/output_spf.mat', cont_pred_dirs{i}) ;
  L = load(fnames{i}, 'SPF_info') ;
  SPF_info{i} = L.SPF_info ;
end ;

fprintf('Processing %i contigs: \n', length(contigs)) ;
for j=1:length(contigs)
  if mod(j, 5)==0, 
    fprintf('%i ... ', j) ; 
  end ;

  contig_block_ids = find(ismember({blocks.chr}, contigs{j}));
  strands = unique({blocks.strand});
  for k=1:length(strands)
    strand_block_ids = intersect(strmatch(strands(k), {blocks.strand}, 'exact'), contig_block_ids);
    for m=1:length(strand_block_ids)
      len = blocks(strand_block_ids(m)).stop - blocks(strand_block_ids(m)).start +1;
      blocks(strand_block_ids(m)).content_pred = zeros(length(cont_pred_dirs),len);
    end
    for l = 1:length(cont_pred_dirs)
      SPF_contig_idx = strmatch(contigs{j}, {SPF_info{l}.contig_name}, 'exact') ;
      SPF_strand_idx = strmatch(strands{k}, {SPF_info{l}.strand}, 'exact') ;
      SPF_contig_strand_idx = intersect(SPF_contig_idx, SPF_strand_idx) ;
      assert(length(SPF_contig_strand_idx)==1) ;

      field_str = sprintf('SPF_%i', SPF_contig_strand_idx-1) ;
      LL=load(fnames{l}, field_str) ;
      LL=LL.(field_str) ;
      assert(isequal(LL.contig_name, contigs{j})) ;
      assert(isequal(LL.strand, strands{k})) ;
      %keyboard
      for m=1:length(strand_block_ids)
        start_pos = max(1, bsearch(LL.pos,  blocks(strand_block_ids(m)).start)-10) ;
        stop_pos = min(length(LL.pos), bsearch(LL.pos,  blocks(strand_block_ids(m)).stop)+10) ;
        idx11 = start_pos:stop_pos ;
        %[tmp,idx1,idx2] = intersect(LL.pos, blocks(strand_block_ids(m)).start:blocks(strand_block_ids(m)).stop) ;
        %idx11 = find(LL.pos>=blocks(strand_block_ids(m)).start & LL.pos<=blocks(strand_block_ids(m)).stop) ;
        [tmp, idx1, idx2] = intersect(LL.pos(idx11), blocks(strand_block_ids(m)).start:blocks(strand_block_ids(m)).stop) ;
        idx1=idx11(idx1) ;
        start_idx = idx11(find(LL.pos(idx11)==blocks(strand_block_ids(m)).start)) ;
        if isempty(start_idx),
          [tmp,start_idx] = min(abs(LL.pos(idx11)-blocks(strand_block_ids(m)).start)) ;
          start_idx = idx11(start_idx) ;
        end ;
        start_score = LL.score(start_idx) ;
        blocks(strand_block_ids(m)).content_pred(l,idx2) = LL.score(idx1) - start_score;
      end 
    end
  end
end 
fprintf('Done.\n') ;

for i=1:length(blocks), 
  idx=find(isnan(blocks(i).content_pred)) ; 
  if ~isempty(idx), 
    fprintf('found %i nan/inf values in content predictions of block %i (%s, %c) => setting to 0\n', length(idx), i, blocks(i).chr, blocks(i).strand) ; 
    blocks(i).content_pred(idx) = 0 ; 
  end ;
end ;

return

function [tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir]=sort_signals(tss_dir_, tis_dir_, acc_dir_, don_dir_, stop_dir_, cleave_dir_)
% [tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir]=sort_signals(tss_dir_, tis_dir_, acc_dir_, don_dir_, stop_dir_, cleave_dir_)

dirs = {tss_dir_, tis_dir_, acc_dir_, don_dir_, stop_dir_, cleave_dir_} ;

for i=1:length(dirs),
  signals{i} = determine_signal(dirs{i}) ;
end ;

%signals_exp = {'tss', 'tis', 'acc', 'don', 'cdsStop', 'cleave'} ;

idx = find(ismember(signals, 'tss')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the TSS signal from input (not or multiple times present)') ;
end ;
tss_dir = dirs{idx} ;

idx = find(ismember(signals, 'tis')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the TIS signal from input (not or multiple times present)') ;
end ;
tis_dir = dirs{idx} ;

idx = find(ismember(signals, 'acc')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the acceptor splice site signal from input (not or multiple times present)') ;
end ;
acc_dir = dirs{idx} ;

idx = find(ismember(signals, 'don')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the donor splice site signal from input (not or multiple times present)') ;
end ;
don_dir = dirs{idx} ;

idx = find(ismember(signals, 'cdsStop')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the STOP signal from input (not or multiple times present)') ;
end ;
stop_dir = dirs{idx} ;

idx = find(ismember(signals, 'cleave')) ;
if length(idx)~=1,
  for i=1:length(signals), fprintf('found signal %s\n', signals{i}) ; end ;
  error('problems identifying the CLEAVE signal from input (not or multiple times present)') ;
end ;
cleave_dir = dirs{idx} ;

return

function [exon_dir, intron_dir, intergenic_dir, utr5_dir, utr3_dir]=sort_contents(exon_dir_, intron_dir_, intergenic_dir_, utr5_dir_, utr3_dir_)
% [exon_dir, intron_dir, intergenic_dir, utr5_dir, utr3_dir]=sort_contents(exon_dir_, intron_dir_, intergenic_dir_, utr5_dir_, utr3_dir_)

dirs = {exon_dir_, intron_dir_, intergenic_dir_, utr5_dir_, utr3_dir_} ;

for i=1:length(dirs),
  contents{i} = determine_content(dirs{i}) ;
end ;

%contents_exp = {'tss', 'tis', 'acc', 'don', 'cdsStop', 'cleave'} ;

idx = find(ismember(contents, 'cds_exon')) ;
if length(idx)~=1,
  for i=1:length(contents), fprintf('found content %s\n', contents{i}) ; end ;
  error('problems identifying the CDS_EXON content from input (not or multiple times present)') ;
end ;
exon_dir = dirs{idx} ;

idx = find(ismember(contents, 'utr5exon')) ;
if length(idx)~=1,
  for i=1:length(contents), fprintf('found content %s\n', contents{i}) ; end ;
  error('problems identifying the UTR5_EXON content from input (not or multiple times present)') ;
end ;
utr5_dir = dirs{idx} ;

idx = find(ismember(contents, 'utr3exon')) ;
if length(idx)~=1,
  for i=1:length(contents), fprintf('found content %s\n', contents{i}) ; end ;
  error('problems identifying the UTR3_exon content from input (not or multiple times present)') ;
end ;
utr3_dir = dirs{idx} ;

idx = find(ismember(contents, 'intergenic')) ;
if length(idx)~=1,
  for i=1:length(contents), fprintf('found content %s\n', contents{i}) ; end ;
  error('problems identifying the INTERGENIC content from input (not or multiple times present)') ;
end ;
intergenic_dir = dirs{idx} ;

idx = find(ismember(contents, 'intron')) ;
if length(idx)~=1,
  for i=1:length(contents), fprintf('found content %s\n', contents{i}) ; end ;
  error('problems identifying the INTRON content from input (not or multiple times present)') ;
end ;
intron_dir = dirs{idx} ;

return

function signal_name = determine_signal(dir_name)
% ok=check_signal(dir_name, signal_name)

try,
  %L=load(sprintf('%s/Conf_cum_spf.mat', dir_name), 'SPF_info') ;
  L=load(sprintf('%s/Conf_cum_spf.mat', dir_name), 'SPF_0') ;
  
  signal_name = L.SPF_0.signal_name  ;
catch
  signal_name = 'invalid' ;
end ;

return

function content_name = determine_content(dir_name)
% content_name = determine_content(dir_name)

try
  L=load(sprintf('%s/output_spf.mat', dir_name), 'SPF_0') ;
  
  content_name = L.SPF_0.signal_name  ;
catch
  content_name = 'invalid' ;
end ;

return

function ok=check_signal(dir_name, signal_name)
% ok=check_signal(dir_name, signal_name)

signal_name_obs = determine_signal(dir_name) ;

ok = ( isequal(signal_name_obs, signal_name) ) ;
if ~ok
  warning('Expected signal "%s", got signal "%s".', signal_name, signal_name_obs) ;
end ;

return

function y = prctile( x, p, dim );

% --- check input
assert( ndims(p) == 2 );
if( size(p,1) == 1 )
  p = p';
end;
assert( size(p,2) == 1 );
assert( all( 0 <= p & p <= 100 ) );
if( ~exist('dim','var') )
  dim = 1;
  if( sum(size(x)>1) == 1 )
    dim = find( size(x) > 1 );
  end;
end;
assert( 1 <= dim & dim <= ndims(x) );

% --- convert percentiles to indices
n = size( x, dim );
j = ( p * n / 100 ) + 0.5;
j0 = floor( j );
nu = j - j0;
assert(all( 0 <= j0 & j0 <= n ));
assert(all( 0 <= nu & nu < 1 ));

% --- special case: 1-dim. data
if( sum(size(x)>1) <= 1 )
  x1 = squeeze( x );
  if( size(x1,1) == 1 )
    x1 = x1';
  end;
  y = prctile1( x1, j0, nu );
  % simulate matlab's dimension inconsistency
  y = y';
  return;
end;

% --- general case: n-dim. data
N = ndims( x );
ydims = size( x );
ydims(dim) = length( p );
y = repmat( nan, ydims );
S = [];
S.type = '()';
S.subs = cell( N, 1 );
% - iterate thru all sub-vectors
% use "subsref" to sequentially single out each vector "x1" from "x"
error( '"prctile" not yet implemented for n-dim. objects "x"' );



% === percentiles of a single vector

function y = prctile1( x1, j0, nu );

assert( ndims(x1) == 2 );
assert( size(x1,2) == 1 );
s = sort( x1 );
s = [ s(1) ; s ; s(end) ];
y = (1-nu) .* s(j0+1) + nu .* s(j0+2);



return

function flag = issorted(a) ;
% flag = issorted(a) ;

flag = isequal(sort(a), a) ;

return

function [c,ia,ib] = intersect(a,b,flag)
%INTERSECT Set intersection.
%   INTERSECT(A,B) when A and B are vectors returns the values common
%   to both A and B. The result will be sorted.  A and B can be cell
%   arrays of strings.
%
%   INTERSECT(A,B,'rows') when A and B are matrices with the same
%   number of columns returns the rows common to both A and B.
%
%   [C,IA,IB] = INTERSECT(...) also returns index vectors IA and IB
%   such that C = A(IA) and C = B(IB) (or C = A(IA,:) and C = B(IB,:)).
%
%   Class support for inputs A,B:
%      float: double, single
%
%   See also UNIQUE, UNION, SETDIFF, SETXOR, ISMEMBER.

%   Copyright 1984-2005 The MathWorks, Inc.
%   $Revision: 1.21.4.6 $  $Date: 2005/06/21 19:36:22 $

%   Cell array implementation in @cell/intersect.m

nIn = nargin;

if nIn < 2
  error('MATLAB:INTERSECT:NotEnoughInputs', 'Not enough input arguments.');
elseif nIn > 3
  error('MATLAB:INTERSECT:TooManyInputs', 'Too many input arguments.');
end

if nIn == 2
  flag = [];
end

isrows = strcmpi(flag,'rows');

rowsA = size(a,1);
colsA = size(a,2);
rowsB = size(b,1);
colsB = size(b,2);

rowvec = ~((rowsA > 1 && colsB <= 1) || (rowsB > 1 && colsA <= 1) || isrows);

numelA = numel(a);
numelB = numel(b);
nOut = nargout;

if isempty(flag)
  
  if length(a)~=numelA || length(b)~=numelB
    error('MATLAB:INTERSECT:AandBvectorsOrRowflag',...
          'A and B must be vectors, or ''rows'' must be specified.');
  end
  
  c = [a([]);b([])];    % Predefined to determine class of output
  
  % Handle empty: no elements.
  
  if (numelA == 0 || numelB == 0)
    % Predefine index outputs to be of the correct type.
    ia = [];
    ib = [];
    % Ambiguous if no way to determine whether to return a row or column.
    ambiguous = ((size(a,1)==0 && size(a,2)==0) || length(a)==1) && ...
      ((size(b,1)==0 && size(b,2)==0) || length(b)==1);
    if ~ambiguous
      c = reshape(c,0,1);
      ia = reshape(ia,0,1);
      ib = reshape(ia,0,1);
    end
    
    % Handle scalars: one element.
    
  elseif (numelA == 1)
    % Scalar A: pass to ISMEMBER to determine if A exists in B.
    [tf,pos] = ismember(a,b);
    if tf
      c = a;
      ib = pos;
      ia = 1;
    else
      ia = []; ib = [];
    end
  elseif (numelB == 1)
    % Scalar B: pass to ISMEMBER to determine if B exists in A.
    [tf,pos] = ismember(b,a);
    if tf
      c = b;
      ia = pos;
      ib = 1;
    else
      ia = []; ib = [];
    end
    
    % General handling.
    
  else
    
    % Convert to columns.
    a = a(:);
    b = b(:);
    
    % Convert to double arrays, which sort faster than other types.
    
    whichclass = class(c);    
    isdouble = strcmp(whichclass,'double');
    
    if ~isdouble    
      if ~strcmp(class(a),'double')
        a = double(a);
      end
      if ~strcmp(class(b),'double')
        b = double(b);
      end
    end
    
    % Switch to sort shorter list.
    
    if numelA < numelB
      if nOut > 1
        [a,ia] = sort(a);           % Return indices only if needed.
      else
        a = sort(a);
      end
      
      [tf,pos] = ismember(b,a);     % TF lists matches at positions POS.
      
      where = zeros(size(a));       % WHERE holds matching indices
      where(pos(tf)) = find(pos);   % from set B, 0 if unmatched.
      tfs = where > 0;              % TFS is logical of WHERE.
      
      % Create intersection list.
      c = a(tfs);                     
      
      if nOut > 1                  
        % Create index vectors if requested.
        ia = ia(tfs);
        if nOut > 2
          ib = where(tfs);
        end
      end
    else
      if nOut > 1
        [b,ib] = sort(b);           % Return indices only if needed.
      else
        b = sort(b);
      end
      
      [tf,pos] = ismember(a,b);     % TF lists matches at positions POS.
      
      where = zeros(size(b));       % WHERE holds matching indices
      where(pos(tf)) = find(pos);   % from set B, 0 if unmatched.
      tfs = where > 0;              % TFS is logical of WHERE.
      
      % Create intersection list.
      c = b(tfs);
      
      if nOut > 1                  
        % Create index vectors if requested.
        ia = where(tfs);
        if nOut > 2
          ib = ib(tfs);
        end
      end
    end  
    
    % Re-convert to correct output data type using FEVAL.
    if ~isdouble
      c = feval(whichclass,c);
    end
  end
  
  % If row vector, return as row vector.
  if rowvec
    c = c.';
    if nOut > 1
      ia = ia.';
      if nOut > 2
        ib = ib.';
      end 
    end
  end
  
else    % 'rows' case
  if ~isrows
    error('MATLAB:INTERSECT:UnknownFlag', 'Unknown flag.');
  end
  
  % Automatically pad strings with spaces
  if ischar(a) && ischar(b)
    if colsA > colsB
      b = [b repmat(' ',rowsB,colsA-colsB)];
    elseif colsA < colsB 
      a = [a repmat(' ',rowsA,colsB-colsA)];
      colsA = colsB;
    end
  elseif colsA ~= colsB && ~isempty(a) && ~isempty(b)
    error('MATLAB:INTERSECT:AandBColnumAgree',...
          'A and B must have the same number of columns.');
  end
  
  % Predefine the size of c, since it may return empty.
  c = zeros(0,colsB);
  ia = [];
  ib = [];
    
  % Remove duplicates from A and B.  Only return indices if needed.
  if nOut <= 1
    a = unique(a,flag);
    b = unique(b,flag);
    c = sortrows([a;b]);
  else
    [a,ia] = unique(a,flag);
    [b,ib] = unique(b,flag);
    [c,ndx] = sortrows([a;b]);
  end
  
  % Find matching entries in sorted rows.
  [rowsC,colsC] = size(c);
  if rowsC > 1 && colsC ~= 0
    % d indicates the location of matching entries
    d = c(1:rowsC-1,:) == c(2:rowsC,:);
  else
    d = zeros(rowsC-1,0);
  end
  
  d = find(all(d,2));
  
  c = c(d,:);         % Intersect is list of matching entries
    
  if nOut > 1
    n = size(a,1);
    ia = ia(ndx(d));      % IA: indices of first matches
    ib = ib(ndx(d+1)-n);  % IB: indices of second matches
  end
  
  % Automatically deblank strings
  if ischar(a) && ischar(b)
    rowsC = size(c,1);
    c = deblank(c);
    c = reshape(c,rowsC,size(c,2));
  end
end

return

% bsearch(x,var)
% Written by Aroh Barjatya
% Binary search for values specified in vector 'var' within data
% vector 'x'
% The data has to be pre-sorted in ascending or decending order
% There is no way to predict how the function will behave if there 
% are multiple numbers with same value.
% returns the index values of the searched numbers

function index = bsearch(x,var)

xLen = length(x);
[xRow xCol] = size(x);
if x(1) > x(xLen)% means x is in descending order
    if xRow==1
        x = fliplr(x);  
    else
        x = flipud(x);
    end
    flipped = 1;
elseif x(1) < x(xLen)% means x is in ascending order
    flipped = 0;
else
    'badly formatted data. Type ''help bsearch\'')';
    return;
end

for i = 1:length(var)
    low = 1;
    high = xLen;
    if var(i) <= x(low)
        index(i) = low;
        continue;
    elseif var(i) >= x(high)
        index(i) = high;
        continue;
    end
    flag = 0;
    while (low <= high)
        mid = round((low + high)/2);
        if (var(i) < x(mid))
            high = mid;
        elseif (var(i) > x(mid))
            low = mid;
        else
            index(i) = mid;
            flag = 1;
            break;
        end
        if (low == high - 1)
            break
        end
    end
    if (flag == 1)
        continue;
    end
    if (low == high)
        index(i) = low;
    elseif ((x(low) - var(i))^2 > (x(high) - var(i))^2)
        index(i) = high;
    else
        index(i) = low;
    end
end

if flipped
    index = xLen - index + 1;
end

return

function [c,ia,ib] = intersect(a,b,flag)
%INTERSECT Set intersection.
%   INTERSECT(A,B) when A and B are vectors returns the values common
%   to both A and B. The result will be sorted.  A and B can be cell
%   arrays of strings.
%
%   INTERSECT(A,B,'rows') when A and B are matrices with the same
%   number of columns returns the rows common to both A and B.
%
%   [C,IA,IB] = INTERSECT(...) also returns index vectors IA and IB
%   such that C = A(IA) and C = B(IB) (or C = A(IA,:) and C = B(IB,:)).
%
%   Class support for inputs A,B:
%      float: double, single
%
%   See also UNIQUE, UNION, SETDIFF, SETXOR, ISMEMBER.

%   Copyright 1984-2005 The MathWorks, Inc.
%   $Revision: 1.21.4.6 $  $Date: 2005/06/21 19:36:22 $

%   Cell array implementation in @cell/intersect.m

nIn = nargin;

if nIn < 2
  error('MATLAB:INTERSECT:NotEnoughInputs', 'Not enough input arguments.');
elseif nIn > 3
  error('MATLAB:INTERSECT:TooManyInputs', 'Too many input arguments.');
end

if nIn == 2
  flag = [];
end

isrows = strcmpi(flag,'rows');

rowsA = size(a,1);
colsA = size(a,2);
rowsB = size(b,1);
colsB = size(b,2);

rowvec = ~((rowsA > 1 && colsB <= 1) || (rowsB > 1 && colsA <= 1) || isrows);

numelA = numel(a);
numelB = numel(b);
nOut = nargout;

if isempty(flag)
  
  if length(a)~=numelA || length(b)~=numelB
    error('MATLAB:INTERSECT:AandBvectorsOrRowflag',...
          'A and B must be vectors, or ''rows'' must be specified.');
  end
  
  c = [a([]);b([])];    % Predefined to determine class of output
  
  % Handle empty: no elements.
  
  if (numelA == 0 || numelB == 0)
    % Predefine index outputs to be of the correct type.
    ia = [];
    ib = [];
    % Ambiguous if no way to determine whether to return a row or column.
    ambiguous = ((size(a,1)==0 && size(a,2)==0) || length(a)==1) && ...
      ((size(b,1)==0 && size(b,2)==0) || length(b)==1);
    if ~ambiguous
      c = reshape(c,0,1);
      ia = reshape(ia,0,1);
      ib = reshape(ia,0,1);
    end
    
    % Handle scalars: one element.
    
  elseif (numelA == 1)
    % Scalar A: pass to ISMEMBER to determine if A exists in B.
    [tf,pos] = ismember(a,b);
    if tf
      c = a;
      ib = pos;
      ia = 1;
    else
      ia = []; ib = [];
    end
  elseif (numelB == 1)
    % Scalar B: pass to ISMEMBER to determine if B exists in A.
    [tf,pos] = ismember(b,a);
    if tf
      c = b;
      ia = pos;
      ib = 1;
    else
      ia = []; ib = [];
    end
    
    % General handling.
    
  else
    
    % Convert to columns.
    a = a(:);
    b = b(:);
    
    % Convert to double arrays, which sort faster than other types.
    
    whichclass = class(c);    
    isdouble = strcmp(whichclass,'double');
    
    if ~isdouble    
      if ~strcmp(class(a),'double')
        a = double(a);
      end
      if ~strcmp(class(b),'double')
        b = double(b);
      end
    end
    
    % Switch to sort shorter list.
    
    if numelA < numelB
      if nOut > 1
        [a,ia] = sort(a);           % Return indices only if needed.
      else
        a = sort(a);
      end
      
      [tf,pos] = ismember(b,a);     % TF lists matches at positions POS.
      
      where = zeros(size(a));       % WHERE holds matching indices
      where(pos(tf)) = find(pos);   % from set B, 0 if unmatched.
      tfs = where > 0;              % TFS is logical of WHERE.
      
      % Create intersection list.
      c = a(tfs);                     
      
      if nOut > 1                  
        % Create index vectors if requested.
        ia = ia(tfs);
        if nOut > 2
          ib = where(tfs);
        end
      end
    else
      if nOut > 1
        [b,ib] = sort(b);           % Return indices only if needed.
      else
        b = sort(b);
      end
      
      [tf,pos] = ismember(a,b);     % TF lists matches at positions POS.
      
      where = zeros(size(b));       % WHERE holds matching indices
      where(pos(tf)) = find(pos);   % from set B, 0 if unmatched.
      tfs = where > 0;              % TFS is logical of WHERE.
      
      % Create intersection list.
      c = b(tfs);
      
      if nOut > 1                  
        % Create index vectors if requested.
        ia = where(tfs);
        if nOut > 2
          ib = ib(tfs);
        end
      end
    end  
    
    % Re-convert to correct output data type using FEVAL.
    if ~isdouble
      c = feval(whichclass,c);
    end
  end
  
  % If row vector, return as row vector.
  if rowvec
    c = c.';
    if nOut > 1
      ia = ia.';
      if nOut > 2
        ib = ib.';
      end 
    end
  end
  
else    % 'rows' case
  if ~isrows
    error('MATLAB:INTERSECT:UnknownFlag', 'Unknown flag.');
  end
  
  % Automatically pad strings with spaces
  if ischar(a) && ischar(b)
    if colsA > colsB
      b = [b repmat(' ',rowsB,colsA-colsB)];
    elseif colsA < colsB 
      a = [a repmat(' ',rowsA,colsB-colsA)];
      colsA = colsB;
    end
  elseif colsA ~= colsB && ~isempty(a) && ~isempty(b)
    error('MATLAB:INTERSECT:AandBColnumAgree',...
          'A and B must have the same number of columns.');
  end
  
  % Predefine the size of c, since it may return empty.
  c = zeros(0,colsB);
  ia = [];
  ib = [];
    
  % Remove duplicates from A and B.  Only return indices if needed.
  if nOut <= 1
    a = unique(a,flag);
    b = unique(b,flag);
    c = sortrows([a;b]);
  else
    [a,ia] = unique(a,flag);
    [b,ib] = unique(b,flag);
    [c,ndx] = sortrows([a;b]);
  end
  
  % Find matching entries in sorted rows.
  [rowsC,colsC] = size(c);
  if rowsC > 1 && colsC ~= 0
    % d indicates the location of matching entries
    d = c(1:rowsC-1,:) == c(2:rowsC,:);
  else
    d = zeros(rowsC-1,0);
  end
  
  d = find(all(d,2));
  
  c = c(d,:);         % Intersect is list of matching entries
    
  if nOut > 1
    n = size(a,1);
    ia = ia(ndx(d));      % IA: indices of first matches
    ib = ib(ndx(d+1)-n);  % IB: indices of second matches
  end
  
  % Automatically deblank strings
  if ischar(a) && ischar(b)
    rowsC = size(c,1);
    c = deblank(c);
    c = reshape(c,rowsC,size(c,2));
  end
end

return

function make_mgene_predictor(predictor_fn, predictor_dir,...
			tss_fn, tss_dir, tis_fn, tis_dir, acc_fn, acc_dir, don_fn, don_dir, cdsStop_fn, cdsStop_dir, cleave_fn, cleave_dir,...
			intergenic_fn, intergenic_dir, utr5exon_fn, utr5exon_dir, cds_exon_fn, cds_exon_dir, intron_fn, intron_dir, utr3exon_fn, utr3exon_dir,...
			fn_out, output_dir)

fd=fopen(fn_out, 'w+') ;
[ret, timedate] = unix('date') ;
fprintf(fd, 'mGene Predictor created on %s\n\n', timedate)
fclose(fd) ;

%signals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
train_dirs.tss = tss_dir;
train_files.tss = tss_fn;

train_dirs.tis = tis_dir;
train_files.tis = tis_fn;

train_dirs.acc = acc_dir;
train_files.acc = acc_fn;

train_dirs.don = don_dir;
train_files.don = don_fn;

train_dirs.cdsStop = cdsStop_dir;
train_files.cdsStop = cdsStop_fn;

train_dirs.cleave = cleave_dir;
train_files.cleave = cleave_fn;


%contents
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
train_dirs.intergenic = intergenic_dir;
train_files.intergenic = intergenic_fn;

train_dirs.utr5exon = utr5exon_dir;
train_files.utr5exon = utr5exon_fn;

train_dirs.cds_exon = cds_exon_dir;
train_files.cds_exon = cds_exon_fn;

train_dirs.intron = intron_dir;
train_files.intron = intron_fn;

train_dirs.utr3exon = utr3exon_dir;
train_files.utr3exon = utr3exon_fn;


%genes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
train_dirs.gene_predictor = predictor_dir;
train_files.gene_predictor = predictor_fn;

fn={'gene_predictor', 'tss', 'tis', 'acc', 'don', 'cdsStop', 'cleave', ...
   'intergenic', 'utr5exon', 'cds_exon', 'intron', 'utr3exon'} ;
for i=1:length(fn)
  system(sprintf('cat %s >> %s && echo >> %s && echo ------ >> %s && echo >> %s ', train_files.(fn{i}), ...
                 fn_out, fn_out, fn_out, fn_out)) ;
end ;

nice_mkdir(output_dir) ;

save(sprintf('%s/mgene_predictor.mat', output_dir), 'train_dirs', ...
     'train_files', '-v7');


%function fuse_mGene_predictor(	fn_fused_mGene_predictor, fn_gene_predictor,...
%				tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir,...
%				exon_dir, intron_dir, utr_dir, intergenic_dir)


%% collect signal classifiers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%signal_dirs = {tss_dir, tis_dir, acc_dir, don_dir, stop_dir, cleave_dir};
%
%for k=1:length(signal_dirs)
%  l = load([signal_dirs{k} '/PAR.mat'], 'PAR');
%  signal = l.PAR.Signal_name;
%  for j=1:5
%    fn_SVM = sprintf('%sbest_partition=%i.mat', l.PAR.FN.output_sig.(signal).fn_SVMs, j);
%    signal_classifiers.(signal) = load(fn_SVM);
%  end
%end
%
%clear l fn_SVM
%% collect content classifiers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%content_dirs = {exon_dir, intron_dir, utr_dir, intergenic_dir}
%
%for k=1:length(content_dirs)
%  if fexist([content_dirs{k} '/PAR.mat'])
%    l = load([content_dirs{k} '/PAR.mat'], 'PAR');
%    content = l.PAR.Content_name;
%    for j=1:5
%      fn_SVM = sprintf('%sbest_partition=%i.mat', l.PAR.FN.output_cont.(content).fn_SVMs, j);
%      if fexist(fn_SVM)
%        content_classifiers.(content) = load(fn_SVM);
%      end
%    end
%  end
%end
%
%% get gene structure predictor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%gene_predictor = struct;
%if fexist(fn_gene_predictor)
% gene_predictor = load(fn_gene_predictor);
%else
%  warning('no gene predictor found')
%end
%
%save(fn_fused_mGene_predictor, 'content_classifiers', 'signal_classifiers', 'gene_predictor')
%
%
%
%

return

function PAR=A_thaliana_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=init_PAR;
elseif nargin<2
  create_dir=0;
end
PAR = A_thaliana(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end

return

function PAR=A_thaliana_don_SVM_WD(PAR,create_dir)

if nargin<1
  PAR = init_PAR;
end
if nargin<2
  create_dir=0;
end
  
PAR = A_thaliana(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);
if create_dir
  create_directories(PAR);
end


return

function PAR=C_elegans_acc_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=C_elegans_don_SVM_WD(PAR,create_dir)
  
if nargin<1
  PAR=[];
end
if nargin<2
  create_dir=0;
end
  
PAR = C_elegans(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = make_partitions(PAR);
PAR = create_filenames(PAR);

if create_dir
  create_directories(PAR);
end

return

function PAR=D_melanogaster_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = acceptor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on = 1;
PAR.tasks.pred_from_seq = 0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_melanogaster_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_melanogaster(PAR);
PAR = donor(PAR);
PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

%PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
%PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

% this is the setting for the splice paper
PAR.regions.cluster_paralogs = 0;

PAR.RPROC.MEMREQ = 8000 ;

PAR.method.kernels{1}.order=22 ;
PAR.method.par_ms.C = 3 ;

return

function PAR=D_rerio_acc_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = acceptor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='acceptor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function PAR=D_rerio_don_SVM_WD(PAR)
  
if nargin<1
  PAR=[];
end
  
PAR = D_rerio(PAR);
PAR = donor(PAR);

% test only one model ... 
PAR.Signals.(PAR.Signal_name).order = 22;
PAR.Signals.(PAR.Signal_name).shift = 0;
PAR.Signals.(PAR.Signal_name).C = 3; 

PAR = SVM_WD(PAR);

PAR = make_partitions(PAR);

PAR.date=date;
PAR.user='Jonas';
PAR.exp_name='donor';

PAR.method.kernels{1}.lwin=PAR.Signals.(PAR.Signal_name).lwin;
PAR.method.kernels{1}.rwin=PAR.Signals.(PAR.Signal_name).rwin;
PAR.method.center = PAR.Signals.(PAR.Signal_name).lwin_big;

PAR = create_filenames(PAR);

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=0;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;

PAR.SETs.train_subsample_pos = 1 ;
PAR.SETs.train_subsample_neg = 0.33 ;

PAR.RPROC.options.ncpus = 12 ; 

return

function blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset,which_fields)
% blocks = add_signals2blocks(blocks, PAR, signal_names, which_dir, offset)

if nargin<5
  warning('I think this should be a parameter')
  offset = 100;  
end
if nargin<4 || isempty(which_dir)
  which_dir = 'pred_dir' ;
end
genome_info = init_genome(PAR.FN.input.fn_genome_config);
which_pos.local = 1;
which_pos.contig = 1;
base_dirs    = {};

for i=1:length(signal_names),
  if isequal(which_dir,'pred_dir')'
    tmp = PAR.FN.output_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_pred;
    fields = {'Conf_cum'};
  elseif isequal(which_dir,'cand_dir')'
    tmp = PAR.FN.input_sig.(signal_names{i});
    base_dirs{i} = tmp.fn_candsites;
    fields = {'label','altgenic','alt_intron','est_conf','est_conf_skip','cDNA_conf','cDNA_conf_skip'};
  end
end
if nargin>5
  fields = which_fields;
end

if ~isfield(blocks,'Signals')
  blocks(1).Signals=struct;
end

% blocks = load_sequence(blocks);
fprintf('Processing %i blocks: ', length(blocks));
for r=1:length(blocks)
  block=blocks(r);
  if mod(r,50)==0, fprintf('%i .. ', r); end ;
  % fill the field genestr
  for s=1:length(signal_names),
    f_name = sprintf('%scontig_%i%s.pos', base_dirs{s},block.chr_num,block.strand);
    d = dir(f_name) ;
    if isempty(d), error('file not found: %s',f_name);end
    if d.bytes>0
      block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields, which_pos, offset);
      % block = retrieve_signals(block, base_dirs{s},signal_names{s}, fields,which_pos,0);
    else
      warning('hack!\n')
      block.Signals.(signal_names{s})=struct ;
      block.Signals.(signal_names{s}).pos = [];
      block.Signals.(signal_names{s}).contig_pos = [];
      for k=1:length(fields)
        block.Signals.(signal_names{s}).(fields{k}) =  [];
      end ;
    end
  end
  blocks(r)=block;
end
fprintf('Done.\n') ;

% load /fml/ag-raetsch/share/projects/nGASP/LSL/test_data/cat_1/blocks_2006-12-19.mat

% eof

return

function ok_idx = check_block_model_consistency(blocks,model)
%function ok_idx = check_block_model_consistency(blocks,model)
% walk through block.features along the 'true' path and 
% check consistency with model

ok = zeros(1,length(blocks));
for b=1:length(blocks)
  if mod(b,10)==0, fprintf('%i/%i ... ', b, length(blocks)) ; end 
  ok(b) = check_block(blocks(b),model,b);
end
ok_idx = find(ok);
fprintf('Done.\n') ;

return

function ok = check_block(block,model,num)
seq = lower(block.seq);
ok=1;
for t=1:length(block.truth)
  for j=1:length(block.truth(t).path)-1
    from_state   = block.truth(t).path(j);
    to_state     = block.truth(t).path(j+1);
    from_sig_pos = block.truth(t).pos_idx(j);
    to_sig_pos   = block.truth(t).pos_idx(j+1);
    from_pos     = block.all_pos(block.truth(t).pos_idx(j));
    to_pos       = block.all_pos(block.truth(t).pos_idx(j+1));
    

    assert(from_pos<to_pos)
    assert(from_sig_pos<to_sig_pos)

    % check segment lengths
    if strcmp(model.states(from_state).name,'intergenic_long') && strcmp(model.states(to_state).name,'intergenic_long')
	%this special case is necessary because the transition is zero 
	lengthrange  = [0 inf] ;%model.lengths_range.intergenic_long;
    %elseif strcmp(model.states(to_state).name,'intergenic_long')
	%lengthrange  = model.lengths_range.intergenic;
    else
      clear length_name
      transition   = model.transition_pointers(to_state,from_state,1);%transiton_pointers is transposed compared to model.A
      fields = fieldnames(model.lengths);
      length_name = '' ;
      for j=1:length(fields), if model.lengths.(fields{j})==transition, length_name = fields{j}; break, end,end
      assert(~isempty(length_name)) ;
      lengthrange  = model.lengths_range.(length_name);
    end
    if (to_pos-from_pos<lengthrange(1) ||to_pos-from_pos>lengthrange(2))%&~isequal(length_name,'intergenic_long')
      fprintf(1,'discard block %i, because its length (%i) is out of bounds (%i,%i)\n',num,to_pos-from_pos,lengthrange(1),lengthrange(2));
      ok=0;
      %keyboard
      return
    end
    % check don/acc_1b transitions: does splicing lead to in frame stop codons?
    if strcmp(model.states(from_state).name,'CDS_don_1b')
      assert(seq(from_pos-1)=='t');
      assert(strcmp(model.states(to_state).name,'CDS_acc_1b'));
      if ismember(seq(to_pos+[0,1]),model.states(to_state).non_consensus)
        fprintf(1,'discard block %i because of a in frame stop codon\n',num);
        ok=0;
        return
      end
    end
    % check don/acc_2b transitions: does splicing lead to in frame stop codons?
    if strcmp(model.states(from_state).name,'CDS_don_2b')
      assert(seq(from_pos-2)=='t');
      assert(seq(from_pos-1)=='a'||seq(from_pos-1)=='g');
      assert(strcmp(model.states(to_state).name,'CDS_acc_2b'));
      if ~ismember(seq(to_pos+[-2:0]),model.states(to_state).consensus)
        fprintf(1,'discard block %i because of a in frame stop codon\n',num);
        ok=0;
        return
      end
    end
    for con=1:length(model.states(to_state).consensus)
      eq(con) = strcmp(model.states(to_state).consensus{con},seq((to_pos:to_pos+length(model.states(to_state).consensus{con})-1)-model.states(to_state).consensus_pos(con)));
    end
    if length(model.states(to_state).consensus)>0 && ~any(eq)
      fprintf(1,'discard block %i because of wrong %s-consensus at %i\n',num,model.states(to_state).name ,to_pos);
      ok=0;
      return
    end
    if isinf(model.A(from_state,to_state))
      fprintf(1,'discard block %i because it uses forbidden transition from %i to %i\n',num,from_state,to_state);
      ok=0;
      % keyboard
      return
    end

    if isinf(block.features{1}(to_state,to_sig_pos))
      error('discard block %i: feature entry is inf; to_state: %i to_pos: %i\n',num,to_state,to_sig_pos);
    end
  end
end
return

return

function [SEG]=correct_cleave_and_tss_pos(PAR, SEG, block, model)
% 
% if there are no predictions for the annotated cleave and tss positions then 
% set cleave and tss to the nearest predicted  position 

col4 = SEG.segments(:,4);
num_genes = length(unique(col4))-1;
assert(all(unique(col4)'==0:num_genes))


for i=1:num_genes
  gene_lines = find(col4==i);
  segments = SEG.segments(gene_lines,1:3);

  upos = unique(segments(:,1:2));
  compos = intersect(block.all_pos,upos);
  if 1
    diffpos = setdiff(upos,compos);

    % find cleave position
    %------------------------------------------------------------
    cleave_idx = find(segments(:,3)== model.segments.polya_tail,1,'last');
    if isempty(cleave_idx)
      cleave_idx = find(segments(:,3)== model.segments.utr3exon,1,'last');
    end
    cleave_pos = segments(cleave_idx,2);

    res = PAR.Signals.cleave.export_settings.resolution;
    lower_bound = segments(cleave_idx,1);
    upper_bound = SEG.segments(min(gene_lines)-1+cleave_idx+1,2);

    new_pos = find_nearest_pos(cleave_pos, block.Signals.cleave.pos,res, ...
                               lower_bound, upper_bound);
    if isempty(new_pos), SEG = []; return, end
    SEG.segments(min(gene_lines)-1+cleave_idx,2) = new_pos;
    SEG.segments(min(gene_lines)-1+cleave_idx+1,1) = new_pos;

    % find tss position
    %------------------------------------------------------------
    tss_idx = find(segments(:,3) == model.segments.utr5exon, 1, 'first');
    tss_pos = segments(tss_idx,1);

    res = PAR.Signals.tss.export_settings.resolution;
    lower_bound = SEG.segments(min(gene_lines)-1+tss_idx-1,1);
    upper_bound = segments(tss_idx,2);

    new_pos = find_nearest_pos(tss_pos, block.Signals.tss.pos, res, lower_bound, upper_bound);
    if isempty(new_pos), SEG = []; return, end
    SEG.segments(min(gene_lines)-1+tss_idx,1) = new_pos;
    SEG.segments(min(gene_lines)-1+tss_idx-1,2) = new_pos;

    if ~all(ismember(diffpos,cleave_pos)|ismember(diffpos, tss_pos)),
      warning('position in truth not represented in position list (this is probably a bug) => ignoring this block') ;
      SEG=[] ;
      return ;
    end ;
    assert(all(ismember(diffpos,cleave_pos) | ismember(diffpos, tss_pos)));
    upos = unique(SEG.segments(gene_lines,1:2));
    assert(length(intersect(block.all_pos,upos))==length(upos));
  end
end

return

function pos=find_nearest_pos(old_pos, all_pos, resolution,lb,ub)

  idx = find(all_pos<old_pos,1,'last');
  if isempty(idx), 
    pos=[] ; 
    warning('correct_cleave_and_tss_pos:find_nearest_pos', 'cannot find correct position for tss/cleave');
    return ; 
  end ;
  if idx+1>length(all_pos),
    pos = all_pos(idx);
    return ;
  end ;
  if ( (old_pos-all_pos(idx)<all_pos(idx+1)-old_pos) || (all_pos(idx+1)<=lb || all_pos(idx+1)>=ub)) && all_pos(idx)>lb && all_pos(idx)<ub,
    pos = all_pos(idx);
    %assert(old_pos-pos<2*resolution+1);
    % this is ok, for random sampling...
  elseif all_pos(idx+1)>lb && all_pos(idx+1)<ub
    pos = all_pos(idx+1);
    %assert(pos-old_pos<2*resolution+1);
    % this is ok, for random sampling...
  else
    pos = [];
    warning('correct_cleave_and_tss_pos:find_nearest_pos', 'cannot find correct position for tss/cleave');
  end
return

return

function features = full_features(sp)
% features = full_features(sp)

assert(length(sp)==2)
assert(iscell(sp))
assert(isequal(size(sp{1}),size(sp{2})))

features = full(sp{1});
features(:,:,2) = full(sp{2});

features(features==0) = -inf;
features = features-1;

return

function blocks = gen_block_segmentation(genes,blocks,model)
% blocks = gen_block_segmentation(genes,blocks,model)
%  
% genes: need to have a field 'segments' with a gene segmentation in global chr-coordinates (allways on positive strand)
% 
% segmentation: 4 columns
%         col 1: start of segment: must be equal to col2 of previous line
%         col 2: end of segment
%         col 3: segment type (e.g. intergenic, exon, intron,...) integer coded. definition: model.segments 
%         col 4: gene count: 1:length(blocks.gene_ids); 0 if segment is intergenic



for b=1:length(blocks)
  current = 1;
  seg = {[1 length(blocks(b).seq) model.segments.intergenic 0]};
  for g=1:length(blocks(b).gene_ids)
    gene = genes(blocks(b).gene_ids(g));
    if ~gene.is_complete==1, continue; end
    if gene.start<blocks(b).start||gene.start>blocks(b).stop||gene.stop<blocks(b).start||gene.stop>blocks(b).stop
      error('gene boundaries not within block boundaries')
    end
    assert(gene.id==blocks(b).gene_ids(g))
    for s = 1:length(seg)
      for t=1:length(gene.exons)
        if ~gene.transcript_complete(t)||~gene.transcript_valid(t), continue; end
        if blocks(b).strand=='+'%
          local_segments = [gene.segments{t}(:,1:2)-blocks(b).start+1 gene.segments{t}(:,3)];
          local_segments(:,end+1)=g;
        else %
          local_segments = [blocks(b).stop-gene.segments{t}(end:-1:1,2:-1:1)+1 gene.segments{t}(end:-1:1,3)];
          local_segments(:,end+1)=length(blocks(b).gene_ids)-g+1;
        end%
        seg{end+1} = insert_gene_segments(seg{s},local_segments,model,blocks(b).strand);
      end
    end
    seg(1:s)=[];
  end
  for j=1:length(seg)
    blocks(b).truth(j).segments=seg{j};
  end
end
return

function segments = insert_gene_segments(segments,gene_segments,model,strand)
  % example for positive strand: 
  % segments = [1 1000 0]
  % gene_segments = [300 400 1]
  %                 [400 500 2]
  % result = [1   300  0]
  %          [300 400  1]
  %          [400 500  2]
  %          [500 1000 0]
  %
line = find(segments(:,1)<gene_segments(1,1),1,'last');

% gene_segments have to fit into one single intergenic region
if ~(segments(line,3)==model.segments.intergenic)
  segments = [];
  return
end
assert(segments(line,2)>=gene_segments(end,2))

new_line = [gene_segments(end,2) segments(line,2)  model.segments.intergenic 0];
segments(line,:)=[segments(line,1) gene_segments(1,1) model.segments.intergenic 0];
segments = [segments(1:line,:); gene_segments; new_line; segments(line+1:end,:)]; 
return



return

function blocks = gen_features(blocks, model)
% blocks = gen_features(block, model)

%blocks=blocks(1);

signal_names = fieldnames(model.signals) ;
signal_ids = zeros(1,length(signal_names)) ;
for i=1:length(signal_names) 
  signal_ids(i) = model.signals.(signal_names{i}) ;
end ;

%for i=1:length(model.states)
%  motifs{i}=[];
%  if ~isempty(model.states(i).consensus)
%    for j=1:length(model.states(i).consensus)
%      motifs{i}{j}=lower(model.states(i).consensus{j});
%    end
%  end
%end

blocks(1).features = {[]};
blocks(1).all_pos = [];
fprintf('Processing %i blocks: ', length(blocks)) ;
for b_idx = 1:length(blocks)
  if mod(b_idx, 50)==0,
    fprintf('%i ... ', b_idx);
  end ;
  block = blocks(b_idx);
  all_pos= [] ;
  for i=1:length(signal_names) 
    new_pos = block.Signals.(signal_names{i}).pos ;
    all_pos = [all_pos new_pos(:)'] ;
  end ;
  all_pos = unique(all_pos) ;

  thirddim =0;
  for i=1:length(model.states)
    thirddim=max(thirddim,length(model.states(i).signal));
  end ;

  features = -inf(model.cnt_states, length(all_pos), thirddim) ;
  genestr  = lower(block.seq) ;
  
  for idx_states=1:length(model.states) 
    state = model.states(idx_states);
    if isempty(state.signal)
      continue ; 
    end ;
    for j=1:length(state.signal)
      sig_name = signal_names{find(state.signal(j)==signal_ids)} ;
      sig_pos_temp{j} = block.Signals.(sig_name).pos ;
      sig_scores_temp{j} = block.Signals.(sig_name).Conf_cum ;
      assert(isequal(sort(sig_pos_temp{j}), sig_pos_temp{j}));
      assert(length(sig_pos_temp{j})==length(unique(sig_pos_temp{j})));
      % [sig_pos_temp{j},iidx] = unique(sig_pos_temp{j}) ;
      % sig_scores_temp{j} = sig_scores_temp{j}(iidx) ;
    end
    sig_pos=sig_pos_temp{1};
    for j=2:length(state.signal)
      sig_pos=intersect(sig_pos, sig_pos_temp{j}) ;
    end ;
    sig_scores =[];
    for j=1:length(state.signal)
      sig_scores{j} = -inf(size(sig_pos));
      [tmp,idx1,idx2] = intersect(sig_pos,sig_pos_temp{j});
      sig_scores{j}(idx1) = sig_scores_temp{j}(idx2) ;
      assert(length(idx1)==length(sig_scores{j}));
    end ;clear sig_pos_temp sig_scores_temp

    if ~isempty(state.consensus) 
      for k=1:length(state.consensus) 
	motifs{k}=lower(state.consensus{k});
	%motifl(k)=length(motifs{k});
      end
      cons_found = check_consensus(genestr, motifs, state.consensus_pos, sig_pos'-1);
      clear motifs;
      % if sum(cons_found==0)~=0
      %   keyboard
      % end
      %assert(all(cons_found)) ;
      sig_pos(~cons_found)=[] ;
      for j=1:length(sig_scores)
        sig_scores{j}(~cons_found)=[] ;
      end
    end ;
  
    if ~isempty(state.non_consensus) 
      ncons_found = zeros(1,length(sig_pos)) ;
      for j=1:length(sig_pos),
        for k=1:length(state.non_consensus)
          offset = state.non_consensus_pos(k) ;
          motif = lower(state.non_consensus{k}) ;
          if sig_pos(j)-offset+ length(motif)-1>length(genestr) || sig_pos(j)-offset<1
            continue
          end
          if isequal(genestr(sig_pos(j)-offset:sig_pos(j)-offset+ length(motif)-1), motif), %teuer
            ncons_found(j) = 1 ;
          end ;
        end ;
      end 
      sig_pos(ncons_found~=0)=[] ;
      for j=1:length(sig_scores)
        sig_scores{j}(ncons_found~=0)=[] ;
      end
    end ;
    
    [tmp,idx1,idx2]=intersect(all_pos,sig_pos) ;
    assert(length(idx2)==length(unique(sig_pos))) ;
    for j=1:length(sig_scores)
      features(idx_states, idx1, j) = sig_scores{j}(idx2) ;
    end
  end ; %%loop over states
  
  features(model.state_ids.seq_start, 1, 1) = 0 ;
  features(model.state_ids.seq_end, end, 1) = 0 ;
  if isfield(model.state_ids,'intergenic_long')
    features(model.state_ids.intergenic_long, :, 1) = 0 ;
  end
  if isfield(model.state_ids,'intron_long')
    features(model.state_ids.intron_long, :, 1) = 0 ;
  end
  block.all_pos = all_pos;
  block.features = sparse_features(features); % teuer
  blocks(b_idx)=block;
  
  %%%check don
  state_ids = model.state_ids.don;
  num=0;
  for j=state_ids
    num=num+1;
    cons{num} = model.states(j).consensus;
    nocons{num} = model.states(j).non_consensus;
  end
  for j=2:length(cons)
    for k=1:j-1
      if isequal(cons{j},cons{k})&&isequal(nocons{j},nocons{k})
        %%!!!
        assert(isequal(features(state_ids(j),:,:),features(state_ids(k),:,:)))
      end
    end
  end
  
  %%%check acc
  state_ids = model.state_ids.acc;
  num=0;
  for j=state_ids
    num=num+1;
    cons{num} = model.states(j).consensus;
    nocons{num} = model.states(j).non_consensus;
  end
  for j=2:length(cons)
    for k=1:j-1
      if isequal(cons{j},cons{k})&&isequal(nocons{j},nocons{k})
        assert(isequal(features(state_ids(j),:,:),features(state_ids(k),:,:)))
      end
    end
  end
  assert(all(any(~isinf(features(:,:,1)))))
end
fprintf('Done.\n') ;

return
%features(features>20)=-1 ;
%figure;imagesc(features(:,:,1)) ;
%figure;imagesc(features(:,:,2)) ;




return

function genes = gen_gene_segmentation(P)
% genes = gen_gene_segmentation(P)
% INPUT
% P.genes
% P.segment_ids  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate segmentation for all valid transcripts of a gene %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
genes = P.genes;
segment_ids = P.segment_ids;  
  
for g=1:length(genes)
  if ~genes(g).is_complete
    continue;
  end
  exons = genes(g).exons;
  for j=1:length(exons),
    if ~genes(g).transcript_valid(j)
      genes(g).segments{j}=[];
      continue;
    end
    if ~genes(g).transcript_complete(j)
      continue;
    end
    if genes(g).strand=='+'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR POSITIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform = [];
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
      % if there is an intron directly before the tis site (AGATG):
      %----------------------------------------------------
      if  ~isempty(genes(g).utr5_exons{j})
        if genes(g).utr5_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
           isoform = [isoform; [genes(g).utr5_exons{j}(end,2) genes(g).cds_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        else
          % set end of last cds_exon to the beginning of the cdsStop-consensus
          isoform(end,2)=isoform(end,2)-3;
        end
      end   
      % insert an intron between last cds_exon and 3'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr3_exons{j}) 
        if  genes(g).cds_exons{j}(end,2)<genes(g).utr3_exons{j}(1,1)
          %isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
          isoform = [isoform; [isoform(end,2) genes(g).utr3_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        if k==1 && isoform(end,3)==segment_ids.cds_exon
          % set end of first 3'utrexon to the beginning of the cdsStop-consensus
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1)-3 genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        % insert polya_tail if applicable
        elseif k==size(genes(g).utr3_exons{j},1)&&length(genes(g).polya)>=j&&...
               genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.utr3exon]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.polya_tail]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        end
      end   
    elseif genes(g).strand=='-'
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % INSERT SEGMENTS FOR NEGATIVE STRAND %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      isoform=[];
      % insert a row for each 3'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr3_exons{j},1)
        % insert polya_tail if applicable
        if k==1&&length(genes(g).polya)>=j&&genes(g).polya(j)>genes(g).utr3_exons{j}(k,1)&&...
           genes(g).polya(j)<genes(g).utr3_exons{j}(k,2)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1) genes(g).polya(j) segment_ids.polya_tail]];
          isoform = [isoform; [genes(g).polya(j) genes(g).utr3_exons{j}(k,2) segment_ids.utr3exon]];
        else
          isoform = [isoform; [genes(g).utr3_exons{j}(k,1:2) segment_ids.utr3exon]];
        end
        if k<size(genes(g).utr3_exons{j},1)
          isoform = [isoform; [genes(g).utr3_exons{j}(k,2) genes(g).utr3_exons{j}(k+1,1) segment_ids.intron]];
        else
          isoform(end,2)=isoform(end,2)+3;
        end
      end
      % if there is an intron between cdsStop and first utr3_exon:
      %----------------------------------------------------
      if  ~isempty(genes(g).utr3_exons{j})
        if genes(g).utr3_exons{j}(end,2)<genes(g).cds_exons{j}(1,1)
          isoform(end,2) = isoform(end,2)-3;%undo moving the last utr3-exon position
          isoform = [isoform; [isoform(end,2) genes(g).cds_exons{j}(1,1)+3 segment_ids.intron]];
        end
      end
      % insert a row for each cds_exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).cds_exons{j},1)
        if k==1 %&& ~isempty(isoform)&&isoform(end,3)==segment_ids.utr3exon
          isoform = [isoform; [genes(g).cds_exons{j}(k,1)+3 genes(g).cds_exons{j}(k,2) segment_ids.cds_exon]];
        else
          isoform = [isoform; [genes(g).cds_exons{j}(k,1:2) segment_ids.cds_exon]];
        end  
        if k<size(genes(g).cds_exons{j},1)
          isoform = [isoform; [genes(g).cds_exons{j}(k,2) genes(g).cds_exons{j}(k+1,1) segment_ids.intron]];
        end
      end  
      % insert an intron between last cds_exon and 5'utr if necessary
      %----------------------------------------------------   
      if  ~isempty(genes(g).utr5_exons{j}) 
        if genes(g).cds_exons{j}(end,2)<genes(g).utr5_exons{j}(1,1)
          isoform = [isoform; [genes(g).cds_exons{j}(end,2) genes(g).utr5_exons{j}(1,1) segment_ids.intron]];
        end
      end
      % insert a row for each 5'utr exon and each intron between
      %----------------------------------------------------
      for k=1:size(genes(g).utr5_exons{j},1)
        isoform = [isoform; [genes(g).utr5_exons{j}(k,1:2) segment_ids.utr5exon]];
        if k<size(genes(g).utr5_exons{j},1)
          isoform = [isoform; [genes(g).utr5_exons{j}(k,2) genes(g).utr5_exons{j}(k+1,1) segment_ids.intron]];
        end
      end
    end %strand case
    % assert a tight segmentation
    assert(all(isoform(2:end, 1) == isoform(1:end-1, 2)));
    if ~(all(isoform(2:end, 1) == isoform(1:end-1, 2)))
       if genes(g).utr5_exons{j}(end,2)<genes(g).tis(j)
         fprintf('last utr5_exon stop: %i \n tis: \t%i \n first cds_exon_start: %i\n\n',...
                genes(g).utr5_exons{j}(end,2),genes(g).tis(j),genes(g).cds_exons{j}(1,1))
       else 
       end
    end

    % assert that there is an ORF
    assert(sum(isoform(:,3)==segment_ids.cds_exon ...
               | isoform(:,3)==segment_ids.utr5exon ...
               | isoform(:,3)==segment_ids.utr3exon)==size(genes(g).cds_exons{j},1)+size(genes(g).utr5_exons{j},1)+size(genes(g).utr3_exons{j},1));
    cds_idx = find(isoform(:,3)==segment_ids.cds_exon);
    assert(mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)==0)
    %mod(sum(isoform(cds_idx,2)-isoform(cds_idx)),3)
    %keyboard

    %str = [];
    %for k=1:length(cds_idx),
    %  str = [str block.genestr(isoform(cds_idx(k),1):isoform(cds_idx(k),2)-1)];
    %end
    %f = find_open_frames(lower(str),1); 
    %assert(f(1)~=0);
    genes(g).segments{j} = isoform;
  end
end

return



return

function [blocks genes split] = gen_lsl_blocks(genes, blocks, fn_genome_config, model, fn_split,run_locally)
% [blocks genes split] = gen_lsl_blocks(genes, blocks, fn_genome_config, model, fn_split,run_locally)

% find genes that are invalid or not complete
%------------------------------------------------------------
fprintf(1,'check genes for completeness\n')
which_checks.exons_sorted = 1;
which_checks.intron_length = 1;
which_checks.exon_length = 1;
which_checks.splicesites = 1;
which_checks.orf = 0 ;
which_checks.gene_length = 1;
which_checks.graph = 1;
which_checks.transacc = 0;
which_checks.complete = 1;

genome_info = init_genome(fn_genome_config);

if ~run_locally
  chunks = 20;
  parts = ceil((length(genes)/chunks)*(0:chunks));
  for j=2:length(parts)
    outfiles{j-1}= ['~' tempname];
    Data.outfile = outfiles{j-1};
    Data.genome_info = genome_info;
    Data.which_checks = which_checks;
    Data.genes = genes(parts(j-1)+1:parts(j));
    jobinfo(j-1)=rproc('check_genes_rproc_wrapper',Data,1000,10*60);
  end
  jobinfo = rproc_wait(jobinfo,30,1,0);
  checked_genes = [];
  for j=1:length(jobinfo)
    l = load(outfiles{j},'genes');
    checked_genes=[checked_genes l.genes];
  end  
  assert(length(genes)==length(checked_genes))
  genes = checked_genes;
else
  genes = check_genes(genes,genome_info, which_checks);
end

valid_idx = find([genes.is_complete]&[genes.is_valid]);
fprintf('\n\n %i of %i genes are valid and have at least one complete transcript \n', length(valid_idx),length(genes))

%assert(length(valid_idx)>0)

clear checked_genes valid_idx j chunks parts

% remove blocks that contain no or incomplete genes
%--------------------------------------------------------------
block_rm_idx = [];
for j=1:length(blocks)
  gene_ids = blocks(j).gene_ids;
  if isempty(gene_ids)
    block_rm_idx = [block_rm_idx j];
    continue;
  end
  for id=gene_ids
    if ~genes(id).is_complete
      block_rm_idx = [block_rm_idx j];
      break;
    end
  end
end

fprintf('remove %i (%i) blocks containing incomplete genes\n',length(block_rm_idx),length(blocks))
blocks(block_rm_idx)=[];
clear curr_genes curr_blocks s_blocks_idx s_genes_idx c_blocks_idx c_genes_idx s j k g block_rm_idx gene_idxs

% filter out blocks that contain overlapping genes
%------------------------------------------------------------
fprintf('filter out blocks that contain overlapping genes\n')
rm_idx = [];
for j=1:length(blocks)
  if length(blocks(j).gene_ids) <= 1, continue; end
  starts = [genes(blocks(j).gene_ids).start];
  stops = [genes(blocks(j).gene_ids).stop];
  [starts sort_idx] = sort(starts);
  stops = stops(sort_idx);
  for k=2:length(starts)
    if ~(starts(k)>stops(k-1))
      rm_idx = [rm_idx j];
      break; 
    end
  end
end
fprintf('remove %i (%i) blocks containing overlapping genes\n',length(rm_idx),length(blocks))
blocks(rm_idx) = [];
clear rm_idx sort_idx starts stops

% add segmentation to genes
%-----------------------------------------------------------
tmp.genes = genes;
tmp.segment_ids = model.segments;
genes = gen_gene_segmentation(tmp);
clear tmp


% add segmentation to blocks
%-------------------------------------------------------------
[blocks.config] = deal(fn_genome_config);
blocks = load_sequence(blocks);
blocks = gen_block_segmentation(genes,blocks,model);

% get split from training blocks
%-------------------------------------------------------------
split = [];
if ~isfield(blocks,'split')||isempty(blocks(1).split)
  l = load(fn_split,'split');
  split = -1*ones(1,length(blocks));
  for j=1:length(blocks), 
    for k=1:length(fieldnames(l.split)), 
      idx = find(l.split.(['split_' num2str(k)])==blocks(j).id); 
      if ~isempty(idx), 
        split(j)=k;
        blocks(j).split=k; 
      end, 
    end, 
  end
  assert(isequal(unique(split), 1:5))
  clear l j k idx
end
% remove blocks with empty truth 
%-------------------------------------------------------------
fprintf(1,'filter out blocks with empty truth\n')
rm_idx = [];
val_idx = [];
for b = 1:length(blocks)
  if isempty(blocks(b).truth)
    rm_idx = [rm_idx b];
    continue;
  end
end
blocks(rm_idx) = [];
fprintf('remove %i (%i) blocks where generation of truth has failed\n',length(rm_idx), length(blocks))



return

function blocks = make_prediction_blocks(PAR, blocks, model, generate_features)
% blocks = make_prediction_blocks(PAR, blocks, model, generate_features)

if nargin<4, 
  generate_features=1 ;
end ;
% load sequence
%-------------------------------------------------------------
if ~isfield(blocks, 'seq')
  blocks = load_sequence(blocks);
end
% add signals to blocks
%-------------------------------------------------------------
fprintf(1,'load signals and append to blocks \n')

if isfield(PAR.LSL.method, 'model_name')&&strcmp(PAR.LSL.method.model_name, 'onlyorf')

  fprintf(1,'load signals for acc, don, tis, cdsStop only \n and make up some dummy positions with zero values for the other signals \n')
  blocks = add_signals2blocks(blocks,PAR,{'acc','don','tis','cdsStop'},'pred_dir',1); 
  for b = 1:length(blocks)
    len = length(blocks(b).seq);
    additional_pos = (PAR.regions.offset+1:10:blocks(b).stop-blocks(b).start-PAR.regions.offset-1);
    blocks(b).Signals.tss.pos = blocks(b).Signals.tis.pos-1;
    idx = find(blocks(b).Signals.tis.pos<=1);
    blocks(b).Signals.tss.pos(idx)=[];
    assert(all(blocks(b).Signals.tss.pos<=len))
    blocks(b).Signals.tss.Conf_cum = zeros(length(blocks(b).Signals.tis.pos),1);
    blocks(b).Signals.cleave.pos = unique([blocks(b).Signals.cdsStop.pos+4; additional_pos']);
    idx = find(blocks(b).Signals.cleave.pos>=len);
    blocks(b).Signals.cleave.pos(idx)=[];
    blocks(b).Signals.cleave.Conf_cum = zeros(length(blocks(b).Signals.cleave.pos),1);
    blocks(b).Signals.polya.pos = [];
    blocks(b).Signals.polya.Conf_cum = [];
  end
else
  signal_names = fieldnames(PAR.model.signals) ;
  blocks = add_signals2blocks(blocks,PAR, signal_names, 'pred_dir', 0);
end

% generate feature matrix states x pos x features 
%-------------------------------------------------------------
if generate_features,
  fprintf(1,'generate feature matrix for blocks \n')
  blocks = gen_features(blocks, model);
end ;


return

function blocks_new=merge_blocks(blocks, model, num_blocks_per_blocks) ;
% blocks=merge_blocks(blocks, model, num_blocks_per_blocks) ;

%idx=randperm(length(blocks)) ;
%idx = reshape(idx(1:(floor(length(blocks)/num_blocks_per_blocks)*num_blocks_per_blocks)), floor(length(blocks)/num_blocks_per_blocks), num_blocks_per_blocks) ;

% merge blocks, that have similar intergenic segment lengths. this tries not to spoil the intergenic length  distribution
left_len = zeros(1,length(blocks)) ;
right_len = zeros(1,length(blocks)) ;
for i=1:length(blocks)
  left_len(i) = blocks(i).truth(1).segments(1,2)-blocks(i).truth(1).segments(1,1) ;
  right_len(i) = blocks(i).truth(1).segments(end,2)-blocks(i).truth(1).segments(end,1) ;
end 

taken=zeros(1,length(blocks)) ;
for i=1:floor(length(blocks)/num_blocks_per_blocks),
  iidx=find(taken==0) ;
  iidx=iidx(randperm(length(iidx))) ;
  idx(i,1) = iidx(1) ;
  taken(iidx(1))=1 ;
  for j=2:num_blocks_per_blocks,
    iidx=find(taken==0) ;
    [tmp,iidx2]=min(abs(left_len(iidx)-right_len(idx(i,j-1)))) ;
    idx(i,j) = iidx(iidx2) ;
    taken(iidx(iidx2))=1 ;
    %[tmp abs(left_len(idx(i,j))-right_len(idx(i,j-1)))]
  end ;
end ;
    
blocks_new=blocks([]) ;
for i=1:size(idx,1),
  blocks_new(i).id = i ;
  blocks_new(i).seq = '' ;
  blocks_new(i).orig_block_ids = idx(i,:) ;
  blocks_new(i).orig_offsets = [] ;
  blocks_new(i).truth.segments = [] ;
  blocks_new(i).is_alt = 0 ;
  blocks_new(i).gene_complete = [] ;

  signal_names=fieldnames(blocks(1).Signals) ;
  for s=1:length(signal_names),
    blocks_new(i).Signals.(signal_names{s})=struct ;
    blocks_new(i).Signals.(signal_names{s}).pos = [] ;
    blocks_new(i).Signals.(signal_names{s}).contig_pos = [] ;
    blocks_new(i).Signals.(signal_names{s}).Conf_cum = [] ;
  end 
  cnt=0 ;
  for j=idx(i,:),
    cnt = cnt + 1 ;
    if length(blocks(j).truth)>1,
      warning('merge_blocks:multiple_truths', 'ignoring multiple truths') ;
    end ;

    blocks_new(i).orig_offsets(end+1) = length(blocks_new(i).seq) ;
    if cnt==1,
      blocks_new(i).start = blocks(j).start ;
    end ;
    if cnt == num_blocks_per_blocks,
      blocks_new(i).stop = blocks(j).stop + blocks_new(i).orig_offsets(end) ;
    end ;

    if isfield(blocks, 'content_pred') && cnt==1
      blocks_new(i).content_pred = [ blocks(j).content_pred ] ;
    elseif isfield(blocks, 'content_pred')
      blocks_new(i).content_pred = [ blocks_new(i).content_pred blocks(j).content_pred+blocks_new(i).content_pred(end) ] ;
    end

    blocks_new(i).seq = [ blocks_new(i).seq blocks(j).seq ] ;
    blocks_new(i).is_alt = blocks_new(i).is_alt | blocks(j).is_alt ;

    blocks_new(i).gene_status = [blocks_new(i).gene_status blocks(j).gene_status] ;
    blocks_new(i).gene_complete = [blocks_new(i).gene_complete blocks(j).gene_complete] ;
    blocks_new(i).truth.segments = [  blocks_new(i).truth.segments;
                        [blocks(j).truth(1).segments(:,1:2)+blocks_new(i).orig_offsets(end) blocks(j).truth(1).segments(:,3) blocks(j).truth(1).segments(:,4)*cnt] ] ;

    signal_names=fieldnames(blocks(j).Signals) ;
    for s=1:length(signal_names),
      blocks_new(i).Signals.(signal_names{s}).pos = [blocks_new(i).Signals.(signal_names{s}).pos;
                          blocks(j).Signals.(signal_names{s}).pos+blocks_new(i).orig_offsets(end)] ;
      blocks_new(i).Signals.(signal_names{s}).Conf_cum = [blocks_new(i).Signals.(signal_names{s}).Conf_cum;
                          blocks(j).Signals.(signal_names{s}).Conf_cum] ;
    end 
  end ;

  while (1),
    truth_type = blocks_new(i).truth.segments(:,3) ;
    idx_t = find(truth_type(1:end-1)==model.segments.intergenic & truth_type(2:end)==model.segments.intergenic, 1, 'first') ;
    if isempty(idx_t), 
      break ;
    end ;
    S= blocks_new(i).truth.segments ;
    blocks_new(i).truth.segments = [S(1:idx_t-1, :);
                        S(idx_t,1) S(idx_t+1, 2:end);
                        S(idx_t+2:end, :) ] ;
  end ;

end ;



return

function [max_start,max_end,max_atg]=find_max_orfs(str, atg)
% [max_start,max_end,max_atg]=find_max_orfs(str, atg)

str=lower(str) ;
if ~exist('atg'),
  atg = '' ;
end ;

idx = sort([strfind(str,'tag'), strfind(str,'taa'), strfind(str,'tga')]) ;
idx_atg = sort(strfind(str,atg)) ;

map=zeros(1,length(str)) ;
map(idx)=1 ;

max_len(1:3) = 0 ; max_start(1:3)=0; max_end(1:3)=0 ; max_atg(1:3) = 0 ;

for j=0:2
  last_start=-2 ; 
  for i=idx(mod(idx-1,3)==j)
    if ~isempty(atg)
      p = idx_atg(find(mod(idx_atg-1,3)==j & idx_atg>=last_start, 1, 'first')) ;
      if isempty(p),
        p=i ; % no atg -> no orf
      end ;
    else
      p=last_start+3; 
    end ;
    if i-p>max_len(j+1),
      max_atg(j+1)=p ;
      max_start(j+1)=last_start+3 ;
      max_end(j+1)=i-1 ;
      max_len(j+1)=i-p ;
    end ;
    last_start=i ; 
  end ;
  if isempty(idx(mod(idx-1,3)==j))
    if ~isempty(atg)
      p = idx_atg(find(mod(idx_atg-1,3)==j, 1, 'first')) ;
      if isempty(p), 
        p=length(p) ;
      end ;
    else
      p=-2+3; 
    end ;
    max_atg(j+1)=p ;
    max_start(j+1)=-2+3 ;
    max_end(j+1)=length(str) ;
    max_len(j+1)=length(str)-p ;
    continue ;
  end ;
  if length(str)-i>max_len(j+1),
    if ~isempty(atg)
      p = idx_atg(find(mod(idx_atg-1,3)==j & idx_atg>=i, 1, 'first')) ;
      if isempty(p),
        p=length(str) ;
      end ;
    else
      p=i ; 
    end ;
    max_atg(j+1)=p ;
    max_start(j+1)=i ;
    max_end(j+1)=length(str)-1 ;
    max_len(j+1)=length(str)-p ;
  end ;
end ;

return

function sp = sparse_features(features)
% sp = sparse_features(features)

for i=1:size(features,1),
  features_ = features(i,:,:) ;
  assert(all(all(all(features_(~isinf(features_))>=0)))) ;
end ;
clear features_ ;
features = features+1 ;

for i=1:size(features,1),
  features_ = features(i,:,:) ;
  features_(isinf(features_)) = 0 ;
  features(i,:,:) = features_ ;
end ;
clear features_ ;

sp={} ;
for i=1:size(features,3)
  sp{i} = sparse(features(:,:,i)) ;
end ;


return

function genes = check_genes(genes, genome_info,which_checks) ;
% genes = check_genes(genes, genome_info,which_checks) ;

% TO DO CHECK SPLICE GRAPH: 
%  CHECK if splice graoh is connected. SHOULd WORK NOW
%  
%addpath ~/svn/tools/genomes/

min_boundary_dist=20 ;

if nargin<3 | isempty(which_checks), 
  fprintf('no checks performed\n')
  return
end

if ~isfield(which_checks,'exons_sorted')
  which_checks.exons_sorted = 1;
end ;
if ~isfield(which_checks,'intron_length')
  which_checks.intron_length = 0;
end
if ~isfield(which_checks,'exon_length')
  which_checks.exon_length = 0;
end
if ~isfield(which_checks,'splicesites');
  which_checks.splicesites = 0;
end
if ~isfield(which_checks,'orf')
  which_checks.orf = 0;
end
if ~isfield(which_checks,'gene_length')
   which_checks.gene_length = 0;
end
if ~isfield(which_checks,'graph')
  which_checks.graph = 0;
end ;
if ~isfield(which_checks,'transacc')
  which_checks.transacc = 0;
end ;
if ~isfield(which_checks,'complete')
  which_checks.complete = 0;
end ;
which_checks;


num_invalid_transcripts = 0 ;
num_invalid_genes = 0 ;
num_no_exons = 0;
num_no_trans = 0;
num_splice_not_ok = 0 ;
num_intron_not_ok = 0 ;
num_orf_not_ok = 0 ;
num_tis_not_ok = 0 ;
num_cds_exon_not_ok = 0 ;
num_stop_not_ok = 0 ;
num_graphs_not_ok = 0 ;
num_transacc_not_ok = 0;
num_at_boundary = 0;

genes_not_ok = [];

fprintf('checking %i genes: ', length(genes)) ;
for i=1:length(genes), 
  if mod(i,1000)==0||i==length(genes) 
    %fprintf(['gene=%i  invalid_genes=%i  invalid_transcripts=%i  no_exons=%i no_transcripts=%i ' ...
    %         'num_splice_not_ok=%i cds_problems=%i,%i,%i,%i num_graphs_not_ok =%i num_trans_not_ok =%i \r'],...
    %        i,num_invalid_genes,num_invalid_transcripts , num_no_exons,num_no_trans,num_splice_not_ok,...
    %        num_tis_not_ok, num_cds_exon_not_ok, num_orf_not_ok,
    %        num_stop_not_ok, num_graphs_not_ok,
    %        num_transacc_not_ok); 
    fprintf('%i ... ', i) ;
  end
  transcript_valid = [] ; 
  genes(i).is_valid = 0 ; % make sure it is set...
  is_valid = 1 ;
  
  if isempty(genes(i).exons)||all(cellfun('isempty',genes(i).exons))
    genes(i).is_valid = 0;
    num_invalid_genes = num_invalid_genes+1;
    num_no_exons = num_no_exons+1;
    continue
  end
  if isempty(genes(i).transcripts)||all(cellfun('isempty',genes(i).transcripts))
    genes(i).is_valid = 0;
    num_no_trans = num_no_trans+1;
    num_invalid_genes = num_invalid_genes+1;
    continue
  end
  
  
  % CHECK TRANSACC CONSENSUS
  trans_not_ok = 0;
  rm_trans=[];
  if which_checks.transacc 
    [transacc,idx] = unique(genes(i).transacc);
    for trans_=transacc'
      [mRNA, tmp, ok] =  load_genomic(genes(i).chr, char(genes(i).strand),...
                                      trans_-5, trans_+5,genome_info,1);
      if ~ok,
        warning('load genomic failed') ;
      else
        if ~isequal(mRNA(4:5),'ag')% &genes(i).strand=='-';
          mRNA(2:6)
          idx=find(ismember(genes(i).transacc,trans_));
          genes(i).transacc_info
          trans_not_ok = trans_not_ok+1;
          rm_trans=[rm_trans trans_];
        end
      end ;
    end
    idx=find(ismember(genes(i).transacc,rm_trans));
    if trans_not_ok
      warning('trans_not_ok');
      %keyboard
    end
    genes(i).transacc(idx)=[];
    genes(i).transacc_info(idx)=[];
    genes(i).transacc_conf(idx)=[];
    num_transacc_not_ok = num_transacc_not_ok+trans_not_ok; 
    
  end

  for j=1:length(genes(i).exons),


    [tmp,idx] = sort(genes(i).exons{j}(:,1)) ;
    
    genes(i).exons{j} = genes(i).exons{j}(idx,:) ;
    if ~isempty(genes(i).exons_confirmed) && ~isempty(genes(i).exons_confirmed{j})
      genes(i).exons_confirmed{j} = genes(i).exons_confirmed{j}(idx,:) ;
    end
    

    %%% CHECK BOUNDARY
    % keyboard
    boundary_not_ok=0;
    if isempty(genes(i).start) || isempty(genes(i).stop)
      boundary_not_ok=1;
    else
      exons = genes(i).exons{j};
      if any(any(exons(:,1:2)<genes(i).start-1|exons(:,1:2)>genes(i).stop+1))
        warning('overlaying transcript found');
        boundary_not_ok=1;
      end
    end ;
   
    %%% CHECK EXONS SORTED
    exons_not_ok = 0 ;
    if isempty(genes(i).exons{j}),
      exons_not_ok = 1 ;
    else
      if which_checks.exons_sorted
        [tmp,idx] = sort(genes(i).exons{j}(:,2)) ;
        if ~isequal(idx',1:length(idx)), 
          fprintf('exons not sorted\n') ;
          exons_not_ok = 1 ;
        end ;
      end
    end ;
    %%% CHECK EXON LENGTH
    if which_checks.exon_length
      exon_length = genes(i).exons{j}(:,2)-genes(i).exons{j}(:,1);
      if any(exon_length<10)
        exons_not_ok = 1;
      end ;
    end

    
    %%% SPLICE SITE CONSENSUS
    splice_not_ok = 0 ;  
    
    if which_checks.splicesites && size(genes(i).exons{j},1)>1 && isempty(genes(i).exons{j}),
      % if any(genes(i).exons{j}(:,1)>genes(i).exons{j}(:,2))
      %    valid_splice(1:2,1:2)=0
      % else
      [mRNA, valid_splice, ok] =  load_genomic(genes(i).chr, char(genes(i).strand),...
                                              genes(i).exons{j}(:,1), ...
                                              genes(i).exons{j}(:,2),genome_info,1);
      if ~ok,
        splice_not_ok = 1 ;
      else
        % end
        if genes(i).strand=='+'
          splice_not_ok = sum(sum(valid_splice(2:end-1,:)')<2) + ~valid_splice(1,2)+~valid_splice(end,1);
        else
          splice_not_ok = sum(sum(valid_splice(2:end-1,:)')<2) + ~valid_splice(1,1)+~valid_splice(end,2);
        end
      end ;
      if splice_not_ok
        genes_not_ok = [genes_not_ok i];
      end
      num_splice_not_ok = num_splice_not_ok+splice_not_ok; 
    end
    
    %%% CHECK INTRON LENGTH
    intron_not_ok = 0 ;
    if which_checks.intron_length
      intron_length = genes(i).exons{j}(2:end,1)-genes(i).exons{j}(1:end-1,2);
      if any(intron_length>genome_info.max_intron_len)|...      
            any(intron_length<genome_info.min_intron_len)
        
        % fprintf('\nintron lengths: %i - %i \n',min(intron_length),max(intron_length)) ;
        intron_not_ok = sum(intron_length>genome_info.max_intron_len |...
                            intron_length<genome_info.min_intron_len);
      end ;
      num_intron_not_ok = num_intron_not_ok+intron_not_ok; 
    end
    transcript_valid(j) = ~any([exons_not_ok splice_not_ok intron_not_ok boundary_not_ok]);
  end %%loop over exons
  
  %%% CHECK ORF
  for j=1:length(genes(i).cds_exons), 
    if which_checks.orf ,
      cds_exons = genes(i).cds_exons{j};
      if isempty(cds_exons), 
        transcript_valid(j)=0;
        continue; 
      end
      if all(cds_exons(:)==0),
         genes(i).cds_exons{j}=[] ;
         transcript_valid(j)=0;
         continue ;
      end ;
      if genes(i).strand=='+'
        if ~isempty(genes(i).tis)
          if ~(genes(i).tis{j}==cds_exons(1,1)),
            warning('this was an assert') ;
            transcript_valid(j) =0;
            %keyboard
            continue ;
          end ;
        end
        if ~isempty(genes(i).cdsStop)
          if ~(genes(i).cdsStop{j}==cds_exons(end,2)-3),
            warning('this was an assert') ;
            transcript_valid(j) =0;
            %keyboard ;
            continue ;
          end ;
        end
        cds_exons(:,2) = cds_exons(:,2)-1;
      else
        if ~isempty(genes(i).tis)
          assert(genes(i).tis{j}==cds_exons(end,2))
        end
        if ~isempty(genes(i).cdsStop) 
          assert(genes(i).cdsStop{j}==cds_exons(1,1)+3);
        end
        cds_exons(:,1) = cds_exons(:,1)+1;
      end

      [cds, valid_splice, ok] =  load_genomic(genes(i).chr, char(genes(i).strand), cds_exons(:,1), cds_exons(:,2), genome_info, 1);
      if ~ok,
        transcript_valid(j) =0;
        continue ;
      end ;
      f = find_open_frames(cds,1) ;

      % keyboard
      % if first (last) cds_exon is smaler than 3 the start (stop) codon is 
      % spliced together, thus we will not have predictions for it
      too_short = cds_exons(1,2)-cds_exons(1,1)+1<3 || cds_exons(end,2)-cds_exons(end,1)+1<3;

      if ~any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))
        transcript_valid(j) =0;
        num_stop_not_ok = num_stop_not_ok + 1;
      end ;
      if any(cds(1:3)~='atg'),
        transcript_valid(j) =0;
        num_tis_not_ok = num_tis_not_ok + 1;
      end ;
      if too_short,
        num_cds_exon_ok = num_cds_exon_not_ok + 1 ;
        transcript_valid(j) =0;
      end ;
      if mod(length(cds),3)~=0 || f(1)~=1 || isempty(cds) 
        num_orf_not_ok  = num_orf_not_ok + 1 ;
        transcript_valid(j) =0;
      end ;
    end
  end ; %% loop over cds_exons

  num_invalid_transcripts = num_invalid_transcripts + sum(~transcript_valid);

  % if isfield(genes(i),'transcript_valid') && ~isempty(genes(i).transcript_valid)
  %   genes(i).transcript_valid = genes(i).transcript_valid.*transcript_valid ;  
  % else
  genes(i).transcript_valid = transcript_valid ;
  % end
    
  %%% check genes length (only the region of the valid transcripts)
  length_not_ok = 0;
  if which_checks.gene_length,
    starts = [] ;
    ends = [] ;
    for j=1:length(genes(i).exons),
      if genes(i).transcript_valid(j)&&~isempty(genes(i).exons{j})
        starts(end+1)= genes(i).exons{j}(1,1) ;
        ends(end+1)  = genes(i).exons{j}(end,2) ;
      end ;
    end ;
    if max(ends)-min(starts)>genome_info.max_intron_len*10,
      % fprintf('gene length: %i\n', max(ends)-min(starts)) ;
      length_not_ok = 1;
    end ;
  end
  
  %%% check splicegraph
  %% please document me ... (some more) !!
  graph_not_ok = 0;
  if which_checks.graph && isfield(genes, 'splicegraph'), 
    edges = genes(i).splicegraph{2} ;
    if ~isequal(edges,0)
      ss = 0 ;
      for e=1:size(edges,1), 
        ss = ss + edges^e ; 
      end ; 
      if any(ss==0)
        graph_not_ok = 1;
        % viewsplicegraph(genes(i))
        % keyboard
      end
      num_graphs_not_ok = num_graphs_not_ok+graph_not_ok ;
    end
  end
  % if ~isfield(genes(i),'is_valid') || isempty(genes(i).is_valid)
  %   genes(i).is_valid = 1;
  % end    
  genes(i).is_valid = any(genes(i).transcript_valid)&&~length_not_ok&&...
      ~graph_not_ok&& ~isempty(genes(i).exons);
  % genes(i).is_valid = genes(i).is_valid&any(genes(i).transcript_valid)&~length_not_ok&...
  %     ~graph_not_ok& ~isempty(genes(i).exons);
  
  num_invalid_genes = num_invalid_genes + ~genes(i).is_valid; 
  
end ; %%loop over genes
fprintf('Done.\n') ;

genes(1).transcript_complete = [];
if which_checks.complete
  for i=1:length(genes),
    if ~any(genes(i).transcript_valid==1),
      genes(i).is_complete = 0 ;
      continue ;
    end ;
    for k=1:length(genes(i).transcripts)
      complete = 1;

% does not work since exons can be split into a cds and a utr part      
%      intersection = intersect(genes(i).cds_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).cds_exons,1)
%        error('adsfadfasdf')
%      end
%      intersection = intersect(genes(i).utr3_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).utr3_exons,1)
%        error('adsfadfasdf')
%      end
%      intersection = intersect(genes(i).utr5_exons,genes(i).exons,'rows');
%      if ~(size(intersection,1)==size(genes(i).utr5_exons,1)
%        error('adsfadfasdf')
%      end
%      clear intersection

      if isempty(genes(i).exons)|| isempty(genes(i).exons{k})
        genes(i).transcript_complete(k) = 0;
        continue
      end
      if length(genes(i).cds_exons)<k||isempty(genes(i).cds_exons{k})
        complete = 0;
      end
      if length(genes(i).utr3_exons)<k||isempty(genes(i).utr3_exons{k})
        complete = 0;
      end
      if length(genes(i).utr5_exons)<k||isempty(genes(i).utr5_exons{k})
        complete = 0;
      end
      if ~isempty(genes(i).tis) && length(genes(i).tis)>=k && ~isempty(genes(i).tis{k})
        if genes(i).tis{k}<genes(i).start||genes(i).tis{k}>genes(i).stop
          complete = 0;
        end
      else
        complete = 0;
      end

      if ~isempty(genes(i).cdsStop)&&length(genes(i).cdsStop)>=k &&~isempty(genes(i).cdsStop{k})
        if genes(i).cdsStop{k}<genes(i).start||genes(i).cdsStop{k}>genes(i).stop
          complete = 0;
        end
      else
        complete = 0;
      end
    
      if ~isempty(genes(i).tss)&& length(genes(i).tss)>=k&&~isempty(genes(i).tss{k})
        if genes(i).tss{k}==genes(i).tis{k}||genes(i).tss{k}<genes(i).start||genes(i).tss{k}>genes(i).stop
          complete = 0;
        end
      else 
        complete = 0;
      end
      
      if ~isempty(genes(i).cleave)&& length(genes(i).cleave)>=k && ~isempty(genes(i).cleave{k})
        if genes(i).cleave{k}==genes(i).cdsStop{k} || genes(i).cleave{k}<genes(i).start || genes(i).cleave{k}>genes(i).stop
          complete = 0;
        end
      else 
        complete = 0;
      end
      
      for j=1:length(genes(i).exons),
        if ~(all(all(genes(i).exons{j}(:,1:2)>=genes(i).start& ...
                     genes(i).exons{j}(:,1:2)<=genes(i).stop+1)))
          genes(i).start = min(min(genes(i).exons{j}))
          genes(i).stop = max(max(genes(i).exons{j}))
        end
      end

      gene_contig_len=contig_len(genes(i).chr, genome_info) ;
      if genes(i).start<min_boundary_dist || genes(i).stop>gene_contig_len-min_boundary_dist,
        genes(i).is_valid=0 ;
        num_at_boundary = num_at_boundary + 1 ;
      end ;
      
      genes(i).transcript_complete(k) = complete & genes(i).transcript_valid(k);
    end
    if ~any(genes(i).transcript_complete)
      %keyboard
    end
    genes(i).is_complete = any(genes(i).transcript_complete);
  end
end

fprintf('\nProblems found:\n') ;
	    fprintf('\ttotal invalid genes\t\t%i\n',num_invalid_genes) ;
	    fprintf('\twithout transcripts\t\t%i\n', num_no_trans) ;
	    fprintf('\ttotal invalid transcripts\t%i\n', num_invalid_transcripts) ;
	    fprintf('\twithout exons\t\t\t%i\n', num_no_exons) ;
	    fprintf('\tsplice consensus missing\t%i\n', num_splice_not_ok) ;
	    fprintf('\ttis consensus missing\t\t%i\n', num_tis_not_ok) ;
	    fprintf('\tcds exon too short\t\t%i\n', num_cds_exon_not_ok) ;
	    fprintf('\tORF missing\t\t\t%i\n', num_orf_not_ok) ;
	    fprintf('\tstop consensus missing\t\t%i\n', num_stop_not_ok) ;
	    fprintf('\tgraphs not ok\t\t\t%i\n', num_graphs_not_ok) ;
	    fprintf('\ttransacceptor not ok\t\t%i\n', num_transacc_not_ok) ;
	    fprintf('\tgenes at contig boundary\t%i', num_at_boundary) ;

fprintf('\n\n')

return

function genes = correct_tis_stop(genes, genome_info, min_orf_len, min_orf_sep, verb) ;
% genes = correct_tis_stop(genes, genome_info, min_orf_len, min_orf_sep, verb) ;

if nargin<3,
  min_orf_len = 300 ;
end ;
if nargin<4,
  min_orf_sep = 0.7 ;
end ;
if nargin<5,
  verb=1 ;
end ;

num_tis_not_ok = 0 ;
num_tis_shift_first = 0 ;
num_tis_shift_max = 0 ;
num_stop_not_ok = 0 ;
num_stop_shift_3 = 0 ;
num_stop_shift_max = 0 ;
num_tis_stop = 0 ;

for i=1:length(genes), 
  if verb==1 && mod(i,1000)==0,
    fprintf('%i/%i ... ', i, length(genes)) ;
  end ;
    

  if isempty(genes(i).exons)||all(cellfun('isempty',genes(i).exons))
    continue
  end
  if isempty(genes(i).transcripts)||all(cellfun('isempty',genes(i).transcripts))
    continue
  end
  
  for j=1:length(genes(i).exons),
    %%% CHECK ORF
    for j=1:length(genes(i).cds_exons), 
      cds_exons = genes(i).cds_exons{j};
      if isempty(cds_exons), 
        continue; 
      end
      if all(cds_exons(:)==0),
        genes(i).cds_exons{j}=[] ;
        continue ;
      end ;
      if genes(i).strand=='+'
        %if ~isempty(genes(i).tis)
        %  assert(genes(i).tis{j}==cds_exons(1,1)) ;
        %end ;
        %if ~isempty(genes(i).cdsStop)
        %  assert(genes(i).cdsStop{j}==cds_exons(end,2)-3) ;
        %end
        cds_exons(:,2) = cds_exons(:,2)-1;
      else
        %if ~isempty(genes(i).tis)
        %  assert(genes(i).tis{j}==cds_exons(end,2))
        %end
        %if ~isempty(genes(i).cdsStop) 
        %  assert(genes(i).cdsStop{j}==cds_exons(1,1)+3);
        %end
        cds_exons(:,1) = cds_exons(:,1)+1;
      end
      
      [cds, tmp, ok] =  load_genomic(genes(i).chr, char(genes(i).strand), cds_exons(:,1), cds_exons(:,2), genome_info, 1);
      if ~ok,
        continue ;
      end ;
      num_tis_stop = num_tis_stop + 1 ;
      
      % check if stop codon is not included in cds exon
      if ~any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))
        num_stop_not_ok = num_stop_not_ok + 1 ;  
        if genes(i).strand=='+'
          cds_exons(end, 2) = cds_exons(end, 2) + 3 ;
        else
          cds_exons(1,1) = cds_exons(1,1) - 3 ;
        end
        [cds] =  load_genomic(genes(i).chr, char(genes(i).strand), cds_exons(:,1), cds_exons(:,2), genome_info, 1);
        
        % messy cds-stop correction
        if any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end)))
          if genes(i).strand=='+'
            if ~isempty(genes(i).utr3_exons{j}),
              num_stop_shift_3 = num_stop_shift_3 + 1 ;
              genes(i).cds_exons{j}(end,2) = genes(i).cds_exons{j}(end,2)+3;
              genes(i).cdsStop{j} = genes(i).cdsStop{j}+3;
              genes(i).utr3_exons{j}(1,1) = genes(i).utr3_exons{j}(1,1) + 3 ;
              if genes(i).utr3_exons{j}(1,2)-genes(i).utr3_exons{j}(1,1)<1,
                ii=find(genes(i).exons{j}(:,2)==genes(i).utr3_exons{j}(1,2));
                if length(ii)~=1,
                  if verb>=2,
                    fprintf('When trying to correct CDS region: did not find correct exon (+).\n') ;
                  end ;
                  % undo action
                  genes(i).cds_exons{j}(end,2) = genes(i).cds_exons{j}(end,2)-3;
                  genes(i).cdsStop{j} = genes(i).cdsStop{j}-3;
                  genes(i).utr3_exons{j}(1,1) = genes(i).utr3_exons{j}(1,1) - 3 ;
                else
                  if genes(i).exons{j}(ii,2) < genes(i).utr3_exons{j}(1,1),
                    if verb>=2,
                      fprintf('Found (corrected) stop codon does not fit into annotated exon (%i-%i, %i-%i, +)\nDo not extend CDS %s by stop codon.\n', ...
                              genes(i).exons{j}(ii,1), genes(i).exons{j}(ii,2), genes(i).utr3_exons{j}(1,1), genes(i).utr3_exons{j}(1,2), ...
                              genes(i).transcripts{j}) ;
                    end ;
                    % undo action
                    genes(i).cds_exons{j}(end,2) = genes(i).cds_exons{j}(end,2)-3;
                    genes(i).cdsStop{j} = genes(i).cdsStop{j}-3;
                    genes(i).utr3_exons{j}(1,1) = genes(i).utr3_exons{j}(1,1) - 3 ;
                  else
                    genes(i).utr3_exons{j}(1,:)=[] ;
                  end
                end ;
              end ;
            end ;
          else
            if ~isempty(genes(i).utr3_exons{j}),
              num_stop_shift_3 = num_stop_shift_3 + 1 ;
              genes(i).cds_exons{j}(1,1) = genes(i).cds_exons{j}(1,1)-3;
              genes(i).cdsStop{j} = genes(i).cdsStop{j}-3;
              genes(i).utr3_exons{j}(end,2) = genes(i).utr3_exons{j}(end,2) - 3 ;
              if genes(i).utr3_exons{j}(end,2)-genes(i).utr3_exons{j}(end,1)<1,
                ii=find(genes(i).exons{j}(:,1)==genes(i).utr3_exons{j}(end,1));
                if length(ii)~=1,
                  if verb>=2,
                    fprintf('When trying to correct CDS region: did not find correct exon (-).\n') ;
                  end ;
                  % undo action
                  genes(i).cds_exons{j}(1,1) = genes(i).cds_exons{j}(1,1)-3;
                  genes(i).cdsStop{j} = genes(i).cdsStop{j}-3;
                  genes(i).utr3_exons{j}(end,2) = genes(i).utr3_exons{j}(end,2) - 3 ;
                end ;
                if genes(i).exons{j}(ii,1) > genes(i).utr3_exons{j}(end,2),
                  if verb>=2,
                    fprintf('Found (corrected) stop codon does not fit into annotated exon (%i-%i, %i-%i, -)\nDo not extend CDS %s by stop codon.\n', ...
                            genes(i).exons{j}(ii,1), genes(i).exons{j}(ii,2), genes(i).utr3_exons{j}(end,1), genes(i).utr3_exons{j}(end,2), ...
                            genes(i).transcripts{j}) ;
                  end ;
                  % undo action
                  genes(i).cds_exons{j}(1,1) = genes(i).cds_exons{j}(1,1)-3;
                  genes(i).cdsStop{j} = genes(i).cdsStop{j}-3;
                  genes(i).utr3_exons{j}(end,2) = genes(i).utr3_exons{j}(end,2) - 3 ;
                end
                genes(i).utr3_exons{j}(end,:)= [] ;
              end ;
            end ;
          end
        end
      end
      
      %have_stop1 = any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end))) ;
      
      cds_exons = genes(i).cds_exons{j};
      if genes(i).strand=='+'
        cds_exons(:,2) = cds_exons(:,2)-1;
      else
        cds_exons(:,1) = cds_exons(:,1)+1;
      end
      [cds] =  load_genomic(genes(i).chr, char(genes(i).strand), cds_exons(:,1), cds_exons(:,2), genome_info, 1);
      
      have_tis = all(cds(1:3)=='atg') ;
      have_stop = any(ismember({'taa', 'tag', 'tga'}, cds(end-2:end))) ;
      
      %assert(have_stop==have_stop1) ;
      
      if have_tis && have_stop,
        continue ;
      end ;

      if verb>=2,
        fprintf('CDS region of gene %s ', genes(i).name) ;
      end ;
      if ~have_tis,
        num_tis_not_ok = num_tis_not_ok + 1 ;  
        if verb>=2,
          fprintf('does not start with ATG') ;
        end ;
      end ;
      if ~have_tis && ~have_stop,
        if verb>=2,
          fprintf(' and ') ;
        end ;
      end ;
      if ~have_stop,
        if verb>=2,
          fprintf('does not end with TAA, TGA, or TAG.\n') ;
        end ;
      else
        if verb>=2,
          fprintf('.\n') ;
        end ;
      end ;

      %if isequal(genes(i).name, 'gene:AGAP004308'), keyboard; end ;

      if ~have_tis && have_stop,
        idx=strfind(cds, 'atg') ;
        idx=idx(mod(idx,3)==1) ;
        if ~isempty(idx),
          atg_pos=idx(1) ;
          if length(cds)-atg_pos<min_orf_len,
            if verb>=2,
              fprintf('=> Shifting TIS to first ATG would lead to very short ORF (%i)\n', length(cds)-atg_pos) ;
            end ;
            continue ;
          end ;

          tis_pos = map_rna_pos(genes(i).cds_exons{j}, genes(i).strand, atg_pos) ;
          if genes(i).strand=='+',
            [cds_exons, utr5_exons, utr3_exons, ok] = split_exons_cds(genes(i).exons{j}, tis_pos, genes(i).cdsStop{j}+3, genes(i).strand) ;
          else
            [cds_exons, utr5_exons, utr3_exons, ok] = split_exons_cds(genes(i).exons{j}, tis_pos, genes(i).cdsStop{j}-3, genes(i).strand) ;
          end ;
          if ok,
            genes(i).tis{j} = tis_pos ;
            genes(i).cds_exons{j} = cds_exons ;
            genes(i).utr5_exons{j} = utr5_exons ;
            genes(i).utr3_exons{j} = utr3_exons ;
            % make sure tss and cleave exist
            if genes(i).strand=='+'
              genes(i).tss{j} = genes(i).exons{j}(1,1);
              genes(i).cleave{j} = genes(i).exons{j}(end,2) ;
              assert(genes(i).tis{j}==genes(i).cds_exons{j}(1,1)) ;
              assert(genes(i).cdsStop{j}==genes(i).cds_exons{j}(end,2)-3) ;
            else
              genes(i).tss{j} = genes(i).exons{j}(end,2) ;
              genes(i).cleave{j} = genes(i).exons{j}(1,1) ;
              assert(genes(i).tis{j}==genes(i).cds_exons{j}(end,2))
              assert(genes(i).cdsStop{j}==genes(i).cds_exons{j}(1,1)+3);
            end ;
            %if isequal(genes(i).name, 'gene:AGAP004714'), keyboard ; end ;
            
            % sanity check
            if genes(i).strand=='+'
              cds_exons(:,2) = cds_exons(:,2)-1;
            else
              cds_exons(:,1) = cds_exons(:,1)+1;
            end
            [genestr] = load_genomic(genes(i).chr, genes(i).strand, cds_exons(:,1), cds_exons(:,2), genome_info, 1);
            assert(isequal(genestr(1:3), 'atg')) ;
            assert(any(ismember({'taa', 'tag', 'tga'}, genestr(end-2:end)))) ;

            num_tis_shift_first = num_tis_shift_first + 1 ;
            if verb>=2,
              fprintf('=> Shifting TIS to first ATG in annotated CDS region\n') ;
            end ;
          else            
            if verb>=2,
              fprintf('=> Problems mapping TIS and/or cdsStop\n') ;
            end ;
          end ;
          continue ;
        else
          % it is probably not fixable...
          if verb>=2,
            fprintf('=> No ATG found in annotated CDS region\n') ;
          end ;
          continue ;
        end ;
      end ;
      %if have_tis && ~have_stop,
      %  fprintf('=> Cannot find stop codon\n') ;
      %  continue ;
      %end ;

      exons = genes(i).exons{j};
      if genes(i).strand=='+'
        exons(:,2) = exons(:,2)-1;
      else
        exons(:,1) = exons(:,1)+1;
      end
      [genestr] = load_genomic(genes(i).chr, genes(i).strand, exons(:,1), exons(:,2), genome_info, 1);
      [s, e, atg]=find_max_orfs(genestr, 'atg') ;
      cand = find(atg~=0 & e<length(genestr)-2) ; % exclude cases with no atg or stop
      cand2 = find(atg~=0) ; % exclude cases with no atg or stop
      [cds_len, idx] = max(e(cand)-atg(cand)) ;
      if length(cds_len)~=1 
        if verb>=2,
          fprintf('=> Cannot suitable identify ORF\n') ;
        end ;
        continue ;
      end ;
      cds_lens = sort(e(cand2)-atg(cand2)) ;
      if length(cds_lens)>1 && cds_len*min_orf_sep<max(cds_lens(end-1)),
        if verb>=2,
          fprintf('=> Cannot unambiguously identify ORF (%i vs %i)\n', cds_len, cds_lens(end-1)) ;
        end ;
        continue ;
      end ;

      if ~isempty(cand) && ~isempty(idx) && cds_len>min_orf_len,
        tis_pos     = map_rna_pos(genes(i).exons{j}, genes(i).strand, atg(cand(idx))) ;
        cdsStop_pos = map_rna_pos(genes(i).exons{j}, genes(i).strand, e(cand(idx))+3+1) ;
        
        [cds_exons, utr5_exons, utr3_exons, ok] = split_exons_cds(genes(i).exons{j}, tis_pos, cdsStop_pos, genes(i).strand) ;
        if ok,
          genes(i).tis{j} = tis_pos ;
          genes(i).cds_exons{j} = cds_exons ;
          genes(i).utr5_exons{j} = utr5_exons ;
          genes(i).utr3_exons{j} = utr3_exons ;
          % make sure tss and cleave exist
          if genes(i).strand=='+'
            genes(i).cdsStop{j} = cdsStop_pos-3 ;
            genes(i).tss{j} = genes(i).exons{j}(1,1);
            genes(i).cleave{j} = genes(i).exons{j}(end,2) ;
            assert(genes(i).tis{j}==genes(i).cds_exons{j}(1,1)) ;
            assert(genes(i).cdsStop{j}==genes(i).cds_exons{j}(end,2)-3) ;
          else
            genes(i).cdsStop{j} = cdsStop_pos+3 ;
            genes(i).tss{j} = genes(i).exons{j}(end,2) ;
            genes(i).cleave{j} = genes(i).exons{j}(1,1) ;
            assert(genes(i).tis{j}==genes(i).cds_exons{j}(end,2))
            assert(genes(i).cdsStop{j}==genes(i).cds_exons{j}(1,1)+3);
          end ;

          % sanity check
          if genes(i).strand=='+'
            cds_exons(:,2) = cds_exons(:,2)-1;
          else
            cds_exons(:,1) = cds_exons(:,1)+1;
          end
          [genestr] = load_genomic(genes(i).chr, genes(i).strand, cds_exons(:,1), cds_exons(:,2), genome_info, 1);
          assert(isequal(genestr(1:3), 'atg')) ;
          assert(any(ismember({'taa', 'tag', 'tga'}, genestr(end-2:end)))) ;

          num_tis_shift_max = num_tis_shift_max + 1 ;
          num_stop_shift_max = num_stop_shift_max + 1 ;
          
          if verb>=2,
            fprintf('=> successfully determined open reading frame of length %i\n', cds_len) ;
          end ;
        else
          if verb>=2,
            fprintf('=> Problems mapping TIS and/or cdsStop\n') ;
          end ;
        end ;
      else
        if verb>=2,
          if isempty(cand) || isempty(idx),
            fprintf('=> Cannot find appropriate ORF\n') ;
          else
            fprintf('=> Cannot find long enough ORF (%i)\n', cds_len) ;
          end ;
        end ;
      end ;
    end
  end ; %% loop over cds_exons
end ; %%loop over genes
if verb==1,
  fprintf('Done.\n') ;
end ;

if verb>=1,
  fprintf('\n\nChecked %i TIS and cdsStop positions.\nProblems found:\n', num_tis_stop) ;
  fprintf('\ttis consensus missing\t\t\t\t%i\n', num_tis_not_ok) ;
  fprintf('\t\t=> corrected to first ATG in frame\t%i\n', num_tis_shift_first) ;
  fprintf('\t\t=> successfully determined maximal ORF\t%i\n', num_tis_shift_max) ;
  fprintf('\tstop consensus missing\t\t\t\t%i\n', num_stop_not_ok) ;
  fprintf('\t\t=> shifted by stop codon by 3nt\t\t%i\n', num_stop_shift_3) ;
  fprintf('\t\t=> successfully determined maximal ORF\t%i\n', num_stop_shift_max) ;
  fprintf('\n\n')
end ;

%keyboard

return

function PAR=create_PAR(Exp, PAR_config, verb)
%  PAR=create_PAR(Exp,PAR_config,verb)
%
% creates PAR with all parameters for one genefinding
% experiment. For a new Organism X a new template file must be added to /template_files/Organisms/X.m. 
% (e.g. C_elegans.m) 
% For each signal a standard method is applied (usually SVM with one or
% more spectrum kernels)
%  
% INPUT  - Exp: filename of current experiment  (e.g. exp_C_elegans_nGASP)
%        - verb: verbose
%
% OUTPUT - PAR. struct with all information needed for genefinding experiment  
  
Signals = {'acceptor','donor','tis','cdsStop','tss','cleave','polya','transacceptor'};
sig_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
Methods = {'SVM_WD','SVM_WD','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_COMBINED','SVM_WD'};

content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
Methodc = 'SVM_Cont';
changeable_fields = {'organism','label_source','Signals','Contents','LSL','RPROC','SETs','regions','tasks','Source_organisms','FN'} ;

if nargin<3,
  verb=0;
end


PAR = init_PAR;

if nargin>0
  fprintf('parameters from exp file: %s \n',Exp)
  if ~exist('PAR_config') || isempty(PAR_config),
    PAR_exp = feval(Exp) ;
  else
    try,
      PAR_exp = feval(Exp, PAR_config) ;
    catch
      PAR_exp = feval(Exp) ;
    end 
    clear PAR_config ;
  end ;
  assert(all(ismember(fieldnames(PAR_exp),changeable_fields)))
  if ~isfield(PAR_exp,'Contents')
    PAR_exp.Contents = struct;
  end
  if ~isfield(PAR_exp,'Signals')
    PAR_exp.Signals = struct;
  end
  PAR.organism = set_default(PAR_exp,'organism',PAR.organism)  ;  
  PAR.label_source = set_default(PAR_exp,'label_source',PAR.label_source) ;
  PAR.RPROC = set_default(PAR_exp,'RPROC',PAR.RPROC)  ; 
  PAR.SETs = set_default(PAR_exp,'SETs',PAR.SETs)  ; 
  PAR.SETs = make_partitions(PAR.SETs);
  PAR.regions = set_default(PAR_exp,'regions',PAR.regions)  ; 
  PAR.tasks = set_default(PAR_exp,'tasks',PAR.tasks)  ; 
  PAR.Source_organisms = set_default(PAR_exp,'Source_organisms',PAR.Source_organisms)  ; 
end

if isempty(PAR.organism.name),
  [organism, label_source] = dummy_organism(PAR.label_source, sig_names);
else
  [organism, label_source] = feval(PAR.organism.name,PAR.label_source,sig_names);
end
PAR.organism = set_default(PAR,'organism',organism)  ;
PAR.label_source = set_default(PAR,'label_source',label_source)  ;

%-----------------------------
%%% prepare all signals
%-----------------------------

for i=1:length(Signals)
  Signal = feval(Signals{i},PAR.label_source);
  if exist('PAR_exp')
    if isfield(PAR_exp.Signals, sig_names{i}) && isfield(PAR_exp.Signals.(sig_names{i}),'method')
      me = PAR_exp.Signals.(sig_names{i}).method;
      PAR_exp.Signals.(sig_names{i}) = rmfield(PAR_exp.Signals.(sig_names{i}),'method');
    end
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
  end
  Signal.method = feval(Methods{i},Signal);
  if exist('me')
    PAR_exp.Signals.(sig_names{i}).method = me;
    Signal = set_default(PAR_exp.Signals,sig_names{i},Signal)  ;
    clear me
  end
  % Signal = Signal.domain_adapt.transfer_fct(Signal);
  PAR.Signals.(sig_names{i}) = Signal;
end

%-----------------------------
%%% prepare all contents
%-----------------------------
for i=1:length(content_names)
  Content = feval(content_names{i});
  if exist('PAR_exp')
    if isfield(PAR_exp.Contents, content_names{i}) && isfield(PAR_exp.Contents.(content_names{i}),'method')
      me = PAR_exp.Contents.(content_names{i}).method;
      PAR_exp.Contents.(content_names{i}) = rmfield(PAR_exp.Contents.(content_names{i}),'method');
    end
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
  end
  Content.method = feval(Methodc,Content);
  if exist('me')
    PAR_exp.Contents.(content_names{i}).method = me;
    Content = set_default(PAR_exp.Contents,content_names{i},Content)  ;
    clear me
  end
  % Content = Content.domain_adapt.transfer_fct(Content);
  PAR.Contents.(content_names{i}) = Content;
end

%-----------------------------
%%% prepare LSL
%-----------------------------

LSL = lsl_template; 
% LSL = LSL.domain_adapt.transfer_fct(LSL);
if exist('PAR_exp')
  LSL = set_default(PAR_exp,'LSL',LSL)  ; 
end
PAR.LSL=LSL;

%-----------------------------
%%% prepare filenames
%-----------------------------
FN = init_FN;
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end

FN = create_filenames(FN,PAR.organism,PAR.Signals,PAR.Contents,verb);
if  exist('PAR_exp')
  FN = set_default(PAR_exp,'FN',FN)  ;
end
PAR.FN=FN;

%-----------------------------
%%% Add polya consensus if file available
%-----------------------------

if fexist(PAR.FN.input_sig.polya.fn_polya)
  load(PAR.FN.input_sig.polya.fn_polya,'consensus')
  PAR.Signals.polya.consensus = consensus;
end

%PAR.FN.source_directory = '~/svn/projects/genefinding/';
%[tmp,PAR.FN.svn_info]=unix('svn info');

PAR.FN.svn_info = '' ;

fprintf('loading model from %s\n', PAR.LSL.method.model_fct)
PAR.model = feval(PAR.LSL.method.model_fct,PAR.organism,PAR.LSL, ...
                  PAR.Signals);


return

function create_directories(FN,organism,verb,signal)

% create_directories(PAR)
%   
% function creates all necessary directories for a gene finding
% experiment as defined by PAR.
  
  
if nargin<3
  verb=0;
end

if  nargin>3
  signalnames{1} = signal;
else
  signalnames = fieldnames(FN.input_sig);
end


%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_dir = sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end
nice_mkdir(input_dir);


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);
output_dir = sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
nice_mkdir(output_dir);

nice_mkdir(FN.exp_dir);


%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC Dirs
%%%%%%%%%%%%%%%


for s=1:length(signalnames)
  if ~isfield( FN.input_sig, signalnames{s})
    continue
  end
  signals = FN.input_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  signalnames = fieldnames(FN.output_sig);
end
  
for s=1:length(signalnames)
  if ~isfield( FN.output_sig, signalnames{s})
    continue
  end
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    name = signals.(signal_fns{i}) ;
    if isempty(strmatch(FN.output_dir,name)) &verb
      fprintf(['incomplete signal output file name, signal output directories not generated; '...
               'run config for %s\n'],signalnames{s})
      break
    end
    iii = max(strfind(name,'/'));
    if ~isempty(iii)
      nice_mkdir(name(1:iii));
    end 
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC Dirs
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  if nargin<4
    signalnames = fieldnames(FN.input_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.input_cont, signalnames{s})
      continue
    end
    signals = FN.input_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% CONTENT SPECIFIC OUTPUT DIRs 
%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  if nargin<4
    signalnames = fieldnames(FN.output_cont);
  end
  for s=1:length(signalnames)
    if ~isfield( FN.output_cont, signalnames{s})
      continue
    end
    signals = FN.output_cont.(signalnames{s});
    signal_fns = fieldnames(signals);
    for i=1:length(signal_fns)
      name = signals.(signal_fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf(['incomplete signal output file name, signal output directories not generated; '...
                 'run create_filenames first for\n'])
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%
%% LSL SPECIFIC
%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
  
  if isfield(FN, 'output_lsl')
    fns = fieldnames(FN.output_lsl);
    for i=1:length(fns)
      name = FN.output_lsl.(fns{i}) ;
      if isempty(strmatch(FN.output_dir,name)) &verb
        fprintf('incomplete signal output file name, signal output directories not generated;')
        break
      end
      iii = max(strfind(name,'/'));
      if ~isempty(iii)
        nice_mkdir(name(1:iii));
      end 
    end
  end
  if isfield(FN, 'input_lsl')
    fns = fieldnames(FN.input_lsl);
    for i=1:length(fns)
      if ~iscell(FN.input_lsl.(fns{i}))
        name = FN.input_lsl.(fns{i}) ;
        if isempty(strmatch(FN.input_dir,name)) &verb
          fprintf('incomplete signal input file name, signal input directories not generated;')
          break
        end
        iii = max(strfind(name,'/'));
        if ~isempty(iii)
          nice_mkdir(name(1:iii));
        end 
      else
        for j=1:length(FN.input_lsl.(fns{i}))
          name = FN.input_lsl.(fns{i}){j} ;
          if isempty(strmatch(FN.input_dir,name)) &verb
            fprintf('incomplete signal input file name, signal input directories not generated;')
            break
          end
          iii = max(strfind(name,'/'));
          if ~isempty(iii)
            nice_mkdir(name(1:iii));
          end 
        end
      end
    end
  end
  
  fn = sprintf('%s/%s/current_predictions', FN.input_dir, organism.name);
  nice_mkdir(fn);

end
return

function FN = create_filenames(FN,organism,Signals,Contents,verb)

%
% FN requires fields :
% 	- input_dirC_
% 	- output_dir
%	- exp_name
%	- FN.fn_genes_train
%	- date
%	- user
%	- organism
%

if nargin<2
  verb=0;
end

if ~isfield(FN,'dir_name_ext')
  dir_name_ext = '';
else
  dir_name_ext = [ '_', FN.dir_name_ext ];
end;

if isempty(FN.dir_name)
  FN.dir_name ='';
end
if isempty(FN.exp_name)
  FN.exp_name = '';
end
if isempty(FN.lsl_exp_name)
  FN.lsl_exp_name = '';
end

assert(isfield(FN,'exp_name'))
assert(isfield(FN,'dir_name'))

%%%%%%%%%%%%%%%
%%% GENERAL INPUT GENOME FILES 
%%%%%%%%%%%%%%%
exp_base_in = sprintf('%s/%s/%s/%s',FN.genome_dir,organism.name,organism.release,organism.genebuilt);

input_fns = fieldnames(FN.genome);
for i=1:length(input_fns)
  old_name = getfield(FN.genome,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.genome_dir,old_name{j}))
        new_name{j}  = sprintf('%s/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  elseif length(strfind(old_name,'/'))==1
    exp_base_in_ = sprintf('%s/%s/%s',FN.genome_dir,organism.name,organism.release);
    new_name  = sprintf('%s/%s',exp_base_in_,old_name );
  elseif isempty(strmatch(FN.genome_dir,old_name)) 
    new_name  = sprintf('%s/%s',exp_base_in,old_name );
  else
    new_name  = old_name ;
  end
  FN.genome = setfield(FN.genome,input_fns{i},new_name);
end  
if verb
  fprintf(1, 'genebuild from dir: \n%s\n\n', exp_base_in)
end

%%%%%%%%%%%%%%%
%%% GENERAL INPUT FILES 
%%%%%%%%%%%%%%%

exp_base_in = sprintf('%s/%s/%s',FN.input_dir,organism.name,FN.dir_name);

input_fns = fieldnames(FN.input);
for i=1:length(input_fns)
  old_name = getfield(FN.input,input_fns{i});
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.input_dir,old_name{j}))
        new_name{j}  = sprintf('%s/orig_data/%s',exp_base_in,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/orig_data/%s',exp_base_in,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.input = setfield(FN.input,input_fns{i},new_name);
end  
input_dir=sprintf('%s/orig_data/',exp_base_in);
if verb
  fprintf(1, 'dir for original data: \n%s\n\n', input_dir)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%
%% OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%
exp_base_out = sprintf('%s/%s/%s',FN.output_dir,organism.name,FN.dir_name);

output_fns = fieldnames(FN.output);
for i=1:length(output_fns)
  old_name = getfield(FN.output,output_fns{i});
  if isempty(old_name)
    old_name = '';
  end 
  if iscell(old_name)
    new_name ={};
    for j=1:length(old_name)
      if isempty(strmatch(FN.output_dir,old_name{j})) 
        new_name{j}  = sprintf('%s/genome_data/%s',exp_base_out,old_name{j} );
      else
        new_name{j}  = old_name{j} ;
      end
    end
  else
    if  isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/genome_data/%s',exp_base_out,old_name );
    else
      new_name  = old_name ;
    end
  end
  FN.output = setfield(FN.output,output_fns{i},new_name);
end  

output_dir=sprintf('%s/genome_data/',exp_base_out);
if verb
  fprintf(1, 'dir for genes, split and regions: \n%s\n\n', output_dir)
end
if ~isfield( FN,'exp_dir')||isempty(FN.exp_dir)|| isempty(strmatch(FN.input_dir,FN.exp_dir)) 
  FN.exp_dir = sprintf('%s/%s',exp_base_out,FN.exp_name);
end

%%%%%%%%%%%%%%%
%%% SIGNAL SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

signalnames = fieldnames(FN.input_sig);
for s=1:length(signalnames)
  signals = getfield(FN.input_sig,signalnames{s});
  signal_fns = fieldnames(signals);
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    if isempty(old_name)
       old_name = '';
    end 
    if isempty(strmatch(FN.input_dir,old_name)) 
      new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
      % new_name  = sprintf('%s/%s/sensors/%s%s/%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name );
    else
      new_name  = old_name ; 
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end 
  FN.input_sig = setfield(FN.input_sig,signalnames{s},signals);
  
  % sig_input = sprintf('%s/%s/sensors/%s%s',exp_base_in,FN.exp_name,signalnames{s},dir_name_ext,old_name);
  if verb
    fprintf(1, 'dir for label_files and examples: \n%s\n\n', new_name)
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SIGNAL SPECIFIC OUTPUT DIRs AND FNs  
%%%%%%%%%%%%%%%%%%%%%%%%%



%output_fns = fieldnames(FN.output);
%for i=1:length(output_fns)
%  old_name = getfield(FN.output,output_fns{i});
%  new_name  = sprintf('%s/%s_%s/%s',FN.output_dir,FN.exp_name,...
%                      organism.name,old_name );
%  FN.output = setfield(FN.output,output_fns{i},new_name);
%end  

signalnames = fieldnames(FN.output_sig);
for s=1:length(signalnames)
  signals = FN.output_sig.(signalnames{s});
  signal_fns = fieldnames(signals);
  method = Signals.(signalnames{s}).method;
  for i=1:length(signal_fns)
    old_name = getfield(signals,signal_fns{i}) ;
    dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
    % dir_name = sprintf('%s/%s/sensors/%s%s',exp_base_out,FN.exp_name,signalnames{s},dir_name_ext);
    if isfield(FN, 'custom_output')
      dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
    else
      dir_name = sprintf('%s/output_%s',dir_name,method.name);
      if isequal(method.name, 'SVM')
        if length(method.kernels)==1&&~isempty(method.kernels{1})
          if isfield(method.kernels{1},'name')
            kernel_names = method.kernels{1}.name; 
          else
            kernel_names = method.kernels{1}.kernel_name;
          end
        else
          kernel_names = 'COMBINED';
        end
        dir_name = sprintf('%s%s',dir_name,kernel_names);
      end
    end
    if isempty(strmatch(FN.output_dir,old_name)) 
      new_name  = sprintf('%s/%s',dir_name, old_name );
    else
      new_name = old_name;
    end
    signals = setfield(signals,signal_fns{i},new_name);
  end
  FN.output_sig = setfield( FN.output_sig, signalnames{s}, signals );
  if verb
    fprintf(1, 'dir for training and test data: \n%s\n\n', dir_name) 
  end
end


%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC INPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'input_cont')
  cont_names = fieldnames(FN.input_cont);
  for s=1:length(cont_names)
    content = getfield(FN.input_cont,cont_names{s});
    cont_fns = fieldnames(content);
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      if isempty(strmatch(FN.input_dir,old_name)) 
        new_name  = sprintf('%s/%s/%s/%s',exp_base_in,FN.exp_name,dir_name_ext,old_name );
        % new_name  = sprintf('%s/%s/contents/%s%s/%s',exp_base_in,FN.exp_name, cont_names{s},dir_name_ext,old_name );
      else
        new_name  = old_name ; 
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.input_cont = setfield(FN.input_cont,cont_names{s},content);
  end

  %cont_input = sprintf('%s/%s/contents/%s',exp_base_in,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific input: \n%s\n\n', cont_input)
  end
end

%%%%%%%%%%%%%%%
%%% CONTENT SPECIFIC OUTPUT FILES 
%%%%%%%%%%%%%%%

if isfield(FN, 'output_cont')
  cont_names = fieldnames(FN.output_cont);
  for s=1:length(cont_names)
    content = getfield(FN.output_cont,cont_names{s});
    cont_fns = fieldnames(content);
    method = Contents.(cont_names{s}).method;
    for i=1:length(cont_fns)
      old_name = getfield(content,cont_fns{i}) ;
      dir_name = sprintf('%s/%s/%s',exp_base_out,FN.exp_name,dir_name_ext);
      if isfield(FN, 'custom_output')
        dir_name = sprintf('%s/%s',dir_name,FN.custom_output);
      else
        dir_name = sprintf('%s/output_%s',dir_name,method.name);
        if isequal(method.name, 'SVM')
          if length(method.kernels)==1&&~isempty(method.kernels{1})
            if isfield(method.kernels{1},'name')
              kernel_names = method.kernels{1}.name; 
            else
              kernel_names = method.kernels{1}.kernel_name;
            end
          else
            kernel_names = 'COMBINED';
          end
          dir_name = sprintf('%s%s',dir_name,kernel_names);
        end
      end
      if isempty(strmatch(FN.output_dir,old_name)) 
        new_name  = sprintf('%s/%s',dir_name, old_name );
      else
        new_name = old_name;
      end
      content = setfield(content,cont_fns{i},new_name);
    end 
    FN.output_cont = setfield(FN.output_cont, cont_names{s},content);
  end

  %cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
  if verb
    fprintf(1, 'dir for content specific output: \n%s\n\n', cont_output)
  end
end

%%%%%%%%%%%%%%%
%%% LSL STUFF
%%%%%%%%%%%%%%%

if isfield(FN, 'input_lsl')
  fns = fieldnames(FN.input_lsl);
  FN.input_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			     exp_base_out,organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    if ~iscell(FN.input_lsl.(fns{i}))
      old_name = FN.input_lsl.(fns{i}) ;
      if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
            &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
        new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
      else
        new_name  = old_name ; 
      end
      FN.input_lsl.(fns{i}) = new_name;
    else
      for j=1:length(FN.input_lsl.(fns{i}))
        old_name = FN.input_lsl.(fns{i}){j} ;
        if isempty(strmatch(FN.input_lsl.dir,old_name))&~isequal(fns{i},'fn_old_training_blocks')...
              &~isequal(fns{i},'fn_old_test_blocks')&~isequal(fns{i},'fn_old_split')
          new_name  = sprintf('%s/%s',FN.input_lsl.dir,old_name);
        else
          new_name  = old_name ; 
        end
        FN.input_lsl.(fns{i}){j} = new_name;
      end    
    end
  end
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific input: \n%s\n\n', cont_output)
end

if isfield(FN, 'output_lsl')
  fns = fieldnames(FN.output_lsl);
  FN.output_lsl.dir = sprintf('%s/%s/%s/%s/lsl/%s/', ...
			      exp_base_out, organism.name,FN.dir_name,FN.exp_name,FN.lsl_exp_name ); 
  for i=1:length(fns)
    old_name = FN.output_lsl.(fns{i}) ;
    if isempty(strmatch(FN.output_lsl.dir,old_name))
      new_name  = sprintf('%s/%s',FN.output_lsl.dir,old_name);
    else
      new_name  = old_name ; 
    end
    FN.output_lsl.(fns{i}) = new_name;
  end 
end
%cont_output = sprintf('%s/%s/contents/%s',exp_base_out,FN.exp_name,signal_dir_name);
if verb
  fprintf(1, 'dir for lsl specific output: \n%s\n\n', cont_output)
end


return

function regions=init_regions(fn_genome_config, organism)
% regions=init_regions(fn_genome_config,organism)

  
regions(1).chr = [];
regions(1).chr_num = [];
regions(1).strand = [];
regions(1).start = [];
regions(1).stop = [];
regions(1).offset =[];
regions(1).id = [];
regions(1).organism = [];
regions(1).config = [];

if ~exist('fn_genome_config')
  return ;
else
  regions = regions([]) ;
  genome_config = init_genome(fn_genome_config);
  strands = '+-';
  for c=1:length(genome_config.contig_names)
    %+strand
    d = dir(genome_config.flat_fnames{c}) ;
    for s=1:length(strands)
      region = [];
      region.chr=genome_config.contig_names{c};
      region.chr_num = c;
      region.strand=strands(s);
      region.start = 1;
      region.stop = d.bytes ;
      switch strands(s)
       case '+',
        region.offset      = region.start - 1;
       case '-',
        region.offset      = region.stop + 1;
      end
      region.id = (c-1)*2+s;
      region.organism = [];
      if exist('organism')
        region.organism = organism;
      end
      region.config = fn_genome_config;
      regions(region.id) = region;
    end
  end
end


return

function blocks = make_blocks_from_genes(genes, fn_config, margin, merge_blocks, min_dist)
% blocks = make_blocks_from_genes(genes, fn_config, margin=2000, merge_blocks=1, min_dist=100)

%fn_config = '/fml/ag-raetsch/share/databases/genomes//F_graminearum/F_graminearumOct2007/genebuild_noproteins/genome.config';
genome_info = init_genome(fn_config);

if ~exist('margin', 'var')
  margin = 2000;
end ;
if ~exist('merge_blocks', 'var')
  merge_blocks = 1 ;
end ;
if ~exist('min_dist', 'var')
  min_dist = 100;
end ;

%blocks = genes;
blocks = struct('strand',{genes.strand},'chr',{genes.chr},'chr_num',{genes.chr_num},'start',{genes.start},...
		'stop',{genes.stop},'gene_ids',{genes.id},'paralogs',{genes.paralogs},'is_alt',{genes.is_alt},...
		'gene_complete',{genes.is_complete}, 'config' ,fn_config );

for i=1:length(blocks)
  blocks(i).chr_num = strmatch(blocks(i).chr, genome_info.contig_names, 'exact') ;
  assert(length(blocks(i).chr_num)==1) ;
end ;

% ADJUST BOUNDARIES
for j=1:length(blocks)
  margin_left = margin ;
  margin_right = margin ;

  if isfield(genes(j), 'next_gene_left') && ~isempty(genes(j).next_gene_left),
    if margin_left > round((genes(j).start-genes(j).next_gene_left)/2),
      margin_left = round((genes(j).start-genes(j).next_gene_left)/2) ;
    end ;
  end ;
  if isfield(genes(j), 'next_n_left') && ~isempty(genes(j).next_n_left),
    if margin_left > genes(j).start-genes(j).next_n_left,
      margin_left = genes(j).start-genes(j).next_n_left ;
    end ;
  end ;
  if isfield(genes(j), 'next_gene_right') && ~isempty(genes(j).next_gene_right),
    if margin_right > round((genes(j).next_gene_right-genes(j).stop)/2),
      margin_right = round((genes(j).next_gene_right-genes(j).stop)/2) ;
    end ;
  end ;
  if isfield(genes(j), 'next_n_right') && ~isempty(genes(j).next_n_right),
    if margin_right > genes(j).next_n_right-genes(j).stop,
      margin_right = genes(j).next_n_right-genes(j).stop ;
    end ;
  end ;

  blocks(j).gene_ids = genes(j).id;
  blocks(j).paralogs = [];
  blocks(j).gene_status = max(genes(j).transcript_status); 
  blocks(j).id = j; 
  if blocks(j).start<1
    if keyboard_allowed()
      keyboard
    end
  end
  flat_fn = genome_info.flat_fnames{blocks(j).chr_num};
  d = dir(flat_fn);
  len = d.bytes;
 
  if blocks(j).stop+margin_right<=len
    blocks(j).stop=blocks(j).stop+margin_right;
  else
    assert(blocks(j).stop<=len)
    blocks(j).stop=len;
  end
  if blocks(j).start-margin_left>1
    blocks(j).start=blocks(j).start-margin_left;
  else
    assert(blocks(j).start>=1)
    blocks(j).start=1;
  end
end

if merge_blocks,
  % MERGE BLOCKS
  len = length(blocks)
  for s='+-'
    strands = [blocks.strand];
    str_blocks = blocks(strands==s);
    x = unique({str_blocks.chr});
    for c = 1:length(x)
      c_idx = find(strcmp({str_blocks.chr},x(c)));
      contig_blocks = str_blocks(c_idx);
      %keyboard
      starts = [contig_blocks.start];
      stops  = [contig_blocks.stop];
      [starts sort_idx] = sort(starts);
      stops = stops(sort_idx);
      contig_blocks = contig_blocks(sort_idx);
      dist = starts(2:end)-stops(1:end-1);
      merge_idx = find(dist<min_dist);
      
      rm_idx = [];
      for j=merge_idx;
        contig_blocks(j+1)=merge(contig_blocks(j),contig_blocks(j+1));
        %keyboard
        rm_idx = [rm_idx j];
      end 
      contig_blocks(rm_idx) = []; 
      blocks(c_idx) = [];
      blocks = [blocks contig_blocks];
    end
  end
  len = length(blocks)
end ;


for j=1:length(blocks)
  blocks(j).id = j;
  off = [blocks(j).start-1 blocks(j).stop+1];
  blocks(j).offset = off((blocks(j).strand=='-')+1); 
end

return

function block = merge(block1,block2)
  block = block1;
  block.gene_ids = [block1.gene_ids block2.gene_ids];
  block.paralogs = unique([block1.paralogs block2.paralogs]);
  block.gene_status = [block1.gene_status block2.gene_status];
  block.start = min([block1.start block2.start]);
  block.stop = max([block1.stop block2.stop]);
return



return

function SETs = make_partitions(SETs)
 

if SETs.num_partitions==1
  return
end
  
SETs.partitions = eye(SETs.num_partitions,SETs.num_partitions)+1;  
for i = 1:2:SETs.num_partitions-1
  SETs.partitions(i:i+1,i:i+1) = SETs.partitions(i:i+1,i:i+1)+1;
end
if mod(SETs.num_partitions,2)
  SETs.partitions(end,end-1:end) =  SETs.partitions(end,end-1:end)+1;
end
if SETs.num_splits~=SETs.num_partitions
  assert(mod(SETs.num_splits,SETs.num_partitions)==0)
  SETs.partitions = repmat(SETs.partitions,1,SETs.num_splits/SETs.num_partitions);
end

if ~isfield(SETs,'partition_train_on') 
  [tmp,idx] = unique(SETs.partitions==1,'rows');
  SETs.partition_train_on = zeros(1,SETs.num_partitions);
  SETs.partition_train_on(idx) = 1;
end



SETs.info.train = 1;
SETs.info.eval = 2;
SETs.info.plif = 2;
SETs.info.pred = 3;

for i=1:SETs.num_partitions
  assert(isequal(unique(SETs.partitions(i,:)),unique([SETs.info.train ...
                      SETs.info.eval SETs.info.plif SETs.info.pred ])))
end

assert(all(sum(SETs.partitions==SETs.info.pred)==1))
assert(length(unique(sum(SETs.partitions'==SETs.info.train)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.eval)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.plif)))==1)
assert(length(unique(sum(SETs.partitions'==SETs.info.pred)))==1)




return


function [ ok ] = nice_mkdir( dir );
if ~isdir(dir)
  ok = ( 0 == unix( [ 'mkdir -p ', dir ] ) );
  ok = ok & ( 0 == unix( [ 'chmod g+t ', dir ] ) );
end


return

function [trivial_regions,blocks,training,test] = prepare_regions(PAR);
%  [trivial_regions,training_split,training_blocks,test_split,test_blocks] = prepare_regions(PAR,genes);


% generate trivial regions
if fexist(PAR.FN.output.fn_trivial_regions)
  fprintf(1,'loading trivial regions from file %s\n',PAR.FN.output.fn_trivial_regions)
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  assert(isequal(trivial_regions(1).config,PAR.FN.input.fn_genome_config));
else
  fprintf(1,'initializing trivial regions\n')
  trivial_regions = init_regions(PAR);
  save(PAR.FN.output.fn_trivial_regions,'trivial_regions');
end

% generate training and test blocks and splits, 
% considering training and test region
if fexist(PAR.FN.output.fn_training_blocks)& fexist(PAR.FN.output.fn_test_blocks)
  Old = load(PAR.FN.output.fn_training_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_training_blocks)
    training = load(PAR.FN.output.fn_training_blocks)
    assert(length(fieldnames(training.split))==PAR.SETs.num_splits)
  end
  Old = load(PAR.FN.output.fn_test_blocks, 'PAR');
  assert(isequal(Old.PAR.FN.output.fn_trivial_regions,PAR.FN.output.fn_trivial_regions));
  assert(isequal(Old.PAR.FN.input.fn_genome_config,PAR.FN.input.fn_genome_config));
  diff_fields = compare_PAR(PAR,Old.PAR) ;
  if ismember('regions',diff_fields)
    warning('regions exist, Parameters are different ')
    %fprintf('regions exist, Parameters are different ')
  else
    fprintf(1,'loading split and blocks from file %s \n',PAR.FN.output.fn_test_blocks)
    test = load(PAR.FN.output.fn_test_blocks)
    assert(length(fieldnames(test.split))==PAR.SETs.num_splits)
  end
else
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  [training, test] = assign_subsets(trivial_regions,PAR.FN.input.fn_trainings_regions,PAR.FN.input.fn_test_regions,genes,PAR.SETs,PAR.regions,PAR.tasks)
end

if 1
  if isfield(PAR.FN.output,'fn_genes_merged') && fexist(PAR.FN.output.fn_genes_merged)
    load(PAR.FN.output.fn_genes_merged,'genes');
  else
    genes=[];
  end
  load(PAR.FN.input.fn_trainings_regions,'training_regions');
  load(PAR.FN.input.fn_test_regions,'test_regions');
  load(PAR.FN.output.fn_trivial_regions,'trivial_regions');
  if  ~exist('training')
    training = load(PAR.FN.output.fn_training_blocks);
  end
  if  ~exist('test')
    test = load(PAR.FN.output.fn_test_blocks);
  end
  % load(PAR.FN.output.fn_blocks,'blocks');
  blocks =test.blocks;
  check_and_plot_splits(trivial_regions(1:min(10,length(trivial_regions))),training_regions,test_regions,genes,blocks,training, test)
end
  

return

function [genes, FAULTS_id,FAULTS_name] = process_gff3_genes(genes, genome_info, name_of_CDS,model) 
% genes = process_gff3_genes(genes, genome_info, name_of_CDS)  


if nargin<3
  name_of_CDS = 'CDS';
end

if nargin<4
  m.exon = 0;
  m.transexon = 1;
  m.utr5exon = 2;
  m.utr3exon = 4; 
  m.cds_exon = 100;
  if any(ismember(name_of_CDS, {'CDS','coding_exon'}))
    m.cds_exon = 3;
  end
  if any(ismember(name_of_CDS, {'exon'}))
    m.cds_exon = [m.cds_exon 0];
  end
else
  if isfield(model.segments, 'exon')
    m.exon = model.segments.exon;
  else
    m.exon = -1e7;
  end
  if isfield(model.segments, 'trans_exon')
    m.transexon = model.segments.trans_exon;
  else
    m.transexon = -1e7; 
  end
  m.utr5exon = model.segments.utr5exon;
  m.utr3exon = model.segments.utr3exon;
  m.cds_exon = model.segments.cds_exon;
end

gff_contig_names = unique({genes.chr}) ;
unannotated_contigs = setdiff(genome_info.contig_names, gff_contig_names) ;
if ~isempty(unannotated_contigs),
  fprintf(1, '\nThe genome information object contains %i/%i contigs not annotated in the GFF3 file.\n', length(unannotated_contigs), length(genome_info.contig_names)) ;
  if length(unannotated_contigs)<=10,
    for i=1:length(unannotated_contigs), 
      fprintf(1, '  %s\n', unannotated_contigs{i}) ;
    end;
  end ;
end ;
unknown_contigs = setdiff(gff_contig_names, genome_info.contig_names) ;
if ~isempty(unknown_contigs),
  fprintf(2, '\nThe GFF3 file contains the following %i contigs not present in the genome information object:\n', length(unknown_contigs)) ;
  for i=1:length(unknown_contigs), 
    fprintf(2, '  %s\n', unknown_contigs{i}) ;
  end;
  error('Cannot continue.') ;
end ;


FAULTS_name.exons.overlapping = {};
FAULTS_name.exons.unknown_type = {};
FAULTS_name.exons.no_transcripts = [];
FAULTS_name.exons.no_exons = [];
FAULTS_name.exons.TAGT_cases = [];

FAULTS_name.cdsUTR5.ok = {};
FAULTS_name.cdsUTR5.TISdiff = {};
FAULTS_name.cdsUTR5.mult_overlap = {}; 
FAULTS_name.cdsUTR5.gap = {};
FAULTS_name.cdsUTR5.TSSdiff = {};
FAULTS_name.cdsUTR5.no_utr5 = {};

FAULTS_name.cdsUTR3.ok = {};
FAULTS_name.cdsUTR3.STOPdiff = {};
FAULTS_name.cdsUTR3.mult_overlap = {}; 
FAULTS_name.cdsUTR3.gap = {};
FAULTS_name.cdsUTR3.CLEAVEdiff = {};
FAULTS_name.cdsUTR3.no_utr3 = {};

FAULTS_name.exonUTR5.gap = {};
FAULTS_name.exonUTR5.TSSdiff = {};
FAULTS_name.exonUTR3.CLEAVEdiff = {};
FAULTS_name.exonUTR3.gap = {};
FAULTS_name.noCDS.noCDS = {};



FAULTS_id.exons.overlapping = [];
FAULTS_id.exons.unknown_type = [];
FAULTS_id.exons.no_transcripts = [];
FAULTS_id.exons.no_exons = [];
FAULTS_id.exons.TAGT_cases = [];

FAULTS_id.cdsUTR5.ok = [];
FAULTS_id.cdsUTR5.TISdiff = [];
FAULTS_id.cdsUTR5.mult_overlap = []; 
FAULTS_id.cdsUTR5.gap = [];
FAULTS_id.cdsUTR5.TSSdiff = [];
FAULTS_id.cdsUTR5.no_utr5 = [];

FAULTS_id.cdsUTR3.ok = [];
FAULTS_id.cdsUTR3.STOPdiff = [];
FAULTS_id.cdsUTR3.mult_overlap = []; 
FAULTS_id.cdsUTR3.gap = [];
FAULTS_id.cdsUTR3.CLEAVEdiff = [];
FAULTS_id.cdsUTR3.no_utr3 = [];

FAULTS_id.exonUTR5.gap = [];
FAULTS_id.exonUTR5.TSSdiff = [];
FAULTS_id.exonUTR3.CLEAVEdiff = [];
FAULTS_id.exonUTR3.gap = [];
FAULTS_id.noCDS.noCDS = [];

fprintf('processing genes\n')
for i=1:length(genes)
  if (mod(i,1000)==0) || i==length(genes), 
    fprintf('%i/%i\r',i,length(genes));
  end
  genes(i).id = i;
  if exist('genome_info','var') && ~isempty(genome_info) && (~isfield(genes,'chr_num') || isempty(genes(i).chr_num))
    genes(i).chr_num= strmatch(upper(genes(i).chr), upper(genome_info.contig_names), 'exact');
    if isempty(genes(i).chr_num)
      error('sequence ID in gff file (%s) does not match any sequence IDs in fasta file (%s)', upper(genes(i).chr), upper(genome_info.contig_names));
    end
  end
  genes(i).tis = {} ;
  genes(i).cdsStop = {} ;
  genes(i).tss = {} ;
  genes(i).cleave = {} ;
  if ~isfield(genes(i), 'cds_exons') || isempty(genes(i).cds_exons)
    genes(i).cds_exons = {} ;
  end ;
  if ~isfield(genes(i), 'utr5_exons') || isempty(genes(i).utr5_exons)
    genes(i).utr5_exons = {} ;
  end ;
  if ~isfield(genes(i), 'utr3_exons') || isempty(genes(i).utr3_exons)
    genes(i).utr3_exons = {} ;
  end ;
  gene = genes(i) ;
  if isempty(gene.exons) || (length(gene.exons)==1&&isempty(gene.exons{1}))
    FAULTS_id.exons.no_exons(end+1)=i;
    FAULTS_name.exons.no_exons{end+1}=gene.name;
    continue
  end
  if isempty(gene.transcripts) 
    FAULTS_id.exons.no_transcripts(end+1) = i;
    FAULTS_name.exons.no_transcripts{end+1} = gene.name;
    continue
  end
  
  for j=1:length(gene.transcripts)
    gene.exons_confirmed{j} =[]; 
    exons = gene.exons{j} ;
    if size(exons,2)<3, 
      %warning('no exon meta info, identifying as exons') ;
      exons(:,3)=m.exon ; 
    end ;
    [exons(:,1),idx] = sort(exons(:,1)) ;
    exons(:,2:end) = exons(idx,2:end) ;
    %assert(all((exons(:,2)-exons(:,1))>=0)) ;
    if ~all((exons(:,2)-exons(:,1))>=0),
      warning('exon_end<exon_start; gene invalid') ;
      continue ;
    end ;
      
    idx_trans = find(exons(:,3)==m.transexon); 
    if ~isempty(idx_trans)
      exons(idx_trans,:) = [];
      warning('transexon removed')
    end
    %remove redundant information 
    if all(ismember([m.exon,m.cds_exon],unique(exons(:,3))))
      idx_cds = find(exons(:,3)==m.cds_exon);
      idx_exon = find(exons(:,3)==m.exon);
      [tmp,tmp,idx2] = intersect(exons(idx_cds,1:2),exons(idx_exon,1:2),'rows');
      exons(idx_exon(idx2),:)=[];
    end
    %% overlapping exons are merged
    if ~all((exons(2:end,1)-exons(1:end-1,2))>=0)
      idx = find(exons(2:end,1)-exons(1:end-1,2)<0) ;
      if (exons(idx,3)==exons(idx+1,3))
        FAULTS_id.exons.overlapping(end+1) = i;
        FAULTS_name.exons.overlapping{end+1} = gene.name;
        % warning('overlapping exons found within one transcript\n')  
	if keyboard_allowed(),
          keyboard
        end ;
	exons(idx,2) =  exons(idx+1,2);
        exons(idx+1,:) =[];
        if ~all((exons(:,2)-exons(:,1))>=0)
	  error('corrupted exons found: length of exon is <= zero');
	end
	if ~all((exons(2:end,1)-exons(1:end-1,2))>=0)
	  error('overlapping exons found within a single transcript');
	end
      end
    end	
    
    transcript = [exons(1) exons(2)];
    
    all_exons = exons;
    cds_exons = exons(find(ismember(exons(:,3),m.cds_exon)),1:2) ;
    utr5_exons = exons(find(exons(:,3)==m.utr5exon), 1:2) ;
    utr3_exons = exons(find(exons(:,3)==m.utr3exon), 1:2) ;
    other = exons(find(~ismember(exons(:,3),[m.cds_exon m.utr5exon m.utr3exon m.exon])),:) ;
    exons = exons(find(exons(:,3)==m.exon),1:2) ;
    assert(size(all_exons,1)==size(cds_exons,1)+size(utr5_exons,1)+size(utr3_exons,1)+size(exons,1)+size(other,1))
    
    if size(other,1)>0
      FAULTS_name.exons.unknown_type{end+1} = gene.name;
      FAULTS_id.exons.unknown_type(end+1) = i;  
      warning('unknown childs ignored')
    end
    tis = [];
    Stop = [];
    tss = [];
    cleave = [];
    if gene.strand=='+',
      
      if ~isempty(cds_exons)  
        tis = cds_exons(1,1) ;
        Stop = cds_exons(end,2) ;
        if ~isempty(utr5_exons)  %% 
          if utr5_exons(end,2)==cds_exons(1,1)
            %% all good
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR5.TISdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TISdiff(end+1) = i; 
            % warning(sprintf('last UTR-5 exon end (%i) is not equal to the start of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr5_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr5_exons = [] ;
          end
	elseif ~isempty(exons) && any(exons(:,1)<tis) 
          idx = find(exons(:,1)<tis);
          utr5_exons = exons(idx,:);
          if sum(utr5_exons(:,2)>tis)>1
            FAULTS_name.cdsUTR5.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps start of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr5_exons = [];
          elseif ~any(utr5_exons(:,2)>tis)
            utr5_exons(end,2) = cds_exons(1,1);
            FAULTS_name.cdsUTR5.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.gap(end+1) = i; 
            % warning(sprintf('extending 5'' UTR to start of CDS exon\n', gene.name, gene.chr));
          elseif utr5_exons(end,2)~=cds_exons(1,2)
            FAULTS_name.cdsUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TSSdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr5_exons(end,2) = cds_exons(1,1);
          else
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
            utr5_exons(end,2) = cds_exons(1,1);
          end
        else
          FAULTS_name.cdsUTR5.no_utr5{end+1} = gene.name;
          FAULTS_id.cdsUTR5.no_utr5(end+1) = i; 
          % warning(sprintf('cannot find 5'' UTR \n', gene.name, gene.chr));
        end ;
        
        if ~isempty(utr3_exons)  %% 
          if utr3_exons(1,1)==cds_exons(end,2)
            %% all good
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR3.STOPdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.STOPdiff(end+1) = i; 
            % warning(sprintf('first UTR-3 exon start (%i) is not equal to the end of the first coding exon (%i) (gene %s, contig %s)\nignoring 3''UTR\n', utr3_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr3_exons = [] ;
          end
        elseif ~isempty(exons) && any(exons(:,2)>Stop) 
          idx = find(exons(:,2)>Stop);
          utr3_exons = exons(idx,:);
          if sum(utr3_exons(:,1)<Stop)>1
            FAULTS_name.cdsUTR3.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps end of the first coding exon (gene %s, contig %s) \nignoring 3''UTR \n', gene.name, gene.chr));
            utr3_exons = [];
          elseif ~any(utr3_exons(:,1)<Stop)
            utr3_exons(1,1) = cds_exons(end,2);
            FAULTS_name.cdsUTR3.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.gap(end+1) = i; 
            % warning(sprintf('extending 3'' UTR to end of CDS exon\n', gene.name, gene.chr));
          elseif utr3_exons(1,1)~=cds_exons(end,1)
            FAULTS_name.cdsUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.CLEAVEdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr3_exons(1,1) = cds_exons(end,2);
          else
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
            utr3_exons(1,1) = cds_exons(end,2);
          end
        else
          FAULTS_name.cdsUTR3.no_utr3{end+1} = gene.name;
          FAULTS_id.cdsUTR3.no_utr3(end+1) = i; 
          % warning(sprintf('cannot find 3'' UTR \n', gene.name, gene.chr));
        end ; % isempty(utr3_exons)
        
      % start no_cds_exons given
      elseif ~isempty(utr5_exons) & ~isempty(exons) & ~isempty(utr3_exons)
        % inferring cds_exons
        tis = utr5_exons(end,2) ;
        Stop = utr3_exons(1,1) ;
        idx = find(exons(:,2)>tis&exons(:,1)<Stop);
        cds_exons = exons(idx,:);
        if cds_exons(1,1)<tis 
          if cds_exons(1,1)~= utr5_exons(end,1) ;
            FAULTS_name.exonUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.exonUTR5.TSSdiff(end+1) = i; 
            % warning(['first potential coding exon has different start than ' ...
            %          'last 5''UTR exon']) 
          end
          cds_exons(1,1)=tis;
        else
          FAULTS_name.exonUTR5.gap{end+1} = gene.name;
          FAULTS_id.exonUTR5.gap(end+1) = i; 
          % warning(['first potential coding exon starts downstream of tis']) 
          cds_exons(1,1)=tis;
        end
        if cds_exons(end,2)>Stop 
          if cds_exons(end,2)~= utr3_exons(1,2) ;
            FAULTS_name.exonUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.exonUTR3.CLEAVEdiff(end+1) = i; 
            % warning(['last potential coding exon has different end than ' ...
            %          'first 3''UTR exon']) 
          end
          cds_exons(end,2)=Stop;
        else
          FAULTS_name.exonUTR3.gap{end+1} = gene.name;
          FAULTS_id.exonUTR3.gap(end+1) = i; 
          % warning(['last potential coding exon ends upstream of cdsStop']) 
          cds_exons(end,2)=Stop;
        end 
      else
        tis = [] ;
        Stop = [] ;
        FAULTS_name.noCDS.noCDS{end+1} = gene.name;
        FAULTS_id.noCDS.noCDS(end+1) = i; 
        % warning(sprintf('cannot infer UTRs and coding regions \n', gene.name, gene.chr));
      end
      
      Stop=Stop-3 ;

      if size(utr3_exons,1)>1 && utr3_exons(1,2)-utr3_exons(1,1)<3
        utr3_exons = [utr3_exons(1,1) utr3_exons(end,2)];
        FAULTS_name.exons.TAGT_cases{end+1} = gene.name;
        FAULTS_id.exons.TAGT_cases(end+1) = i;
        % warning('TAGT case: invalid donor!')
      end
      % determine phase of cds exon
      cds_exons(1,3) = 0;
      cds_length = 0;
      for e=2:size(cds_exons,1)
        cds_length = cds_length + cds_exons(e-1,2)-cds_exons(e-1,1);
        phase = mod(cds_length,3);
        switch phase
         case 0
          cds_exons(e,3) = phase;
         case 1
          cds_exons(e,3) = 2;
         case 2
          cds_exons(e,3) = 1;
        end
      end
      exons = utr5_exons;
      if ~isempty(utr5_exons)
        tss =  utr5_exons(1,1);
      end ;
      if ~isempty(cds_exons)
        if ~isempty(exons)
          exons(end,2) = cds_exons(1,2) ;
          exons = [exons; cds_exons(2:end,1:2)] ;
        else
          exons = cds_exons(:,1:2) ;
        end ;
      end
      if ~isempty(utr3_exons)
        cleave =  utr3_exons(end,2);
        if ~isempty(exons)
          exons(end,2) = utr3_exons(1,2) ;
          exons = [exons; utr3_exons(2:end,:)] ;
        else
          exons = utr3_exons(:,1:2) ;
        end ;
      end
      
    else % gene.strand=='-',
      if ~isempty(cds_exons)  
        tis = cds_exons(end,2) ;
        Stop = cds_exons(1,1) ;
        if ~isempty(utr5_exons)  %% 
          if utr5_exons(1,1)==cds_exons(end,2)
            %% all good
            FAULTS_id.cdsUTR5.ok(end+1) = i;
            FAULTS_name.cdsUTR5.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR5.TISdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TISdiff(end+1) = i; 
            % warning(sprintf('last UTR-5 exon end (%i) is not equal to the start of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr5_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr5_exons = [] ;
          end
	elseif ~isempty(exons) && any(exons(:,2)>tis) 
          idx = find(exons(:,2)>tis);
          utr5_exons = exons(idx,:);
          if sum(utr5_exons(:,1)<tis)>1
            FAULTS_name.cdsUTR5.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps start of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr5_exons = [];
          elseif ~any(utr5_exons(:,1)<tis)
            FAULTS_name.cdsUTR5.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR5.gap(end+1) = i; 
            utr5_exons(1,1) = cds_exons(end,2);
            % warning(sprintf('extending 5'' UTR to start of CDS exon\n', gene.name, gene.chr));
          elseif utr5_exons(1,1)~=cds_exons(end,1)
            FAULTS_name.cdsUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR5.TSSdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr5_exons(1,1) = cds_exons(end,2);
          else
            utr5_exons(1,1) = cds_exons(end,2);
          end
        else
          FAULTS_name.cdsUTR5.no_utr5{end+1} = gene.name;
          FAULTS_id.cdsUTR5.no_utr5(end+1) = i; 
          % warning(sprintf('cannot find 5'' UTR \n', gene.name, gene.chr));
        end ;
        
        if ~isempty(utr3_exons)  %% 
          if utr3_exons(end,2)==cds_exons(1,1)
            %% all good
            FAULTS_id.cdsUTR3.ok(end+1) = i;
            FAULTS_name.cdsUTR3.ok{end+1} = gene.name;
          else
            FAULTS_name.cdsUTR3.STOPdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.STOPdiff(end+1) = i; 
            %  warning(sprintf('first UTR-3 exon start (%i) is not equal to the end of the first coding exon (%i) (gene %s, contig %s)\nignoring 5''UTR\n', utr3_exons(end,2), cds_exons(1,1), gene.name, gene.chr));
            utr3_exons = [] ;
          end
        elseif ~isempty(exons) && any(exons(:,1)<Stop) 
          idx = find(exons(:,1)<Stop);
          utr3_exons = exons(idx,:);
          if sum(utr3_exons(:,2)>Stop)>1
            FAULTS_name.cdsUTR3.mult_overlap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.mult_overlap(end+1) = i; 
            % warning(sprintf('more than one exons overlaps end of the first coding exon (gene %s, contig %s) \nignoring 5''UTR \n', gene.name, gene.chr));
            utr3_exons = [];
          elseif ~any(utr3_exons(:,2)>Stop)
            FAULTS_name.cdsUTR3.gap{end+1} = gene.name;
            FAULTS_id.cdsUTR3.gap(end+1) = i; 
            utr3_exons(end,2) = cds_exons(1,1);
            % warning(sprintf('extending 3'' UTR to end of CDS exon\n', gene.name, gene.chr));
          elseif utr3_exons(end,2)~=cds_exons(1,2)
            FAULTS_name.cdsUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.cdsUTR3.CLEAVEdiff(end+1) = i; 
            % warning(sprintf('TIS overlapping exon has different end than first cds_exon\n', gene.name, gene.chr));
            utr3_exons(end,2) = cds_exons(1,1);
          else
            utr3_exons(end,2) = cds_exons(1,1);
          end
        else
          FAULTS_name.cdsUTR3.no_utr3{end+1} = gene.name;
          FAULTS_id.cdsUTR3.no_utr3(end+1) = i; 
          % warning(sprintf('cannot find 3'' UTR \n', gene.name, gene.chr));
        end ;

      % start no_cds_exons given
      elseif ~isempty(utr5_exons) & ~isempty(exons) & ~isempty(utr3_exons)
        % inferring cds_exons
        tis = utr5_exons(1,1) ;
        Stop = utr3_exons(end,2) ;
        idx = find(exons(:,1)<tis&exons(:,2)>Stop);
        cds_exons = exons(idx,:);
        if cds_exons(end,2)>tis 
          if cds_exons(end,2)~= utr5_exons(1,2) ;
            FAULTS_name.exonUTR5.TSSdiff{end+1} = gene.name;
            FAULTS_id.exonUTR5.TSSdiff(end+1) = i; 
            % warning(['first potential coding exon has different start than ' ...
            %          'last 5''UTR exon']) 
          end
          cds_exons(end,2)=tis;
        else
          FAULTS_name.exonUTR5.gap{end+1} = gene.name;
          FAULTS_id.exonUTR5.gap(end+1) = i; 
          % warning(['first potential coding exon starts downstream of tis']) 
          cds_exons(end,2)=tis;
        end
        if cds_exons(1,1)<Stop 
          if cds_exons(1,1)~= utr3_exons(end,1) ;
            FAULTS_name.exonUTR3.CLEAVEdiff{end+1} = gene.name;
            FAULTS_id.exonUTR3.CLEAVEdiff(end+1) = i; 
            % warning(['last potential coding exon has different end than ' ...
            %          'first 3''UTR exon']) 
          end
          cds_exons(1,1)=Stop;
        else
          FAULTS_name.exonUTR3.gap{end+1} = gene.name;
          FAULTS_id.exonUTR3.gap(end+1) = i; 
          % warning(['last potential coding exon ends upstream of Stop']) 
          cds_exons(1,1)=Stop;
        end 
      else
        tis = [] ;
        Stop = [] ;
        FAULTS_name.noCDS.noCDS{end+1} = gene.name;
        FAULTS_id.noCDS.noCDS(end+1) = i; 
        % warning(sprintf('cannot infer UTRs and coding regions \n', gene.name, gene.chr));
      end
      
      Stop=Stop+3 ;
      
      if size(utr3_exons,1)>1 && utr3_exons(end,2)-utr3_exons(end,1)<3
        utr3_exons = [utr3_exons(1,1) utr3_exons(end,2)];
        FAULTS_name.exons.TAGT_cases{end+1} = gene.name;
        FAULTS_id.exons.TAGT_cases(end+1) = i;
        % warning('TAGT case: invalid donor!')
      end
      
      % determine phase of cds exon
      if ~isempty(cds_exons)
        cds_exons(end,3) = 0;
        cds_length = 0;
        for e=size(cds_exons,1)-1:-1:1
          cds_length = cds_length + cds_exons(e+1,2)-cds_exons(e+1,1);
          phase = mod(cds_length,3);
          switch phase
           case 0
            cds_exons(e,3) = phase;
           case 1
            cds_exons(e,3) = 2;
           case 2
            cds_exons(e,3) = 1;
          end
        end
      end    
      exons = utr3_exons;
      if ~isempty(utr3_exons)
        cleave = utr3_exons(1,1);
      end 
      if ~isempty(cds_exons)
        if ~isempty(exons),
          exons(end,2) = cds_exons(1,2) ;
          exons = [exons; cds_exons(2:end,1:2)] ;
        else
          exons = cds_exons(:,1:2) ;
        end ;
      end
      if ~isempty(utr5_exons)
        tss = utr5_exons(end,2);
        if ~isempty(exons)
          exons(end,2) = utr5_exons(1,2) ;
          exons = [exons; utr5_exons(2:end,:)] ;
        else
          exons = utr5_exons(:,1:2) ;
        end 
      end
      
    end ; %strand==-'

    gene.exons{j} = exons ;
    gene.cds_exons{j} = cds_exons;
    gene.utr5_exons{j} = utr5_exons;
    gene.utr3_exons{j} = utr3_exons;
    
    gene.tis{j}=tis ;
    gene.cdsStop{j}=Stop ;
    gene.tss{j} = tss;
    gene.cleave{j} = cleave;
    
  end ; %%loop over transcripts
  
  gene.start = min(gene.exons{1}(:)) ;
  gene.stop = max(gene.exons{1}(:)) ;
  for j=2:length(gene.exons)
    gene.start = min(gene.start, min(gene.exons{j}(:))) ;
    gene.stop = max(gene.stop, max(gene.exons{j}(:))) ;
  end ;
  
  genes(i)=gene ;
end ;%%loop over genes



F1=fieldnames(FAULTS_name);
for f=1:length(F1)
  F=fieldnames(FAULTS_name.(F1{f}));
  for i=1:length(F)
    [tmp,idx] = unique(FAULTS_id.(F1{f}).(F{i}));
    FAULTS_name.(F1{f}).(F{i}) = FAULTS_name.(F1{f}).(F{i})(idx);
    FAULTS_id.(F1{f}).(F{i})= FAULTS_id.(F1{f}).(F{i})(idx);
  end
end

fprintf('\n\n');
fprintf('number of genes with overlapping exons:\t\t\t\t%i \n',length(unique(FAULTS_id.exons.overlapping)));
fprintf('number of genes with exons of unknown type:\t\t\t%i \n',length(unique(FAULTS_id.exons.unknown_type)));
fprintf('number of genes with invalid donor (TAGT):\t\t\t%i \n' ,length(unique(FAULTS_name.exons.TAGT_cases)));
        
fprintf('number of genes with cds and UTR5 derived TIS different:\t%i \n',length(unique(FAULTS_id.cdsUTR5.TISdiff)));
fprintf('number of genes with multiple UTR5s overlapping TIS: \t\t%i \n',length(unique(FAULTS_id.cdsUTR5.mult_overlap)));
fprintf('number of genes with gap between 5'' UTR and CDS: \t\t%i \n',length(unique(FAULTS_id.cdsUTR5.gap)));
fprintf('number of genes with different end of first CDS exon:\t\t%i \n',length(unique(FAULTS_id.cdsUTR5.TSSdiff)));
fprintf('number of genes with coding region but no 5'' UTR:\t\t%i \n',length(unique(FAULTS_id.cdsUTR5.no_utr5)));

fprintf('number of genes with cds and UTR3 derived STOP different:\t%i \n',length(unique(FAULTS_id.cdsUTR3.STOPdiff)));
fprintf('number of genes with multiple UTR3s overlapping STOP:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.mult_overlap)));
fprintf('number of genes with gap between 3'' UTR and CDS:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.gap)));
fprintf('number of genes with different start of last CDS exon:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.CLEAVEdiff)));
fprintf('number of genes with coding region but no 3'' UTR:\t\t%i \n',length(unique(FAULTS_id.cdsUTR3.no_utr3)));

fprintf('number of genes with exon and UTR5 derived TSS different:\t%i \n',length(unique(FAULTS_id.exonUTR5.TSSdiff)));
fprintf('number of genes with gap between 5'' UTR and exon:\t\t%i \n',length(unique(FAULTS_id.exonUTR5.gap)));
fprintf('number of genes with exon and UTR3 derived CLEAVE different:\t%i \n',length(unique(FAULTS_id.exonUTR3.CLEAVEdiff)));
fprintf('number of genes with gap between 3'' UTR and exon:\t\t%i \n',length(unique(FAULTS_id.exonUTR3.gap)));
fprintf('number of genes without coding or UTR information:\t\t%i \n',length(unique(FAULTS_id.noCDS.noCDS)));



return

function [cds_exons, utr5_exons, utr3_exons, ok] = split_exons_cds(exons, tis_pos, cdsStop_pos, strand) ;
% [cds_exons, utr5_exons, utr3_exons, ok] = split_exons_cds(exons, tis_pos, cdsStop_pos, strand) ;

cds_exons = [] ;
utr5_exons = [] ;
utr3_exons = [] ;
ok=1 ;

cds_exons = exons ;
tis_exon = find(cds_exons(:,1)<=tis_pos & cds_exons(:,2)>=tis_pos) ;
if length(tis_exon)~=1,
  ok=0 ;
  return ;
end ;

if strand == '+',
  utr5_exons = cds_exons(1:tis_exon,:) ;
  utr5_exons(end,2) = tis_pos ;
  if utr5_exons(end,2)-utr5_exons(end,1)<=0,
    utr5_exons(end,:) = [] ;
    ok = 0 ;
    return ;
  end ;
  
  cds_exons = cds_exons(tis_exon:end, :) ;
  cds_exons(1,1) = tis_pos ;
else
  utr5_exons = cds_exons(tis_exon:end, :) ;
  utr5_exons(1,1) = tis_pos ;
  if utr5_exons(1,2)-utr5_exons(1,1)<=0,
    utr5_exons(1,:) = [] ;
    ok = 0 ;
    return ;
  end ;
  
  cds_exons = cds_exons(1:tis_exon, :) ;
  cds_exons(end,2) = tis_pos ;
end ;

stop_exon = find(cds_exons(:,1)<=cdsStop_pos & cds_exons(:,2)>=cdsStop_pos) ;
if length(stop_exon)~=1,
  ok=0 ;
  return ;
end ;

if strand == '+',
  utr3_exons = cds_exons(stop_exon:end, :) ;
  utr3_exons(1,1) = cdsStop_pos ;
  if utr3_exons(1,2)-utr3_exons(1,1)<=0,
    utr3_exons(1,:) = [] ;
  end ;
  
  cds_exons = cds_exons(1:stop_exon, :) ;
  cds_exons(end,2) = cdsStop_pos ;
else
  utr3_exons = cds_exons(1:stop_exon,:) ;
  utr3_exons(end,2) = cdsStop_pos ;
  if utr3_exons(end,2)-utr3_exons(end,1)<=0,
    utr3_exons(end,:) = [] ;
  end ;
  
  cds_exons = cds_exons(stop_exon:end, :) ;
  cds_exons(1,1) = cdsStop_pos ;
end ;

ok=1 ;
return

function [blocks, QP, viterbi_nbest,num_new_constraints] = add_constraints(blocks, block_idx, num_examples, PAR, QP, viterbi_nbest);
%   [blocks, QP, viterbi_nbest] = add_constraints(blocks,block_idx,num_examples,PAR,QP,viterbi_nbest);
% 
%
if ~isfield(PAR, 'fn_log')
  %logfile = '~/tmp/logfile'
  fid = 1;%fopen(logfile, 'a+');
else
  fid = fopen(PAR.fn_log,'a+') ;
end

verb_level=1 ;

%addpath /fml/ag-raetsch/share/software/matlab_tools/cplex10
%addpath ~/svn/projects/genefinding/utils
num_new_constraints = 0;
max_neg  = PAR.LSL.method.max_neg ;
opt_step = PAR.LSL.method.opt_step;
model = PAR.model ;

clear A1 A1_sparse
A1 = zeros(opt_step,model.cnt_parameters) ;
A2 = sparse([], [], [], length(blocks)*max_neg, size(QP.A,2)-model.cnt_parameters, length(blocks)*max_neg) ; 
b = zeros(opt_step, 1) ;
q = 0 ; r = 0 ; q2 = 0 ;

if ~isequal(blocks, fix_blocks(blocks, model))
  fprintf('add_constraints: fix_blocks\n') ;
  if keyboard_allowed(),
    keyboard ;
  end ;
  blocks = fix_blocks(blocks, model) ;
end ;
if ~isequal(model, fix_model(model))
  fprintf('add_constraints: fix_model\n') ;
  if keyboard_allowed(),
    keyboard ;
  end ;
  model = fix_blocks(model) ;
end ;

assert(isequal(model, fix_model(model))) ;
assert(isequal(blocks, fix_blocks(blocks, model))) ;

[xis, parameters] = res2param(QP.res, num_examples, model, QP, 0, PAR.weight_names);
param_vec = weights2vector(parameters, PAR.weight_names) ;
num_correct = 0 ;
fprintf('\n') ;
losses = [] ;

w_fields = PAR.weight_names ;%fieldnames(init_weights(model)) ;

% check the weights from time to time
check_compute_weights=(rand(1)>0.95) ;
for id = 1:length(blocks),
  %blocks(id).pred_use_approximation = 0 ;
  block = blocks(id) ;

  %fprintf(fid,'\n block: %i, example %i\n ',block.id,id);
  while ~fexist(sprintf('%spred_block%i.mat', PAR.FN.output_lsl.fn_train_block_preds, block.id))
    fprintf('.') ;
    pause(10)
  end
  %fprintf('block: %i, example %i\r', block.id, id);
  % load(sprintf('%s_viterbi_block%i.mat', PAR.fn_train, block.id ), 'pred');   
  load(sprintf('%spred_block%i.mat', PAR.FN.output_lsl.fn_train_block_preds, block.id), 'pred'); 
  assert(isfield(pred, 'loss')) ;

  if isempty(pred), warning('empty'); continue; end ;

  all_identical = 1 ;
  for pred_num = 1:length(pred)
    pred(pred_num).weights = reorder_fields(pred(pred_num).weights, w_fields);
    
    if check_compute_weights,
      [pred2(pred_num).weights, loss2(pred_num)] = compute_weights(pred(pred_num), block, model, PAR.FN, parameters) ;    
      fnw = fieldnames(pred2(pred_num).weights) ;
      for kk=1:length(fnw),
        diff = norm(pred2(pred_num).weights.(fnw{kk})-pred(pred_num).weights.(fnw{kk})) ;
        if diff>1e-4,
          fprintf('compute weights diff: %s => %1.2f\n', fnw{kk}, diff) ;
          if keyboard_allowed(),
            keyboard ;
          end ;
        end ;
      end ;
    end ;

    loss(pred_num) = pred(pred_num).loss ;

    identical = 0 ;
    for t=1:length(block.truth)
      identical = equal(pred(pred_num).path, block.truth(t).path) & equal(pred(pred_num).pos, block.truth(t).pos) ;
      identical = identical | (loss(pred_num)==0) ;
      if identical, break ; end ;
    end ;
    if ~identical, all_identical=0; end ;

    vec = weights2vector(block.truth(1).weights, PAR.weight_names) - weights2vector(pred(pred_num).weights, PAR.weight_names);
    diff_before = -sum(param_vec.*vec);
    
    if identical,
      if loss(pred_num)>0,
        warning('unexpected: loss>0') ;
        if keyboard_allowed(), keyboard ; end ;
        %assert(~loss(pred_num)) ;
      end ;
      if pred_num==1, 
        num_correct = num_correct+1;
      end
      if verb_level>2,
        fprintf(fid, 'disabling gen_path approximation\n') ;
      end ;
      block.pred_use_approximation = 0 ;
    else
      all_identical=0 ;
      assert(norm(vec)>1e-8) ;
      %if loss(pred_num==0) && keyboard_allowed, keyboard; end 
      assert(loss(pred_num)>0) ;
      
      if diff_before+loss(pred_num)-xis(block_idx(id)) < 1e-7
        %fprintf(fid,'no margin violation') ;
        %fprintf('\nno margin violation\n') ;
        if verb_level>1,
          fprintf('No margin violation: %f > %f = %f - %f\n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
        end ;
        if -diff_before - loss(pred_num)>1e-3 && block.pred_use_approximation==0,
  	  warning('true path scores higher than max_path (%1.4f)', -diff_before - loss(pred_num))
   	  if keyboard_allowed,
            keyboard
          end 
        end
        if pred_num==length(pred)
          if verb_level>2,
            fprintf(fid,'no constraint added\n');
          end ;
        end
        if verb_level>2,
          fprintf(fid, 'disabling gen_path approximation\n') ;
        end ;
	block.pred_use_approximation = 0 ;
        continue
      end
      if verb_level>1,
        fprintf('Adding violated constraint %f > %f = %f - %f \n', -diff_before, loss(pred_num) - xis(block_idx(id)), loss(pred_num), xis(block_idx(id))) ;
      end ;
      num_new_constraints = num_new_constraints+1;
      q  = q+1 ; 
      q2 = q2+1 ;
      A1(q2,1:model.cnt_parameters) = -vec;
      b(q2)= -loss(pred_num) ;
      A2(q, block_idx(id)) = -1 ; % slack
      
      if q2==size(A1,1)
        fprintf('*\n') ;  
        A1 = sparse(A1(1:q2,:)) ;
        assert(q==size(A1,1));
        A2 = A2(1:q,:) ;
        A = [A1 A2] ;
        QP.A = [QP.A; A] ;
        QP.b = [QP.b; b] ;
        
        %tic;[res,lambda,how] = qp_solve(QP.lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, PAR.solver);toc;
        %tic;[res,lambda,how] = qp_solve(QP.lpenv, QP.Q, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'sift');toc;
        
        %OBJ = sum(res.*f) + 0.5*res'*QP.Q*res ;
        %[xis,parameters,hvar,H] = res2param(res,num_examples,model);
        %sum_xis = sum(xis) ;

        q2 = 0; q = 0 ;
        A1 = zeros(opt_step,model.cnt_parameters) ;
        A2 = sparse([],[],[],length(blocks)*max_neg, size(QP.A, 2)-model.cnt_parameters, length(blocks)*max_neg) ; 
        b = zeros(opt_step, 1) ;
      end ;
      if isempty(block.pred)
        block.pred = pred(pred_num) ; 
      else
        if ~isfield(block.pred, 'loss')
          block.pred(end).loss=nan ;
          block.pred=orderfields(block.pred) ;
        end ;
        if length(block.pred)==1,
          block.pred = orderfields(block.pred) ;
        end ;
        block.pred(end+1) = pred(pred_num) ; 
      end ;
      block.pred(end).loss = loss(pred_num) ;

      %fprintf(fid, 'enabling gen_path approximation\n') ;
      %block.pred_use_approximation = 1 ;
    end ;
  end ; %%% loop over predictions 
  %if all_identical,
    %viterbi_nbest(id)=viterbi_nbest(id)+1 ;
    %fprintf('increasing number of nbest path to %i\n', viterbi_nbest(id)) ;
  %end ;
  blocks(id) = block ;
  losses(id) = mean(loss) ;

end ;%%% loop over blocks

if verb_level>0,
  if fid~=1,
    fprintf(fid,'id=%i    correct=%i(%2.2f)  average_loss=%2.2f  num_new_constraints=%i\r', id,num_correct, num_correct/id, mean(losses), num_new_constraints);
  end ;
  fprintf('\nid=%i    correct=%i(%2.2f)  average_loss=%2.2f  num_new_constraints=%i\n', id, num_correct,num_correct/id, mean(losses), num_new_constraints);
end ;

A1     = sparse(A1(1:q2,:)) ;
b      = b(1:q2) ;
A2     = A2(1:q,:) ;
A      = [A1 A2] ;
QP.A   = [QP.A;A] ;
QP.b   = [QP.b;b] ;

if fid>1
  fclose(fid) ;
end

% it does not help anyways (GR)
viterbi_nbest(:)=1 ;

return

function [weights, loss, score, losses, scores, ok] = compute_weights(SEG, block, model, FN, parameters, check_result) ;
% weights=compute_weights(segmentation, block, model) ;

if nargin<5
  parameters = [];
end
if nargin<6
  check_result = 1 ;
end
ok=true ;
[penalty_array, content_weights, state_signals, a_trans] = gen_penalty_array(model, block.split, FN, parameters) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%for i=1:length(segmentation.genes)
%  segmentation.genes{i}(1,3)=model.segments.utr5exon ;
%  segmentation.genes{i}(end,3)=model.segments.utr3exon ;
%end ;

% simple state sequence -> generate this from segmentation
% if ~isfield(SEG,'path') | isfield(SEG,'pos_idx') 
%   [my_path,pos_idx,ok]=segmentation2path(SEG.segmentation, block, model);
% else
%   my_path = SEG.path ;
%   pos_idx = SEG.pos_idx
% end
my_path = SEG.path ;
pos_idx = SEG.pos_idx ;

% keyboard
% sg('send_command', 'loglevel WARNING') ;
% sg('send_command', 'loglevel ALL') ;
%model.loss.segments(:)=1 ;
%model.loss.segments(:,9:16)=2 ;
%keyboard

%[my_path,pos_idx,pos,ok]=segmentation2path(SEG, block, model) ;

seg_path = block.truth(1).seg_path;
% if the alternatives disagree, then don't incur a loss
for i=2:length(block.truth)
  if isempty(block.truth(i).seg_path), continue ; end ;
  idx = find(block.truth(i).seg_path(1,:) ~= block.truth(1).seg_path(1,:)) ;
  seg_path(2,idx) = 0 ;
end ;
if iscell(block.features)
  features = full_features(block.features);
else 
  features = block.features;
end

if isfield(model, 'use_conservation') && model.use_conservation
  error('not yet implemented')
else
  use_orf=1;
  num_svms=8;
  sg_set_plif(penalty_array)
  sg('init_dyn_prog', num_svms)
  if isfield(block, 'content_pred')
    block.content_pred(end+1:num_svms,:) = deal(0);
    sg('set_lin_feat', block.seq, int32(block.all_pos-1), block.content_pred);
  else
    sg('precompute_content_svms',block.seq,int32(block.all_pos-1), content_weights');
  end
  sg('set_model', model.transition_pointers, use_orf, int32(model.mod_words), int32(state_signals),int32(model.orf_info))
  sg('set_feature_matrix',features)
  if isfield(model, 'tiling')
    sg('precompute_tiling_features',block.intensities, int32(block.probe_local_pos-1), int32(model.tiling_plif_ids-1))
  end
  if isfield(model, 'rna_seq')
    sg('precompute_tiling_features',block.coverage, int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1))
  end
  if isfield(model, 'rna_seq_intron_track')
    sg('precompute_tiling_features',block.intron_coverage, int32(0:length(block.seq)-1), int32(model.rna_seq_intron_track_plif_ids-1))
  end

  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', model.p', model.q', seg_path, a_trans, model.loss.segments, int32(my_path-1), int32(pos_idx-1));
%  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', my_path-1, pos_idx-1, model.p', ...
%        model.q',a_trans, block.features, block.all_pos-1, lower(block.seq)',model.transition_pointers, state_signals, ...
%	penalty_array, content_weights', model.mod_words, model.loss.segments, seg_path) ;
%
end ;

loss = sum(losses) ;
score = sum(scores) ;

if isempty(parameters) && check_result
  if ~sum(scores)==0
    idx = find(scores)
    pos_idx(idx(1)+[-1:1]) 
    block.all_pos(pos_idx(idx(1)+[-1:1]))
    my_path(idx(1)+[-1:1])
    block.id
    error('invalid path: sum(scores) ~= 0')
    %weights = [];
    %ok=false ;
    %return
  end
else
  if check_result,
    assert(~isinf(sum(scores))) ;
    if isinf(sum(scores)), 
      ok=false; 
    end ;
  end ;
end

weights = init_weights(model) ;
for i=1:size(penalty_deriv,1)
  pen = penalty_array{i} ;
  weights = setfield(weights, pen.name, penalty_deriv(pen.id,:)) ;
end ;

%%% transitions
weights.transitions(:) = a_deriv(model.active_transitions) ;



%%%% for debugging
if 0 
  dbup
  dbstack
  
  idx = find(isinf(scores))
  
  SEG.path(idx:idx+1)
  
  %%%check state

  block.seq(SEG.path(idx),SEG.pos_idx(idx))
  %%%check transition 
  
  trans_lengths = SEG.pos(idx+1)-SEG.pos(idx:idx+1)
  idx2 = find(ismember(SEG.segments(:,1:2),SEG.pos(idx:idx+1),'rows'));
  seg_type = SEG.segments(idx2,3)
  
  range = model.lengths_range
end

return

function model = define_addContentFeatures(model)
  
% plifs for tiling array data
if model.use.addfeatures.tiling
  tiling_start =  model.cnt_contents + model.cnt_lengths + model.cnt_signals;
  content_names = fieldnames(model.contents);
  model.tiling_plif_ids = [];
  for s = 1:length(content_names)
    model.tiling.(content_names{s}) = tiling_start+s ;
    model.tiling_plif_ids = [model.tiling_plif_ids tiling_start+s];
  end
  model.cnt_tiling = length(fieldnames(model.tiling));
  % declare range
  model.tiling_range = struct;
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    model.tiling_range = setfield(model.tiling_range, tiling_names{s},[6,11]);% ca. 2%-98% Percentile
  end
else
  model.cnt_tiling = 0;
end

return

function model = define_contents(model)
  
% declare content types

fid = 1;  
model.mod_words = [];
% the first contents consider all k-mers. the last three
% frame-content sensors only consider codons, with an offset from
% the end of the segment

content_start =  1 ;
cnt=0;

model.contents.intergenic = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.utr_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.cds_exon   = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
model.contents.intron     = content_start+cnt ;cnt=cnt+1;
model.mod_words = [model.mod_words; 1 0] ;
if model.use.segments.intercistronic
  model.contents.intercistronic = content_start+cnt;cnt=cnt+1;
  model.mod_words = [model.mod_words; 1 0] ;
end
if model.use.contents.frame
  model.contents.cds_frame0   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 0] ;
  model.contents.cds_frame1   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 1] ;
  model.contents.cds_frame2   = content_start+cnt ;cnt=cnt+1;
  model.mod_words = [model.mod_words; 3 2] ;
end
model.cnt_contents = length(fieldnames(model.contents));

%hack
if model.cnt_contents<8
  fprintf(fid,'introducing dummy contents\n')
  for i= 1:8-model.cnt_contents
    if i==1
      model.contents.('dummy')= content_start+cnt;cnt=cnt+1;
    else
      model.contents.(['dummy' num2str(i)])= content_start+cnt;cnt=cnt+1;
    end
  end
  model.cnt_contents = length(fieldnames(model.contents));
  model.mod_words = [model.mod_words; 1 0] ;
end

% declare content range
model.contents_range = struct;
content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  model.contents_range = setfield(model.contents_range, content_names{s},[-10,10]);
end


return

function model = define_lengths(model,organism)
  
fid = 1; 
% declare length types
length_start = model.cnt_contents+1 ;
cnt = 0;

model.lengths.intergenic = length_start+cnt; cnt = cnt+1;
model.lengths.utr5exon = length_start+cnt ;cnt = cnt+1;
if model.use.segments.transexon 
  model.lengths.transexon = length_start+cnt ;cnt = cnt+1;
end
model.lengths.single_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.first_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.middle_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.last_cds_exon = length_start+cnt ;cnt = cnt+1;
model.lengths.utr3exon = length_start+cnt;cnt = cnt+1;
if model.use.segments.polya_tail 
  model.lengths.polya_tail = length_start+cnt;cnt = cnt+1;
end
model.lengths.intron = length_start+cnt ;cnt = cnt+1;
if model.use.segments.intercistronic
  model.lengths.intercistronic = length_start+cnt ;cnt = cnt+1;
  model.lengths.intergenictrans = length_start+cnt ;cnt = cnt+1;
end
model.lengths.intergenic_long = length_start+cnt ;cnt = cnt+1;

model.cnt_lengths = length(fieldnames(model.lengths));


% declare length range
s1=which([organism.name '_lengths']);
s2=which([organism.clade '_lengths']);
if ~isempty(s1)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.name)
  model.lengths_range = feval([organism.name '_lengths']);
elseif ~isempty(s2)
  fprintf(fid,'taking lengths range from file: %s_lengths.m \n',organism.clade)
  model.lengths_range = feval([organism.clade '_lengths']);
else
  fprintf(fid,'no lengths range specified, using default settings\n')
  model.lengths_range = feval('default_lengths');
end

return

function model = define_loss(model)
  

% create loss matrices

equiv = [];
  
different = [model.segments.intergenic, model.segments.cds_exon, 3 ;
             model.segments.intergenic, model.segments.intron, 3;
             model.segments.intron, model.segments.cds_exon, 2];

%--------------------------------
if ~model.use.loss.orf_only
  different = [different;
               model.segments.intergenic, model.segments.utr5exon, 2;
               model.segments.intergenic, model.segments.utr3exon, 2];
  if model.use.segments.intercistronic
    different = [different;
                 model.segments.intercistronic, model.segments.utr5exon, 2;
                 model.segments.intercistronic, model.segments.utr3exon, 2;
                 model.segments.intergenictrans, model.segments.utr5exon, 2;
                 model.segments.intergenictrans, model.segments.utr3exon, 2];
    
  end
else
  equiv = [equiv;
           model.segments.intergenic model.segments.utr3exon ;
           model.segments.intergenic model.segments.utr5exon] ;
  if model.use.segments.intercistronic
    equiv = [equiv;
             model.segments.intercistronic model.segments.utr3exon ; 
             model.segments.intercistronic model.segments.utr5exon ;
             model.segments.intergenictrans model.segments.utr3exon ;
             model.segments.intergenictrans model.segments.utr5exon] ;
  end
end
%--------------------------------
if model.use.segments.polya_tail
  equiv = [equiv;
           model.segments.utr3exon, model.segments.polya_tail];
  different = [different;
               model.segments.intergenic, model.segments.polya_tail, 1];    
end
 %-------------------------------- 
if model.use.segments.intercistronic
  
  different = [different;
               model.segments.intercistronic, model.segments.cds_exon, 3 ;
               model.segments.intercistronic, model.segments.intron, 3;
               model.segments.intercistronic, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.intercistronic, 0.1;
               model.segments.intergenictrans, model.segments.cds_exon, 3 ;
               model.segments.intergenictrans, model.segments.intron, 3;
               model.segments.intergenictrans, model.segments.transexon, 2;
               model.segments.intergenic, model.segments.intergenictrans, 0.1];

  if model.use.segments.polya_tail
    different = [different;
                 model.segments.intercistronic, model.segments.polya_tail,1;
                 model.segments.intergenictrans, model.segments.polya_tail, 1];
  end
end
%--------------------------------
if model.use.segments.transexon
  equiv = [equiv;
           model.segments.utr5exon, model.segments.transexon] ;     
  different = [different;
               model.segments.intergenic, model.segments.transexon, 1] ;
  if isfield(model.segments, 'intergenictrans'),
    different = [different;
                 model.segments.intergenictrans, model.segments.transexon, 1];    
  end ;
end
%--------------------------------

model.loss.segments = ones(model.cnt_segments) - eye(model.cnt_segments);
for i=1:size(different,1)
  model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3) ;
  % model.loss.segments(different(i,1)+1, different(i,2)+1) = different(i,3)+1 ;warning('make loss matrix asymetric to enhance sensitivity')
  model.loss.segments(different(i,2)+1, different(i,1)+1) = different(i,3) ;
end ;

for i=1:size(equiv,1)
  model.loss.segments(equiv(i,1)+1, equiv(i,2)+1) = 0 ;
  model.loss.segments(equiv(i,2)+1, equiv(i,1)+1) = 0 ;
end ;


% tripple loss for coding
%model.loss.segments(model.segments.cds_exon+1, :) = 3*model.loss.segments(model.segments.cds_exon+1, :) ;
%model.loss.segments(:, model.segments.cds_exon+1) = 3*model.loss.segments(:, model.segments.cds_exon+1) ;

% first part is for segments, the second for nucleotides
model.loss.segments = [model.loss.segments 0.0*model.loss.segments] ;

if model.use.signals.tss
  model.loss.zero_range.tss = 10 ;
end
model.loss.zero_range.tis = 0;
model.loss.zero_range.don = 0;
model.loss.zero_range.acc = 0;
if model.use.signals.trans
  model.loss.zero_range.trans = 0;
end
model.loss.zero_range.cdsStop = 0;
if model.use.signals.polya
  model.loss.zero_range.polya = 10 ;
end
if model.use.signals.cleave
  model.loss.zero_range.cleave = 10 ;
end

model.loss.confirmed = 1 ;
model.loss.confirmed_tol = 1000 ; % region around confirmed gene
model.loss.unconfirmed = 1 ;

% model.loss.mustbe_corr.intergenic = 0 ;
% model.loss.mustbe_corr.utr5exon   = 0;
% model.loss.mustbe_corr.transexon  = 0 ;
% model.loss.mustbe_corr.cds_exon   = 1 ;
% model.loss.mustbe_corr.utr3exon   = 0 ;
% model.loss.mustbe_corr.polya_tail = 0 ;
% model.loss.mustbe_corr.intercistronic  = 0 ;
% model.loss.mustbe_corr.intron     = 1 ;
return ;

return

function model = define_mappings(model)
  

%%mappings between length and contents
  
model.plif_links = [] ;
model.plif_links(end+1,:) = [model.lengths.intergenic      model.contents.intergenic] ;
model.plif_links(end+1,:) = [model.lengths.first_cds_exon  model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.last_cds_exon   model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.middle_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.single_cds_exon model.contents.cds_exon] ;
model.plif_links(end+1,:) = [model.lengths.intron          model.contents.intron] ;
if model.use.segments.transexon
  model.plif_links(end+1,:) = [model.lengths.transexon       model.contents.utr_exon] ;
end
model.plif_links(end+1,:) = [model.lengths.utr5exon        model.contents.utr_exon] ;
model.plif_links(end+1,:) = [model.lengths.utr3exon        model.contents.utr_exon] ;
if model.use.segments.polya_tail
  model.plif_links(end+1,:) = [model.lengths.polya_tail      model.contents.utr_exon] ;
end
if model.use.segments.intercistronic
  model.plif_links(end+1,:) = [model.lengths.intercistronic  model.contents.intercistronic] ;
  model.plif_links(end+1,:) = [model.lengths.intergenictrans  model.contents.intergenic] ;
end
model.plif_links(end+1,:) = [model.lengths.intergenic_long model.contents.intergenic] ;


if isfield(model.state_ids, 'intron_long') 
  model.plif_links(end+1,:) = [model.lengths.intron_long model.contents.intron] ;  
end

%%mappings between length and additional contents (tiling)

if model.use.addfeatures.tiling
  model.tiling_links = [] ;   
  model.tiling_links(end+1,:) = [model.lengths.intergenic      model.tiling.intergenic] ;
  model.tiling_links(end+1,:) = [model.lengths.first_cds_exon  model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.last_cds_exon   model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.middle_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.single_cds_exon model.tiling.cds_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.intron          model.tiling.intron] ;
  if model.use.segments.transexon
    model.tiling_links(end+1,:) = [model.lengths.transexon       model.tiling.transexon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.utr5exon        model.tiling.utr_exon] ;
  model.tiling_links(end+1,:) = [model.lengths.utr3exon        model.tiling.utr_exon] ;
  if model.use.segments.polya_tail
    model.tiling_links(end+1,:) = [model.lengths.polya_tail      model.tiling.utr_exon] ;
  end
  model.tiling_links(end+1,:) = [model.lengths.intergenic_long model.tiling.intergenic] ;
  if model.use.segments.intercistronic
    model.tiling_links(end+1,:) = [model.lengths.intercistronic  model.tiling.intercistronic] ;
  end
end

%%mappings between length and segments

model.seg_links = [] ;
model.seg_links(end+1,:) = [model.lengths.intergenic      model.segments.intergenic] ;
model.seg_links(end+1,:) = [model.lengths.first_cds_exon  model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.last_cds_exon   model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.middle_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.single_cds_exon model.segments.cds_exon] ;
model.seg_links(end+1,:) = [model.lengths.intron          model.segments.intron] ;
if model.use.segments.transexon
  model.seg_links(end+1,:) = [model.lengths.transexon       model.segments.transexon] ;
end
model.seg_links(end+1,:) = [model.lengths.utr5exon        model.segments.utr5exon] ;
model.seg_links(end+1,:) = [model.lengths.utr3exon        model.segments.utr3exon] ;
if model.use.segments.polya_tail
  model.seg_links(end+1,:) = [model.lengths.polya_tail      model.segments.polya_tail] ;
end
if model.use.segments.intercistronic
  model.seg_links(end+1,:) = [model.lengths.intergenictrans  model.segments.intergenictrans] ;
  model.seg_links(end+1,:) = [model.lengths.intercistronic  model.segments.intercistronic] ;
end
model.seg_links(end+1,:) = [model.lengths.intergenic_long model.segments.intergenic] ;
%

return

function model = define_segments(model)
% declare segment types

  cnt=0;  
model.segments.intergenic = cnt ;cnt=cnt+1;
model.segments.utr5exon   = cnt ;cnt=cnt+1;
if model.use.segments.transexon 
  model.segments.transexon  = cnt ;cnt=cnt+1;
end
model.segments.cds_exon   = cnt ;cnt=cnt+1;
model.segments.utr3exon   = cnt ;cnt=cnt+1;
if model.use.segments.polya_tail
  model.segments.polya_tail = cnt ;cnt=cnt+1;
end
if model.use.segments.intercistronic
  model.segments.intercistronic  = cnt ;cnt=cnt+1;
  model.segments.intergenictrans  = cnt ;cnt=cnt+1;
end
model.segments.intron     = cnt ;cnt=cnt+1;

model.cnt_segments = length(fieldnames(model.segments));


return

function model = define_signals(model,polya_sixmers)
  
% declare signal types
cnt=1;
signal_start = model.cnt_contents + model.cnt_lengths ;

if model.use.signals.tss
  model.signals.tss = signal_start+cnt;
  model.consensus.tss = {} ;
  model.consensus_pos.tss = [];
  cnt=cnt+1;
end

model.signals.tis = signal_start+cnt;
model.consensus.tis = {'atg'} ;
model.consensus_pos.tis = [0];
cnt=cnt+1;

model.signals.don = signal_start+cnt;
model.consensus.don = {'gt','gc'} ;
model.consensus_pos.don = [0,0];
cnt=cnt+1;

model.signals.acc = signal_start+cnt;
model.consensus.acc = {'ag'} ;
model.consensus_pos.acc = [2];
cnt=cnt+1;

model.signals.cdsStop = signal_start+cnt ;
model.consensus.cdsStop = {'taa','tga','tag'} ;
model.consensus_pos.cdsStop = [0,0,0];
cnt=cnt+1;

if model.use.signals.polya
  model.signals.polya = signal_start+cnt ;
  model.consensus.polya = polya_sixmers;
  model.consensus_pos.polya = zeros(1,length(polya_sixmers)) ;
  cnt=cnt+1;
end
if model.use.signals.cleave
  model.signals.cleave = signal_start+cnt ;
  model.consensus.cleave = {} ;
  model.consensus_pos.cleave = [];
  cnt=cnt+1;
end

if model.use.signals.trans
  model.signals.trans = signal_start+cnt;
  model.consensus.trans = {'ag'} ;
  model.consensus_pos.trans = [2];
  cnt=cnt+1;
end
if model.use.signals.transacc
  model.signals.transacc = signal_start+cnt;
  model.consensus.transacc = {'ag'} ;
  model.consensus_pos.transacc = [2];
  cnt=cnt+1;
end

model.cnt_signals = length(fieldnames(model.signals));


% declare signal ranges for PLiFs

if model.use.signals.tss
  model.signals_range.tss = [0,1] ;
end
model.signals_range.tis = [0,1] ;
model.signals_range.don = [0,1] ;
model.signals_range.acc = [0,1] ;
model.signals_range.cdsStop = [0,1] ;
if model.use.signals.polya
  model.signals_range.polya = [0,1] ;
end
if model.use.signals.cleave
  model.signals_range.cleave = [0,1] ;
end
if model.use.signals.trans
  model.signals_range.trans = [0,1] ;
end
if model.use.signals.transacc
  model.signals_range.transacc = [0,1] ;
end



return

function model = define_states(model)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define states
model.states(1) = struct ;
cnt=0 ;

% sequence start 
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_start' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_start=cnt ;

signal_names = fieldnames(model.signals) ;
for i=1:model.cnt_signals
  if strcmp(signal_names{i}, 'transacc'), continue; end ;
  if ~strcmp(signal_names{i},'acc') && ~strcmp(signal_names{i},'don') ...
        && ~strcmp(signal_names{i}, 'tis') && ~strcmp(signal_names{i}, 'cdsStop')...
        && ~strcmp(signal_names{i}, 'trans') && ~strcmp(signal_names{i}, 'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},cnt) ;
  elseif strcmp(signal_names{i},'cleave') 
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt]) ;
    if model.use.segments.intercistronic
      cnt=cnt+1 ;
      model.states(cnt).name = signal_names{i};
      model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
      model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(cnt).non_consensus = {} ;
      model.states(cnt).non_consensus_pos = [] ;
      model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
      model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    end
  elseif strcmp(signal_names{i},'tis') | strcmp(signal_names{i},'cdsStop')
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).consensus = getfield(model.consensus,signal_names{i});
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,signal_names{i});
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = getfield(model.signals,signal_names{i}) ;
    cnt=cnt+1 ;
    model.states(cnt).name = signal_names{i};
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.state_ids=setfield(model.state_ids, signal_names{i},[cnt-1 cnt]) ;
    if strcmp(signal_names{i}, 'tis')
    model.states(cnt).name = 'tis_acc' ;
      model.states(cnt).consensus = {'agatg'} ;
      model.states(cnt).consensus_pos = 2 ;
      model.states(cnt).signal = [model.signals.acc model.signals.tis] ;%%%% !!! 2 Zahlen
    else
      model.states(cnt).name = 'cdsStop_acc';
      model.states(cnt).consensus = {'agtaa', 'agtag', 'agtga'} ;
      model.states(cnt).consensus_pos = [2 2 2] ;
      model.states(cnt).signal = [model.signals.acc model.signals.cdsStop] ;%%%% !!! 2 Zahlen
    end
  elseif strcmp(signal_names{i},'trans') & model.use.states.SL1 & model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL1';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'SL2';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;%%%% !!! 3 Zahlen
    model.state_ids = setfield(model.state_ids, 'SL2', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'trans') & ~model.use.states.SL1 & ~model.use.states.SL2
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = getfield(model.consensus,'trans');
    model.states(cnt).consensus_pos = getfield(model.consensus_pos,'trans');
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc]  ;
    cnt=cnt+1 ;
    model.states(cnt).name = 'trans';
    model.states(cnt).consensus = {'agatg'} ; 
    model.states(cnt).consensus_pos = 2;
    model.states(cnt).non_consensus = {} ;
    model.states(cnt).non_consensus_pos = [] ;
    model.states(cnt).signal = [model.signals.trans model.signals.transacc model.signals.tis]  ;
    model.state_ids = setfield(model.state_ids, 'SL1', [cnt-1 cnt]) ;
  elseif strcmp(signal_names{i},'don')
    cnt_don = cnt ;
    cnt=cnt+8 ;
    for id = cnt_don+1:cnt_don+8 ;
      model.states(id).consensus = getfield(model.consensus,signal_names{i});
      model.states(id).consensus_pos = getfield(model.consensus_pos,signal_names{i});
      model.states(id).non_consensus = {} ;
      model.states(id).non_consensus_pos = [] ;
      model.states(id).signal=model.signals.don ;
    end ;
    model.states(cnt_don+1).name='UTR5_don' ;
    model.states(cnt_don+2).name='CDS_don_0' ;
    model.states(cnt_don+3).name='CDS_don_1a' ;
    model.states(cnt_don+3).non_consensus = {'t'} ;
    model.states(cnt_don+3).non_consensus_pos= [1] ;
    model.states(cnt_don+4).name='CDS_don_1b' ;
    model.states(cnt_don+4).consensus = {'tgt','tgc'} ;
    model.states(cnt_don+4).consensus_pos= [1,1] ;
    model.states(cnt_don+5).name='CDS_don_2a' ;
    model.states(cnt_don+5).non_consensus = {'ta', 'tg'} ;
    model.states(cnt_don+5).non_consensus_pos= [2,2] ;
    model.states(cnt_don+6).name='CDS_don_2b' ;
    model.states(cnt_don+6).consensus = {'tagt', 'tagc'} ;
    model.states(cnt_don+6).consensus_pos= [2,2] ;
    model.states(cnt_don+7).name='CDS_don_2c' ;
    model.states(cnt_don+7).consensus = {'tggt', 'tggc'} ;
    model.states(cnt_don+7).consensus_pos= [2,2] ;
    model.states(cnt_don+8).name='UTR3_don' ;
    
    model.state_ids.don=cnt_don+1:cnt_don+8 ;
  elseif strcmp(signal_names{i},'acc')
    cnt_acc = cnt ;
    cnt=cnt+8 ;
    for id = cnt_acc+1:cnt_acc+8 ;
      model.states(id).consensus = {'ag'} ;
      model.states(id).consensus_pos= [2] ;
      model.states(id).non_consensus={} ;
      model.states(id).non_consensus_pos=[] ;
      model.states(id).signal=model.signals.acc ;
    end ;
    model.states(cnt_acc+1).name='UTR5_acc' ;
    model.states(cnt_acc+2).name='CDS_acc_0' ;
    model.states(cnt_acc+3).name='CDS_acc_1a' ;
    model.states(cnt_acc+4).name='CDS_acc_1b' ;
    model.states(cnt_acc+4).non_consensus = {'aa','ag','ga'} ;
    model.states(cnt_acc+4).non_consensus_pos= [0,0,0] ;
    model.states(cnt_acc+5).name='CDS_acc_2a' ;
    model.states(cnt_acc+6).name='CDS_acc_2b' ;
    model.states(cnt_acc+6).consensus = {'agt', 'agc'} ;
    model.states(cnt_acc+6).consensus_pos= [2, 2] ;
    model.states(cnt_acc+7).name='CDS_acc_2c' ;
    model.states(cnt_acc+7).non_consensus = {'a'} ;
    model.states(cnt_acc+7).non_consensus_pos= [0] ;
    model.states(cnt_acc+8).name='UTR3_acc' ;
 
    model.state_ids.acc=cnt_acc+1:cnt_acc+8 ;

    if isfield(model.use, 'intron_long_state')&& model.use.intron_long_state
      cnt_intron_long = cnt;
      cnt = cnt+6;
      model.states(cnt_intron_long+1).consensus={} ;
      model.states(cnt_intron_long+1).consensus_pos=[] ;
      model.states(cnt_intron_long+1).non_consensus={} ;
      model.states(cnt_intron_long+1).non_consensus_pos=[] ;
      model.states(cnt_intron_long+1).name='intron_long_0' ;
      model.states(cnt_intron_long+1).signal=[] ;

      model.states(cnt_intron_long+2).consensus={} ;
      model.states(cnt_intron_long+2).consensus_pos=[] ;
      model.states(cnt_intron_long+2).non_consensus={} ;
      model.states(cnt_intron_long+2).non_consensus_pos=[] ;
      model.states(cnt_intron_long+2).name='intron_long_1a' ;
      model.states(cnt_intron_long+2).signal=[] ;
    
      model.states(cnt_intron_long+3).consensus={} ;
      model.states(cnt_intron_long+3).consensus_pos=[] ;
      model.states(cnt_intron_long+3).non_consensus={} ;
      model.states(cnt_intron_long+3).non_consensus_pos=[] ;
      model.states(cnt_intron_long+3).name='intron_long_1b' ;
      model.states(cnt_intron_long+3).signal=[] ;

      model.states(cnt_intron_long+4).consensus={} ;
      model.states(cnt_intron_long+4).consensus_pos=[] ;
      model.states(cnt_intron_long+4).non_consensus={} ;
      model.states(cnt_intron_long+4).non_consensus_pos=[] ;
      model.states(cnt_intron_long+4).name='intron_long_2a' ;
      model.states(cnt_intron_long+4).signal=[] ;

      model.states(cnt_intron_long+5).consensus={} ;
      model.states(cnt_intron_long+5).consensus_pos=[] ;
      model.states(cnt_intron_long+5).non_consensus={} ;
      model.states(cnt_intron_long+5).non_consensus_pos=[] ;
      model.states(cnt_intron_long+5).name='intron_long_2b' ;
      model.states(cnt_intron_long+5).signal=[] ;

      model.states(cnt_intron_long+6).consensus={} ;
      model.states(cnt_intron_long+6).consensus_pos=[] ;
      model.states(cnt_intron_long+6).non_consensus={} ;
      model.states(cnt_intron_long+6).non_consensus_pos=[] ;
      model.states(cnt_intron_long+6).name='intron_long_1a' ;
      model.states(cnt_intron_long+6).signal=[] ;

      model.state_ids.intergenic_long= cnt_intron_long+1:cnt_intron_long+6;
    end
%    cnt
  end
end

% intergenic_long
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='intergenic_long' ;
model.states(cnt).signal=[] ;
model.state_ids.intergenic_long=cnt ;

% sequence end
cnt=cnt+1 ;
model.states(cnt).consensus={} ;
model.states(cnt).consensus_pos=[] ;
model.states(cnt).non_consensus={} ;
model.states(cnt).non_consensus_pos=[] ;
model.states(cnt).name='seq_end' ;
model.states(cnt).signal=[] ;
model.state_ids.seq_end = cnt ;

model.cnt_states = cnt ; 

return

function model = define_transition_pointers(model)
  
A = model.A;
mRNA_start_states_out=model.mRNA_start_states_out;
%%% transition_pointers contain for each allowed transition pointers to lengths plifs
transition_pointers = zeros(size(A)) ;
transition_pointers(~isinf(A))=A(~isinf(A)) ;
transition_pointers = transition_pointers' ;

%%% add pointers to content PLIFs
transition_pointers(:,:,2) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
for i=1:size(model.plif_links,1)
  [idx1,idx2] = find(transition_pointers(:,:,1)==model.plif_links(i,1));
  for j=1:length(idx1)
    transition_pointers(idx1(j),idx2(j),2) = model.plif_links(i,2);
  end ;
end


%%% add pointers to tiling PLIFs
if model.use.addfeatures.tiling
  dim3 = size(transition_pointers,3);
  transition_pointers(:,:,dim3+1) = zeros(size(transition_pointers,1), size(transition_pointers,2), 1) ;
  for i=1:size(model.tiling_links,1)
    [idx1,idx2] = find(transition_pointers(:,:,1)==model.tiling_links(i,1));
    for j=1:length(idx1)
      transition_pointers(idx1(j),idx2(j),3) = model.tiling_links(i,2);
    end ;
  end
end

%%% add pointers to frame PLIFs
tmp = zeros(size(transition_pointers(:,:,1))) ;
tmp(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% single cds exon  

if ~isfield(model.use.contents, 'pre_comp') || ~model.use.contents.pre_comp,
  tmp(mRNA_start_states_out, model.state_ids.don(2)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(mRNA_start_states_out, model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% first cds exon  
  tmp(mRNA_start_states_out, model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% first cds exon 
  
  tmp(model.state_ids.acc(2:7), model.state_ids.don(2)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(3:4)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.don(5:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
else
  tmp(mRNA_start_states_out, model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% first cds exon 
  tmp(model.state_ids.acc(2),model.state_ids.don(2:7)) = model.contents.cds_frame0 ;% middle cds exon 
  tmp(model.state_ids.acc(3:4),model.state_ids.don(2:7)) = model.contents.cds_frame1 ;% middle cds exon 
  tmp(model.state_ids.acc(5:7),model.state_ids.don(2:7)) = model.contents.cds_frame2 ;% middle cds exon 
  tmp(model.state_ids.acc(2), model.state_ids.cdsStop(1)) = model.contents.cds_frame0 ;% last cds exon 
  tmp(model.state_ids.acc(3:4), model.state_ids.cdsStop(1)) = model.contents.cds_frame1 ;% last cds exon 
  tmp(model.state_ids.acc(5:7), model.state_ids.cdsStop(1)) = model.contents.cds_frame2 ;% last cds exon 
end ;


dim3 = size(transition_pointers,3);
transition_pointers(:,:,dim3+1) = tmp' ; % don't touch this---tmp has to be transposed!

a_trans = zeros(4,sum(~isinf(A(:)))) ;
% [from,to,val,segment]
k = 0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  pen = transition_pointers(idx,i,1) ;
  a_trans(1,k+1:k+length(idx))=i-1 ;
  a_trans(2,k+1:k+length(idx))=idx-1 ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  for q=1:length(pen) 
    idx_=find(model.seg_links(:,1)==pen(q)) ;
    if isempty(idx_), 
      assert(pen(q)==0) ; 
    else
      assert(length(idx_)==1) ;
      pen(q) = model.seg_links(idx_,2) ;
    end ;
  end ;
  a_trans(4,k+1:k+length(idx))= pen;
  k=k+length(idx) ;
end ;
%a_trans = a_trans' ;
%[tmp,idx] = sort(a_trans(:,2)) ;
%a_trans = a_trans(idx,:) ;

model.transition_pointers = transition_pointers ;
model.a_trans = a_trans;

return

function res = evaluate(fn_blocks, fn_pred_iter, model)

%l = load('/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp/lsl/tiling_on_Umaps_blocks/train/init.mat');
%l = load('/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp/lsl/Umaps/train/init.mat');
%l = load('/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp/lsl/on_Umaps_blocks/train/init.mat');
%l = load('/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp_new_model/lsl/tiling_old_model/train/init.mat');
%l = load('/fml/ag-raetsch/share/projects/genefinding/A_thaliana/jonas_new_genes/exp_new_model/lsl/abinitio_old_model/train/init.mat');
%PAR1 = l.PAR;
%clear l


%iteration1 = find_iteration(fn_pred);
%l = load(sprintf('%siteration_%i/blocks.mat', PAR1.FN.output_lsl.fn_pred,iteration1)) ;
%blocks1 = load_struct(PAR1.FN.input_lsl.fn_val_blocks, 'blocks');
if ischar(fn_blocks) && ischar(fn_pred_iter),
  blocks1 = load_struct(fn_blocks, 'blocks');

  for j=1:length(blocks1)
    fname = sprintf('%s_viterbi_block%i.mat',fn_pred_iter,blocks1(j).id);
    load(fname,'pred');
    blocks1(j).pred = pred;
    clear pred
  end
else
  blocks1 = fn_blocks ;
  assert(isempty(fn_pred_iter)) ;
end ;
%blocks1 = l.blocks;
%fn_pred_iter = [fn_pred 'iteration_' num2str(iteration1) '/'];

%blocks1([blocks1.strand]=='-')=[];%used for a check
%blocks1([blocks1.strand]=='+')=[];%used for a check
%perm = randperm(length(blocks1));
%blocks1 = blocks1(perm(1:floor(length(blocks1)*7/10)));

  res_all.cds_nucleotides.num_obsv =0; 
  res_all.cds_nucleotides.num_pred =0; 
  res_all.cds_nucleotides.num_corr =0; 
  res_all.cds_exons.num_obsv       =0; 
  res_all.cds_exons.num_pred       =0; 
  res_all.cds_exons.num_corr       =0; 
  res_all.cds_transcripts.num_obsv =0; 
  res_all.cds_transcripts.num_pred =0; 
  res_all.cds_transcripts.num_corr =0; 

%transcripts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for j=1:length(blocks1)
  segments = blocks1(j).truth(1).segments;
  res_all.cds_transcripts.num_obsv =res_all.cds_transcripts.num_obsv+max(segments(:,4)); 
  if ~isempty(blocks1(j).pred.genes{1})
    res_all.cds_transcripts.num_pred =res_all.cds_transcripts.num_pred+length(blocks1(j).pred.genes); 
  end
  for k=1:max(segments(:,4))
    for l=1:length(blocks1(j).pred.genes)
      if isempty(blocks1(j).pred.genes{1})
        continue;
      end
      trans1 = blocks1(j).pred.genes{l}(blocks1(j).pred.genes{l}(:,3)==model.segments.cds_exon,:);
      trans2 = segments(segments(:,3)==model.segments.cds_exon&segments(:,4)==k,1:3);
      if isequal(trans1,trans2)
         res_all.cds_transcripts.num_corr = res_all.cds_transcripts.num_corr +1;
      end
    end
  end
end

%exons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for j=1:length(blocks1)
  all_exons1 = [];
  all_exons2 = blocks1(j).truth(1).segments(blocks1(j).truth(1).segments(:,3)==model.segments.cds_exon, 1:3);
  for k=1:length(blocks1(j).pred.genes)
    if isempty(blocks1(j).pred.genes{1})
      continue;
    end
    trans1 = blocks1(j).pred.genes{k}(blocks1(j).pred.genes{k}(:,3)==model.segments.cds_exon,:);
    all_exons1 = [all_exons1;trans1];
  end

  common_exons = intersect(all_exons1, all_exons2, 'rows');
  res_all.cds_exons.num_pred=res_all.cds_exons.num_pred+size(all_exons2,1);
  res_all.cds_exons.num_obsv=res_all.cds_exons.num_obsv+size(all_exons1,1);
  res_all.cds_exons.num_corr=res_all.cds_exons.num_corr+size(common_exons,1);
  if size(common_exons,1)<size(all_exons2,1)
    %keyboard
  end
end


% Compute sensitivity and specificity
%res_all.cds_nucleotides.SN = res_all.cds_nucleotides.num_corr  /res_all.cds_nucleotides.num_obsv;
%res_all.cds_nucleotides.SP = res_all.cds_nucleotides.num_corr  /res_all.cds_nucleotides.num_pred;

res_all.cds_exons.SN       = res_all.cds_exons.num_corr        /res_all.cds_exons.num_obsv;
res_all.cds_exons.SP       = res_all.cds_exons.num_corr        /res_all.cds_exons.num_pred;


res_all.cds_transcripts.SN = res_all.cds_transcripts.num_corr  /res_all.cds_transcripts.num_obsv;
res_all.cds_transcripts.SP = res_all.cds_transcripts.num_corr  /res_all.cds_transcripts.num_pred;
res_all.cds_transcripts.F  = 2*res_all.cds_transcripts.SN*res_all.cds_transcripts.SP/(res_all.cds_transcripts.SP+res_all.cds_transcripts.SN)

res = res_all;

%save(sprintf('%sresults',PAR.fn_pred),'res');

return

return

function blocks = fix_blocks(blocks, model) ;
% blocks = fix_blocks(blocks, model) ;

assert(isequal(model, fix_model(model))) ;
w_fields = fieldnames(init_weights(model)) ;

for i=1:length(blocks),
	if isfield(blocks, 'truth')
		for j=1:length(blocks(i).truth),
			blocks(i).truth(j).weights = reorder_fields(blocks(i).truth(j).weights, w_fields) ;
        	end ;
	end
	if isfield(blocks, 'pred')
		for j=1:length(blocks(i).pred),
			blocks(i).pred(j).weights = reorder_fields(blocks(i).pred(j).weights, w_fields) ;
        	end ;
	end
end ;

return

function model=fix_model(model) ;
% model = fix_model(model) ;

model.contents = reorder_fields(model.contents, model.contents_fields) ;
model.lengths = reorder_fields(model.lengths, model.lengths_fields) ;
model.signals = reorder_fields(model.signals, model.signals_fields) ;
model.segments = reorder_fields(model.segments, model.segments_fields) ;

return

function decoys = gen_decoys(block, model, max_num)
% decoys = gen_decoys(block, model, max_num)

 
gene_idx = setdiff(unique(block.truth(1).segments(:,4)),0);
for j=1:length(gene_idx)
  idx = find(block.truth(1).segments(:,4)==gene_idx(j));
  genes{j} = block.truth(1).segments(idx,1:4) ;
end
decoys.genes = genes ;
%decoys.genes = block.truth(1).genes ;

modified = 0 ;
for i = 1:length(decoys.genes)
  genes = decoys.genes{i} ;
  idx = find(genes(:,3)==model.segments.cds_exon) ;
  idx = idx(2:end-1) ;
  idx1 = find(mod(genes(idx,2)-genes(idx,1),3)==0);
  if length(idx1)>0,
    decoys.genes{i}(idx(idx1)-1,2) = decoys.genes{i}(idx(idx1)+1,2) ;
    decoys.genes{i}(idx(idx1):idx(idx1)+1,:) = [] ;
    modified = 1 ;
  end ;
end ;

if ~modified, 
  decoys = [] ; 
  return ; 
end ;

idx = find(block.truth(1).segments(:,3)==model.segments.intergenic);
if isfield(model.segments, 'intercistronic')
  idx = [idx find(block.truth(1).segments(:,3)==model.segments.intercistronic)];
end
decoys.segments = [] ;
for i = 1:length(decoys.genes)
  decoys.segments = [decoys.segments; block.truth(1).segments(idx(i),1:4)];
  decoys.segments = [decoys.segments; decoys.genes{i}] ;
end ;
decoys.segments = [decoys.segments;block.truth(1).segments(end,1:4)];




[decoys.path, decoys.pos_idx, decoys.pos] = segmentation2path(decoys, block, model);

decoys = orderfields(decoys) ;
return ;




















truepos = block.truth.exons{1}(:,1:2) ;
truedescr = block.truth.exons{1}(:,3) ;
content_type =  unique(truedescr) ;

paths = {} ;
descr = {} ;
num = 0 ;

% simulate gene skip
gene_id = find(truedescr == getfield(model.contents,'intergenic')) ;
true_intergenic = truepos(idx) ;
for i = 1:size(gene_id,1)-2,
  num = num+1 ;
  paths{num} = [truepos(1:gene_id(i)-1,:);
                truepos(gene_id(i),1) truepos(gene_id(i)+1,2) 
                truepos(gene_id(i)+2:end,:)] ;
  descr{num} = [truedescr(1:gene_id(i));
                truedescr(gene_id(i)+2:end)] ;
end ;


% simulate merged gene 
for i = 1:size(gene_id,1)-1,
  num = num+1 ;
  paths{num} = [truepos(1:gene_id(i)-1,:);
                truepos(gene_id(i+1):end,:)] ;
  descr{num} = [truedescr(1:gene_id(i)-1);truedescr(gene_id(i+1):end)] ;
end ;


% simulate intron skip
for g = 1:length(gene_id)
  truepos_g = truepos(gene_id(g):gene_id(g+1)-1,:);
  for i = 1:size(truepos_g,1)-2,
    if truedescr(i) == truedescr(i+1)
      num = num+1 ;
      paths{num} = truepos ;
      
      paths{num} = [truepos(1:i-1,:);
                    truepos(i,1) truepos(i+1,2) 
                    truepos(i+2:end,:)] ;
      descr{num} = [truedescr(1:i);
                    truedescr(i+2:end)] ;
    end
  end ;
end

% simulate exon skip
for i=2:size(truepos,1)-1,
  num=num+1 ;
  paths{num}=[truepos(1:i-1,:);
              truepos(i+1:end,:)] ;
  descr{num} = [truedescr(1:i-1);truedescr(i+1:end)] ;
end ;

% simulate alternative donor
for w = 1:length(unique(content_type))
  truepos_w = truepos(:,truedescr==w)
  for i = 1:size(truepos_w,1)-1,
    num = num+1 ;
    paths{num} = truepos ;
    descr{num} = truedescr
    range = truepos_w(i,1)+3:truepos_w(i+1,1) ;
    idx = range(strfind(seq(range),'gt')) ;
    idx = idx(idx~=truepos(i,2)) ;
    idx2 = randperm(length(idx)) ;
    if isempty(idx2),
      num = num-1 ;
    else
      paths{num}(i,2) = idx(idx2(1)) ;
    end ;
  end ;
end

% simulate alternative acceptor
for w = 1:length(unique(content_type))
  truepos_w = truepos(:,truedescr==w)
  for i = 1:size(truepos,1)-1,
    num = num+1 ;
    paths{num} = truepos ;
    descr{num} = truedescr
    range = truepos_w(i,2):truepos_w(i+1,2)-3 ;
    idx = range(strfind(seq(range),'ag'))+2 ;
    idx = idx(idx~=truepos(i+1,1)) ;
    idx2 = randperm(length(idx)) ;
    if isempty(idx2),
      num = num-1 ;
    else
      paths{num}(i+1,1)=idx(idx2(1)) ;
    end ;
  end ;
end

% simulate alternative starts 
for w = 1:length(unique(content_type))
  truepos_w = truepos(:,truedescr==w)
  for i = 1:size(truepos_w,1)-1,
    num = num+1 ;
    paths{num} = truepos ;
    descr{num} = truedescr
    range = truepos_w(i,1)+3:truepos_w(i+1,1) ;
    idx = range(strfind(seq(range),'gt')) ;
    idx = idx(idx~=truepos(i,2)) ;
    idx2 = randperm(length(idx)) ;
    if isempty(idx2),
      num = num-1 ;
    else
      paths{num}(i,2) = idx(idx2(1)) ;
    end ;
  end ;
end


% simulate alternative PolyA

% simulate alternative TIS

% simulate alternative STOP


for i=1:num
  assert(all(paths{i}(:,1)<paths{i}(:,2))) ;
  assert(all(paths{i}(1:end-1,2)<paths{i}(2:end,1))) ;
end ;

idx = randperm(num) ;
paths = paths(idx(1:min(max_num,num))) ;


for i=1:length(paths)
  decoys(i).exons = paths{i} ;
  decoys(i).path = [] ;
end ;

return

function truth = gen_loss_matrix(truth, all_pos, gene_confirmed, model)
% truth = gen_loss_matrix(truth, all_pos, gene_confirmed, model)
% gene_confirmed: -1: predicted
%                  0: partially confirmed
%                  1: confirmed

% path = truth.path;
% pos = truth.pos;
% seg_path = ones(2,length(all_pos));
% [Pos,idx1,idx2]=intersect(all_pos,pos);
% st=1;
% for i = 2:length(idx1)
%   ii = find(model.a_trans(1,:)==(path(st));
%   ii2 = find(model.a_trans(2,ii)==(path(i));
%   seg_path(st:idx(i)) = model.a_trans(3,ii(ii2));
%   ii = find(all_pos>= Pos(i)- model.loss.zero_range(path(st),path(i))...
%              &all_pos<= Pos(i)+ model.loss.zero_range(path(st),path(i));
%   seg_path(2,idx1(i)) = 1; 
% end

seg = truth.segments;
%seg_path = ones(2,length(all_pos));
seg_path = zeros(2,max(all_pos));

for i=1:size(seg,1)
  seg_path(1, seg(i,1):seg(i,2)-1) = seg(i,3) ;
end ;

% confirmed genes are worth more than unconfirmed ones
seg_path(2,:) = model.loss.unconfirmed ;

gene_idx = setdiff(unique(truth.segments(:,4)),0);
for j=1:length(gene_idx)
  idx = find(truth.segments(:,4)==gene_idx(j));
  genes{j} = truth.segments(idx,1:3) ;
end

assert(length(genes)==length(gene_confirmed)) ;
for i=1:length(genes)
  if gene_confirmed(i)==1%confirmed
    start = genes{i}(1,1) ;
    stop  = genes{i}(end,2) ;
    seg_path(2, max(1,start-model.loss.confirmed_tol) : min(size(seg_path,2), stop+model.loss.confirmed_tol) ) = model.loss.confirmed ;
  elseif gene_confirmed(i)==0%partially confirmed 
    start = genes{i}(1,1) ;
    stop  = genes{i}(end,2) ;
    seg_path(2, max(1,start-model.loss.confirmed_tol) : min(size(seg_path,2), stop+model.loss.confirmed_tol) ) = (model.loss.confirmed+model.loss.unconfirmed)/2 ;
  end ;
end ;

pos = truth.pos;
[Pos,idx1,idx2]=intersect(all_pos, pos);
assert(length(Pos)==length(pos)) ;
signal_names = fieldnames(model.signals) ;
for i = 1:length(idx1)
  signal = model.states(truth.path(idx2(i))).signal ;
  if length(signal)~=1, continue ; end ;
  found = 0 ;
  for j=1:model.cnt_signals,
    if getfield(model.signals, signal_names{j})==signal
      signal = j ;
      found = 1 ;
      break ;
    end ;
  end ;
  assert(found~=0) ;
  tol = getfield(model.loss.zero_range, signal_names{signal}) ;
  if tol==0, continue ; end ;
  idx = max(1,(Pos(i) - tol)):min((Pos(i)+ tol),size(seg_path,2)) ;
  seg_path(2,idx) = 0 ; 
end

seg_path = seg_path(:, all_pos) ;

truth.seg_path = seg_path ;

return

function [pred, dummy] = gen_paths(PAR)
% pred = gen_paths(PAR)
  
%sg_fn = which('sg')
%unix(sprintf('ldd %s', sg_fn))

% dummy return argument
dummy=[] ;

%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;%GLOBAL_blocks;
if isempty(GLOBAL_cPAR),
  L = load(PAR.cPAR_fname, 'cPAR') ;
  GLOBAL_cPAR = L ;
  %blocks = load_struct(L.cPAR.blocks_fname) ; 
  %GLOBAL_blocks = blocks ;
else
  L = GLOBAL_cPAR ;
  %blocks = GLOBAL_blocks ;
end ;

%block = blocks(PAR.block_id) ;
block = PAR.block ;

% fix some field ordering
L.cPAR.model = fix_model(L.cPAR.model) ;
L.cPAR.parameters = reorder_fields(L.cPAR.parameters, L.cPAR.parameters_fields) ;
block = fix_blocks(block, L.cPAR.model) ;

model = L.cPAR.model;
parameters = L.cPAR.parameters ;

%%%%%%%%%%%%%%%%%

if ~isfield(PAR, 'max_len')
  PAR.max_len=[] ;
end ;
if ~isfield(PAR, 'prediction')
  PAR.prediction=0 ;
end ;
block.seq(find(block.seq~='A'&block.seq~='C'&block.seq~='G'&block.seq~='T'))='A';


% subset the positions to approximate the full dynamic program
if isfield(PAR, 'approximate') && isequal(PAR.approximate, 1),

  %if keyboard_allowed(),
  %  keyboard
  %end ;

  % make sure all previously predicted positions, true positions and strong positions are in the list
  used_positions = [] ;
  for i=1:length(block.pred),
	  used_positions = [used_positions block.pred(i).pos] ;
  end ;
  true_positions = [] ;
  for i=1:length(block.truth),
    true_positions = [true_positions block.truth.pos] ;
  end ;
  strong_positions_acc = block.Signals.acc.pos(find(block.Signals.acc.Conf_cum>0.75)) ;
  strong_positions_don = block.Signals.don.pos(find(block.Signals.don.Conf_cum>0.75)) ;
  strong_positions = union(strong_positions_acc, strong_positions_don) ;
  good_positions = union(union(used_positions, strong_positions), true_positions) ;

  [tmp,idx_pos]=intersect(block.all_pos, good_positions) ;

  block.all_pos = block.all_pos(idx_pos) ;
  block.pos = block.pos(idx_pos) ;

  if iscell(block.features)
    for i=1:length(block.features)
      block.features{i} = block.features{i}(:, idx_pos) ;
    end ;
  else
    block.features = block.features(:, idx_pos, :) ;
  end ;
  block.content_pred = block.content_pred(:, idx_pos) ;

  % update seg_path in truth (this is all_pos based)
  for t=1:length(block.truth)
    block.truth(t) = gen_loss_matrix(block.truth(t), block.all_pos, block.gene_status, model) ;
  end ;
else
  idx_pos = 1:length(block.all_pos) ;
end ;

% generate full feature matrix, if still in sparse format
if iscell(block.features)
  block.features = full_features(block.features);
end

[penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model, block.split, PAR.FN, parameters, PAR.max_len) ;

loss = model.loss.segments ;
if PAR.prediction,
  loss(:)=0 ;
  seg_path = zeros(2,length(block.all_pos)) ;
else
  %block.truth(1) = gen_loss_matrix(block.truth(1), block.pos, ...
  %                                 block.gene_confirmed, model) ;
  
  seg_path = block.truth(1).seg_path;
  % if the alternatives disagree, then don't incur a loss
  for i=2:length(block.truth)
    idx = find(block.truth(i).seg_path(1,:) ~= block.truth(1).seg_path(1,:)) ;
    seg_path(2,idx) = 0 ;
  end ;
end ;

if 0,
  if keyboard_allowed(), keyboard ; end ;
  old_model = model ;
  old_a_trans = a_trans ;
  old_block = block ;
  old_state_signals = state_signals ;
  
  [model, a_trans, block, state_signals] = duplicate_states4transcript_loss(model, a_trans, block, state_signals) ;
end ;

% SHOGUN interface
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  use_orf = 1;
  num_svms = 8;
  sg_set_plif(penalty_array)
  sg('init_dyn_prog', num_svms)
  if isfield(block, 'content_pred')
    block.content_pred(end+1:num_svms,:) = deal(0);
    sg('set_lin_feat', block.seq, int32(block.all_pos-1), block.content_pred);
  else
    sg('precompute_content_svms',block.seq,int32(block.all_pos-1), content_weights');
  end
  sg('set_model', model.transition_pointers, use_orf, int32(model.mod_words), int32(state_signals),int32(model.orf_info))
  sg('set_feature_matrix', block.features)
  if isfield(model, 'tiling')
    sg('precompute_tiling_features',double(block.intensities), int32(block.probe_local_pos-1), int32(model.tiling_plif_ids-1))
  end
  if isfield(model, 'rna_seq')
    if isfield(block, 'coverage')
      sg('precompute_tiling_features',double(block.coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1))
    elseif ~isfield(model, 'rna_seq_intron_track')% hack for backward compatibility
    sg('precompute_tiling_features',double(block.intron_coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_plif_ids-1)) 
    end
  end
  if isfield(model, 'rna_seq_intron_track')
    sg('precompute_tiling_features',double(block.intron_coverage), int32(0:length(block.seq)-1), int32(model.rna_seq_intron_track_plif_ids-1))
  end
  if length(PAR.viterbi_nbest)<2,
    PAR.viterbi_nbest(2) = 0 ;
  end ;
  [path_scores, path, ppos]= sg('best_path_trans', model.p', model.q', int32(PAR.viterbi_nbest), seg_path, a_trans, loss);


% very important check to assert that feature generation and veterbi decoding are equivalent
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~(isfield(PAR, 'skip_checks') && isequal(PAR.skip_checks,1)), 
  path_ = path(1,ppos(1,:)~=-1);
  ppos_ = ppos(1,ppos(1,:)~=-1);
  [p_deriv, q_deriv, a_deriv, penalty_deriv, scores, losses]= sg('best_path_trans_deriv', model.p', model.q', seg_path, a_trans, loss, int32(path_), int32(ppos_));
  %%% test
  A = model.A ;
  A(~isinf(A)) = parameters.transitions ;
  A(isinf(A)) = 0 ;
  score = sum(sum(a_deriv.*A))+sum(losses);
  for i=1:size(penalty_deriv,1)
    score = score + sum(penalty_deriv(i,:).*penalty_array{i}.penalties);
  end ;
  assert(all(losses>=0))
  if ~(abs(score-path_scores(1))<1e-4)
    fprintf('unexpected score differences: %1.3f %1.3f (diff=%1.3f)\n', ...
            score, path_scores, score-path_scores) ;
    if keyboard_allowed(),
      keyboard ;
    end ;
  end ;
end ;

pred = []; loss=[] ;
for i=1:sum(PAR.viterbi_nbest)
  pred(i).loss = nan ;
  pred(i).path = path(i,ppos(i,:)~=-1)+1;
  pred(i).pos_idx = ppos(i,ppos(i,:)~=-1)+1 ;
  pred(i).pos = block.all_pos(pred(i).pos_idx);
  [pred(i).segments, pred(i).genes] = path2segmentation(pred(i).path, pred(i).pos_idx, block, model);
  if ~PAR.prediction
    [pred(i).weights, pred(i).loss, pred_x(i).score, pred_x(i).losses, pred_x(i).scores, pred_x(i).ok] = compute_weights(pred(i), block, model, PAR.FN, parameters) ;   
    if isfield(PAR, 'approximate') && isequal(PAR.approximate, 1),
      if keyboard_allowed(),
        %keyboard
      end ;
    end ;
  end ;
  % undo position subseting in index to positions (done for approximation)
  pred(i).pos_idx = idx_pos(pred(i).pos_idx) ;
end
pred = orderfields(pred) ;

%disp(['saving pred to ' PAR.fn_save]) ;
cPAR = L.cPAR;
save(PAR.fn_save, 'pred', 'PAR', '-v7')

return



return

function flag=gen_paths_approximation_enough(P, pred)
% flag=gen_paths_approximation_enough(P, pred)

flag=1 ;

return ;

return

function [penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, FN,  parameters, max_len) ;
% [penalty_array,content_weights,state_signals,a_trans] = gen_penalty_array(model,cont_subset_id, parameters, max_len) ;
  
if nargin<4 || isempty(parameters)
  use_param = 0 ;
else
  use_param = 1 ;
end ;

content_names = fieldnames(model.contents) ;
%idx=sort_numeric_fields(model.contents) ;
%content_names = content_names(idx) ;

length_names = fieldnames(model.lengths) ;
%idx=sort_numeric_fields(model.lengths) ;
%length_names = length_names(idx) ;

signal_names = fieldnames(model.signals) ;
%idx=sort_numeric_fields(model.signals) ;
%signal_names = signal_names(idx) ;

if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling);
end
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq);
end
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generate penalty_array. set plif penalties to zero 

if nargin<5 || isempty(max_len)
  max_len = 0 ;
  for s = 1:model.cnt_lengths,
    len = getfield(model.lengths_range, length_names{s}) ;
    if len(2)>max_len,
      max_len = len(2) ;
    end ;
  end ;
end ;

penalty_array = {} ;
content_weights = [] ;
num_svms = 0 ;
for s = 1:model.cnt_contents,
  pen.id        = getfield(model.contents, content_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['contents_' content_names{s}] ;
  pen.limits    = getfield(model.boundaries.contents, content_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  % keyboard
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  num_svms = num_svms + 1 ;
  pen.use_svm   = num_svms ;
  if strcmp(content_names{s},'dummy')
    content_weights(num_svms, :) = deal(0);
  else
    %fn_content_all_models = [FN.output_cont.(content_names{s}).fn_SVMs '_partition=' num2str(cont_subset_id) '.mat'];
    if ~isempty(FN) && isfield(FN, 'output_cont') && isfield(FN.output_cont, content_names{s})
      fn_content_all_models = FN.output_cont.(content_names{s}).fn_SVMs;
    else
      fn_content_all_models = '';
    end
    if ~fexist(fn_content_all_models) || exist(fn_content_all_models, 'dir') 
      %fprintf('file %s with content svms not found, use zeros as weights\n', fn_content_all_models);
    elseif ~ismember('w', who_file(fn_content_all_models)), 
      %fprintf('no weight vector found in file %s, use zeros as weights\n', fn_content_all_models);
    else
      l = load(fn_content_all_models, 'w');
      content_weights(num_svms, :) = l.w;
    end
    clear l
  end
  penalty_array{end+1} = pen ;
end ;

if size(content_weights,1)<8
  % current shogun requirement
  content_weights(end+1:8,:)=0 ;
end ;

for s = 1:model.cnt_lengths,
  pen.id        = getfield(model.lengths, length_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['lengths_' length_names{s}] ;
  pen.limits    = getfield(model.boundaries.lengths, length_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = 'log(+1)' ;
  len=getfield(model.lengths_range, length_names{s}) ;
  pen.min_value   = len(1) ;
  pen.max_value   = min(len(2), max_len) ;
  pen.use_cache = 1 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;

for s = 1:model.cnt_signals,
  pen.id        = getfield(model.signals, signal_names{s}) ;
  assert(pen.id == length(penalty_array)+1) ;
  pen.name      = ['signals_' signal_names{s}] ;
  pen.limits    = getfield(model.boundaries.signals, signal_names{s}) ;
  pen.limits    = pen.limits(1:end-1) ;
  pen.penalties = zeros(1,length(pen.limits)) ;
  if ~use_param
    pen.penalties = zeros(1,length(pen.limits)) ;
  else
    pen.penalties = getfield(parameters, pen.name) ;
    pen.penalties = pen.penalties(:)' ;
  end ;
  pen.transform = '' ;
  pen.min_value   = -1e6 ;
  pen.max_value   = 1e6 ;
  pen.use_cache = 0 ;
  pen.use_svm   = 0 ;
  penalty_array{end+1} = pen ;
end ;

if isfield(model,'tiling')
  for s = 1:model.cnt_tiling,
    pen.id        = getfield(model.tiling, tiling_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['tiling_' tiling_names{s}] ;
    pen.limits    = getfield(model.boundaries.tiling, tiling_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end
if isfield(model,'rna_seq')
  for s = 1:model.cnt_rna_seq,
    pen.id        = getfield(model.rna_seq, rna_seq_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['rna_seq_' rna_seq_names{s}] ;
    pen.limits    = getfield(model.boundaries.rna_seq, rna_seq_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end
if isfield(model,'rna_seq_intron_track')
  for s = 1:model.cnt_rna_seq_intron_track,
    pen.id        = getfield(model.rna_seq_intron_track, rna_seq_intron_track_names{s}) ;
    assert(pen.id == length(penalty_array)+1) ;
    pen.name      = ['rna_seq_intron_track_' rna_seq_intron_track_names{s}] ;
    pen.limits    = getfield(model.boundaries.rna_seq_intron_track, rna_seq_intron_track_names{s}) ;
    pen.limits    = pen.limits(1:end-1) ;
    if ~use_param
      pen.penalties = zeros(1,length(pen.limits)) ;
    else
      pen.penalties = getfield(parameters, pen.name) ;
      pen.penalties = pen.penalties(:)' ;
    end ;
    % keyboard
    pen.transform = '' ;
    pen.min_value   = -1e6 ;
    pen.max_value   = 1e6 ;
    pen.use_cache = 0 ;
    num_svms = num_svms + 1 ;
    pen.use_svm   = num_svms ;
    penalty_array{end+1} = pen ;
  end ;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num_signals=0;
for i=1:model.cnt_states
  num_signals=max([num_signals length(model.states(i).signal)]);
end

state_signals=zeros(model.cnt_states, num_signals) ;
for i=1:model.cnt_states,
  state_signals(i,1:length(model.states(i).signal))=model.states(i).signal ;
end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


A = model.A ;
if use_param
  A(~isinf(A)) = parameters.transitions ;
  model.A = A;
end

a_trans = model.a_trans ;
k=0 ;
for i=1:size(A,1)
  idx = find(~isinf(A(i,:))) ;
  val = A(i,idx) ;
  assert(all(a_trans(1,k+1:k+length(idx))==i-1)) ;
  assert(all(a_trans(2,k+1:k+length(idx))==idx-1)) ;  
  a_trans(3,k+1:k+length(idx))=val ;    
  k=k+length(idx) ;
end ;
a_trans=a_trans' ;
[tmp,idx]=sort(a_trans(:,2)) ;
a_trans = a_trans(idx,:) ;


function idx=sort_numeric_fields(s)
% idx=sort_numeric_fields(s)

names=fieldnames(s) ;
values=[] ;
for i=1:length(names),
  values(i) = s.(names{i}) ;
end ;
[tmp,idx]=sort(values) ;

return

function model = gen_transition_matrix(model,PAR)

use=model.use;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define transitions

A = -inf(model.cnt_states, model.cnt_states);
p = -inf(model.cnt_states,1);
q = -inf(model.cnt_states,1) ;
p(model.state_ids.seq_start) = 0 ;
q(model.state_ids.seq_end) = 0 ;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% start constructing transition matrix A

orf_from = -1*ones(model.cnt_states,1) ;
orf_to = -1*ones(model.cnt_states,1) ;

orf_from(model.state_ids.acc(2:7))= [ 0 1 1 2 2 2] ;
orf_from(model.state_ids.tis)     = 0 ; % AGAGT case
if model.use.states.SL1 & model.use.states.SL2
  orf_from(model.state_ids.SL1(2)) = 0 ; % AGAGT case
  orf_from(model.state_ids.SL2(2)) = 0 ; % AGAGT case
elseif model.use.signals.trans
  orf_from(model.state_ids.trans(2)) = 0 ; % AGAGT case
end
orf_to(model.state_ids.don(2:7))  = [0 1 1 2 2 2] ;
orf_to(model.state_ids.cdsStop(1))   = 0 ;
orf_info=[orf_from, orf_to] ;

%%%%%%%%%%%
%% no gene / intergenic stuff
%%%%%%%%%%%
if ~use.model.one_gene,
  A(model.state_ids.seq_start,  model.state_ids.seq_end) = model.lengths.intergenic ;
end
A(model.state_ids.intergenic_long, model.state_ids.intergenic_long) = 0; % no penalty et al
A(model.state_ids.seq_start, model.state_ids.intergenic_long) = model.lengths.intergenic_long ;
A(model.state_ids.intergenic_long, model.state_ids.seq_end) = model.lengths.intergenic_long ;

%%%%%%%%%%%
% 5' UTR 
%%%%%%%%%%%
gene_start_states_in = [];
gene_start_states_out = [];
mRNA_start_states_out = [model.state_ids.tis];

if use.signals.tss,
  gene_start_states_in = [gene_start_states_in model.state_ids.tss] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.tss] ;
  if use.model.allow_missing_tss,
    assert(false) ;
    gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
  end ;
else
  assert(false)
  gene_start_states_in = [gene_start_states_in model.state_ids.tis] ;
end ;

if use.signals.trans,
  gene_start_states_in = [gene_start_states_in model.state_ids.SL1] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL1(1)] ;
  gene_start_states_out = [gene_start_states_out model.state_ids.SL2(1)] ;
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL1(2)] ;%'AGAGT cases'
  mRNA_start_states_out = [mRNA_start_states_out model.state_ids.SL2(2)] ;%'AGAGT cases'
  if use.signals.tss, 
    A(model.state_ids.tss, model.state_ids.SL1)    = model.lengths.transexon ;  
  end ;
  if use.segments.intercistronic & use.signals.cleave ,
    A(model.state_ids.cleave(1), model.state_ids.SL1) = model.lengths.intergenictrans ;
    A(model.state_ids.cleave(2), model.state_ids.SL1) = model.lengths.intercistronic ;
    A(model.state_ids.cleave(2), model.state_ids.SL2) = model.lengths.intercistronic ;
  end ;
end ;

% gene start
A(model.state_ids.seq_start,gene_start_states_in) = model.lengths.intergenic ;
if use.model.use_multi
  A(model.state_ids.intergenic_long, gene_start_states_in) = model.lengths.intergenic_long ; 
end
    
A(gene_start_states_out,model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(gene_start_states_out,model.state_ids.don(1)) = model.lengths.utr5exon ;
% spliced 5'utr
A(model.state_ids.acc(1), model.state_ids.don(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.acc(1)) = model.lengths.intron ;
% last 5'utr exon
A(model.state_ids.acc(1),model.state_ids.tis(1)) = model.lengths.utr5exon ;
A(model.state_ids.don(1), model.state_ids.tis(2)) = model.lengths.intron ; 


%%%%%%%%%%%
% CDS STUFF
%%%%%%%%%%%
A(mRNA_start_states_out, model.state_ids.cdsStop(1)) = model.lengths.single_cds_exon ; % trans(2) ok 
A(mRNA_start_states_out, model.state_ids.don(2:7)) = model.lengths.first_cds_exon ; % trans(2) ok
for i=2:7,
  A(model.state_ids.don(i), model.state_ids.acc(i)) = model.lengths.intron ;
  if isfield(model.state_ids, 'intron_long')
    A(model.state_ids.don(i), model.state_ids.intron_long(i-1)) = model.lengths.intron;
    A(model.state_ids.intron_long(i-1), model.state_ids.acc(i)) = model.lengths.intron_long;
    A(model.state_ids.intron_long(i-1),model.state_ids.intron_long(i-1)) = model.lengths.intron_long;
  end
end ;
A(model.state_ids.acc(2:7), model.state_ids.don(2:7)) = model.lengths.middle_cds_exon ;
A(model.state_ids.acc(2:7), model.state_ids.cdsStop(1)) = model.lengths.last_cds_exon ;
% agstop 
A(model.state_ids.don(2), model.state_ids.cdsStop(2)) = model.lengths.intron ;



%%%%%%%%%%%
% 3' UTR
%%%%%%%%%%%
A(model.state_ids.cdsStop, model.state_ids.don(8)) = model.lengths.utr3exon ;
A(model.state_ids.don(8), model.state_ids.acc(8)) = model.lengths.intron ;
A(model.state_ids.acc(8), model.state_ids.don(8)) = model.lengths.utr3exon ;

if use.signals.polya,
  % polyA has to be in last 3'UTR exon
  A(model.state_ids.cdsStop, model.state_ids.polya) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.polya)=model.lengths.utr3exon ;
end ;

gene_stop_states = [];    
if use.signals.cleave,
  gene_stop_states = [gene_stop_states model.state_ids.cleave(1)];
  A(model.state_ids.cdsStop, model.state_ids.cleave) = model.lengths.utr3exon ;
  A(model.state_ids.acc(8), model.state_ids.cleave) = model.lengths.utr3exon ;
  if use.signals.polya,
    A(model.state_ids.polya, model.state_ids.cleave) = model.lengths.polya_tail ;
  end
else
  gene_stop_states = [gene_stop_states model.state_ids.cdsStop];
  if use.signals.polya,
    gene_stop_states = [gene_stop_states model.state_ids.polya];
  end
end ;
A(gene_stop_states, model.state_ids.seq_end)    = model.lengths.intergenic ; 
A(gene_stop_states, model.state_ids.intergenic_long) = model.lengths.intergenic_long ; 
if use.model.use_multi,
  A(gene_stop_states, gene_start_states_in) = model.lengths.intergenic ;    
end ;

if use.segments.intercistronic 
  for k=model.state_ids.SL1,
    idx = find(A(:,k)==model.lengths.intergenic) ;
    A(idx,k) = model.lengths.intergenictrans ;   
  end ;
end
%%%%%%%%%%%
% end transition definition
%%%%%%%%%%%


%%% technical stuff for intergenic_long

transitions = [];
for i=1:length(gene_start_states_in)
  transitions = [transitions; model.state_ids.seq_start gene_start_states_in(i)];
  for j=1:length(gene_stop_states)
    transitions = [transitions; gene_stop_states(j) gene_start_states_in(i)];
    transitions = [transitions; gene_stop_states(j) model.state_ids.seq_end] ;
  end
end

model.intergenic_long.transitions = unique(transitions,'rows');
model.intergenic_long.max_short_length = model.lengths_range.intron(2) ;
model.intergenic_long.step = model.lengths_range.intron(2); % use the intron length as maximal step size
model.intergenic_long.long_segment_parts = model.lengths_range.intron(2)/2-25 ;
%model.intergenic_long.step = 30 ; % makes about 30 (step) * 40 (grid)=1200nt per step

%%% 

q(gene_stop_states) = 0 ;

model.gene_start_states_in = gene_start_states_in;
model.gene_start_states_out = gene_start_states_out;
model.mRNA_start_states_out = mRNA_start_states_out;
model.gene_stop_states = gene_stop_states;

model.A = A;
model.p = p;
model.q = q;
model.orf_info = orf_info;
model.active_transitions = find(~isinf(A)) ;
model.cnt_transitions = length(model.active_transitions) ;



return

function [QP,parameters]= init_path(PAR, blocks, lpenv, weight_names)
% [blocks,QP,parameters]= init_path(PAR)
%

%addpath /fml/ag-raetsch/share/software/matlab_tools/cplex10


%blocks = load_struct(PAR.FN.input_lsl.fn_training_blocks, 'blocks') ;


if fexist(PAR.FN.output_lsl.fn_init) & ~PAR.LSL.method.ignore_init_file
  disp('loading initialization data') ;
  load(PAR.FN.output_lsl.fn_init, 'QP') ;
  Old = load(PAR.FN.output_lsl.fn_init, 'PAR') ;
  assert(isequal(PAR.model,Old.PAR.model)) 
  assert(isequal(PAR.LSL.method,Old.PAR.LSL.method))
else
  % initialize matrices for quadratic program
  QP = initialize_QP(blocks, PAR.LSL.method, PAR.model, 1, weight_names) ;
  if ~fexist(fileparts(PAR.FN.output_lsl.fn_init))
    unix(sprintf('mkdir -p %s', fileparts(PAR.FN.output_lsl.fn_init)));
  end 
  save(PAR.FN.output_lsl.fn_init, 'QP', 'PAR', '-v7') ;
end ;


QP.res = zeros(size(QP.lb)) ;
QP.lpenv = lpenv;
QP = solve_qp(QP, PAR) ;

OBJ = sum(QP.res.*QP.f)+0.5*QP.res'*QP.Q*QP.res ;
parameters = init_weights(PAR.model) ;
parameters = vector2weights(QP.res(1:PAR.model.cnt_parameters)', parameters, PAR.weight_names) ;



return

function lpenv = init_solver() ;
%  lpenv = init_solver() ;
%

global OPTIMIZER


if ~isequal(OPTIMIZER, 'cplex');

  lpenv = inf ;

else

  global last_p_lp last_num_con
  last_p_lp = -1 ;
  %last_p_lp=[] ;
  %last_num_con=0 ;
  
  lpenv = cplex_license(1,1) ;
  if lpenv == 0,
    pause(100) ;
    rproc_rerun('could not obtain cplex license') ; 
  end ;
  
  how = lp_set_param(lpenv,'CPX_PARAM_AGGIND', 1,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_THREADS', 2,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_BARTHREADS', 2,1) ;
  disp('trying 4 threads') ;
  how = lp_set_param(lpenv,'CPX_PARAM_THREADS', 4,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_BARTHREADS', 4,1) ;
  disp('trying 8 threads') ;
  how = lp_set_param(lpenv,'CPX_PARAM_THREADS', 8,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_BARTHREADS', 8,1) ;
  how = lp_set_param(lpenv,'CPX_PARAM_PRECOMPRESS', 1,1) ;
  % how = lp_set_param(lpenv,'CPX_PARAM_SIFTALG', 1,1) ; % dual
  % how = lp_set_param(lpenv,'CPX_PARAM_SIFTALG', 4,1) ; % Barrier
  how = lp_set_param(lpenv,'CPX_PARAM_SIFTALG', 3,1) ; 
  how = lp_set_param(lpenv,'CPX_PARAM_SIFTDISPLAY', 2,1) ; % verbose
  
end

return

function weight_names=init_weight_names(model) ;
%  weight_names=weight_names(model) ;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weight_names{model.contents.(content_names{s})} = ['contents_' content_names{s}] ;
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weight_names{model.lengths.(length_names{s})} = ['lengths_' length_names{s}] ;
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weight_names{model.signals.(signal_names{s})} = ['signals_' signal_names{s}] ;
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weight_names{model.tiling.(tiling_names)} = ['tiling_' tiling_names{s}] ;
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weight_names{model.rna_seq.(rna_seq_names{s})} = ['rna_seq_' rna_seq_names{s}] ;
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weight_names{model.rna_seq_intron_track.(rna_seq_intron_track_names{s})} = ['rna_seq_intron_track_' rna_seq_intron_track_names{s}] ;
  end
end

%%% transitions
weight_names{end+1}='transitions' ;

return

function weights=init_weights(model) ;
% weights=init_weights(model) ;

weights = struct;

%%% content detectors
content_names = fieldnames(model.contents) ;
for s = 1:model.cnt_contents,
  weights = setfield(weights, ['contents_' content_names{s}], zeros(1,model.bins));
end

%%% length detectors
length_names = fieldnames(model.lengths) ;
for s = 1:model.cnt_lengths,
  weights = setfield(weights, ['lengths_' length_names{s}], zeros(1,model.bins));
end

%%% signal detectors
signal_names = fieldnames(model.signals) ;
for s = 1:model.cnt_signals,
  weights = setfield(weights, ['signals_' signal_names{s}], zeros(1,model.bins));
end

%%% tiling detectors
if isfield(model, 'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:model.cnt_tiling,
    weights = setfield(weights, ['tiling_' tiling_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq detectors
if isfield(model, 'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:model.cnt_rna_seq,
    weights = setfield(weights, ['rna_seq_' rna_seq_names{s}], zeros(1,model.bins));
  end
end
%%% rna_seq_intron_track detectors
if isfield(model, 'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:model.cnt_rna_seq_intron_track,
    weights = setfield(weights, ['rna_seq_intron_track_' rna_seq_intron_track_names{s}], zeros(1,model.bins));
  end
end

%%% transitions
weights.transitions = zeros(1,model.cnt_transitions) ;

return

function QP = initialize_QP(blocks, method, model, ...
                            generate_neg_constraints, weight_names)
% QP= initialize_QP(blocks , method, model, generate_neg_constraints=1)
  

INF = method.INF;
C_regul = method.par_ms.C_regul; 
bins = method.plif_bins;
cnt_plif_detector = model.cnt_signals + model.cnt_contents  ;
cnt_plif_lengths = model.cnt_lengths  ;
cnt_transitions = model.cnt_transitions ;
max_neg = method.max_neg ;
margin = method.margin;

%%%%%%%


content_names = fieldnames(model.contents) ;
id_contents = [] ;

for i=1:length(content_names),
  m = getfield(model.contents, content_names{i}) ;
  id_contents(end+1) = m;
  C_regul.id2C(m) = C_regul.contents;
end ;
signal_names = fieldnames(model.signals) ;
id_signals = [] ;
for i=1:length(signal_names),
  m = getfield(model.signals, signal_names{i}) ;
  id_signals(end+1) = m ;
  C_regul.id2C(m) = C_regul.signals;
end ;
length_names = fieldnames(model.lengths) ;
id_length = [] ;
for i=1:length(length_names),
  m = getfield(model.lengths, length_names{i}) ;
  id_length(end+1) = m ;
  C_regul.id2C(m) = C_regul.lengths;
end ;
if isfield(model,'tiling')
  tiling_names = fieldnames(model.tiling) ;
  id_tiling = [] ;
  for i=1:length(tiling_names),
    m = getfield(model.tiling, tiling_names{i}) ;
    id_tiling(end+1) = m ;
    C_regul.id2C(m) = C_regul.tiling;
  end ;
end
if isfield(model,'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  id_rna_seq = [] ;
  for i=1:length(rna_seq_names),
    m = getfield(model.rna_seq, rna_seq_names{i}) ;
    id_rna_seq(end+1) = m ;
    C_regul.id2C(m) = C_regul.rna_seq;
  end ;
end
if isfield(model,'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  id_rna_seq_intron_track = [] ;
  for i=1:length(rna_seq_intron_track_names),
    m = getfield(model.rna_seq_intron_track, rna_seq_intron_track_names{i}) ;
    id_rna_seq_intron_track(end+1) = m ;
    C_regul.id2C(m) = C_regul.rna_seq_intron_track;
  end ;
end

%%%%%%
monoton_id = [id_contents id_signals] ;
nof_monoton = 0;
for m = monoton_id,
  param_ids = model.plif2param{m};
  nof_monoton = nof_monoton + length(param_ids) - 1;
end ;
smooth_id = [monoton_id id_length] ;

if isfield(model,'tiling')
  nof_tiling = 0;
  for m = id_tiling,
    param_ids = model.plif2param{m};
    nof_tiling = nof_tiling + length(param_ids) - 1;
  end ;
  smooth_id = [smooth_id id_tiling] ;
end
if isfield(model,'rna_seq')
  nof_rna_seq = 0;
  for m = id_rna_seq,
    param_ids = model.plif2param{m};
    nof_rna_seq = nof_rna_seq + length(param_ids) - 1;
  end ;
  smooth_id = [smooth_id id_rna_seq] ;
end
if isfield(model,'rna_seq_intron_track')
  nof_rna_seq_intron_track = 0;
  for m = id_rna_seq_intron_track,
    param_ids = model.plif2param{m};
    nof_rna_seq_intron_track = nof_rna_seq_intron_track + length(param_ids) - 1;
  end ;
  smooth_id = [smooth_id id_rna_seq_intron_track] ;
end

nof_smooth = 0;
for m = smooth_id,
  param_ids = model.plif2param{m};
  nof_smooth = nof_smooth + length(param_ids) - 1;
end ;

nof_trans_smooth = model.cnt_transitions ;
nof_smooth = nof_smooth + nof_trans_smooth ;

%%%%%%

clear A A1 A1_sparse
A1 = zeros(10000, model.cnt_parameters) ;
A1_sparse = [] ;
A2 = sparse([],[],[],length(blocks)*max_neg, length(blocks)+nof_smooth,...
            length(blocks)*max_neg) ; 

q = 0 ; r = 0 ; q2 = 0 ;
num_complete = 0 ;


for block_id = 1:length(blocks)
  if mod(block_id,50)==0, 
    block_id
  end ;
  block = blocks(block_id);
  if generate_neg_constraints,
    % change: max_path was not initialized before
    max_path = length(block.pred);
    for i=1:max_path,
      q = q+1 ; 
      q2 = q2+1 ; 
      A1(q2,1:model.cnt_parameters) = weights2vector(block.pred(i).weights, weight_names)-weights2vector(block.truth(1).weights, weight_names) ;    
      if all(abs(A1(q2,:))<1e-3)
        error('haeh')
      end ;
      if q2==size(A1,1)
        A1_sparse = [A1_sparse; sparse(A1)] ;
        q2 = 0 ;
        A1(:) = 0 ;
      end ;
      A2(q, block_id)= -1 ; 
    end ;
  end ;
end ;

A1 = [A1_sparse; sparse(A1(1:q2,:))] ;
assert(q==size(A1,1));
clear A1_sparse
A2 = A2(1:q,:) ;
A = [A1 A2] ;
clear A1 A2
b = -margin*ones(q,1) ;

% penalization of 1st derivatives
B = spzeros(2*nof_smooth, model.cnt_parameters+length(blocks)+nof_smooth, ...
            6*nof_smooth) ;
QP.f = zeros(model.cnt_parameters+length(blocks)+nof_smooth,1) ;
QP.f( model.cnt_parameters+(1:length(blocks)) ) = 1;
q = 0 ;
for m = smooth_id
  param_ids = model.plif2param{m};
  for i = param_ids(1:end-1) 
    q = q + 1;
    B(q+q-1,i) = 1 ;
    B(q+q-1,i+1)=-1 ;
    B(q+q-1,model.cnt_parameters+length(blocks)+q)=-1 ;
    B(q+q,i)   = -1 ;
    B(q+q,i+1) = +1 ;
    B(q+q,model.cnt_parameters+length(blocks)+q)=-1 ;
    QP.f(model.cnt_parameters+length(blocks)+q) = C_regul.id2C(m) ;
  end;
end ;

% add L1 constraints and penalty for transitions
param_ids = model.transitions2param ;
for i = param_ids
  q = q + 1;
  B(q+q-1,i) = 1 ;
  B(q+q-1,model.cnt_parameters+length(blocks)+q)=-1 ;
  B(q+q,i)   = -1 ;
  B(q+q,model.cnt_parameters+length(blocks)+q)=-1 ;
  QP.f(model.cnt_parameters+length(blocks)+q) = C_regul.transitions ;
end;
% figure; imagesc(B)
assert( q == nof_smooth );
A = [A;B]; 
b = [b; zeros(2*nof_smooth,1)] ;
clear B


% monotonicity for all PLIFS

C = spzeros(nof_monoton, model.cnt_parameters+length(blocks)+nof_smooth, 2*nof_monoton) ;
q = 0 ;
for m = monoton_id,
  param_ids = model.plif2param{m};
  for i = param_ids(1:end-1)
    q = q+1 ;
    C(q,i) = 1 ;
    C(q,i+1) = -1 ;
  end ;
end ;
% figure;imagesc(C)
QP.A = [A;C]; 
QP.b = [b; zeros(nof_monoton,1)] ;

if isfield(model,'tiling')
  T = spzeros(nof_tiling, model.cnt_parameters+length(blocks)+nof_smooth, 2*nof_tiling) ;
  q = 0 ;
  for m = id_tiling,
    param_ids = model.plif2param{m};
    tiling_names = fieldnames(model.tiling);
    for j=1:length(tiling_names), if model.tiling.(tiling_names{j})==m, break;end,end
    name = tiling_names{j};
    if ismember(name,{'intergenic','intron'})
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = -1 ;
        T(q,i+1) = 1 ;
      end ;
    else
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = 1 ;
        T(q,i+1) = -1 ;
      end ;
    end
  end ;
%  figure;imagesc(T)
  QP.A = [QP.A;T]; 
  QP.b = [QP.b; zeros(nof_tiling,1)] ;
end
if isfield(model,'rna_seq')
  T = spzeros(nof_rna_seq, model.cnt_parameters+length(blocks)+nof_smooth, 2*nof_rna_seq) ;
  q = 0 ;
  for m = id_rna_seq,
    param_ids = model.plif2param{m};
    rna_seq_names = fieldnames(model.rna_seq);
    for j=1:length(rna_seq_names), if model.rna_seq.(rna_seq_names{j})==m, break;end,end
    name = rna_seq_names{j};
    if ismember(name,{'intergenic','intron'})
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = -1 ;
        T(q,i+1) = 1 ;
      end ;
    else
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = 1 ;
        T(q,i+1) = -1 ;
      end ;
    end
  end ;
%  figure;imagesc(T)
  QP.A = [QP.A;T]; 
  QP.b = [QP.b; zeros(nof_rna_seq,1)] ;
end
if isfield(model,'rna_seq_intron_track')
  T = spzeros(nof_rna_seq_intron_track, model.cnt_parameters+length(blocks)+nof_smooth, 2*nof_rna_seq_intron_track) ;
  q = 0 ;
  for m = id_rna_seq_intron_track,
    param_ids = model.plif2param{m};
    rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track);
    for j=1:length(rna_seq_intron_track_names), if model.rna_seq_intron_track.(rna_seq_intron_track_names{j})==m, break;end,end
    name = rna_seq_intron_track_names{j};
    if ismember(name,{'cds_exon', 'utr_exon'})
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = -1 ;
        T(q,i+1) = 1 ;
      end ;
    else
      for i = param_ids(1:end-1)
        q = q+1 ;
        T(q,i)   = 1 ;
        T(q,i+1) = -1 ;
      end ;
    end
  end ;
%  figure;imagesc(T)
  QP.A = [QP.A;T]; 
  QP.b = [QP.b; zeros(nof_rna_seq_intron_track,1)] ;
end

QP.lb = [-INF*ones(model.cnt_parameters,1) ; zeros(length(blocks)+nof_smooth,1)] ;
QP.ub = [+INF*ones(model.cnt_parameters+length(blocks)+nof_smooth,1)]  ;
% QP.ub = [];

Q = spzeros(model.cnt_parameters+length(blocks)+nof_smooth) ;
for i=1:model.cnt_parameters,
  if( i < model.cnt_parameters-model.cnt_transitions )
    Q(i,i) = C_regul.plif_ys_sq;
  else
    Q(i,i) = C_regul.transitions_sq ;
  end;
end ;
for i=model.cnt_parameters+length(blocks)+(1:nof_smooth-nof_trans_smooth)
  Q(i,i) = Q(i,i)+C_regul.smoothness_sq ;
end ;
QP.Q = Q;

if isfield(method, 'fn_regul_solution') ,
  L=load(method.fn_regul_solution) ;
  fprintf('regularizing with C=%1.2f against solution (norm=%1.2f)\nfrom file %s \n', ...
          C_regul.solution, norm(L.QP.res(1:model.cnt_parameters)), ...
          method.fn_regul_solution) ;
  assert(L.TRAIN_PAR.model.cnt_parameters==model.cnt_parameters) ;
  for k=1:L.TRAIN_PAR.model.cnt_parameters
    QP.Q(k,k) = QP.Q(k,k) + C_regul.solution ;
    QP.f(k) = QP.f(k) - C_regul.solution*L.QP.res(k) ;
  end ;
end ;

%%   checks
r = zeros(size(QP.f)) ;
r(model.cnt_parameters+(1:length(blocks))) = 1;
assert(all(QP.A*r-QP.b==0)) ;

return

function blocks = initialize_blocks(blocks, model, max_neg, FN)

% blocks = initialize_blocks(blocks, model)
% - transforms conservation to DNA_alphabet 
% - generates weights and loss for truth
% - adds decoy paths 

%%%%%%%

if isfield(model, 'use_conservation') && model.use_conservation 
  disp('Converting conservation levels') ;
  for id = 1:length(blocks),
    blocks(id).genestr_conserv = conservation2DNA_alphabet(blocks(id).conservation) ;
  end ;  
end ;


%%%%%%

blocks(1).pred = struct;
fprintf('Processing %i blocks: ', length(blocks)) ;
for block_id = 1:length(blocks)
  if mod(block_id,50)==0, 
    fprintf('%i ... ', block_id) ;
  end ;
  block = blocks(block_id);
  block.truth(1).seg_path = [] ;
  block.seq=upper(block.seq);
  notacgt_idx = find(block.seq~='A'&block.seq~='C'&block.seq~='G'&block.seq~='T');
  if length(notacgt_idx)>0
    fprintf('Change %i non-[ACGT] symbold to ''A''\n',length(notacgt_idx))
    block.seq(notacgt_idx)='A';
  end
  for t=1:length(block.truth)
    %block.truth(t) = gen_loss_matrix(block.truth(t), block.all_pos, block.gene_confirmed, model) ;
    block.truth(t) = gen_loss_matrix(block.truth(t), block.all_pos, block.gene_status, model) ;
    [weights,loss,score,losses,scores] = compute_weights(block.truth(t), block, model, FN,[],1);
    if t==1, assert(loss==0) ; end
    if isempty(weights)
      %block.truth(t).weights = blocks(1).truth(1).weights ; % hack
      error('empty weights') ;
    else
      block.truth(t).weights = weights ;
    end
  end
  % generate wrong paths
  block.pred = gen_decoys(block, model, max_neg) ;
  % draw_pred(block.truth,block.pred,model)
  max_path = length(block.pred) ;
  good_idx = [] ;
  for i = 1:max_path,
    [weights,loss] = compute_weights(block.pred(i), block, model,FN,[],0);
    if isempty(weights)
      block.pred(i) = [];
      warning('invalid decoy path')
      max_path = max_path-1;
    else
      if loss>0,
        good_idx(end+1) = i ;
        assert(loss~=0) ;
        block.pred(i).weights = weights ;
        block.pred(i).loss = loss ;
      end ;
    end
  end
  block.pred = block.pred(good_idx) ;
  blocks(block_id) = block ;
  clear weights ;
end ;
fprintf('Done.\n') ;
return

function [model] = make_model(organism,LSL,Signals,old)
% [model] = make_model(bins)

if nargin<4
  old=0;
end
  
model.bins = LSL.method.plif_bins ;
% model.use_gc =  LSL.method.use_gc;
model.use = LSL.method.use;
polya_sixmers = Signals.polya.consensus;

model = define_segments(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define features

model = define_contents(model);
model = define_lengths(model,organism);
model = define_signals(model,polya_sixmers);
model = define_addContentFeatures(model);

model.cnt_plifs = model.cnt_contents+model.cnt_lengths+model.cnt_signals+model.cnt_tiling;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% remember the ordering of these fields
model.segments_fields = fieldnames(model.segments) ;
model.contents_fields = fieldnames(model.contents) ;
model.lengths_fields = fieldnames(model.lengths) ;
model.signals_fields = fieldnames(model.signals) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%define states

model = define_states(model);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% define mappings between length, contents and segments

model = define_mappings(model);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

model = gen_transition_matrix(model) ;
% transition_pointers used to be called penalties
model = define_transition_pointers(model);
if old
  model.penalties = model.transition_pointers;
end
model.A(~isinf(model.A))=0 ;



model.cnt_parameters = model.cnt_transitions + model.cnt_plifs*model.bins ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% define loss 
model =  define_loss(model);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mappings of PLIFS to parameters

if ~old
  map_plif2param = init_weights(model);
  map_plif2param = rmfield(map_plif2param,'transitions');
  weights_name = fieldnames(map_plif2param);
  start=0;
  for w=1:length(weights_name)
    len=length(getfield(map_plif2param, weights_name{w}));
    model.plif2param{w} = start+(1:len);
    start = start + len;
  end
  model.transitions2param = start+1:model.cnt_parameters ;
  
  assert(model.cnt_parameters-model.cnt_transitions==start)
end

return

function [segments,genes,ok] = path2segmentation(my_path,my_pos_idx,block, model)
% [segmentation,ok] = path2segmentation(path,pos_idx,block, model)
%pos = [0     length(block.all_pos)-1] ;
%path = [model.state_ids.seq_start    model.state_ids.seq_end]-1 ;
  
segment_names = fieldnames(model.segments) ;
segments = [] ;
ok = 1 ;
genes = {} ;

my_pos_idx(my_path==getfield(model.state_ids,'intergenic_long'))=[];
my_path(my_path==getfield(model.state_ids,'intergenic_long'))=[];
% same for intron long
if isfield(model.state_ids,'intron_long')
  for i=getfield(model.state_ids,'intron_long')
    del_idx=my_path==i;
    my_pos_idx(del_idx)=[];
    my_path(del_idx)=[];
  end
end
for i=1:length(my_path)-1
  lengths_idx = model.transition_pointers(my_path(i+1), my_path(i)) ;
  if my_path(i+1) == model.state_ids.tis(2),
    if ~(my_path(i) == model.state_ids.don(1)) ;
      error('invalid path') ;
      %warning('invalid path') ;
      ok=0 ;
      return ;
    end ;
    % handle the agatg-case properly 
    segment_type = model.seg_links(find(model.seg_links(:,1)==lengths_idx),2) ;
    segments = [segments; [block.all_pos(my_pos_idx(i)) block.all_pos(my_pos_idx(i+1)), model.segments.intron ]] ;
    continue ;
  end ;
  
  if lengths_idx~=0
    segment_type = model.seg_links(find(model.seg_links(:,1)==lengths_idx),2) ;
    % hack
    if isfield(model.segments,'intercistronic')&&segment_type==model.segments.intercistronic & ...
          block.all_pos(my_pos_idx(i+1)) - block.all_pos(my_pos_idx(i)) > model.lengths_range.intercistronic(2),
      segment_type = model.segments.intergenic ;
    end 
  else
    segment_type = 0 ;
    warning('unable to decode path') ;
    ok = 0 ;
  end ;
  segments = [segments; [block.all_pos(my_pos_idx(i)) block.all_pos(my_pos_idx(i+1)),segment_type ]] ;
end

segments(1,1) = 1;
segments(end,2) = length(block.seq);
idx =  find(segments(:,3) ==getfield(model.segments,'intergenic'));
if isfield(model.segments,'intercistronic')
  idx = [idx find(segments(:,3) ==getfield(model.segments,'intercistronic'))];
end
n_genes = length(idx)-1 ;
genes{1} = [];
for i=1:n_genes
  genes{i} = segments(idx(i)+1:idx(i+1)-1,:);
end

return

function [blocks,PAR,P] = pred_path(PAR, blocks)
% [blocks,PAR,P] = pred_path(PAR [, blocks])
%
%
check_split = 0 ;

if ~isfield(PAR, 'fn_log')
  fid=1;
PAR.evaluate = 0;
else  
  fid = fopen( sprintf('%spred.txt',PAR.fn_log),'a+') ;
end
fprintf(fid,'\n\npred_path \n');
fprintf(fid,'date: %s\n',date);
%fprintf(fid,'\n start \n');

if ~isfield(PAR, 'gen_paths_func')
  PAR.gen_paths_func = 'gen_paths' ; 
else
  PAR.gen_paths_func 
end ;

%% parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isfield(PAR, 'fn_gene_predictor')
  if ~isfield(PAR, 'iteration')
    iteration = 1 ;
    while (1)
      fname = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration) ;
      if ~fexist(fname), iteration=iteration-1; break ; end 
      iteration=iteration+1 ;
    end ;
  else
    iteration = PAR.iteration;
  end ;
  fname = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration) ;
  fprintf('Loading iteration %i\n', iteration) ;
  if check_split,
    load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
    TRAIN_PAR.model = fix_model(TRAIN_PAR.model) ;
    blocks_train = load_struct(sprintf('%s_iteration%i_blocks.mat', PAR.FN.output_lsl.fn_train,iteration)) ;
    blocks_train = fix_blocks(blocks_train, TRAIN_PAR.model) ;
    idx = [blocks_train.id] ;
    clear blocks_train
  else
    load(fname, 'QP', 'TRAIN_PAR', 'num_examples');
    TRAIN_PAR.model = fix_model(TRAIN_PAR.model) ;
  end ;
else 
  load(PAR.fn_gene_predictor, 'QP', 'TRAIN_PAR', 'num_examples');
  TRAIN_PAR.model = fix_model(TRAIN_PAR.model) ;
end

%% load data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<2
  load(PAR.fn_blocks, 'blocks') ;
  blocks = fix_blocks(blocks, TRAIN_PAR.model) ;
end
if isfield(PAR,'fn_splits')
  load(PAR.fn_splits, 'blocks_split') ; 
  block_idx = [blocks_split{PAR.pred_split_idx}] ;
  if check_split
    if ~isempty(intersect(idx, block_idx))
      disp('test and training are overlapping') ;
      keyboard
      disp('correcting indices') ;
      block_idx = setdiff([blocks_split{:}], idx) ;
    end ;
  end ;
  blocks = blocks(block_idx) ;
end
fprintf('Predicting on %i blocks\n', length(blocks)) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isfield(TRAIN_PAR.model, 'use_conservation') && TRAIN_PAR.model.use_conservation 
  disp('Converting conservation levels') ;
  for id = 1:length(blocks),
    blocks(id).genestr_conserv = conservation2DNA_alphabet(blocks(id).conservation) ;
  end ;  
end ;

if isfield(PAR.LSL.method, 'add_lin_feat') && PAR.LSL.method.add_lin_feat
  assert(iscell(PAR.LSL.method.add_feat_fun))
  assert(iscell(PAR.FN.input_lsl.fn_lin_feat_data))
  assert(length(PAR.FN.input_lsl.fn_lin_feat_data)==length(PAR.LSL.method.add_feat_fun))
  for feat_type=1:length(PAR.LSL.method.add_feat_fun)
    fprintf('load linear features with function: %s from file: %s\n', PAR.LSL.method.add_feat_fun{feat_type}, PAR.FN.input_lsl.fn_lin_feat_data{feat_type}) ;
    blocks = feval(PAR.LSL.method.add_feat_fun{feat_type}, blocks, PAR.FN.input_lsl.fn_lin_feat_data{feat_type}, 80);
  end
end
%if isfield(PAR,'use_tiling')&&PAR.use_tiling&& ~isfield(blocks,'intensities')
%  blocks=add_intensities2blocks(blocks,PAR.FN.input_lsl.fn_tiling_data,80);
%end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%if isfield(PAR, 'use_gc') && ~PAR.use_gc
%  disp('Removing GC splice sites') ;
%  for id = 1:length(blocks),
%    idx=find(blocks(id).genestr(blocks(id).details.pos.don+1)=='T') ;
%    blocks(id).details.pos.don=blocks(id).details.pos.don(idx) ;
%    blocks(id).details.score.don=blocks(id).details.score.don(idx) ;
%    blocks(id).details.output.don=blocks(id).details.output.don(idx) ;
%  end ;  
%end ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% avoid multiple loading of the big constant parameter file
global GLOBAL_cPAR ;
GLOBAL_cPAR=[] ;

cPAR=[] ;
cPAR.model = TRAIN_PAR.model ;

if isfield(PAR.model, 'intron_list'),
  cPAR.model.intron_list = PAR.model.intron_list ;
end ;

%cPAR.blocks_fname = ['~/' tempname '.mat'] ;
%cPAR.blocks = blocks ;
[xis, cPAR.parameters] = res2param(QP.res, num_examples, TRAIN_PAR.model, ...
                                   QP, 0, TRAIN_PAR.weight_names);
cPAR.parameters_fields = fieldnames(cPAR.parameters) ;

%% ADD offset to tss transitions to enhance sensitivity
if isfield(PAR.LSL.method,'tss_offset')
  error('not implemented') ;
  %cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.tss-1)) = cPAR.parameters.transitions(find(cPAR.model.a_trans(2,:)==cPAR.model.state_ids.tss-1))+PAR.LSL.method.tss_offset ;
end
P=[] ;
P.cPAR_fname = sprintf('~/tmp/%i-%1.10f.mat', round(rand(1)*1e8), now) ;
assert(~fexist(P.cPAR_fname)) ;
save(P.cPAR_fname, 'cPAR', '-v7') ;

%save_struct(cPAR.blocks_fname, blocks) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

P.viterbi_nbest = 1;

run_locally = rproc_policy('pred_path:gen_paths', [], length(blocks)) ;

[engine, environment] = determine_engine() ;

num = 0 ; model = TRAIN_PAR.model ;
jobinfo = rproc_empty(1) ;
if run_locally,
  fprintf(fid,'\nComputing %i dynamic programs ...',length(blocks));
else
  fprintf(fid,'\nSubmitting %i dynamic programs ...',length(blocks));
end ;
for block_id = 1:length(blocks),

  %idx = find(blocks(block_id).truth(1).segments(:,3)~=model.segments.intergenic) ;
  %max_len = max(500, max(blocks(block_id).truth(1).segments(idx,2)- ...
  %                       blocks(block_id).truth(1).segments(idx,1))) ;
  %max_len = max_len*1.2+1000 
  %P.max_len = max_len ;
  fprintf('%i ... ', block_id) ;

  P.block_id = block_id ;
  P.block = blocks(block_id) ;
  P.fn_save = sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, blocks(block_id).id);
  system(sprintf('rm -f %s', P.fn_save)) ;

  %if fexist(P.fn_save),
    %continue; 
  %end 
  P.prediction = 1 ;
  P.FN = PAR.FN;
  options = PAR.RPROC.options;
  options.identifier = ['G'  num2str(block_id)] ;
  %options.start_dir =  fileparts(which(PAR.gen_paths_func));
  options.express = 0;
  options.verbosity = 2 ;

  if ~run_locally,
    [mem_req, time_req, opts]=rproc_memtime_policy('gen_paths', length(P.block.seq), options) ;
    num = num+1 ;
    if ~isfield(options, 'immediately_bg') || options.immediately_bg==0,
      if PAR.RPROC.exm_per_batch==1,
        jobinfo(num) = rproc(PAR.gen_paths_func, P, mem_req, opts, time_req) ;
      else
        jobinfo(num) = rproc_create(PAR.gen_paths_func, P, mem_req, opts, time_req) ;
      end ;
    else
      jobinfo(num) = rproc(PAR.gen_paths_func, P, mem_req, opts, time_req) ;
    end ;
  else
    pred = feval(PAR.gen_paths_func, P) ;
  end ;
end
fprintf('Done.\n') ;

if ~run_locally,
  if ~isfield(options, 'immediately_bg') || options.immediately_bg==0,
    if PAR.RPROC.exm_per_batch==1,
        [jobinfo, num_crashed] = rproc_wait(jobinfo, 60, 1, -1, 1);
    else
      [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.RPROC.exm_per_batch) ;
      [meta_jobinfo,num_crashed] = rproc_wait(meta_jobinfo, 60, 1, -1, 1);
    end ;
  else
    [jobinfo,num_crashed] = rproc_wait(jobinfo, 60, 1, -1, 1);
  end ;
  if num_crashed>0,
    error('%i jobs crashed. Cannot continue.', num_crashed) ;
  end ;
end ;

blocks(1).pred = struct;
for block_id = 1:length(blocks),
  block = blocks(block_id) ;
  while ~fexist(sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, block.id ))
    fprintf('.') ;
    pause(10)
  end
  load(sprintf('%s_viterbi_block%i.mat', PAR.fn_pred, block.id ),'pred');   
  blocks(block_id).prediction = pred; 
end

unix(['rm -f ' P.cPAR_fname]) ;

%save([PAR.fn_pred 'blocks'], 'blocks', 'PAR', 'P') ;
if fid>2
  fclose(fid);
end

return

function [blocks,res]=predict(PAR, blocks)
% [blocks,res]=predict(PAR, blocks)

%addpath(PAR.FN.source_directory)
%paths

%if nargin<3 || strcmp(set,'test')
%  testset=1;
%else
%  testset=0;
%end
%
%%PAR.submit=1;
%if ~isfield(PAR, 'iteration')||isempty(PAR.iteration) 
%  PAR.iteration = find_iteration(PAR);
%end
%if testset&&(~isfield(PAR, 'fn_pred')||isempty(PAR.fn_pred))
%  PAR.fn_pred = [PAR.FN.output_lsl.fn_pred 'iteration_' num2str(PAR.iteration) '/'];
%  PAR.fn_blocks = PAR.FN.input_lsl.fn_val_blocks;
%elseif ~testset&&(~isfield(PAR, 'fn_pred')||isempty(PAR.fn_pred))
%  PAR.fn_pred = [PAR.FN.output_lsl.fn_pred 'trainset_iteration_' num2str(PAR.iteration) '/'];
%  PAR.fn_blocks = PAR.FN.input_lsl.fn_training_blocks;
%end

PAR.model = fix_model(PAR.model) ;

if nargin<2
  blocks = load_struct(PAR.fn_blocks, 'blocks');
  blocks = fix_blocks(blocks, PAR.model) ;
end

unix(sprintf('mkdir -p %s',PAR.fn_pred));
blocks = pred_path(PAR, blocks) ;

res = [] ;

if isfield(PAR, 'evaluate') && ~PAR.evaluate
  return
end

res = evaluate(PAR.fn_blocks, PAR.fn_pred, PAR.model)
fprintf('save results to %sresults.mat\n',PAR.fn_pred) 
save(sprintf('%sresults.mat',PAR.fn_pred), 'res', '-v7');

%printf('Nucleotide level accuracy:\n') ;
%print_struct(1, res.cds_nucleotides) ;

fprintf('Exon level accuracy:\n')
print_struct(1, res.cds_exons) ;

fprintf('Transcript level accuracy:\n')
print_struct(1, res.cds_transcripts) ;

fprintf('\n\n\n') ;

res

return

genome_info = init_genome(PAR.FN.input.fn_genome_config);

for j=1:length(blocks)
  load(sprintf('%s_viterbi_block%i',PAR.fn_pred,blocks(j).id),'pred');
  blocks(j).pred = pred;
  clear pred
end

if 1
  load(PAR.FN.output.fn_genes_merged,'genes');
else
  load(PAR.FN.output.fn_genes_anno,'genes');
  for j=1:length(genes)
    genes(j).anno_id = genes(j).id;
    if strcmp(genes(j).transcripts{1},'dummy')
      genes(j).mask_on = 1;
    end
%    genes(j).mask_on = max(genes(j).transcript_status)<1;
  end
end
  res_all.cds_nucleotides.num_obsv =0; 
  res_all.cds_nucleotides.num_pred =0; 
  res_all.cds_nucleotides.num_corr =0; 
  res_all.cds_exons.num_obsv       =0; 
  res_all.cds_exons.num_pred       =0; 
  res_all.cds_exons.num_corr       =0; 
  res_all.cds_transcripts.num_obsv =0; 
  res_all.cds_transcripts.num_pred =0; 
  res_all.cds_transcripts.num_corr =0; 

for j=1:length(blocks)
  if ~isempty(blocks(j).pred.genes{1})
    genes_pred = blocks2genes(blocks(j),PAR.model,'pred',genome_info);
  else
    genes_pred = init_genes(1);
  end
  for k=1:length(genes_pred)
    genes_pred(k).id = k;
    %if isempty(genes_pred(j).exons{1})
    %  warning('exons field empty');
    %end
    %if isempty(genes_pred(j).cds_exons{1})
    %  warning('exons field empty');
    %end
  end
  [res,map] = evaluate_predictions(blocks(j),genes,genes_pred,genome_info,0);
  res_all = addresults(res_all, res);
end

% Compute sensitivity and specificity
%res_all.nucleotides.SN     = res_all.nucleotides.num_corr      /res_all.nucleotides.num_obsv;
%res_all.nucleotides.SP     = res_all.nucleotides.num_corr      /res_all.nucleotides.num_pred;

res_all.cds_nucleotides.SN = res_all.cds_nucleotides.num_corr  /res_all.cds_nucleotides.num_obsv;
res_all.cds_nucleotides.SP = res_all.cds_nucleotides.num_corr  /res_all.cds_nucleotides.num_pred;

res_all.cds_exons.SN       = res_all.cds_exons.num_corr        /res_all.cds_exons.num_obsv;
res_all.cds_exons.SP       = res_all.cds_exons.num_corr        /res_all.cds_exons.num_pred;

%res_all.exons.SN           = res_all.exons.num_corr            /res_all.exons.num_obsv;
%res_all.exons.SP           = res_all.exons.num_corr            /res_all.exons.num_pred;
%
%res_all.transcripts.SN     = res_all.transcripts.num_corr      /res_all.transcripts.num_obsv;
%res_all.transcripts.SP     = res_all.transcripts.num_corr      /res_all.transcripts.num_pred;
%
res_all.cds_transcripts.SN = res_all.cds_transcripts.num_corr  /res_all.cds_transcripts.num_obsv;
res_all.cds_transcripts.SP = res_all.cds_transcripts.num_corr  /res_all.cds_transcripts.num_pred;

res = res_all;

ret = 0%res.cds_transcripts.mean_SNSP;

save(sprintf('%sresults.mat',PAR.fn_pred), 'res', 'ret', '-v7');

return

function res = addresults(r1, r2)

  res.cds_nucleotides.num_obsv 	= r1.cds_nucleotides.num_obsv	+r2.cds_nucleotides.num_obsv;
  res.cds_nucleotides.num_pred 	= r1.cds_nucleotides.num_pred	+r2.cds_nucleotides.num_pred;
  res.cds_nucleotides.num_corr 	= r1.cds_nucleotides.num_corr	+r2.cds_nucleotides.num_corr;
  res.cds_exons.num_obsv 	= r1.cds_exons.num_obsv		+r2.cds_exons.num_obsv;
  res.cds_exons.num_pred 	= r1.cds_exons.num_pred		+r2.cds_exons.num_pred;
  res.cds_exons.num_corr 	= r1.cds_exons.num_corr		+r2.cds_exons.num_corr;
  res.cds_transcripts.num_obsv 	= r1.cds_transcripts.num_obsv	+r2.cds_transcripts.num_obsv;
  res.cds_transcripts.num_pred 	= r1.cds_transcripts.num_pred	+r2.cds_transcripts.num_pred;
  res.cds_transcripts.num_corr 	= r1.cds_transcripts.num_corr	+r2.cds_transcripts.num_corr;
return


function iteration = find_iteration(PAR)

iteration = 1 ;
while (1)
  fname = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration) ;
  if ~fexist(fname), iteration=iteration-1 ; break ; end ;
  iteration=iteration+1 ;
end ;
return

return

function [model,lengths] = prepare_model(PAR, blocks)
% [model,lengths] = prepare_model(PAR)
% - adding content weights
% - getting boundaries for plifs
  
model = PAR.model; 
%fn_contents = PAR.FN.input_lsl.fn_content
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get content weights
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
%content_names = fieldnames(model.contents) ;
%if ~isempty(content_names)
%  model.content_weights = load_content_weights(fn_contents, content_names) ;
%  if isfield(model, 'use_conservation') && model.use_conservation,
%    model.content_weights = load_content_weights(fn_content_conserv, content_names, '_conserv', model.content_weights) ;
%  end ;
%end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% boundaries for signal and content detectors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if fexist(PAR.FN.input_lsl.fn_boundary_model),
  fprintf('using boundary model in %s\n', PAR.FN.input_lsl.fn_boundary_model) ;
  L = load(PAR.FN.input_lsl.fn_boundary_model) ;
  model.boundaries = L.boundaries ;  
  return
end

if nargin<2
  blocks = load_struct(PAR.FN.input_lsl.fn_training_blocks, 'blocks') ;  
end
INF = 1e20 ; 


%%% signal detectors
  
boundaries = struct;
boundaries.signals = struct;
boundaries.contents = struct;
boundaries.lengths = struct;
if isfield(model,'tiling')
  boundaries.tiling = struct;
end
if isfield(model,'rna_seq')
  boundaries.rna_seq = struct;
end
if isfield(model,'rna_seq_intron_track')
  boundaries.rna_seq_intron_track = struct;
end

sig_names = fieldnames(model.signals) ;
for s = 1:length(sig_names)
  range = getfield(model.signals_range, sig_names{s}) ;
  boundaries.signals = setfield(boundaries.signals, sig_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
end

%%% content detectors

content_names = fieldnames(model.contents) ;
for s = 1:length(content_names)
  range = getfield(model.contents_range, content_names{s}) ;
  boundaries.contents = setfield(boundaries.contents, content_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
end

if isfield(model,'tiling')
  tiling_names = fieldnames(model.tiling) ;
  for s = 1:length(tiling_names)
    range = getfield(model.tiling_range, tiling_names{s}) ;
    boundaries.tiling = setfield(boundaries.tiling, tiling_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
  end
end
if isfield(model,'rna_seq')
  rna_seq_names = fieldnames(model.rna_seq) ;
  for s = 1:length(rna_seq_names)
    range = getfield(model.rna_seq_range, rna_seq_names{s}) ;
    boundaries.rna_seq = setfield(boundaries.rna_seq, rna_seq_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
  end
end
if isfield(model,'rna_seq_intron_track')
  rna_seq_intron_track_names = fieldnames(model.rna_seq_intron_track) ;
  for s = 1:length(rna_seq_intron_track_names)
    range = getfield(model.rna_seq_intron_track_range, rna_seq_intron_track_names{s}) ;
    boundaries.rna_seq_intron_track = setfield(boundaries.rna_seq_intron_track, rna_seq_intron_track_names{s},[-INF linspace(range(1),range(2),model.bins-1) INF]);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% length distributions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

length_names = fieldnames(model.lengths) ;
lengths = [] ;
for s = 1:model.cnt_lengths
  lengths = setfield(lengths, length_names{s}, []) ;
end

segments_names = fieldnames(model.segments) ;
for id = 1:length(blocks),
  block = blocks(id) ;
  segments = block.truth(1).segments ;
  idx_genes=setdiff(unique(segments(:,4)),0);
  for i=1:length(idx_genes)
    genes{i} = segments(segments(:,4)==idx_genes(i),1:3);
  end
  for s = 1:length(segments_names)
    s_id = getfield(model.segments, segments_names{s}) ;
    if s_id == model.segments.cds_exon,
      for j=1:length(genes),
        gene_segments = genes{j} ;
        idx = find(gene_segments(:,3)==s_id) ;
        if length(idx)==1,
          lengths.single_cds_exon = [lengths.single_cds_exon ;gene_segments(idx,2)-gene_segments(idx,1)];
        elseif length(idx)>1,
          lengths.first_cds_exon = [lengths.first_cds_exon ;gene_segments(idx(1),2)-gene_segments(idx(1),1)] ;
          lengths.last_cds_exon = [lengths.last_cds_exon ;gene_segments(idx(end),2)-gene_segments(idx(end),1)] ;
          lengths.middle_cds_exon = [lengths.middle_cds_exon ;gene_segments(idx(2:end-1),2)-gene_segments(idx(2:end-1),1)] ;
        else
          assert(0) ;
        end ;
      end ;
    else
      idx = find(segments(:,3)==s_id) ;
      list = getfield(lengths, segments_names{s}) ;
      list = [list; segments(idx,2)-segments(idx,1)] ;
      lengths = setfield(lengths, segments_names{s}, list) ;
    end ;
  end
end
for s = 1:length(length_names)
  if isempty(lengths.(length_names{s}))
    warning('nonexisting field %s. Using default plif values',length_names{s});
    sout = log(logspace(log10(5),log10(1000),1000)+1);
    boundaries.lengths = setfield(boundaries.lengths,length_names{s}, [-INF sout(ceil([1/model.bins:1/model.bins:1-1/model.bins]*length(sout))) INF]) ;
    continue;
  end
  %range = getfield(model.lengths_range, length_names{s}) ;
  sout = getfield(lengths,length_names{s}) ;
  %sout(sout<range(1)|sout>range(2)) = [];
  %sout = sort(log(sout+1))' ;
  %fprintf(1, '%s max_val: %i\n', length_names{s}, max(sout))
  sout = log(sout+1)' ;
  %if length(sout)<100, 
    %fprintf('too few points: %s %i\n',length_names{s}, length(sout));  
    %sout = log(logspace(log10(5),log10(1000),1000)+1) ; 
    %sout = log(logspace(log10(range(1)),log10(range(2)),1000)+1) ; 
  %end ;
  %boundaries.lengths = setfield(boundaries.lengths,length_names{s}, [-INF sout(ceil([1/model.bins:1/model.bins:1-1/model.bins]*length(sout))) INF]) ;
  %keyboard
  boundaries.lengths = setfield(boundaries.lengths,length_names{s}, [-INF prctile(sout,(1/(model.bins-1):1/(model.bins-1):1)*100) INF]) ;
end
model.boundaries=boundaries ;


fprintf('saving boundary model in %s\n', PAR.FN.input_lsl.fn_boundary_model) ;
boundaries  = model.boundaries;
save(PAR.FN.input_lsl.fn_boundary_model, 'boundaries', '-v7') ;

return

function n = reorder_fields(s, fields)
% n = reorder_fields(s, fields)

  if isempty(s),
    n=s ;
    return ;
  end ;

  n=struct ;
  for i=1:length(fields)
    for j=1:length(s) ;
      n(j).(fields{i}) = s(j).(fields{i}) ;
    end ;
  end ;

  remaining = setdiff(fieldnames(s), fields) ;

  if ~isempty(remaining)
    remaining
    error('these fields are unassigned');
  end ;


return

function [xis,parameters,hvar,H,obj] = res2param(res, num_examples, model, QP, C, weight_names)
% [xis,parameters,hvar,H] = res2param(res, num_examples, model)
 

%%%%%% decompose res 

vector = res(1:model.cnt_parameters)' ;
parameters = init_weights(model) ;
param_name = weight_names ;%fieldnames(parameters);
last_pos = 0 ;
for p = 1:length(param_name)
  old_p = getfield(parameters, param_name{p}) ;
  parameters = setfield(parameters, param_name{p}, vector(last_pos+1:last_pos+length(old_p)));
  last_pos = last_pos + length(old_p) ;
end
assert(last_pos==length(vector)) ;


xis = res(model.cnt_parameters+1:model.cnt_parameters+num_examples);
 
hvar = res(model.cnt_parameters+num_examples+1:end);
H = [] ;
obj = struct;
%for i=1:length(model.plif2param{1})
%  H(i).hvar = hvar(model.plif2param{i});
%end


if ~isfield(QP, 'Q')
  return
end

%%%%%% decompose obj 

r = res;
r(1:model.cnt_parameters) = 0;
r(end-length(hvar)+1:end) = 0;
obj.slack = sum(r.*QP.f) ;

r = res;
r(model.cnt_parameters-model.cnt_transitions+1:end) = 0;
obj.plif_ys_sq = 0.5*r'*QP.Q*r;

r = res;
r(1:model.cnt_parameters-model.cnt_transitions) = 0;
r(model.cnt_parameters+1:end) = 0;
obj.transitions_sq = 0.5*r'*QP.Q*r;

r = res;
nof_smooth =(model.cnt_plifs*(model.bins-1))+model.cnt_transitions;% because of trans_smooth
r(1:end-nof_smooth) = 0;
obj.smoothness_sq = 0.5*r'*QP.Q*r;

r(end-model.cnt_transitions+1:end) = 0;
obj.smooth  = sum(r.*QP.f) ;

r = res;
r(1:model.cnt_parameters+num_examples+nof_smooth-model.cnt_transitions) = 0;
obj.transitions = QP.f'*r ;


%%%


obj.OBJ  = obj.slack + obj.smooth + obj.plif_ys_sq + obj.transitions_sq + obj.smoothness_sq + obj.transitions ;

OBJ_total = sum(res.*QP.f) + 0.5*res'*QP.Q*res ;

if (abs(OBJ_total-obj.OBJ)>1e-8)
  OBJ_total-obj.OBJ
  keyboard
  obj.OBJ = OBJ_total 
end ;

%%%

return

function [path,pos_idx,pos]=segmentation2path(SEG, block, model)
% [path,pos_idx,pos]=segmentation2path(SEG, block, model)
  

%pos = [0     length(block.all_pos)-1] ;
%path = [model.state_ids.seq_start    model.state_ids.seq_end]-1 ;
  

genestr=lower(block.seq) ;

col4 = SEG.segments(:,4);
num_genes = length(unique(col4))-1;
assert(all(unique(col4)'==0:num_genes))

path = [] ;
pos = [] ;
path(end+1) = model.state_ids.seq_start ;
pos(end+1) = block.all_pos(1) ;

any_invalid = 0 ;
for i=1:num_genes
  segments = SEG.segments(col4==i,1:3);
  if (size(segments,1)==1 & isnan(segments(1,3)))
    path(end+1:end+2) = [model.state_ids.tss(1) model.state_ids.cdsStop(1)] ;
    pos(end+1:end+2) = segments(1:2)';
    warning('gene is invalid') ;
    any_invalid = 1 ;
    continue
  end
  % consider 5' utr
  idx = find(segments(:,3)==model.segments.utr5exon) ;
  if ~isempty(idx),
    utr5_segments = segments(idx,:) ;
    path(end+1) = model.state_ids.tss(1) ;
    pos(end+1) = utr5_segments(1,1) ;
    
    for i=1:size(utr5_segments,1)-1
      path(end+1)= model.state_ids.don(1) ;
      pos(end+1) = utr5_segments(i, 2) ;
      path(end+1)= model.state_ids.acc(1) ;
      pos(end+1) = utr5_segments(i+1, 1) ;
    end ;
  end ;

  if isfield(model.segments,'transexon')
    idx = find(segments(:,3)==model.segments.transexon) ;
  else 
    idx = [];
  end
  if ~isempty(idx),
    if length(idx)==1 && segments(idx(1),1)==segments(idx(1),2),
      % this is the trans-splice agatg case
      path(end+1) = model.state_ids.trans(2) ;
      pos(end+1) = segments(idx,1) ;
    else
      utr5_segments = segments(idx,:) ;
      path(end+1) = model.state_ids.trans(1) ;
      pos(end+1) = utr5_segments(1,1) ;
      
      for i=1:size(utr5_segments,1)-1
        path(end+1)= model.state_ids.don(1) ;
        pos(end+1) = utr5_segments(i, 2) ;
        path(end+1)= model.state_ids.acc(1) ;
        pos(end+1) = utr5_segments(i+1, 1) ;
      end ;
    end ;
  end ;

  % consider coding region
  idx = find(segments(:,3)==model.segments.cds_exon) ;
  coding_segments = segments(idx,:) ;
  if idx(1)>1 && segments(idx(1)-1,3)==model.segments.intron,
    % special case agatg
    path(end+1) = model.state_ids.don(1) ;
    pos(end+1) = segments(idx(1)-1,1) ;
    path(end+1) = model.state_ids.tis(2) ;
    pos(end+1) = segments(idx(1)-1,2) ;
  else
    if ~isfield(model.state_ids,'trans')||path(end)~=model.state_ids.trans(2),
      path(end+1) = model.state_ids.tis(1) ;
      pos(end+1) = coding_segments(1,1) ;
    end ;
  end ;
  %spliced = '' ;
  %for i=1:size(coding_segments,1)
  %  spliced = [spliced genestr(coding_segments(i,1):coding_segments(i,2)-1)] ;
  %end ;
  %assert(mod(length(spliced),3)==0) ;
  %f=find_open_frames(spliced,1)

  frame = mod(coding_segments(1,2)-coding_segments(1,1),3) ;
  for i=1:size(coding_segments,1)-1
    pos(end+1) = coding_segments(i,2) ;
    pos(end+1) = coding_segments(i+1,1) ;
    if frame==0,
      path(end+1)= model.state_ids.don(1+1) ;
      path(end+1)= model.state_ids.acc(1+1) ;
    elseif frame==1,
      if genestr(coding_segments(i,2)-1)~='t'
        path(end+1)=model.state_ids.don(1+2) ;
        path(end+1)=model.state_ids.acc(1+2) ;
      else
        path(end+1)=model.state_ids.don(1+3) ;
        path(end+1)=model.state_ids.acc(1+3) ;
      end ;
    elseif frame==2,
      if genestr(coding_segments(i,2)-2)=='t' & genestr(coding_segments(i,2)-1)=='a'
        path(end+1)=model.state_ids.don(1+5) ;
        path(end+1)=model.state_ids.acc(1+5) ;
      elseif genestr(coding_segments(i,2)-2)=='t' & genestr(coding_segments(i,2)-1)=='g'
        path(end+1)=model.state_ids.don(1+6) ;
        path(end+1)=model.state_ids.acc(1+6) ;
      else
        path(end+1)=model.state_ids.don(1+4) ;
        path(end+1)=model.state_ids.acc(1+4) ;
      end ;
    end ;
    frame = mod(frame+coding_segments(i+1,2)-coding_segments(i+1,1),3) ;
  end ;
  
  % consider 3' utr
  utr3_segments = [] ;

  idx = find(segments(:,3)==model.segments.cds_exon) ;
  if segments(idx(end)+1,3)==model.segments.intron 
    % special case: agstop
    path(end+1)= model.state_ids.don(2) ;
    pos(end+1) = segments(idx(end)+1,1) ;
    path(end+1)= model.state_ids.cdsStop(2) ;
    pos(end+1) = segments(idx(end)+1,2) ;
  else
    % normal stop
    pos(end+1)=coding_segments(end,2) ;
    if frame==0,
      path(end+1)=model.state_ids.cdsStop(1) ;
    elseif frame==1,
      path(end+1)=model.state_ids.cdsStop(1) ;
    elseif frame==2,
      path(end+1)=model.state_ids.cdsStop(1) ;
    else
      error('senseless frame value');
    end ;
  end ;

  idx = find(segments(:,3)==model.segments.utr3exon) ;
  if ~isempty(idx),
    utr3_segments = segments(idx,:) ;
    for i = 1:size(utr3_segments,1)-1
      path(end+1)= model.state_ids.don(end) ;
      pos(end+1) = utr3_segments(i, 2) ;
      path(end+1)= model.state_ids.acc(end) ;
      pos(end+1) = utr3_segments(i+1, 1) ;
    end ;
  end ;

  % consider 3' utr
  idx = find(segments(:,3)==model.segments.polya_tail) ;
  if ~isempty(idx),
    assert(length(idx)==1) ;
    utr3_segments = segments(idx,:) ;
    path(end+1) = model.state_ids.polya(1) ;
    pos(end+1) = utr3_segments(1,1) ;
  end ;

  if ~isempty(utr3_segments),
    path(end+1) = model.state_ids.cleave ;
    pos(end+1) = utr3_segments(end,2) ;
  end ;
end ;

path(end+1) = model.state_ids.seq_end ;
pos(end+1)  = block.all_pos(end) ;

if ~any_invalid,
  for i=1:length(pos)-1
    assert(model.A(path(i),path(i+1))>=-1000) ;
  end ;
end ;

for i=1:size(model.intergenic_long.transitions, 1)
  idx = find([path(1:end-1)==model.intergenic_long.transitions(i,1) & ...
              path(2:end)==model.intergenic_long.transitions(i,2) & ...
              pos(2:end)-pos(1:end-1) >= model.intergenic_long.max_short_length], 1, 'first') ;
  
  while(~isempty(idx))
    excess = pos(idx+1) - pos(idx) - model.intergenic_long.max_short_length ;
    assert(all(excess>=0)) ;
    
    % fill in the minimum number of transitions necessary to generate the true path
    idx2 = find(block.pos>pos(idx)+model.intergenic_long.long_segment_parts & block.pos<pos(idx+1)-model.intergenic_long.long_segment_parts) ;
    num_steps = ceil((block.pos(idx2(end))-block.pos(idx2(1)))/model.intergenic_long.step)+1 ;
    assert(length(idx2)>=num_steps) ;
    %if block.pos(idx2(end))-block.pos(idx2(1)) > model.lengths_range.intron(2),
    %  warning(sprintf('intergenic gap (%i) is longer than maximal intron length (%i), true path might not be realizable',...
    %                  block.pos(idx2(end))-block.pos(idx2(1)), model.lengths_range.intron(2))) ;
    %end ;
    path = [path(1:idx) model.state_ids.intergenic_long*ones(1,num_steps) path(idx+1:end)] ;
    pos = [pos(1:idx) block.pos(idx2([round(linspace(1, length(idx2), num_steps))])) pos(idx+1:end)] ;
    
    idx = find([path(1:end-1)==model.intergenic_long.transitions(i,1) & ...
                path(2:end)==model.intergenic_long.transitions(i,2) & ...
                pos(2:end)-pos(1:end-1) >= model.intergenic_long.max_short_length], 1, 'first') ;
  end ;
end ;

if isfield(model.state_ids,'intron_long')
  path_ext = [];
  pos_ext =[];
  state_idx=1;
  for i=model.state_ids.don(1):model.state_ids.don(end)
  % for all don states possibly followed by a long intron
    idx = find(path==i);
    % following state must be either fitting cds_acc or cdsStop_acc or (UTR5_don->)tis_acc
  
    assert(all(path(idx+1)==model.state_ids.acc(state_idx)|path(idx+1)==model.state_ids.cdsStop(2)|(path(idx)==model.state_ids.don(1)&path(idx+1)==model.state_ids.tis(2))));
    while ~isempty(idx)
      % begin at the end to avoid index updates and make use of flexible first intron length
      intron_length = pos(idx(end)+1)-pos(idx(end));
      if intron_length > model.lengths_range.intron(2)
      % need for (a) long intron(s)
        current_pos = pos(idx(end)+1);
        while intron_length>model.lengths_range.intron(2)
	  next_idx = find(block.all_pos>=current_pos-model.lengths_range.intron_long(2)&block.all_pos<=current_pos-model.lengths_range.intron_long(1),1);
	  assert(~isempty(next_idx));
          path_ext(end+1)=model.state_ids.intron_long(state_idx);
	  pos_ext(end+1)=block.all_pos(next_idx);
 	  intron_length=intron_length - pos(idx(end)+1) + pos_ext(end);
	  current_pos = pos_ext(end);
        end
      end
      idx(end)=[];
    end
    state_idx=state_idx+1;
  end
  pos = [pos, pos_ext];
  [pos,perm]=sort(pos);
  path=[path, path_ext];
  path=path(perm);
end

[tmp, pos_idx, idx] = intersect(block.all_pos, pos) ;

assert(isempty(setdiff(pos, block.all_pos(pos_idx))))
%if ~isempty(setdiff(pos, block.all_pos(pos_idx)))
%  [tmp,idx1]=setdiff(pos, block.all_pos(pos_idx)) ;
%  warning('%i (%i) position(s) not found', length(idx1),length(pos)) ;
%  keyboard
%end ;
path = path(idx) ;
pos = block.all_pos(pos_idx) ;

%%%%

return 

function pos = find_steps(all_pos,range, range_intergenic)
idx = find(all_pos>all_pos(end)-range(2)&all_pos<all_pos(end)-range(1));
assert(~isempty(idx))
pos_sets{1} = all_pos(end);
pos_sets{2} = all_pos(idx);
if ismember(all_pos(1),pos_sets{2})
  pos = [all_pos(1) all_pos(end)];
  return
end

% find all positions that are reachable from the start positions
step = 2;
while 1
  set = pos_sets{step};
  idx = find(all_pos>set(1)-range(2)&all_pos<set(end)-range(1));
  step = step+1;
  if isempty(idx)
    assert(any(set-all_pos(1)<range_intergenic(2)&set-all_pos(1)>range_intergenic(1)))
    pos_sets{step} = all_pos(1);
    pos(1)=all_pos(1);
    break;
  end
  pos_sets{step} = all_pos(idx);
  if any(set-all_pos(1)<range_intergenic(2)&set-all_pos(1)>range_intergenic(1))
    pos_sets{step} = all_pos(1);
    pos(1)=all_pos(1);
    break;
  end
end

% greedy backtrack
for j=1:step-1
  set = pos_sets{step-j};
  if j==1
    idx = find(set>pos(j)+range_intergenic(1)&set<pos(j)+range_intergenic(2));
  else
    idx = find(set>pos(j)+range(1)&set<pos(j)+range(2));
  end
  assert(~isempty(idx))
  pos(j+1) = set(idx(1));
end
return

function pos = find_steps_old(all_pos,range)
idx = find(all_pos>all_pos(1)+range(1)&all_pos<all_pos(1)+range(2));
assert(~isempty(idx))
pos_sets{1} = all_pos(1);
pos_sets{2} = all_pos(idx);
if ismember(all_pos(end),pos_sets{2})
  pos = [all_pos(1) all_pos(end)];
  return
end

% find all positions that are reachable from the start positions
step = 2;
while 1
  set = pos_sets{step};
  idx = find(all_pos>set(1)+range(1)&all_pos<set(end)+range(2));
  if isempty(idx)
    error('cannot solve this problem')
  end
  step = step+1;
  pos_sets{step} = all_pos(idx);
  if ismember(all_pos(end),pos_sets{step})
    pos(step) = all_pos(end);
    break;
  end
end

% greedy backtrack
for j=step-1:-1:1
  set = pos_sets{j};
  idx = find(set>pos(j+1)-range(2)&set<pos(j+1)-range(1));
  assert(~isempty(idx))
  pos(j) = set(idx(1));
end
return





return

function sg_set_plif(penalty_array)
% sg_set_plif(penalty_array)

for j=1:length(penalty_array)
  all_ids(j)		= penalty_array{j}.id;
  all_names{j} 		= penalty_array{j}.name;
  all_limits(:,j) 	= penalty_array{j}.limits;
  all_penalties(:,j) 	= penalty_array{j}.penalties;
  if isempty(penalty_array{j}.transform)
    all_transform{j} 	= 'linear';
  else
    all_transform{j} 	= penalty_array{j}.transform;
  end
  all_min_values(j) 	= penalty_array{j}.min_value;
  all_max_values(j) 	= penalty_array{j}.max_value;
  all_use_cache(j) 	= penalty_array{j}.use_cache;
  all_use_svm(j) 	= penalty_array{j}.use_svm;
  all_do_calc(j) 	= 1;%penalty_array{j}.do_calc;
end

sg('set_plif_struct',int32(all_ids)-1,all_names, all_limits, all_penalties, all_transform, all_min_values, all_max_values, int32(all_use_cache), int32(all_use_svm), int32(all_do_calc)); 
[ret_ids,ret_names, ret_limits, ret_penalties, ret_transform, ret_min_values, ret_max_values, ret_use_cache, ret_use_svm, ret_do_calc]= sg('get_plif_struct');

[engine, env]=determine_engine() ;

assert(isequal(ret_do_calc, all_do_calc))
assert(isequal(ret_use_svm, all_use_svm))
assert(isequal(ret_use_cache, all_use_cache))
assert(isequal(ret_max_values, all_max_values))
assert(isequal(ret_min_values, all_min_values))
if ~isequal(engine, 'octave'),
  assert(isequal(ret_transform', all_transform))
end ;
assert(isequal(ret_penalties, all_penalties))
assert(isequal(ret_limits, all_limits))
if ~isequal(engine, 'octave'),
  assert(isequal(ret_names', all_names))
end ;
assert(isequal(ret_ids+1, all_ids))


return

function QP = solve_qp(QP, PAR) ;
%   QP = solve_qp(QP, PAR) ;
%

global OPTIMIZER

[engine, env, tmp, tmp, licence_type]=determine_engine() ;

if isequal(engine, 'octave'),

  disp('Solving quadratic program') ;      
  idx = find_relevant_constraints(QP);
  QP.con_idx = idx;
  
  if isequal(OPTIMIZER, 'mosek')
    A_lb = -inf*ones(size(QP.b(idx))) ;
    tic;[res] = mosek_qp(QP.res, QP.Q, QP.f, [], [], QP.lb, QP.ub, A_lb, QP.A(idx,:), QP.b(idx)); toc;
  elseif isequal(OPTIMIZER, 'glpk')
    % use glpk QP solver
    ctype = char( 'U' * ones(1, size(QP.A,1) ) ) ;
    tic;[res] = qpng(QP.Q, QP.f, QP.A(idx,:), QP.b(idx), ctype, QP.lb, QP.ub, QP.res); toc;
    
    %tic; [res, OBJ, INFO, LAMBDA] = qp (QP.res, QP.Q, QP.f, [],
    %[], QP.lb, QP.ub, A_lb, QP.A(idx,:), QP.b(idx)); toc;
  else 
    error('Unknown optimizer: %s\n', OPTIMIZER) ;
  end ;
  last_num_con = size(QP.A,1) ;

  %if ~strcmp(how, 'OK')
  %  tic;[res,lambda,how]=lp_resolve(QP.lpenv, last_p_lp, 1, 'bar');toc;
  %end
  QP.res = res ;

  %% check if solution is feasible
  assert(all(QP.A(idx,:)*res-QP.b(idx)<=1e-2)) ;
  if max(QP.A*res-QP.b)>1
    warning('solve_qp:constraints', 'removed some constraints that would have influenced the result (solve again to obtain a more accurate result)')
  end 

else

  if isequal(OPTIMIZER, 'cplex'),

    %how=lp_set_param(QP.lpenv,'CPX_PARAM_BARCROSSALG', 2,1) ; % create lp basis after barrier solving
    
    if nnz(QP.Q)==0
      disp('Solving linear program') ;      
      how=lp_set_param(QP.lpenv,'CPX_PARAM_PREDUAL', 1, 1) ; 
      tic;[res,lambda,how] = lp_solve(QP.lpenv, QP.f, QP.A, QP.b, QP.lb, QP.ub, 0, 1, 'dual');toc;
    else
      disp('Solving quadratic program') ;      
      idx = find_relevant_constraints(QP);
      QP.con_idx = idx;
      tic;[res, lambda, how] = qp_solve(QP.lpenv, QP.Q, QP.f, QP.A(idx,:), QP.b(idx), QP.lb, QP.ub, 0, 0, 'bar');toc;
      
      if ~strcmp(how, 'OK')
        how
      end
      QP.res = res ;
      
      %% check if solution is feasible
      assert(all(QP.A(idx,:)*res-QP.b(idx)<=1e-2)) ;
      if max(QP.A*res-QP.b)>1
        warning('solve_qp:constraints', 'removed some constraints that would have influenced the result (solve again to obtain a more accurate result)')
      end 
    end ;

  elseif isequal(OPTIMIZER, 'mosek') || isequal(OPTIMIZER, 'quadprog'),

    idx = find_relevant_constraints(QP);
    QP.con_idx = idx;

    if isequal(OPTIMIZER, 'mosek'),
      assert(~isempty(strfind(which('quadprog'), 'mosek'))) ;
    end ;

    [res,fval,how,output,lambda] = quadprog(QP.Q, QP.f, QP.A(idx,:), QP.b(idx), [], [] , QP.lb, QP.ub, []) ;
    if ~isequal(how, 1)
      how
    end ;

    QP.res = res ;

  else
    error('Unknown optimizer: %s\n', OPTIMIZER) ;
  end ;
  last_num_con = size(QP.A,1) ;

  %% check if solution is feasible
  assert(all(QP.A(idx,:)*res-QP.b(idx)<=1e-2)) ;
  if max(QP.A*res-QP.b)>1
    warning('solve_qp:constraints', 'removed some constraints that would have influenced the result (solve again to obtain a more accurate result)')
  end 
end ;

return

function idx = find_relevant_constraints(QP)

idx = 1:size(QP.A,1) ;
if 1,
  Diff = (QP.A*QP.res-QP.b) ;
  for p=95:-5:10
    ttt=my_prctile(Diff,p) ;
    if my_prctile(Diff,p)<-50, % some heuristic to make sure the gap is large enough
      idx = find(Diff>my_prctile(Diff,p)) ;
      fprintf('removing %2.0f%% of all constraints (d=%1.2f):\nreduced problem has %i constraints (originally %i)\n', p, my_prctile(Diff,p), length(idx), size(QP.A,1)) ;
      break ;
    end ;
  end ;
end ;


return

return

function [fn_predictor, iteration] = train_path(PAR)
% [fn_predictor, iteration] = train_path(PAR)

fid = 1;% fopen( sprintf('%strain.txt',PAR.FN.output_lsl.fn_log),'a+') ;
fprintf(fid,'\n\n train_path \n');
fprintf(fid,'date: %s\n',date);
fprintf(fid,'\n initializing \n');

PAR.model = fix_model(PAR.model) ;
model = PAR.model ;

PAR.weight_names = init_weight_names(model) ;

method = PAR.LSL.method;
max_neg  = method.max_neg ;
opt_step = method.opt_step;

lpenv = init_solver ;

if isfield(PAR,'iteration'),
  iteration = PAR.iteration 
  PAR = rmfield(PAR,'iteration') ;
else
  iteration = 1 ;
  while (1)
    fname = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration) ;
    if ~fexist(fname), iteration=iteration-1 ; break ; end ;
    iteration=iteration+1 ;
  end ;  
end ;

fname = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration) 
fn_parameters = sprintf('%s_parameters.mat', PAR.FN.output_lsl.fn_train)
if fexist(fname),
  fprintf('restarting at iteration %i\n', iteration) ;
  load(fname, 'TRAIN_PAR', 'QP','num_examples') ;
  %blocks_train = load_struct(sprintf('%s_iteration%i_blocks.mat', PAR.FN.output_lsl.fn_train,iteration)) ;
  blocks_train = load_struct(PAR.FN.input_lsl.fn_training_blocks, 'blocks') ;
  blocks_train = fix_blocks(blocks_train, model) ;
  init_path(PAR, blocks_train, lpenv, PAR.weight_names) ;
  assert(fexist(fn_parameters)==1)
else
  assert(iteration==0)
  blocks_train = load_struct(PAR.FN.input_lsl.fn_training_blocks, 'blocks') ;
  blocks_train = fix_blocks(blocks_train, model) ;
  if isfield(PAR.LSL, 'train_blocks_num') && PAR.LSL.train_blocks_num<length(blocks_train)
	fprintf('reduce number of blocks to %i\n', PAR.LSL.train_blocks_num) ;
	blocks_train = blocks_train(1:PAR.LSL.train_blocks_num);
  end
  if isfield(PAR.LSL.method, 'add_lin_feat')&&PAR.LSL.method.add_lin_feat
    assert(iscell(PAR.LSL.method.add_feat_fun))
    assert(iscell(PAR.FN.input_lsl.fn_lin_feat_data))
    assert(length(PAR.FN.input_lsl.fn_lin_feat_data)==length(PAR.LSL.method.add_feat_fun))
    for feat_type=1:length(PAR.LSL.method.add_feat_fun)
      fprintf('load linear features with function: %s from file: %s\n', PAR.LSL.method.add_feat_fun{feat_type}, PAR.FN.input_lsl.fn_lin_feat_data{feat_type}) ;
      blocks_train = feval(PAR.LSL.method.add_feat_fun{feat_type}, blocks_train, PAR.FN.input_lsl.fn_lin_feat_data{feat_type}, 80);
    end
  end
  blocks_train = initialize_blocks(blocks_train, PAR.model, PAR.LSL.method.max_neg, PAR.FN);
  blocks_train = fix_blocks(blocks_train, model) ;
  [QP,parameters] = init_path(PAR, blocks_train, lpenv, PAR.weight_names);
  num_examples = length(blocks_train) ;
  solve_cnt = 0;
  param_history={};
  save(fn_parameters, 'solve_cnt', 'param_history', '-v7')
  fprintf(fid,'\nStarting from iteration %i  \n\n',iteration);
end ;

QP.lpenv = lpenv;
global last_p_lp last_num_con
last_p_lp=[] ;
last_num_con=0 ;

[tmp sort_idx] = sort([blocks_train.stop]-[blocks_train.start], 'descend');
blocks_train = blocks_train(sort_idx);

% make sure parameters fit to QP.res
[xis,parameters] = res2param(QP.res, num_examples, model, QP, 0, PAR.weight_names);

%method.max_num_iterations=110 ;

% rproc options
options = PAR.RPROC.options;
options.express = 0;
%options.start_dir =  fileparts(which('gen_paths'));
options.verbosity = 0 ;

% initialize approximation flags
for i=1:length(blocks_train),
  blocks_train(i).pred_use_approximation = 1 ;
end ;

jobinfo_predict = rproc_empty(0) ;

old_OBJ = -100 ;
idx_blocks = 1:num_examples;
viterbi_nbest = ones(1,num_examples) ;
while iteration < method.max_num_iterations % | abs(old_OBJ-OBJ)<1e-3,
  iteration = iteration + 1 ;
  num_new_constraints_iteration=0;
  OBJ(iteration) = sum(QP.res.*QP.f)+0.5*QP.res'*QP.Q*QP.res ;
  fprintf(fid,'\nStarting iteration: %i\n-----------------------\n',iteration);
  %fprintf(fid,'\n objective: %i\n',OBJ(iteration) );

  num_correct = 0;
  num_crashed = 0;
  bound_reached = 0;
  last_sum_xis = 0;
  jobinfo = rproc_empty(0) ;

  %fprintf(fid,'\n submitting dynamic programming jobs ...');

  for block_id_start = 1:method.exm_per_solve:num_examples,
    block_ids = block_id_start:min(block_id_start+method.exm_per_solve-1, num_examples) ;
    
    %%%%%%%%%%%%%%%%%%%%%
    global GLOBAL_cPAR ;
    GLOBAL_cPAR=[] ;
    
    cPAR=[] ;
    cPAR.model = model ;
    %cPAR.blocks_fname = ['~/' tempname '.mat'] ;
    [xis, cPAR.parameters] = res2param(QP.res, num_examples, model, QP, 0, PAR.weight_names);
    cPAR.parameters_fields = fieldnames(cPAR.parameters) ;
    
    P=[] ;
    P.cPAR_fname = sprintf('~/tmp/%i-%1.10f.mat', round(rand(1)*1e8), now) ;
    assert(~fexist(P.cPAR_fname)) ;
    save(P.cPAR_fname, 'cPAR', '-v7') ;

    %save_struct(cPAR.blocks_fname, blocks_train) ;

    %%%%%%%%%%%%%%%%%%%%%
    
    % only submit jobs if sufficiently many blocks need the full computation
    frac_full = mean([blocks_train.pred_use_approximation]==0);
    compute_full_gen_paths = frac_full > 0.5 ;

    fprintf('Dynamic programming approximation appropriate for %i/%i jobs\n', sum([blocks_train.pred_use_approximation]==1), length(blocks_train)) ;
    if compute_full_gen_paths,
      fprintf('=> Compute exact dynamic programs for all blocks\n') ;
    else
      fprintf('=> Compute approximate dynamic programs for all blocks\n') ;
    end ;

    if compute_full_gen_paths,
      [run_locally]= rproc_policy('train_path:gen_paths', [], length(block_ids)) ;
    else
      [run_locally]= rproc_policy('train_path:gen_paths', [], length(block_ids)/10) ;
    end ;
    
    if run_locally,
      fprintf('\nRunning %i dynamic programs: ', length(block_ids)) ;
    else
      fprintf('\nSubmitting %i dynamic programs: ', length(block_ids)) ;
    end ;
    jobinfo=rproc_empty(0) ; ii=0 ;
    for block_id = block_ids,
      idx = find(blocks_train(block_id).truth(1).segments(:,3)~=model.segments.intergenic) ;
      max_len = max(500, max(blocks_train(block_id).truth(1).segments(idx,2)- ...
                             blocks_train(block_id).truth(1).segments(idx,1))) ;
      max_len = max_len*1.2+1000 ;
      %max_len = 22000 ;

      P.block_id = block_id ;
      P.FN = PAR.FN;
      P.block = blocks_train(block_id) ;
      %P.model = model ;
      %P.parameters = parameters ;
      P.viterbi_nbest = viterbi_nbest(block_id) ;
      %P.max_len = max_len ;
      P.fn_save = sprintf('%spred_block%i.mat', PAR.FN.output_lsl.fn_train_block_preds, blocks_train(block_id).id); 
      P.prediction = 0 ;

      % get a quick approximation of the best path 
      if ~compute_full_gen_paths %P.block.pred_use_approximation,
        P.approximate = 1 ;
        %P.skip_checks = 0 ;
        
        %tic; pred=gen_paths(P) ; toc;
        %if gen_paths_approximation_enough(P, pred),
        %  continue ;
        %end ;
        %P.approximate=0 ;
        %P.skip_checks=0 ;
        
        if rand(1)<0.05, % randomly disable some blocks ... to guarantee convergence
          %fprintf(fid, 'disabling gen_path approximation\n') ;
          blocks_train(block_id).pred_use_approximation = 0 ;
        end ;
      else
        unix(['rm -f ' P.fn_save]) ;
        P.approximate = 0 ;
        %fprintf(fid, 'enabling gen_path approximation\n') ;
        blocks_train(block_id).pred_use_approximation = 1 ;
      end ;
      
      options.identifier = ['G'  num2str(block_id)] ;
      ii=ii+1 ;
      if mod(ii, 5)==0,
        fprintf('%i ... ', ii) ;
      end ;

      if ~run_locally,
        
        [mem_req, time_req, opts]= rproc_memtime_policy('gen_paths', length(P.block.seq), options, 0) ;
        
        if ~isfield(options, 'immediately_bg') || options.immediately_bg==0,
          if PAR.RPROC.exm_per_batch==1,
            jobinfo(end+1) = rproc('gen_paths', P, mem_req, opts, time_req) ;
          else
            jobinfo(end+1) = rproc_create('gen_paths', P, mem_req, opts, time_req) ;
          end ;
        else
          jobinfo(end+1) = rproc('gen_paths', P, mem_req, opts, time_req) ;
        end ;
      else
        gen_paths(P) ;
      end
    end ;
    fprintf('Done.\n') ;
    
    if ~isempty(jobinfo),
      if ~isfield(options, 'immediately_bg') || options.immediately_bg==0,
        if PAR.RPROC.exm_per_batch==1,
          [jobinfo, num_crashed] = rproc_wait(jobinfo, 30, 1, -1, 1);
        else
          [jobinfo, meta_jobinfo] = rproc_submit_batch(jobinfo, PAR.RPROC.exm_per_batch) ;
          [meta_jobinfo,num_crashed] = rproc_wait(meta_jobinfo, 30, 1, -1, 1);
        end ;
        %[meta_jobinfo,num_crashed] = rproc_wait(jobinfo, 20, 1, 0);
        %unix(['rm -f ' P.cPAR_fname ' ' cPAR.blocks_fname]) ;
      else
        [jobinfo] = rproc_wait(jobinfo, 30, 1, -1, 1);
      end ;
    end ;

    blocks = blocks_train(block_ids) ;
    idx_temp = idx_blocks(block_ids) ;
    
    [blocks, QP, viterbi_nbest(block_ids), num_new_constraints] = add_constraints(blocks, idx_temp, num_examples, PAR, QP, viterbi_nbest(block_ids));
    blocks_train(block_ids) = blocks ;

    if ~isempty(jobinfo),
      rproc_cleanup(jobinfo) ;
      if ~isfield(options, 'immediately_bg') || options.immediately_bg==0,
        if PAR.RPROC.exm_per_batch==1,
          rproc_cleanup(jobinfo) ;
        else
          rproc_cleanup(meta_jobinfo) ;
        end ;
      end  ;
    end ;

    %fprintf(fid,'\n number of new constraints: %i\n',num_new_constraints );
    if num_new_constraints==0, continue; end
    num_new_constraints_iteration = num_new_constraints_iteration+num_new_constraints;

    load(fn_parameters, 'solve_cnt', 'param_history')
    solve_cnt = solve_cnt+1;
    fprintf(fid, '\n solving qp: %i\n', solve_cnt);
    QP = solve_qp(QP, PAR) ;
    [xis,parameters,hvar,H,obj] = res2param(QP.res, num_examples, model, QP, 0, PAR.weight_names);
    %obj
    param_history{end+1} = parameters;
    parameters_fields = fieldnames(parameters) ;
    save(fn_parameters, 'solve_cnt', 'param_history', 'parameters_fields', '-v7')

    obj_solve(solve_cnt) = obj;
    %OBJ = sum(QP.res.*QP.f)+0.5*QP.res'*QP.Q*QP.res ;
    %fprintf(fid,'\n sum(xis)  : %i\n',sum(xis) );
    print_struct(fid, obj);
    save([PAR.FN.output_lsl.fn_train,'.mat'], 'PAR', 'QP','num_examples', '-v7') ;
    %draw_pred(blocks_train(block_id).truth, blocks_train(block_id).pred,model) 
  end

  if sum(xis)-last_sum_xis>100, bound_reached=1;end
  if sum(xis)-last_sum_xis<1e-2 && bound_reached
    if keyboard_allowed()
      keyboard
    end 
  else
    last_sum_xis = sum(xis);
  end

  TRAIN_PAR = PAR;
  if ~exist('obj_solve'),
    obj_solve=struct ;
  end ;
  save(sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train,iteration), 'TRAIN_PAR', 'QP','num_examples','obj_solve', '-v7') ;
  if isfield(PAR, 'debug_mode') && PAR.debug_mode == 1
    save_struct(sprintf('%s_iteration%i_blocks.mat', PAR.FN.output_lsl.fn_train,iteration), blocks_train) ;
  end

  %% evaluation
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if mod(iteration, 20)==0 && PAR.submit,
    fprintf('predict\n');
    myoptions = PAR.RPROC.options;
    myoptions.express = 0;
    %myoptions.start_dir =  fileparts(which('predict'));
    myoptions.verbosity = 2 ;
    PAR.fn_blocks = PAR.FN.input_lsl.fn_val_blocks; 
    PAR.iteration = iteration;
    PAR.fn_pred = [PAR.FN.output_lsl.fn_pred 'iteration_' num2str(iteration) '/'];
    PAR.evaluate = 1;
    [mem_req, time_req, opts] = rproc_memtime_policy('predict', 0, myoptions) ;
    myoptions.verbosity=2 ;
    jobinfo_predict(end+1) = rproc('predict', PAR, mem_req, opts, time_req);

    for i=1:iteration,
      fn_pred = [PAR.FN.output_lsl.fn_pred 'iteration_' num2str(i) '/results.mat'];
      if fexist(fn_pred),
        L=load(fn_pred) ;
        if isfield(L, 'pred'),
          fprintf('\nResults on validation/training set from iteration %i:\n', i) ;
          L.pred
        end ;
      end ;
    end ;
  end ;

  %fprintf('\n\n new constraints during iteration %i: %i\n\n', iteration,num_new_constraints_iteration);
  %if 1%num_new_constraints_iteration/num_examples<0.98
  %  method.exm_per_solve = num_examples;
  %elseif num_new_constraints_iteration/num_examples<0.999
  %  method.exm_per_solve = max(method.exm_per_solve, ceil(num_examples/2));
  %end

  %% termination
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if length(obj_solve)>20 && std([obj_solve(end-4:end).OBJ])<0.1
    fprintf('\n\n objective change very small %i\n\n', iteration);
    break;
  end
  if num_new_constraints_iteration==0,
    fprintf('\n\nno new constraints during iteration %i\n\nfinishing optimization.', iteration);
    break;
  end
end
if fid>2
  fclose(fid) ;
end

if ~isempty(jobinfo_predict)
  fprintf('\nWaiting for predictions on validation/training set\n') ;
  jobinfo_predict = rproc_wait(jobinfo_predict, 60, 1, 0, 2) ;

  for i=1:iteration,
    fn_pred = [PAR.FN.output_lsl.fn_pred 'iteration_' num2str(i) '/results.mat'];
    if fexist(fn_pred),
      L=load(fn_pred) ;
      if isfield(L, 'res'),
        fprintf('\nResults on validation/training set from iteration %i:\n', i) ;
        fprintf('Exon level accuracy:\n')
        print_struct(1, L.res.cds_exons) ;
        fprintf('Transcript level accuracy:\n')
        print_struct(1, L.res.cds_transcripts) ;
      end ;
    end ;
  end ;
  rproc_cleanup(jobinfo_predict) ;
end ;

fn_predictor = sprintf('%s_iteration%i.mat', PAR.FN.output_lsl.fn_train, iteration);

if isfield(PAR, 'save_fname') && ~isempty(PAR.save_fname),
  unix(sprintf('cp %s %s', fn_predictor, PAR.save_fname)) ;
  save_append(PAR.save_fname, 1, 'iter', iteration, 'fn_predictor', fn_predictor) ;
end ;

fprintf('Trained gene predictor in %s\n\n', fn_predictor) ;


return

function weights = vector2weights(vector, weights, weight_names)
% weights = vector2weights(vector, weights)
  
%weights_name = fieldnames(weights);
last_pos = 0 ;
for w = 1:length(weight_names)
  old_w = getfield(weights, weight_names{w}) ;
  weights = setfield(weights, weight_names{w}, vector(last_pos+1:last_pos+length(old_w)));
  last_pos = last_pos + length(old_w) ;
end
assert(last_pos==length(vector)) ;
return

function vector = weights2vector(weights, weight_names)
% vector = weights2vector(weights)
  
vector = [];

%weights_name = fieldnames(weights);
for w=1:length(weight_names)
  vector=[vector, getfield(weights, weight_names{w})];
end

return

function [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)
% [ALL,is_invalid] = basic_cand_stuff(gene,ALL,sig,debug,signal_name)

if nargin<5
  signal_name =[];
end

is_invalid=0; 

genic_idx = find(ALL.POS>gene.start&ALL.POS<gene.stop);
ALL.GENE_ID(genic_idx) = gene.id;

if isfield(gene,'transacc') && ~isempty(gene.transacc)
  ALL.TRANS(genic_idx) = 1;
end
if ~isempty(gene.tss)
  ALL.TSS(genic_idx) = 1;
end

if ~isempty(signal_name)&&~isempty(gene.(signal_name))
  ALL.HAS_SIG(genic_idx) = 1;
end
  
if ~gene.is_valid
  is_invalid=1; 
  ALL.GENE_ID(genic_idx) =  - gene.id;
  return
end

if isequal(sig,'other')
  if gene.is_alt 
    ALL.ALTGENIC(genic_idx) = 1;
  end
else
  if any(gene.exon_in_alt_region)
    alt_reg = get_connected_regions(gene.exon_in_alt_region);  
    
    for i=1:length(unique(alt_reg))-1
      temp=find(alt_reg==i);
      alt_start = gene.splicegraph{1}(1,temp(1));
      alt_stop = gene.splicegraph{1}(2,temp(end));
      alt_genic_idx = find(ALL.POS>alt_start&ALL.POS<alt_stop);
      ALL.ALTGENIC(alt_genic_idx) = 1;
      if debug
        viewsplicegraph(gene,1)
        hold on 
        plot([alt_start alt_start],[1 10],'b')
        plot([alt_stop alt_stop],[1 10],'b')  
      end
    end
  end
end

if gene.is_alt 
  ALL.ALTGENIC(genic_idx) = 1;
end
return

function calc_plifs(PAR_1)
fid=1;
q=0;

fn_SVMs = PAR_1.FN.output_sig.(PAR_1.Signal_name).fn_SVMs;
method = PAR_1.Signals.(PAR_1.Signal_name).method;
num_partitions = PAR_1.SETs.num_partitions;

for p = 1:num_partitions
  fprintf(fid,'\n\n-----------\n');
  fprintf(fid,'PARTITION %i\n',p);
  fprintf(fid,'-----------\n');
  fn_SVM = sprintf('%sbest_partition=%i.mat', fn_SVMs,p) ;

  %--------------------------------------------------------------------------------%
  % load parameters from SVM file
  %--------------------------------------------------------------------------------%
  load(fn_SVM,'Train');
  PAR = Train.PAR;
  PAR.fn_SVMs=fn_SVM;
  clear Train
  
  %--------------------------------------------------------------------------------%
  % learn PLIFS on evaluation set and convert output on test set
  %--------------------------------------------------------------------------------%
  PAR.fn_pred = sprintf('%spartition=%i.mat',fn_SVMs,p);
  PAR.fn_plifs = sprintf('%spartition=%i_plifs.mat',fn_SVMs,p);
  assert(fexist(PAR.fn_pred)==1)
  assert(fexist(PAR.fn_plifs)==1)
  fprintf(fid,'\n already predicted on set ');
  fprintf(fid,'%i',PAR.SETs.which_pred);
        
  %--------------------------------------------------------------------------------%
  %%% learn PLIFS on evaluation set
  %--------------------------------------------------------------------------------%
  clear  prob prob_cum limits Conf Conf_cum
  fprintf('\nload fn_plif\n')
  load(PAR.fn_plifs,'LT', 'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_plifs)))
    if ~all(ismember({'prob','prob_cum', 'limits'},who_file(PAR.fn_SVMs)))
      disp('learn PLIFs')
      assert(length(output)==length(LT))
      [prob, prob_cum,limits] = learn_plifs(output,LT,method.plif.bins,1,method.plif.min_incr );
      %save(PAR.fn_SVMs,'-append','prob', 'prob_cum','limits');
      save_append(PAR.fn_SVMs, 1, 'prob', prob, 'prob_cum', prob_cum, ...
                  'limits', limits);
    else
      load(PAR.fn_SVMs,'prob', 'prob_cum','limits');
    end
    disp('Convert SVM Output to Confidence on fn_plif')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_plifs,'-append','Conf','Conf_cum');
    save_append(PAR.fn_plifs,1, 'Conf', Conf, 'Conf_cum', Conf_cum);
    clear output Conf Conf_cum LT
  end
  %--------------------------------------------------------------------------------%
  %%% Convert SVM Outputs on test set 
  %--------------------------------------------------------------------------------%
  load(PAR.fn_pred,'output');
  if ~all(ismember({'Conf','Conf_cum'},who_file(PAR.fn_pred)))
    disp('Convert SVM Output to Confidence on fn_pred')
    [Conf, Conf_cum] = Out2Conf(output,prob, prob_cum,limits);
    %save(PAR.fn_pred,'-append','Conf','Conf_cum');
    save_append(PAR.fn_pred, 1,'Conf', Conf, 'Conf_cum', Conf_cum);
  end
end%loop over partitions
% [jobinfo,num_crashed] = rproc_wait(jobinfo,20,1,0);

return

function POS=  find_consensus(seq,Signal,strand)
%  POS=  find_consensus(seq,Signal,strand) 

consensus = Signal.consensus ; 
lwin_big = Signal.lwin_big;
rwin_big = Signal.rwin_big;
offset = Signal.offset;



POS = zeros(1,floor(length(seq)/10));
num = 0;
seq=upper(seq);
for m = 1:length(consensus)  
  cons = upper(consensus{m});
  pos_tmp = findstr(seq(lwin_big+1:end-rwin_big),cons)+lwin_big;
  POS(num+[1:length(pos_tmp)]) = pos_tmp;
  num = num + length(pos_tmp);
end
POS(num+1:end)=[];
clear num 
POS = POS + offset;
%if strand=='-' & strcmp(Signal.name,'polya')
%  POS = -POS + length(seq)+1-length(cons)+1;
%elseif strand=='-' 
if strand=='-'
  POS = -POS + length(seq)+1;
end
POS = unique(sort(POS));

return

function ALL = get_coverage(ALL,gene,true_splice,label_2,info,info_names)
% [COV1,COV2] = get_coverage(ALL,gene,true_splice,info_names)
% 
% 
% get sum over all coverage for each true_splicesite and certain transcript()
% 
% INPUT  - gene:  gene structure of length 1;  
%          required fields:transcript_info,transcript_valid,exons,exons_confirmed, is_alt
%        - true_splice_site:    vector of all true splice sites (either
%          donor or acceptor, not both) that occur in the
%          splicegraph of the gene.
%        - info_names: cell which tells, you which transcripts to look
%          at ('anno','cDNA',...)
%   
% OUTPUT - cov1  number of coverage by given transcript type (according to info_names) for each true splice site. 
%        - cov2  number of times skipped by given transcript type (according to info_names) for each true splice site. 

  
COV1 = ALL.COV1 ;
COV2 = ALL.COV2;
POS = ALL.POS;
  
genic_idx = find(POS>gene.start&POS<gene.stop);
genic_pos = POS(genic_idx); 
assert(all(all(COV2(:,genic_idx)==0)))
assert(all(all(COV1(:,genic_idx)==0)))

cov1 = zeros(size(info_names,1),length(true_splice));
cov2 = zeros(size(info_names,1),length(true_splice));

% idx_const = find(sum(label_2==1)==0);
const_exons = gene.splicegraph{1}(:,find(~gene.exon_in_alt_region));
[tmp,idx_const,tmp] = intersect(true_splice,const_exons(:));
assert(all(sum(label_2(:,idx_const)==1)==0))
for type = 1:size(info_names,1) 
  if isequal(info_names{type,1},'anno')
    cov1(type,:) = -inf;
    cov2(type,:) = -inf;
  end
  t_idx = [];
  for tt1 = 1:size(info_names,2)
    if ~isempty(info_names{type,tt1})
      t_idx = [t_idx find(gene.transcript_info==info.(info_names{type,tt1}))];  
    end
  end
  if isempty(t_idx)
    continue
  end
  for t = t_idx
    if ~gene.transcript_valid(t)
      continue
    end
    [cov1_t, cov2_t] = get_coverage4transcript(gene,t,true_splice,info);
    if gene.transcript_info(t)==info.anno
      cov1(type,:) = max(cov1(type,:),cov1_t);
      cov2(type,:) = max(cov2(type,:),cov2_t); 
      if ~isempty(idx_const)&&~all(isinf(cov2_t(idx_const)))
        warning('here seems to be bug that should be fixed ... ') ;
        if keyboard_allowed()
          keyboard
        end
      end
    else
      cov1(type,:) = cov1(type,:) +cov1_t;
      cov2(type,:) = cov2(type,:) +cov2_t;
      if ~isempty(idx_const)&&~all(cov2_t(idx_const)==0)
        if keyboard_allowed()
          keyboard
        end
      end
    end
  end %%% loop over transcripts
end %%% loop over transcript types
idx_anno = find(strcmp(info_names,'anno')) ;
not_anno = setdiff(idx_anno,[1:size(info_names,1)]);

ii_alt2 = find(any(label_2==1));
ii_ir2 = find(label_2(2,:)==1);
if ~isempty(ii_alt2) && ~all(sum(cov2(not_anno,ii_alt2))>0|any(~isinf(cov2(idx_anno,ii_alt2))))
  warning('no cov2')
  % hold off;
  % viewsplicegraph(gene);
  if ~isempty(ii_ir2)
    fprintf('intron retention')
    % for a=1:length(true_alt_ir)
    %    hold on ; plot([true_alt_ir(a) true_alt_ir(a)],[1 10],'b')
    % end
  else
    %keyboard
  end
  gene.id
  warning('keyboard')
  %keyboard
end


[ii,ii1,ii2] = intersect(POS,true_splice);
assert(all(ismember(ii1,genic_idx)))

COV1(:,ii1) = cov1(:,ii2);
COV2(:,ii1) = cov2(:,ii2);

%%% SET COV2 OF NEGATIVE GENIC POSITIONS to min over coverage of
% neigbouring truesplicesites

true_splice = true_splice(ii2);
cov1 = cov1(:,ii2);


genic_neg_pos = setdiff(genic_pos,true_splice);
genic_cov2  =   zeros(size(info_names,1),length(genic_neg_pos));
[temp,ii1,ii2] = intersect(POS,genic_neg_pos);
assert(length(temp)==length(genic_neg_pos));
genic_neg_pos = genic_neg_pos(ii2);
if ~isequal(determine_engine, 'octave'),
  assert(issorted(true_splice))
end ;
for a=2:length(true_splice)
  idx = find(genic_neg_pos<true_splice(a)&genic_neg_pos>true_splice(a-1));
  genic_cov2(:,idx) = repmat(min([cov1(:,a),cov1(:,a-1)]'),length(idx),1)';
end
assert(all(all(COV2(:,ii1)==0)))
assert(all(all(COV1(:,ii1)==0)))
COV2(:,ii1) =genic_cov2;
    






function [cov1, cov2] = get_coverage4transcript(gene,transcript_idx,true_splice,info)

% [cov1, cov2] = get_coverage(gene,transcrit_idx,true_splice)
%
% for a given transcript transcrit_idx of the gene counts number of coverage for each
% of the true splice sites. 
% returned is cov1 the number of times a splice site is used in one
% evidence (e.g. transcript). In the case of ESTs this can be more than
% one, because a single transcript is produced out of a cluster of
% several ESTs.This information is stored in the
% field exons_confirmed{transcript_idx}, which is a Nx2 matrix, with the
% first number giving the confirmation for the exon and the second number
% giving the confirmation for the succeding
% intron. (exons_confirmed{transcript_idx}(end,2) shoeld always be 0, as
% there is no intron after the last exon.)
% The second returned value cov2 gives the number of evidences that
% disagree with the splicesite. e.i. the number of times a splice site is skipped.
   
  

exons = gene.exons{transcript_idx}(:,1:2);
exons_conf = gene.exons_confirmed{transcript_idx};

cov1 = -inf(1,length(true_splice));
cov2 = -inf(1,length(true_splice));

for a = 1:length(true_splice)
  %%% CHECK COVERAGE OF INCLUDED FORM     
  [idx1,idx2] = find(exons==true_splice(a));
  if ~isempty(idx1)
    assert(length(idx1)==1)
    cov1(a) = exons_conf(idx1,1); 
  end
  %%% CHECK EXON COVERAGE OF SKIPPED FORM    
  idx  = find(exons(:,1)<true_splice(a)&exons(:,2)>true_splice(a)) ;
  if ~isempty(idx)   
    assert(length(idx)==1)
    % cov2(a) = exons_conf(idx,2); 
    if gene.transcript_info(transcript_idx)==info.est|gene.transcript_info(transcript_idx)==info.cDNA &&...
          (idx==1|idx==size(exons,1))
      if all(label_2(:,a)==-1)
        continue
      else
        % alternative 3' or 5' or first or last exon; est ends
        % are usually unconfirmed 
      end
    elseif gene.transcript_info(transcript_idx)==info.anno
      if min(abs(true_splice(a)-exons(idx,:)))<3 
        % keyboard
        continue
      end
    end
    cov2(a) = exons_conf(idx,1);
  end  
  %%% CHECK INTRON COVERAGE OF SKIPPED FORM
  idx  = find(exons(1:end-1,2)<true_splice(a)&exons(2:end,1)>true_splice(a)) ;
  if ~isempty(idx)            
    assert(length(idx)==1)           
    if gene.transcript_info(transcript_idx)==info.anno
      cov2(a) = max(cov2(a),exons_conf(idx,2));      
    else
      cov2(a) = cov2(a)+exons_conf(idx,2);
    end
  end 
end %% loop over true_splice sites

assert(~any(~isinf(cov1)&~isinf(cov2)))





return

function  [ALL,label_2] = get_true_alternative_sites(ALL,gene,true_splice,sig,debug)

strand = gene.strand;
POS=ALL.POS;
LABEL_2=ALL.LABEL_2;

if (isequal(sig,'acc') & strand=='+')|...
      (isequal(sig,'don') & strand=='-'),
  which_end = 1;
  other_end = 2;
elseif (isequal(sig,'don')& strand=='+') | ...
      (isequal(sig,'acc')& strand=='-')
  which_end = 2;
  other_end = 1;
end
   
  
label_2 = -ones(3,length(true_splice)); 
% GET TRUE ALTERNATIVE SPLICE SITES

ii_es2 = []; ii_o2 = [];  ii_ir2 = [];  ii_3p2 = [];  ii_5p2 = []; 

if gene.alt_exon %% EXON SKIP
  alt_exons = gene.alt_exon_details.exons;
  true_alt_es = gene.splicegraph{1}(which_end,alt_exons(2,:)) ;
  [temp,ii1,ii_es2] = intersect(true_alt_es,true_splice);
  assert(length(temp)==length(unique(true_alt_es)))
  true_alt_es = true_alt_es(ii1);
  % splice_before_es = gene.splicegraph{1}(don_end,alt_exons(1,ii1)) ;
  label_2(1,ii_es2) = 1;
  if debug
    fprintf(1,'exon skip')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_es)
      plot([true_alt_es(jj) true_alt_es(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end ;
  end
end %if gene.alt_exon

if gene.alt_intron
  alt_introns = gene.alt_intron_details.exons;
  true_alt_ir = gene.splicegraph{1}(which_end,alt_introns(other_end,:));
  [temp,ii1,ii_ir2] = intersect(true_alt_ir,true_splice);
  assert(length(temp)==length(unique(true_alt_ir)))
  true_alt_ir = true_alt_ir(ii1);
  % splice_before_ir = gene.splicegraph{1}(acc_end,alt_introns(1,ii1));
  % intron_ret(1,:) = gene.splicegraph{1}(which_end,alt_introns(other_end,ii1));
  % intron_ret(2,:) = gene.splicegraph{1}(other_end,alt_introns(which_end,ii1));
  label_2(2,ii_ir2) = 1;
  if debug
    fprintf(1,'intron retention')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_ir)
      plot([true_alt_ir(jj) true_alt_ir(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end

if gene.alt_5prime&isequal(sig, 'don') 
  %if ~isempty(gene.alt_5prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_5p = gene.alt_5prime_details.exons(1,:);
  else
    alt_5p = gene.alt_5prime_details.exons(2,:);
  end ;
  alt_5p = unique([alt_5p gene.alt_5prime_details.exons(3,:)]);
  true_alt_5p = gene.splicegraph{1}(which_end,alt_5p);
  [temp,ii1,ii_5p2] = intersect(true_alt_5p,true_splice);
  assert(length(temp)==length(unique(true_alt_5p)))
  true_alt_5p = true_alt_5p(ii1);
  % splice_before_5p = gene.splicegraph{1}(acc_end,);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_5p2) = 1;
  if debug
    fprintf(1,'alt 5prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_5p)
      plot([true_alt_5p(jj) true_alt_5p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard
    end 
  end
end
if gene.alt_3prime&isequal(sig,'acc')
  %if ~isempty(gene.alt_3prime_details), keyboard; end ;
  if gene.strand=='+',
    alt_3p = gene.alt_3prime_details.exons(2,:);
  else
    alt_3p = gene.alt_3prime_details.exons(1,:);
  end ;
  alt_3p = unique([alt_3p gene.alt_3prime_details.exons(3,:)]);
  true_alt_3p = gene.splicegraph{1}(which_end,alt_3p);
  [temp,ii1,ii_3p2] = intersect(true_alt_3p,true_splice);
  assert(length(temp)==length(unique(true_alt_3p)))
  true_alt_3p = true_alt_3p(ii1);
  % splice_before_3p = gene.splicegraph{1}(don_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_before = gene.splicegraph{1}(other_end,[exon_alt_3prime(find(idx_alt_3prime==g_idx)).fiveprimesite]);
  % don_after = gene.splicegraph{1}(other_end,alt_3p);
  label_2(3,ii_3p2) = 1;
  if debug
    fprintf(1,'alt 3prime')
    viewsplicegraph(gene,1)
    hold on
    for jj=1:length(true_alt_3p)
      plot([true_alt_3p(jj) true_alt_3p(jj)],[0 10],'b');
    end
    if keyboard_allowed(),
      keyboard 
    end 
  end
end
[ii,ii1,ii2] = intersect(POS,true_splice);
LABEL_2(:,ii1) = label_2(:,ii2);





return

function ALL = initialize_allconfs(POS,info_names)

ALL.POS = POS;  
  
ALL.GENE_ID = zeros(1,length(POS)); 
ALL.ALTGENIC = zeros(1,length(POS)); 
ALL.COV1 = zeros(length(info_names),length(POS)); 
ALL.COV2 = zeros(length(info_names),length(POS)); 
ALL.LABEL_2 =  -ones(3,length(POS)); 
ALL.ISSPLICE = zeros(1,length(POS)); 

ALL.TRANS = zeros(1,length(POS)); 
ALL.TSS = zeros(1,length(POS)); 
ALL.HAS_SIG = zeros(1,length(POS));
ALL.DIST = zeros(1,length(POS)); 
ALL.CONF_TRANS = zeros(1,length(POS)); 

return

function [Signal,info_names,sig,alt_names] = initialize_cand_search(Signal,info,genes,strand,signal_name,debug)
% [Signal,info,info_names,sig,alt_names] = initialize_cand_search(PAR,genes,strand,signal_name,debug)

rand('seed',1234);

% info = PAR.info_genes.(signal_name);
% Signal = PAR.Signals.(PAR.Signal_name);

if isempty(Signal.consensus) && isfield(Signal, 'consensus_fn') && ~isempty(Signal.consensus_fn)
  load(Signal.consensus_fn,'consensus')
  Signal.consensus = consensus;
end

info_names = fieldnames(info);
use = [];
fprintf(1,'getting conf for ');
for i=1:length(info_names)
  if ismember([info_names{i} '_conf'],Signal.Conf_names)
    use = [use i];
    if isequal(signal_name,'transcript')&~isequal(Signal.name,'transacc')
      assert(ismember([info_names{i} '_conf_skip'],Signal.Conf_names))
    end
    fprintf(1,'%s, ',info_names{i})
  end
end
fprintf('\n')
info_names = info_names(use);

% if ismember('cDNA',info_names)
%   use = strmatch('cDNA',info_names,'exact');
%   info_names{use,2} = 'fulllength';
%   fprintf(1,'\nfulllength cDNAs added to cDNAs\n')
% end

fprintf(1,'\n');

if ~isempty(genes)
  strands = unique([genes.strand]);
  assert(strand==strands(1))
  assert(length(strands)==1);
  assert(length(unique([genes.chr_num]))==1);
end


sig='';
switch Signal.name
 case {'acc','transacc','alt_exon_acc'}
  sig = 'acc';
 case {'don','alt_exon_don'}
  sig = 'don';
 otherwise
  sig = 'other';
end

if isequal(sig,'other') || isequal(Signal.name,'transacc') || ...
      isequal(Signal.name,'acc') || isequal(Signal.name,'don') 
  alt_names ={};
  return
end

alt_names = {'alt_exon', 'alt_intron', 'alt_5prime', 'alt_3prime'};
use = []; 
fprintf(1,'getting label 2 for ');
for i=1:length(alt_names)
  if ismember(alt_names{i},Signal.Conf_names)
    fprintf(1,'%s, ',alt_names{i})
  end
end

return

function [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(Signal,fn_filter_settings,fn_examples, which, filter_mode,subsample_neg,subsample_pos)
% [Data,par] = load_data(PAR, which, filter_mode,subsample_neg,subsample_pos)
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% load data of training set
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% method = PAR.method;
% Signal = PAR.Signals.(PAR.Signal_name);
%  FN = PAR.FN;
% input_sig = FN.input_sig.(Signal_name);
% fn_filter_settings = input_sig.fn_filter_settings;
% fn_examples = input_sig.fn_examples;

method = Signal.method;

XT = []; LT = [] ; POS.region_id = []; POS.pos=[]; POS_tmp = []; 
SC = []; use_sc = [];
if nargin<4
  subsample_neg=1;
end
if nargin<5
  subsample_pos=1;
end

use_cons = Signal.method.use_cons;
% if nargin<6
%   use_cons = 0;
% end
for w = which
  % create filename of input file
  filename = gen_example_filename(Signal,fn_filter_settings,fn_examples,filter_mode,w);
  
  fprintf('loading XT, LT and POS from file %s \n',filename)
  L = load(filename,'LT');
  L.XT = load_matrix(filename, 'XT');
  P = load(filename, 'POS');
  %if isfield(signal,'center')
  %  par.center = signal.center;
  %else
  %  par.center = size(L.XT,1)./2;
  %end
  
  have_pos = isfield(P, 'POS') ;
  have_lt = isfield(L, 'LT') ;
  if ~have_lt, L.LT=[] ; end ;
  have_sc = isfield(L, 'SC_') ;
  if ~have_sc, L.SC_=[] ; end ;  

  [L.XT, L.LT, P.POS, L.SC_] = subsample(L.XT, L.LT, P.POS, L.SC_, subsample_neg, subsample_pos) ;

  if ~have_lt, L=rmfield(L, 'LT') ; end ;
  if ~have_sc, L=rmfield(L, 'SC_') ; end ;
  
  if ~isfield(method,'feature_type') || isempty(method.feature_type) ||isequal(method.feature_type, 'char')
    L.XT_char = char(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = char(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'uint8')
    L.XT_char = uint8(L.XT) ;
    for i=1:size(L.XT,1)
      L.XT_char(i,:) = uint8(double(L.XT(i,:))) ;
    end ;
    XT = [XT L.XT_char];
  elseif isequal(method.feature_type, 'double')
    XT = [XT L.XT];
  else
    error('unknown feature type; should be "char" or "double"')
  end
  if isfield(L,'LT')
    if size(L.LT,1)>size(L.LT,2)
      L.LT = L.LT';
    end
    LT = [LT L.LT] ;
  end
  
  if (isfield(L,'SC_')&& isfield(L.use_sc)) || use_cons
    fprintf('using consensus information\n');
    SC = [SC L.SC_] ;
    use_sc = [use_sc L.use_sc];
  end

  POS.region_id =[POS.region_id P.POS.region_id];
  POS.pos = [POS.pos P.POS.pos];
end ;
%%% just for compatibility with older versions 
  
% check whether everything is ok (matlab bug successfully avoided?)
for i=1:size(XT,1)
  if all(XT(i,:)==0)
    % i
    % keyboard
  end
  if  ~isfield(method,'feature_type') || isequal(method.feature_type, 'char')
    assert(all(XT(i,:)~=0)) ;
  end
end ;

clear L
if ~isempty(LT)
  fprintf('%i positive, %i negative examples (%.2f %% positives), sequences of length %i \n',sum(LT==1),sum(LT==-1),sum(LT==1)/length(LT)*100, size(XT,1));
end

if ~isempty(SC)
  XT(:,~use_sc) = [] ;
  POS(~use_sc) =[];
  LT(~use_sc) = [];
  SC(:,~use_sc) = [];
  if ~isempty(LT)
    fprintf('%i positive, %i negative examples (%.2f %% positives) \n',sum(LT==1),sum(LT==-1),sum(LT==1)/sum(LT==1|LT==-1)*100);
  end
end

Data.LT = LT;
% Data.XT = char(XT) ;
Data.XT = XT;
Data.Pos = POS;
clear XT LT POS
%this assert is very mem consuming
%assert(~any(XT(:)~='A' & XT(:)~='G' & XT(:)~='C' & XT(:)~='T')) ;
% XT = to_upper_case(char(XT)) ;
if ~isempty(SC)
  Data.SC = char(SC) ;
  assert(~any(SC(:)~='A' & SC(:)~='G' & SC(:)~='C' & SC(:)~='T')) ;
  
  % SC = to_upper_case(char(SC)) ;
end


assert(size(Data.XT,2)==length(Data.LT))
assert(length(Data.LT)==length(Data.Pos.pos))
assert(length(Data.LT)==length(Data.Pos.region_id))

return

function regions=load_sequence(regions);

% regions=load_sequence(regions);
  
genome_info = init_genome(regions(1).config);


for i=1:length(regions)
  if isfield(regions(i),'num_fragments')&&~isempty(regions(i).num_fragments)
    num_fragments = regions(i).num_fragments;
  else
    num_fragments = 1;
  end
  for p=1:num_fragments
    if mod(i,100)==0,
      fprintf(1,'region %i of %i\r',i,length(regions))
    end ;
    chr = regions(i).chr_num(p) ;
    str = regions(i).strand(p) ;
    start = regions(i).start(p) ;
    stop = regions(i).stop(p) ;
    seq = upper(load_genomic(genome_info.contig_names{chr},str,start,stop,genome_info));
    assert(ischar(seq)) ;
    idx = (~ismember(seq,'ACGT')) ;
    seq(idx)='N';
    if p==1
      regions(i).seq = seq;
    else
      regions(i).seq = [regions(i).seq ,seq];
    end
  end
end

return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic]=read_sigpred(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name]=read_sigpred(sigpred_fname)

if fexist([sigpred_fname '.gz']) && ~fexist(sigpred_fname),
  ret = unix(sprintf('gzip -d %s',sigpred_fname)) ;
  assert(ret==0) ;
end ;


chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

fd = fopen(sigpred_fname, 'r') ;

rest_text = '' ; part = 0 ; part_offset = 0 ;
line_idx = 0 ;
while ~feof(fd),
  part = part + 1 ;
  text=fread(fd, 100000000, 'uint8=>char') ;
  text=[rest_text text'] ;
  rest_text = '' ;
  if length(text)==0, 
    break ; 
  end ;
  last_idx = find(text==sprintf('\n'), 1, 'last') ;
  if last_idx~=length(text) && ~feof(fd),
    rest_text = text(last_idx+1:end) ;
    text(last_idx:end) = [] ;
  end ;
  text = [text sprintf('\n')] ;
  line_idx = find(text==sprintf('\n')) ;

  if length(line_idx)>2, 
    chr(length(chr)+1 : length(chr) + length(line_idx) -2 ) = nan ; 
    signal_name(length(signal_name)+1:length(signal_name) + length(line_idx)-2) = nan ;
    score_name(length(score_name)+1 : length(score_name) + length(line_idx)-2) = nan ;
    strand(length(strand)+1 : length(strand) + length(line_idx)-2) = ' ' ;
    pos(length(pos)+1 : length(pos) + length(line_idx)-2) = nan ;
    score(length(score)+1 : length(score) + length(line_idx)-2) = nan ;
  end ;

  chr_idx = 0 ;
  signal_name_idx = 0 ;
  score_name_idx = 0 ;
  progress = 0 ;
  for i=1:length(line_idx)-1,
    line=text(line_idx(i)+1:line_idx(i+1)-1) ;
    if isempty(line), 
      chr(i) = nan ;
      score(i) = nan ;
      score_name(i) = nan ;
      signal_name(i) = nan ;
      pos(i) = nan ;
      strand(i) = ' ' ;
      continue; 
    end ;
    if isequal(line(1),'#')
      continue
    end

    elem_idx = find(line==sprintf('\t')) ;
    
    if (i-1)/length(line_idx)>=progress,
      fprintf('\rPart %i: Read line %i/%i (%2.0f%%)     ', part, (i-1), length(line_idx), progress*100);
      progress = progress+0.05 ;
    end ;
    if chr_idx==0 || ~isequal(line(1:elem_idx(1)-1), chr_name_dic{chr_idx}),
      chr_idx = strmatch(line(1:elem_idx(1)-1), chr_name_dic, 'exact') ;
      if isempty(chr_idx),
        chr_name_dic{end+1} = line(1:elem_idx(1)-1) ;
        chr_idx = length(chr_name_dic) ;
      end ;
    end ;
    
    if signal_name_idx==0 || ~isequal(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic{signal_name_idx}),
      signal_name_idx = strmatch(line(elem_idx(1)+1:elem_idx(2)-1), signal_name_dic, 'exact') ;
      if isempty(signal_name_idx),
        signal_name_dic{end+1} = line(elem_idx(1)+1:elem_idx(2)-1) ;
        signal_name_idx = length(signal_name_dic) ;
      end ;
    end ;
    
    if score_name_idx==0 || ~isequal(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic{score_name_idx}),
      score_name_idx = strmatch(line(elem_idx(2)+1:elem_idx(3)-1), score_name_dic, 'exact') ;
      if isempty(score_name_idx),
        score_name_dic{end+1} = line(elem_idx(2)+1:elem_idx(3)-1) ;
        score_name_idx = length(score_name_dic) ;
      end ;
    end ;
    
    chr(i+part_offset) = chr_idx ;
    signal_name(i+part_offset) = signal_name_idx ;
    score_name(i+part_offset) = score_name_idx ;
    strand(i+part_offset) = line(elem_idx(4)+1:elem_idx(5)-1) ;
    pos(i+part_offset) = str2double(line(elem_idx(3)+1:elem_idx(4)-1)) ;
    score(i+part_offset) = str2double(line(elem_idx(5)+1:end)) ;
  end

  part_offset = part_offset + length(line_idx)-1 ;
end ;
fclose(fd) ;

fprintf('\rPart %i: Read line %i/%i (%2.0f%%)              \n', part, ...
        length(line_idx), length(line_idx), 100);
signal_name(isnan(chr)) = [] ;
score_name(isnan(chr)) = [] ;
strand(isnan(chr)) = [] ;
pos(isnan(chr)) = [] ;
score(isnan(chr)) = [] ;
chr(isnan(chr)) = [] ;

assert(all(chr~=0)) ;


return

function [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)
% [chr, strand, pos, score, signal_name, score_name, chr_name_dic, signal_name_dic, score_name_dic, success]=read_sigpred_bin(sigpred_fname)

chr_name_dic = {} ;
score_name_dic = {} ;
signal_name_dic = {} ;

chr=[] ; 
signal_name = [] ;
score_name = [] ;
strand = '' ;
pos = [] ;
score = [] ;

if ~fexist(sigpred_fname),
  success=false ;
  return ;
end ;

cnt = 0; last=0;
while ~last,
  try
    SPF=load(sigpred_fname, sprintf('SPF_%i', cnt)) ;
    if ~isfield(SPF, sprintf('SPF_%i', cnt)),
      break
    end ;
  catch
    break ;
  end 
  if ~isempty(SPF)
    eval(sprintf('SPF=SPF.SPF_%i;', cnt)) ;
    cnt=cnt+1 ;
  %else
  %  SPF=load(sigpred_fname) ;
  %  last = 1 ;
  end ;

  if any(ismember(chr_name_dic, SPF.contig_name)),
    contig_idx = find(ismember(chr_name_dic, SPF.contig_name)) ;
  else
    chr_name_dic{end+1} = SPF.contig_name ;
    contig_idx = length(chr_name_dic) ;
  end ;

  if any(ismember(score_name_dic, SPF.score_name)),
    score_idx = find(ismember(score_name_dic, SPF.score_name)) ;
  else
    score_name_dic{end+1} = SPF.score_name ;
    score_idx = length(score_name_dic) ;
  end ;

  if any(ismember(signal_name_dic, SPF.signal_name)),
    signal_idx = find(ismember(signal_name_dic, SPF.signal_name)) ;
  else
    signal_name_dic{end+1} = SPF.signal_name ;
    signal_idx = length(signal_name_dic) ;
  end ;

  chr(end+1:end+length(SPF.pos))=contig_idx ;
  score_name(end+1:end+length(SPF.pos))=score_idx ;
  signal_name(end+1:end+length(SPF.pos))=signal_idx ;
  pos(end+1:end+length(SPF.pos))=SPF.pos ;
  score(end+1:end+length(SPF.pos))=SPF.score ;
  strand(end+1:end+length(SPF.pos))=SPF.strand ;
end ;

success = true ;

return

function regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos,offset)

% regions = retrieve_signals(regions,base_dir,signal_name,fields,which_pos)
%
% loads all candidate positions and addional informations as specified in
% fields,(i.e.'label','anno_cover','est_cover','cdna_cover','Conf_cum' )
% from binary fiels and adds them to the field Signals.signal_name of the
% structure regions 
%
% see also save_label_files
if nargin<6
  offset = 0;
end
  
if nargin<5
  which_pos.local = 1;
  which_pos.contig = 1;
end

genome_info = init_genome(regions(1).config);

f_name = sprintf('%scontig_%i%s.pos', base_dir,regions(1).chr_num,regions(1).strand );
if ~fexist(f_name),
  fprintf('file %s does not exist\n', f_name);
end

if ~isempty(fields)
  f_idx = [];
  for f=1:length(fields),
    f_name = sprintf('%scontig_%i%s.%s', base_dir,regions(1).chr_num,regions(1).strand, fields{f});
    if fexist(f_name),
      f_idx = [f_idx f];
    else
      fprintf('file %s does not exist\n', f_name);
    end
  end
  if ~isempty(f_idx),
    fields = fields(f_idx);
  else
    error('no files available')
  end
end

if ~isfield(regions,'Signals')
  regions(1).Signals = [];
end


for r=1:length(regions)
  if isfield(regions(r),'num_fragments')&&~isempty(regions(r).num_fragments)
    num_fragments = regions(r).num_fragments;
  else
    num_fragments = 1;
  end
  prefix = 0;
  
  SIGNALS.pos = [];
  SIGNALS.contig_pos = [];
  for f=1:length(fields)
    SIGNALS.(fields{f})= [];
  end
  if ismember('pos2',fields)
    SIGNALS.contig_pos2 = [];
  end
  for p=1:num_fragments
    contig_length = dir(genome_info.flat_fnames{regions(r).chr_num(p)}) ;
    contig_length = contig_length.bytes;
    %if 1%r==5145
    %  fprintf(1, 'regions.stop: %i, regions.start: %ir: %i',regions(r).start,regions(r).stop, r)
    %end
    if regions(r).start(p)<1
      warning('regions.start<1; set to 1!')
      regions(r).start(p) = 1;
    end
    if regions(r).stop(p)>contig_length
      warning('regions.stop>contig length; set to contig_length!')
      regions(r).stop(p) = contig_length;
    end
    f_name = sprintf('%scontig_%i%s',base_dir, regions(r).chr_num(p), ...
                     regions(r).strand(p));
    if fexist([f_name '_sha1sum.mat'])
      [tmp,sha1sum] = unix(sprintf('sha1sum %s.pos',f_name));
      sha1sum = separate(sha1sum,' ');
      Old = load([f_name '_sha1sum'],'sha1sum');
      Old.sha1sum = separate(Old.sha1sum,' ');
      if ~isequal(Old.sha1sum{1},sha1sum{1})
        warning('sha1sum different')
      end
      %  else
      %    warning('no sha1sum check performed')
    end
    
    if isempty(fields)
      fields{1} ='pos';
    end
    if regions(r).start(p)+offset>regions(r).stop(p)-offset
      regions(r).Signals.(signal_name) = SIGNALS;
      continue
    end
    d=dir([f_name '.pos']) ;
    if d(1).bytes==0,
      contig_pos = [] ;
      score = zeros(0, length(fields)) ;
    else
      [contig_pos,score] = interval_query(f_name,fields,[regions(r).start(p)+offset;regions(r).stop(p)-offset]);
    end ;
    if ismember('pos2',fields)
      if ~isempty(contig_pos)
        idx_p = find(strcmp('pos2',fields));
        contig_pos2 = score(:,idx_p);
      else
        contig_pos2 =[];
      end
    end
    assert(isequal(contig_pos, sort(contig_pos))) ;
    %assert(issorted(contig_pos))
    if ~isempty(contig_pos)
      if ~(contig_pos(1)>=regions(r).start(p)&contig_pos(end)<=regions(r).stop(p))
        if keyboard_allowed()
          keyboard
        end;
      end
    end
    if which_pos.local
      if regions(r).strand(p)=='+'
        pos = contig_pos - regions(r).start(p) + 1+prefix;
        if exist('contig_pos2')
          pos2 = contig_pos2 - regions(r).start(p) + 1+prefix;
        end
        SIGNALS.pos = [SIGNALS.pos pos];
      else
        pos = regions(r).stop(p) - contig_pos + 1;
        [pos,idx] = sort(pos);
        score = score(idx,:);
        contig_pos = contig_pos(idx);
        if exist('contig_pos2')
          contig_pos2 = contig_pos2(idx);
          pos2 = regions(r).stop(p) - contig_pos2 + 1;
          SIGNALS.pos = [SIGNALS.pos pos2];
        else
          SIGNALS.pos = [SIGNALS.pos pos];
        end
      end
    end
    if which_pos.contig
      SIGNALS.contig_pos = [SIGNALS.contig_pos contig_pos];
      if exist('contig_pos2')
        SIGNALS.contig_pos2 = [SIGNALS.contig_pos2 contig_pos2];
        if regions(r).strand(p)=='+'
          SIGNALS.pos2 = [SIGNALS.pos2 pos2];
        else
          SIGNALS.pos2 = [SIGNALS.pos2 pos];
        end
      end
    end
    for f=1:length(fields)
      if strmatch(fields{f},'label') ix=f; end
      if ~isequal(fields{f},'pos')&~isequal(fields{f},'pos2')
        SIGNALS.(fields{f})= [SIGNALS.(fields{f}), score(:,f)];
      end
    end
    if exist('ix')
      u = unique(score(:,ix));
      if length(u)==1 assert(u==-1|u==1),
      elseif length(u)==2 assert(all(u==[-1;1])),
      elseif isempty(u),
      else error('unique(label) is not [-1;1]');
      end
    end
    regions(r).Signals.(signal_name) = SIGNALS;
    prefix = prefix +regions(r).stop(p)- regions(r).start(p);
  end %%loop over fragments
end %% loop over blocks

return

function save_predictions_as_sigpred(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.spf'];

if fexist(fn_out) | fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in signal prediction format (spf): %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');

fprintf(fd, '##Seqid\tType\tSource\tStart\tStrand\tScore\n') ;
for j=1:length(pos)
  fprintf(fd, '%s\t%s\t%s\t%i\t%c\t%.3f\n', contig_name, signal_name, score_name, pos(j), strand, score(j)) ;
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out)) ;
  if ret~=0
    warning('gzip failed') ;
  end
end 


return

function save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)
% save_predictions_as_sigpred_bin(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

fn_out=[out_fn_prefix '_' score_name '_spf.mat'];

if fexist(fn_out) 
  fprintf(1, 'file exists, it will NOT be overwritten\n')
  return
end
fprintf(1,'save in binary signal prediction format (spf): %s\n', fn_out);
save(fn_out, '-V7', 'contig_name', 'signal_name', 'score_name', 'pos', 'strand', 'score') ;

return

function fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)
% fn_out = save_predictions_as_wiggle(out_fn_prefix, pos, score, score_name, signal_name, contig_name, strand, do_gzip)

%assert(issorted(pos))
assert(isequal(pos, sort(pos)))
assert(length(pos)==length(score))

if ~exist('do_gzip', 'var'),
  do_gzip = 1 ;
end ;

fn_out=[out_fn_prefix '_' score_name '.wiggle'];

if fexist(fn_out)|fexist([fn_out '.gz'])
  fprintf(1, 'file exists, it will NOT be overwritten\n')
%  keyboard
  return
end

fprintf(1,'save as wiggle: %s\n', fn_out);
[fd msg] = fopen(fn_out, 'w+');
disp(msg)

fprintf(fd,'browser position chr%s:1-50,000\n', contig_name);
fprintf(fd,'browser hide all\n');
fprintf(fd,'browser pack Genes\n');
fprintf(fd,'browser full altGraph\n');
fprintf(fd,'browser viewLimits=%1.2f:%1.2f\n', min(score), max(score));


fprintf(fd,['track type=wiggle_0 name="%s_%s%s" description="%s chr%s %s" ' ...
            '\n'],signal_name, score_name, strand, score_name, ...
        contig_name, strand);
fprintf(fd,'variableStep chrom=chr%s\n',contig_name);

for j=1:length(pos)
  fprintf(fd,'%i\t%.3f\n', pos(j), score(j));
end

fclose(fd);

if do_gzip,
  ret = unix(sprintf('gzip %s',fn_out));
  if ret~=0
    warning('gzip failed')
  end
end ;

return

function flag = segment_overlap(start,stop,starts,stops)
% flag = segment_overlap(start,stop,starts,stops)  

flag = ((start >= starts) & (start<=stops)) | ...
       ((stop >= starts) & (stop<=stops)) | ...
       ((start <=starts) & (stop>=stops)) | ...
       ((start >=starts) & (stop<=stops)) ;
  
return

function CONF = set_CONF(Signal,ALL,sig,info_names,signal_name)
% CONF = set_CONF(Signal,ALL,sig,info_names)

%%% GENERAL 
idx = find(strcmp(Signal.Conf_names,'altgenic'));
if ~isempty(idx)
  CONF(idx,:) = ALL.ALTGENIC;
end
idx = find(strcmp(Signal.Conf_names,'gene_id'));
if ~isempty(idx)
  CONF(idx,:) = ALL.GENE_ID;
end

for i=1:length(info_names)
  idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf']));
  CONF(idx,:) = ALL.COV1(i,:);
  if ~isequal(sig,'other')&~isequal(Signal.name,'transacc')
    idx = find(strcmp(Signal.Conf_names,[info_names{i} '_conf_skip']));
    CONF(idx,:) = ALL.COV2(i,:);
  end
end

%%% ALT SPLICE
idx = find(strcmp(Signal.Conf_names,'alt_exon'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(1,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_intron'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(2,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_3prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end
idx = find(strcmp(Signal.Conf_names,'alt_5prime'));
if ~isempty(idx)
  CONF(idx,:) = ALL.LABEL_2(3,:);
end

idx = find(strcmp(Signal.Conf_names,['is' sig]));
if ~isempty(idx)
  CONF(idx,:) = ALL.ISSPLICE;
end

%%% SPECIAL THINGS
idx = find(strcmp(Signal.Conf_names,['has_' signal_name]));
if ~isempty(idx)
  CONF(idx,:) = ALL.HAS_SIG;
end
idx = find(strcmp(Signal.Conf_names,'has_trans'));
if ~isempty(idx)
  CONF(idx,:) = ALL.TRANS;
end


idx = find(strcmp(Signal.Conf_names,'dist'));
if ~isempty(idx)
  CONF(idx,:) = ALL.DIST;
end

if exist('CONF', 'var') && ~(size(CONF,1)==length(Signal.Conf_names))
  warning('this was an assert') ;
end ;

return

function Content = cds_exon

name = 'cds_exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=content_template(name)

Content.name = name;
Content.consensus = [];
Content.lwin_big = 0;
Content.rwin_big = 0;
%Content.order = [3 4 5];
Content.label_fct = 'get_cand_contents';
Content.load_fct = 'load_content_data';

Content.filter_label.train.USE_ALL = 1;
Content.filter_label.test.USE_ALL = 1;
Content.filter_label.eval.USE_ALL = 1;
Content.Conf_names = {} ;
Content.C = [1]; 

Content.wordlen = { };
Content.stepping = 1;
Content.offset = 0;

Content.export_settings.conf_cum_thresh = 0.5 ;
Content.export_settings.resolution = 1 ;


%PAR.Contents.type_descrs = { 'intergenic', 'utr_exon', 'cds_exon','intron', 'intercistronic' };
%PAR.Contents.type_abbrs = { 'ige' 'utr' 'cex' 'ino' 'ics' };
%PAR.Contents.num_types = length(PAR.Contents.type_descrs); 



return

function Content = frame0(info)

name = 'frame0';

Content = content_template(name);
Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 3;

Content.offset = mod(3-mod(Content.wordlen{1},3),3);
 

return

function Content = intergenic

name = 'intergenic';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=intron

name = 'intron';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content = polya_tail

name = 'polya_tail';

Content = content_template(name);

Content.wordlen = { [3:5]};

return

function Content = utr3exon

name = 'utr3exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5]};
Content.stepping = 1; 
Content.offset = 0;
return

function Content=utr5exon

name = 'utr5exon';

Content = content_template(name);

Content.wordlen = {[3],[4],[5],[6]};
Content.stepping = 1; 
Content.offset = 0;
return

function method = SVM(Signal)
  
method = method_template;

% set parameters, model selection 

method.name = 'SVM';
method.train_fct = 'svm_train';
method.predict_fct = 'svm_predict';
method.kernels = {[]};
method.poim_maxOrder = 8;
method.svm_name = 'LIGHT';
method.qpsize = 50;
method.threads = 16;
method.center = Signal.lwin_big;
method.par_ms.C = Signal.C;

%PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;

% use multi kernel learning
method.use_mkl = 0;





return

function method = SVM_COMBINED(Signal)

% Signals = SVM_COMBINED(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_COMBINMED -> SVM -> method_template


method = SVM(Signal) ;

nofKernels = length(Signal.kernel_name);

assert(length(Signal.lwin)==nofKernels)
assert(length(Signal.rwin)==nofKernels)


for k=1:nofKernels
  for n = Signal.kernel_par_names 
    method.kernels{k}.(n{1}) = Signal.(n{1}){k};
  end
end
if isfield(Signal,'subkernel_weights')
  method.subkernel_weights = Signal.subkernel_weights;
end


return

function method = SVM_Cont(Content)

  
method = SVM(Content);
method.feature_type = 'char';
method.center = [];
method.kernels{1}.name = 'LINEAR';
method.genomewide_predict_fct = 'svm_content_predict';

for k=1:length(Content.wordlen)
  method.kernels{k}.wordlen = Content.wordlen{k};
  method.kernels{k}.stepping = Content.stepping ;
  method.kernels{k}.offset = Content.offset ;
end



return

function method = SVM_WD(Signal)

method = SVM(Signal);
method.feature_type = 'char';

kernel.name = 'WD';
kernel.order = Signal.order;
kernel.shift = 0 ;
kernel.lwin = Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 


return

function method = SVM_WDS(Signal)

% Signals = SVM_WDS(Signals,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field (signal_name) is modified.
%          Particuarly a field method is added.
%        - signal_name can be something in {'acc','don','tss','cleave','polya','tis','cdsStop'};
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: SVM_WDS -> SVM -> method_template

  
method = SVM(Signal) ;
method.feature_type = 'char';


kernel.name = 'WDS';
kernel.order = Signal.order;
kernel.shift = Signal.shift;
kernel.lwin =  Signal.lwin{1};
kernel.rwin = Signal.rwin{1};

method.kernels{1} = kernel; 

return

function method = method_template

% Signals = method_template(Signal,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field Signal.(signal_name).method is added. Default values are set.

method.name = [];
method.train_fct = [];
method.predict_fct = [];

% PAR.method.svm_name = 'GPBTSVM';PAR.method.qpsize=500;
% use conservation information
% method.use_cons = 0;
% load training examples only once and calculate all models in one job
% method.submit_batch=0;

method.plif.bins = 50;
method.plif.min_incr = 1e-6;

method.use_cons = 0;
method.feature_type = [];
%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).lwin_big)

%assert(max(abs([PAR.Signals.(PAR.Signal_name).lwin{:}]))<PAR.Signals.(PAR.Signal_name).rwin_big)




return

function PAR=A_thaliana(PAR)

PAR = organism_template(PAR);

PAR.organism.name = 'A_thaliana';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters = 1;
PAR.label_source.tss.fulllength = 0;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 0;
PAR.label_source.cleave.db = 0;

PAR.label_source.cds.anno = 0;
PAR.label_source.cds.maxorf = 0;


PAR.regions.cluster_paralogs = 1;

return

function [organism label_source] = C_elegans(label_source_orig,signal_names)

[organism label_source] = Nematodes(label_source_orig,signal_names);

organism.name = 'C_elegans';
organism.full_name = 'Caenorhabditis_elegans';
organism.release = '';
organism.version = '';
organism.genebuilt = '';
return

function PAR = D_melanogaster(PAR)


PAR = organism_template(PAR);

PAR.organism.clade = 'insect';
PAR.organism.name = 'D_melanogaster';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 0;
PAR.label_source.tss.db = 0;

return

function PAR=D_rerio(PAR)
  
PAR = organism_template(PAR);

PAR.organism.name = 'D_rerio';

PAR.label_source.use_anno = 0;
PAR.label_source.use_confirmed = 1;

PAR.label_source.tss.est_clusters  = 1;
PAR.label_source.tss.fulllength   = 1;
PAR.label_source.tss.db = 0;

PAR.label_source.cleave.est_clusters = 1;
PAR.label_source.cleave.fulllength = 1;
PAR.label_source.cleave.db = 0;

return

function [organism label_source] = dummy_organism(label_source_orig,signal_names)

[label_source] = organism_template(label_source_orig,signal_names);
  
organism.clade = [];
organism.name = '';
organism.full_name = '';
organism.release = '';
organism.version = '';
organism.genebuilt = '';


return

function [label_source] = organism_template(label_source_orig,signal_names)


% PAR = set_label_source(PAR)
% 
% sets defaults for all label_sources (don't use anno, use EST confirmed only.)
% values that are already assigned will NOT be overriden.
  
  
%% default values for general use of annotation and EST confrimed sequences
label_source = label_source_orig;

if isempty(label_source.use_anno)
  label_source.use_anno = 0;
end
if isempty(label_source.use_confirmed)
  label_source.use_confirmed = 1;
end


%% default values for individual signals


for j=1:length(signal_names)
  signal_name = signal_names{j};
  if label_source.use_anno
    label_source.(signal_name).anno = 1;
  else
    label_source.(signal_name).anno = 0;
  end
  if label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 1;
    label_source.(signal_name).cDNA = 1;
    label_source.(signal_name).fulllength = 1;
  elseif ~label_source.use_confirmed & ~isequal(signal_name,'transacc')
    label_source.(signal_name).est_clusters = 0;
    label_source.(signal_name).cDNA = 0;
    label_source.(signal_name).fulllength = 0;
  end
  label_source.(signal_name).db = 0;
  label_source.(signal_name).from_fn_candsites = 1 ;
end

label_source.tis.maxorf = 0;
label_source.cdsStop.maxorf = 0;

label_source.transacc.anno_SL1 = 0;
label_source.transacc.anno_SL2 = 0;
label_source.transacc.pred = 1;
label_source.transacc.db = 0;



return

function Signal = acceptor(label_source,signal_name)

% Signal = acceptor(label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field acc (transcc) is modified.  
%        - label_source is a struct that should contain a field
%          (signal_name). this is used in the called function splicesite_template.
%        - name can either be 'acc' or 'transacc'  
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
% (function acceptor is also called from transacc)
  
  
if nargin<3 
  signal_name = 'acc';
end

Signal = splicesites_template(label_source,signal_name);

%%%% SET ACCEPTOR STUFF

Signal.consensus = {'AG'};
Signal.offset = 2;

Signal.lwin = {[-60]} ;
Signal.rwin = {[80]} ;

Signal.order = [ 22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_3prime'};

Signals.(signal_name) = Signal;



return

function Signal = cdsStop(label_source)
  
% Signals = cdsStop(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cdsStop is modified.
%        - label_source is a struct that should contain a field cdsStop. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
   
signal_name = 'cdsStop';

Signal = translatends_template(label_source,signal_name);

%%%% SET cdsStop STUFF

Signal.consensus = {'TAA','TAG','TGA'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-110] [0]} ; 
Signal.rwin = {[0] [10] [180]};
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[20],[]};  
Signal.wordlen = {[2] [] [4]}; 
Signal.C = 2.8284; % logC=1.5



return

function Signal = cleave(label_source)

% Signals = cleave(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field cleave is modified.
%        - label_source is a struct that should contain a field cleave. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
signal_name = 'cleave';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET CLEAVE STUFF

Signal.lwin_big = 410;
Signal.rwin_big = 810;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = {[-400], [-60], [0]};
Signal.rwin = {[100], [60], [800]};
Signal.wordlen = {[5], [], [3]}; 

Signal.order = {[],[18],[]};
Signal.shift = {[],[0.1],[]};
Signal.shift_const = {[],[0],[]};
Signal.C = 1; % log2(C)=0



return

function Signal = donor(label_source)

% Signals = donor(Signals,signal_name,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field don is modified.
%        - label_source is a struct that should contain a field don. 
%          this is used in the called function signal_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template

 
signal_name = 'don';

Signal = splicesites_template(label_source,signal_name);

%%%% SET DONOR STUFF

Signal.consensus = {'GC' 'GT'};
Signal.offset = 0;

Signal.lwin = {[-80]};
Signal.rwin = {[60]};

Signal.order = [22];
Signal.shift = 0.5;
Signal.C = [3]; 

Signal.Conf_names = {Signal.Conf_names{:},'alt_5prime'};




return

function Signal = polya(label_source,fn_polya)
  
% Signals = polya(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field polya is modified.
%        - label_source is a struct that should contain a field polya. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: polya -> signal_template

signal_name = 'polya' ; 
Signal = signal_template(label_source,signal_name);

%%%% SET POLYA STUFF

Signal.name = signal_name;
Signal.type = signal_name; 
Signal.label_fct = 'get_cand_others';
Signal.filter_fct = 'filter_cands';

Signal.offset = 0;

Signal.lwin_big = 310;
Signal.rwin_big = 310;

Signal.kernel_name= {'WDS','SP','SP','SP','SP','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.subkernel_weights = [1 0.2 0.2 0.2 0.2 0.2];

Signal.lwin = {-300 -300 -149 1 151 -300} ;
Signal.rwin = {306 -150 0 150 306 306};
Signal.shift={[0 0.1 0.4],[],[],[],[],[]};
%Signal.shift={[0.1],[],[],[],[],[]};
Signal.shift_const={[0 10 20],[],[],[],[],[]};
%Signal.shift_const={[0],[],[],[],[],[]};
Signal.wordlen={[],[4],[4],[4],[4],[4]};
Signal.order={[18],[],[],[],[],[]};
Signal.C = [0.01 0.5 10];

Signal.consensus = {''};
if exist('fn_polya')
  Signal.consensus_fn = fn_polya;
  if fexist(Signal.consensus_fn)
    load(Signal.consensus_fn,'consensus')
    Signal.consensus = consensus;
  end
end

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
%Signal.filter_label.eval.use_intergenic=0;
%Signal.filter_label.eval.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.train.conf_pos.anno = 0;
Signal.filter_label.train.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos = 0; 

%Signal.filter_label.eval.conf_pos.anno = 0;
%Signal.filter_label.eval.conf_pos.from_cleave = [201 301 401];
%Signal.filter_label.eval.conf_pos.db = 0;
%Signal.filter_label.eval.conf_pos.fulllength = 0;
%Signal.filter_label.eval.onlywithpos = 0; 

Signal.filter_label.test.conf_pos.anno = 0;
Signal.filter_label.test.conf_pos.from_cleave = [201 301 401];
Signal.filter_label.test.conf_pos.db = 0;
Signal.filter_label.test.conf_pos.fulllength = 0;
Signal.filter_label.test.onlywithpos = 0; 


Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



Signal.label_source_par.polya.wl = 1;
Signal.label_source_par.polya.POSconstr =1;
Signal.label_source_par.polya.maxpcCov = 95;
Signal.label_source_par.polya.logdir = [];

return

function Signal = signal_template(label_source,signal_name)

% Signals = signal_template(Signal,label_source,signal_name)
% Signals is a struct with a field for each Signal. In this function only
% the field signal_name is added. Default values are set.


if nargin<2
  signal_name = 'dummy';
end

%%%%%

Signal.type = []; 
Signal.name = signal_name;
Signal.consensus = {[]};
Signal.offset = [];

Signal.lwin_big = [];
Signal.rwin_big = [];

Signal.label_fct = [];
Signal.filter_fct = 'filter_cands';
Signal.load_fct = 'load_data';
Signal.score_fct = [];

Signal.Conf_names = { 'altgenic','gene_id' };

Signal.sampling.downsample_neg = 1; % <=1 (for no downsampling: 1)  
Signal.sampling.mindist = 1; % >=1 (for all: 1)  
Signal.sampling.pos_neg_ratio = 1/10;

Signal.resolution   = 1; % for lsl

conf.USE_ALL = 0;
conf.conf_pos = struct;
conf.conf_pos_skip = struct;
conf.any_conf_pos = [];
conf.any_conf_pos_skip = [];
conf.conf_neg = struct;
conf.conf_neg_skip = struct;
conf.any_conf_neg = [];
conf.any_conf_neg_skip = [];
conf.use_intergenic=[];
conf.use_alt = [];
conf.use_label = 'label';

Signal.filter_label.train = conf ; 
Signal.filter_label.test = conf ;
Signal.filter_label.eval = conf ;

Signal.export_settings.conf_cum_thresh = 0 ;
Signal.export_settings.resolution = 1 ;

Signal.domain_adapt.name = 'target_only';
Signal.domain_adapt.Source_organisms = [];

Signal.method = struct;

label_source = label_source.(signal_name);
sources = fieldnames(label_source);
for i=1:length(sources)
  if label_source.(sources{i})
    Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf']};
    if isequal(signal_name,'acc')|isequal(signal_name,'don')
      Signal.Conf_names = {Signal.Conf_names{:},[sources{i} '_conf_skip']};
    end
  end
end




return

function Signal = splicesites_template(label_source,signal_name)

% Signals = splicesites_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name: 'don', 'acc','transacc'
%  
% OUTPUT Signals with modified field signal_name; 
%  
% HIERACHY: acceptor/donor -> splicesite_template -> signal_template
%
% see also transcriptends_template/translat_template
  
  
INF=1e6;

Signal = signal_template(label_source,signal_name);

%%%% SET SPLICE SITE STUFF

Signal.type = 'splicesites'; 
Signal.label_fct = 'get_cand_splicesites';

Signal.lwin_big = 199;
Signal.rwin_big = 199;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.train.use_alt_neg=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;
Signal.filter_label.test.use_alt_neg=0;

Signal.filter_label.train.use_label = 'label';
Signal.filter_label.test.use_label = 'label';


Signal.filter_label.train.use_decoys = 1;
Signal.filter_label.test.use_decoys = 1;


Signal.filter_label.train.conf_pos.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos.cDNA = [0 INF];
Signal.filter_label.train.conf_pos.fulllength = [0 INF];
Signal.filter_label.train.conf_pos.anno = [0:1] ;

Signal.filter_label.train.conf_pos_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_pos_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_pos_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_pos_skip.anno = [-inf] ;

Signal.filter_label.train.conf_neg.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg.cDNA = [0 INF];
Signal.filter_label.train.conf_neg.fulllength = [0 INF];
Signal.filter_label.train.conf_neg.anno = [-1:1] ;

Signal.filter_label.train.conf_neg_skip.est_clusters = [0 INF];
Signal.filter_label.train.conf_neg_skip.cDNA = [0 INF];
Signal.filter_label.train.conf_neg_skip.fulllength = [0 INF];
Signal.filter_label.train.conf_neg_skip.anno = [-1:1] ;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

% Signal.save_confs = 1;
% Signal.save_alts = 1;


Signal.export_settings.conf_cum_thresh = 0.5 ;
Signal.export_settings.resolution = 1 ;


Signal.Conf_names = {Signal.Conf_names{:},'alt_exon','alt_intron'};





return

function Signal = tis(label_source)
  
% Signals = tis(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tis is modified.
%        - label_source is a struct that should contain a field tis. 
%          this is used in the called function signal_template.
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template

    
signal_name = 'tis';

Signal = translatends_template(label_source,signal_name);

%%%% SET TIS STUFF

Signal.consensus = {'ATG'};


Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};
Signal.lwin = {[-200] [-30] [0]} ; 
Signal.rwin = {[0] [110] [180]}; 
Signal.shift={[],[0],[]};
Signal.shift_const={[],[0],[]};
Signal.order={[],[24],[]};  
Signal.wordlen = {[3] [] [5]}; 
Signal.C = 1; % log2(C)=1.0










return

function Signal = transacceptor(label_source)

% Signals = transacceptor(Signals,label_source)
% INPUT  - Signals is a struct with a field for each Signal. In this function only
%          the field transcc is modified.  
%        - label_source is a struct that should contain a field
%          transacc. this is used in the called function splicesite_template.
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tansacceptor -> acceptor -> splicesite_template -> signal_template

INF = 1e6;  
signal_name = 'transacc';

Signal = acceptor(label_source,signal_name);

%%%% SET transacc STUFF

Signal.lwin = {[-190]} ;
Signal.rwin = {[80]};


Signal.filter_label.train.conf.anno_SL1 = 0;
Signal.filter_label.train.conf.anno_SL2 = [0 INF];
Signal.filter_label.train.conf.pred = [0 INF];

Signal.filter_label.eval.conf.anno_SL1 = 0;
Signal.filter_label.eval.conf.anno_SL2 = [0 INF];
Signal.filter_label.eval.conf.pred = [0 INF];


Signal.filter_label.test.conf.anno_SL1 = 0;
Signal.filter_label.test.conf.anno_SL2 = [0 INF];
Signal.filter_label.test.conf.pred = [0 INF];


Signal.filter_label.train.use_intergenic = 1;
Signal.filter_label.train.use_alt = 0;
Signal.filter_label.eval.use_intergenic = 1;
Signal.filter_label.eval.use_alt = 0;
Signal.filter_label.test.use_intergenic = 1;
Signal.filter_label.test.use_alt = 0;
 
Signal.filter_label.train.use_pred = 1;
Signal.filter_label.eval.use_pred = 1;
Signal.filter_label.test.use_pred = 1;

Signal.filter_label.train.only_acc = 1;
Signal.filter_label.eval.only_acc = 1;
Signal.filter_label.test.only_acc = 1;

Signal.filter_label.train.pos_dist = 20;
Signal.filter_label.eval.pos_dist = 20;
Signal.filter_label.test.pos_dist = 20;

Signal.export_settings.conf_cum_thresh = 0.05 ;
Signal.export_settings.resolution = 1 ;

Signal.Conf_names = {Signal.Conf_names{:},'isacc','has_tss','has_trans','dist'};


Signal.label_source_par.transacc.win_anno_SL1 = 50;
Signal.label_source_par.transacc.win_anno_SL2 = 50;
Signal.label_source_par.transacc.win_pred = 5;
Signal.label_source_par.transacc.threshold_pred = 0.5;


return

function Signal = transcriptends_template(label_source,signal_name)

% Signals = transcriptends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tss', 'cleave'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template
%
% see also splicesite_template/translat_template
  
  
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSCRIPT END STUFF

Signal.type = 'transcriptends'; 
Signal.label_fct = 'get_cand_others';


Signal.consensus = [];
Signal.offset = 0;

Signal.sampling.downsample_neg = 1/10; %% take randomly every 10th porsitions 
Signal.sampling.mindist = 10 ;


Signal.resolution   = 20; % for lsl

Signal.filter_label.train.conf_pos.anno = [0 1];
Signal.filter_label.train.conf_pos.est_clusters = [2:3];
Signal.filter_label.train.conf_pos.db = 0;
Signal.filter_label.train.conf_pos.fulllength = 0;
Signal.filter_label.train.onlywithpos=0; 

Signal.filter_label.train.use_intergenic=1;
Signal.filter_label.train.use_alt=0;

Signal.filter_label.test = Signal.filter_label.train;
Signal.filter_label.eval = Signal.filter_label.train;

Signal.export_settings.conf_cum_thresh = 0.01 ;
Signal.export_settings.resolution = 10 ;

Signal.Conf_names = {Signal.Conf_names{:},['has_' signal_name]};



return

function Signal = translatends_template(label_source,signal_name)

% Signals = translatends_template(Signals,label_source,signal_name)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field signal_name is added or
%          modified. 
%          Default values are set.
%        - label_source is a struct that should contain a field (signal_name). 
%          this is used in the called function signal_template.
%        - signal_name:'tis', 'cdsStop'
%
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tis/cdsStop -> translatends_template -> signal_template
%
% see also splicesite_template/transcriptends_template
  
INF=1e6;
  
Signal = signal_template(label_source,signal_name);

%%%% SET TRANSLAT END STUFF

Signal.type = 'translatends' ;
Signal.label_fct = 'get_cand_others';

Signal.offset = 0;
Signal.lwin_big = 210;
Signal.rwin_big = 200;



Signal.filter_label.train.conf_pos.anno = [0 1] ;
Signal.filter_label.train.conf_pos.maxorf = INF;
Signal.filter_label.test.conf_pos.anno = [0 1] ;
Signal.filter_label.test.conf_pos.maxorf =INF;

Signal.filter_label.train.use_intergenic=0;
Signal.filter_label.train.use_alt=0;
Signal.filter_label.test.use_intergenic=0;
Signal.filter_label.test.use_alt=0;

Signal.filter_label.eval = Signal.filter_label.test;

% PAR.label_source.cds.par.max_nof_paths = 100;
% PAR.label_source.cds.par.nof_best_kept = 2;


return

function Signal = tss(label_source,has_transacc)
  
% Signals = tss(Signals,label_source,has_transacc)
% INPUT  - Signals is a struct with a field for each Signal. 
%          In this function only the field tss is modified.
%        - label_source is a struct that should contain a field tss. 
%          this is used in the called function signal_template.
%        - has_transacc: is per default set to zero, only 1 for nematodes. 
%       
% OUTPUT Signals with modified field signal_name; 
%
% HIERACHY: tss/cleave -> transcriptends_template -> signal_template

  
if nargin<3
  has_transacc=0; 
end
   
signal_name = 'tss';

Signal = transcriptends_template(label_source,signal_name);

%%%% SET TSS STUFF


Signal.lwin_big = 601;
Signal.rwin_big = 901;

Signal.kernel_name= {'SP','WDS','SP'};
Signal.kernel_par_names={'kernel_name','order','shift','shift_const','wordlen','lwin','rwin'};

Signal.lwin = { [-600] [-70] [   0] };
Signal.rwin = { [+100] [+70] [+900] };
Signal.wordlen = { [4] [] [4] };

Signal.order ={[] [16] []};
Signal.shift ={[] [0] []};
Signal.shift_const ={[] [28] []};
Signal.C = 2; % log2(C)=1.0

if has_transacc
  Signal.Conf_names = {Signal.Conf_names{:},'has_trans'};
  Signal.filter_label.train.rm_trans = 1;
  Signal.filter_label.eval.rm_trans = 1;
  Signal.filter_label.test.rm_trans = 1;
else
  Signal.filter_label.train.rm_trans = 0;
  Signal.filter_label.eval.rm_trans = 0;
  Signal.filter_label.test.rm_trans = 0;
end

%%%%%% for human
%Signal.sampling.downsample_neg = 1/160; % <=1 (for no downsampling: 1)



return

function PAR=exp_galaxy(PAR_config)
% PAR=exp_galaxy(PAR_config)

PAR.SETs.num_partitions=5 ;
PAR.SETs.num_splits=5 ;

PAR.FN.input_dir=PAR_config.dir ;
PAR.FN.output_dir=PAR_config.dir;
PAR.FN.genome_dir=PAR_config.dir ;
PAR.FN.input.fn_genome_config = PAR_config.genome_config ;

% preparing RPROC settings
PAR.RPROC.MEMREQ = 6000 ;
PAR.RPROC.collect_jobs = 0 ;
PAR.RPROC.exm_per_batch = 1 ;
if 0
PAR.RPROC.options.addpaths = {'~/svn/projects/genefinding/sensors',
                    '~/svn/projects/genefinding/data_processing_sensors',
                    '~/svn/projects/genefinding/contents',
                    '~/svn/projects/genefinding/galaxy/tools/',
                    '~/svn/projects/genefinding/utils',
                    '~/svn/projects/genefinding/shogun',
                    '~/svn/projects/genefinding/data_processing_lsl',
                    '~/svn/tools/genomes',
                    '~/svn/tools/utils'} ;

end

PAR.RPROC.options.addpaths = {} ;

signals = {'acc','don','tis','cdsStop','tss','cleave'};
for i=1:length(signals)
  PAR.label_source.(signals{i}).from_fn_candsites = 1 ;
  PAR.Signals.(signals{i}).filter_label.train.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.test.USE_ALL = 1;
  PAR.Signals.(signals{i}).filter_label.eval.USE_ALL = 1;
  PAR.Signals.(signals{i}).Conf_names = {} ;
  PAR.Signals.(signals{i}).method.threads = 1;
end


if 0,
  PAR.Signals.acc.order = 10 ;
  PAR.Signals.acc.shift = 0 ;

  PAR.Signals.don.order = 10 ;
  PAR.Signals.don.shift = 0 ;
  
  PAR.Signals.tis.order =  {[],[5],[]};
  PAR.Signals.tis.shift =  {[],[0],[]};
  
  PAR.Signals.cdsStop.order = {[],[5],[]};
  PAR.Signals.cdsStop.shift = {[],[0],[]};
  
  PAR.Signals.tss.order = {[],[5],[]};
  PAR.Signals.tss.shift = {[],[0],[]};
  
  PAR.Signals.cleave.order = {[],[5],[]};
  PAR.Signals.cleave.shift = {[],[0],[]};

end


contents = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};

for i=1:length(contents)
  PAR.label_source.(contents{i}).from_fn_candsites = 1 ;
  PAR.Contents.(contents{i}).filter_label.train.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.test.USE_ALL = 1;
  PAR.Contents.(contents{i}).filter_label.eval.USE_ALL = 1;
  PAR.Contents.(contents{i}).Conf_names = {} ;
  PAR.Contents.(contents{i}).method.threads = 1;
end

PAR.LSL.method.use.signals.polya = 0 ;
PAR.LSL.method.exm_per_solve = 100 ;

PAR.regions.max_length = 30000 ;


if 1,

%----- REGULARIZATION
method=PAR.LSL.method ;

%C = 1 ;
C = 1e-3;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-3;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;

PAR.LSL.method=method ;

end

return

function PAR=exp_galaxy_A_thaliana(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'A_thaliana';  

PAR.SETs.num_partitions=5;
PAR.SETs.num_splits=5;
%PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 1;
PAR.tasks.learn_plifs_on =1;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 0 ;
PAR.tasks.train_on = 1;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='' ;
PAR.FN.output_dir='' ;

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;

return

function PAR=exp_galaxy_C_elegans(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'C_elegans'; 

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_melanogaster(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_melanogaster';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function PAR=exp_galaxy_D_rerio(PAR)

if nargin<1 
  PAR=init_PAR;
end

PAR.date=date;
PAR.exp_name='';
PAR.dir_name ='';

PAR.organism.name = 'D_rerio';  

PAR.SETs.num_partitions=1;
PAR.SETs.num_splits=1;
PAR.SETs.which_pred_svm = 1;

PAR.tasks.modelsel_on = 0;
PAR.tasks.learn_plifs_on =0;
PAR.tasks.pred_from_seq=1;
PAR.tasks.train_error_on = 0;
PAR.tasks.all_models_one_job = 0;
PAR.tasks.prepare_labels=0;
PAR.tasks.pred_on_genome = 1;
PAR.tasks.train_on = 0;
PAR.tasks.run_locally=1;

PAR.FN.input_dir='/tmp';
PAR.FN.output_dir='/tmp';

PAR.RPROC.collect_jobs = 1;
PAR.RPROC.options.ncpus = 1;
PAR.RPROC.exm_per_batch = 1;
PAR.RPROC.time_req = 3600;
PAR.RPROC.MEMREQ = 3000;


return

function FN=init_FN(signal_names,content_names)
 
  
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end
 
%--------------------------------------------------------------------------
% Directories
% -------------------------------------------------------------------------------------

FN.genome_dir= '/fml/ag-raetsch/share/databases/genomes/';
FN.input_dir= '/fml/ag-raetsch/share/projects/genefinding/';
FN.output_dir = '/fml/ag-raetsch/share/projects/genefinding/';

FN.dir_name = '';
FN.exp_name = '';
FN.lsl_exp_name = '';
%--------------------------------------------------------------------------
% INPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.genome.fn_confirmed_sequences = 'confirmed_sequences.mat';
FN.genome.fn_genome_config_orig  = 'genome_uncondensed.config';
FN.genome.fn_genome_config  = 'genome.config';
FN.genome.fn_map = 'condense_map.txt';
FN.genome.fn_genes_anno = 'annotated_genes.mat';
FN.genome.fn_genes_merged = '';
FN.genome.fn_genes_conf = 'confirmed_sequences.mat';
FN.genome.fn_name_to_contig_mapping = 'sequences/contig_orig_new_mapping';


FN.input.fn_genome_config = 'genome.config';
FN.input.fn_trainings_regions = 'training_regions.mat';
FN.input.fn_test_regions = 'test_regions.mat';
FN.input.fn_genes_anno_orig ='annotated_genes.mat';
FN.input.fn_genes_conf_orig = 'confirmed_sequences.mat';
FN.input.fn_ESTs_matches = {'est_matches.mat'};

%--------------------------------------------------------------------------
% OUTPUT FILE NAMES
% -------------------------------------------------------------------------------------

FN.output.fn_genes_anno ='anno_genes.mat';
FN.output.fn_genes_conf = 'conf_genes.mat';
FN.output.fn_genes_merged = 'genes_all.mat';
FN.output.fn_genes_signals = 'genes_signals.mat';
FN.output.fn_genes_contents = 'annotated_genes.mat';
FN.output.fn_genes_lsl = 'annotated_genes.mat';
FN.output.fn_ESTs_matches = [];
FN.output.fn_trivial_regions = 'trivial_regions.mat';
FN.output.fn_blocks = 'blocks.mat';
FN.output.fn_training_blocks ='training_blocks.mat';
FN.output.fn_test_blocks = 'test_blocks.mat';

%--------------------------------------------------------------------------
% SIGNAL FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(signal_names)
  sig_name = signal_names{j};
  FN.input_sig.(sig_name).fn_candsites = 'cands/' ;
  FN.input_sig.(sig_name).fn_examples = 'examples';
  FN.input_sig.(sig_name).fn_pos = 'pos/';
  FN.input_sig.(sig_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_sig.(sig_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_sig.(sig_name).fn_models='MS/models';
  FN.output_sig.(sig_name).fn_SVMs='MS/';
  FN.output_sig.(sig_name).fn_pred='pred/';
  FN.output_sig.(sig_name).fn_motifs='MS/motifs';
end


if any(strcmp(signal_names,'tss'))
  FN.input_sig.tss.fn_est_cluster = 'tss_est_clusters.mat';
  FN.input_sig.tss.fn_fulllength  = 'tss_fulllength.mat';
  FN.input_sig.tss.fn_db = 'tss_db.mat' ;
end

if any(strcmp(signal_names,'cleave'))
  FN.input_sig.cleave.fn_est_cluster = 'cleave_est_clusters.mat';
  FN.input_sig.cleave.fn_fulllength = 'cleave_fulllength.mat';
  FN.input_sig.cleave.fn_db  = 'cleave_db.mat';
  FN.input_sig.cleave.fn_anno  = 'cleave_anno.mat';
end
if any(strcmp(signal_names,'polya'))
  FN.input_sig.polya.fn_polya = 'truesites_and_consensus.mat';
  FN.input_sig.polya.fn_log = 'log';
end
if any(strcmp(signal_names,'tis'))
  FN.input_sig.tis.fn_anno = 'tis_anno.mat';
end
if any(strcmp(signal_names,'cdsStop'))
  FN.input_sig.cdsStop.fn_anno = 'cdsStop_anno.mat';
end

if any(strcmp(signal_names,'transacc'))
  FN.input_sig.transacc.fn_anno_SL1 = 'transacc_anno_SL1' ;
  FN.input_sig.transacc.fn_anno_SL2 = 'transacc_anno_SL2' ;
  FN.input_sig.transacc.fn_pred = 'transacc_pred';
  FN.input_sig.transacc.fn_db = []; % only for C_elegans
end

%--------------------------------------------------------------------------
% CONTENT FILE NAMES
% -------------------------------------------------------------------------------------

for j=1:length(content_names)
  content_name = content_names{j};
  FN.input_cont.(content_name) = struct;
  FN.input_cont.(content_name).fn_candsites = 'cands/' ;
  FN.input_cont.(content_name).fn_examples = 'examples';
  FN.input_cont.(content_name).fn_pos = 'pos/';
  FN.input_cont.(content_name).fn_filter_settings= 'filter_settings.mat';
  FN.input_cont.(content_name).fn_example_statistics = 'example_statistics.mat';

  FN.output_cont.(content_name).fn_models = 'MS/models';
  FN.output_cont.(content_name).fn_SVMs = 'MS/';
  FN.output_cont.(content_name).fn_pred = 'pred/';
  FN.output_cont.(content_name).fn_motifs = 'MS/motifs';
end


%--------------------------------------------------------------------------
% LSL FILE NAMES
% -------------------------------------------------------------------------------------

FN.input_lsl.fn_test_blocks = 'data/test_blocks';
FN.input_lsl.fn_training_blocks =  'data/training_blocks.mat';
FN.input_lsl.fn_training_split =  'data/blocks_split.mat';

FN.input_lsl.fn_val_blocks =  'data/validation_blocks.mat';
FN.input_lsl.fn_blocks_all =  'data/blocks_all.mat';
FN.input_lsl.fn_boundary_model = 'data/boundary_model.mat';


FN.output_lsl.fn_init =  'train/init.mat';
FN.output_lsl.fn_train_block_preds = 'train/single_block_predictions/';
FN.output_lsl.fn_train = 'train/train';
FN.output_lsl.fn_pred = 'predictions/';
FN.output_lsl.fn_blocks_pred = 'predictions/blocks_pred';
FN.output_lsl.fn_pred_genome = 'genomewide_predictions/';
FN.output_lsl.fn_log = 'log_files/log_';



return

function PAR=init_PAR(signal_names,content_names);
 
  
% -------------------------------------------------------------------------------------
% Experiments
% -------------------------------------------------------------------------------------
if nargin<1
  signal_names = {'acc','don','tis','cdsStop','tss','cleave','polya','transacc'};
end
if nargin<2
  content_names = {'intergenic','utr5exon','cds_exon','utr3exon','polya_tail','intron','frame0'};
end




PAR.date = date;
PAR.user = whoami;
  
PAR.organism.clade = [];
PAR.organism.name = '';
PAR.organism.full_name = [];
PAR.organism.release = [];
PAR.organism.version = [];
PAR.organism.genebuilt = [];

% -------------------------------------------------------------------------------------
% info regarding original gene structure
% -------------------------------------------------------------------------------------


info_genes.transcript.anno = -1;
info_genes.transcript.est = 0;
info_genes.transcript.cDNA = 1;
info_genes.transcript.fulllength = 2;

info_genes.transcript_status.confirmed = 1;
info_genes.transcript_status.partially_confirmed = 0;
info_genes.transcript_status.predicted = -1;


info_genes.pseudo.dummy = [];

% -------------------------------------------------------------------------------------
% info regarding label source
% -------------------------------------------------------------------------------------

label_source.use_anno = [];
label_source.use_confirmed = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  label_source.(signal_name).anno = [];
  label_source.(signal_name).est = [];
  label_source.(signal_name).cDNA = [];
  label_source.(signal_name).fulllength = [];
  label_source.(signal_name).db = [];
  label_source.(signal_name).from_fn_candsites = [];
  
  info_genes.(signal_name).anno = -1;
  info_genes.(signal_name).est = 0;
  info_genes.(signal_name).cDNA = 1;
  info_genes.(signal_name).fulllength = 2;
  info_genes.(signal_name).db = 3;
  info_genes.(signal_name).from_fn_candsites = 4;
end
for j=1:length(content_names)
  content_name = content_names{j};
  label_source.(content_name).from_fn_candsites = [];
end

label_source.tis.maxorf = [];
info_genes.tis.maxorf = 5;
label_source.cdsStop.maxorf = [];
info_genes.cdsStop.maxorf = 5;

label_source.transacc.anno_SL1 = [];
label_source.transacc.anno_SL2 = [];
label_source.transacc.pred = [];
info_genes.transacc.anno_SL1 = 5;
info_genes.transacc.anno_SL2 = 6;
info_genes.transacc.pred = 7;

label_source.polya.from_cleave = [];
info_genes.polya.from_cleave = 5;


PAR.label_source = label_source;
PAR.info_genes = info_genes;


PAR.Signal_name = [];
for j=1:length(signal_names)
  signal_name = signal_names{j};
  PAR.Signals.(signal_name) = struct;
end

cnt=0;
PAR.Content_name = [];
for j=1:length(content_names)
  PAR.Contents.(content_names{j}) = struct;
  PAR.Contents.(content_names{j}).name = content_names{j};
  PAR.Contents.(content_names{j}).type = cnt;
  cnt=cnt+1;
end

PAR.LSL.method = struct;

% -------------------------------------------------------------------------------------
% memory requirements
% -------------------------------------------------------------------------------------

PAR.RPROC.collect_jobs = 0 ; 
PAR.RPROC.options.ncpus = 1 ; 
PAR.RPROC.options.waitonfull = [];
PAR.RPROC.options.maxjobs = [] ;
PAR.RPROC.options.immediately = 0;
PAR.RPROC.options.immediately_bg = 0;
PAR.RPROC.options.addpaths = [];
PAR.RPROC.options.rmpaths = [];

PAR.RPROC.MEMREQ = [];
PAR.RPROC.time_req = [];
PAR.RPROC.exm_per_batch = 1 ;


% -------------------------------------------------------------------------------------
% SPLITS AND PARTITIONS
% -------------------------------------------------------------------------------------

PAR.SETs.num_splits = 5;
PAR.SETs.num_partitions = 5;
PAR.SETs.train_subsample_neg = 1;
PAR.SETs.test_subsample_neg = 1;
PAR.SETs.train_subsample_pos = 1;
PAR.SETs.test_subsample_pos = 1;
%for lsl
PAR.SETs.num_train = [] ; 
PAR.SETs.ignore_incomplete = 1 ; 


% -------------------------------------------------------------------------------------
% original data processing stuff
% -------------------------------------------------------------------------------------

PAR.regions.max_length = 1000000;
PAR.regions.min_genes_per_region = 1;
PAR.regions.offset_test = 20000;
PAR.regions.offset_train = 1000;
PAR.regions.minCutDist = 250;
PAR.regions.cluster_paralogs = 0;
PAR.regions.merge_confirmed = 0;

% -------------------------------------------------------------------------------------
% TASKS
% -------------------------------------------------------------------------------------

PAR.tasks.signals.prepare_labels = 0;
PAR.tasks.signals.train = 0;
PAR.tasks.signals.modelsel = 0;
PAR.tasks.signals.learn_plifs = 0;
PAR.tasks.signals.get_train_error = 0;

PAR.tasks.signals.pred_on_genome = 0; 
PAR.tasks.signals.verbose = 0; 

PAR.tasks.signals.run_locally = 0;

% -------------------------------------------------------------------------------------
% transfer from other organisms
% -------------------------------------------------------------------------------------

PAR.Source_organisms.names = {};
PAR.Source_organisms.dir_names = {};
PAR.Source_organisms.exp_names = {};


% -------------------------------------------------------------------------------------
% FN
% -------------------------------------------------------------------------------------
PAR.FN = struct;


return

function lengths_range = default_lengths

lengths_range.intergenic = [0, 21000] ;
%lengths_range.intergenictrans = [0, 21000] ;
lengths_range.utr5exon   = [0, 8000] ;
% lengths_range.transexon  = [1, 1000] ;
lengths_range.single_cds_exon = [100, 8000] ; %%%100, from don => acc+stop
lengths_range.first_cds_exon  = [3, 8000] ;
lengths_range.middle_cds_exon   = [3, 8000] ;
lengths_range.last_cds_exon   = [1, 8000] ;
lengths_range.utr3exon    = [1, 8000] ;
lengths_range.polya_tail  = [0, 250] ;
% lengths_range.intercistronic = [1, 5000] ;
lengths_range.intergenic_long = [10500-50, 10500+50] ; % due to irregular spacing 
lengths_range.intron      = [30, 21000] ;%%%30, from don => acc+stop

return

function LSL = lsl_template
  

method.name = 'lsl' ;
method.model_fct = 'make_model';
method.train_fct = 'lsl_train';
method.predict_fct = 'lsl_predict';

method.dt = datestr(now,'dd-mm-yyyy');
method.user = 'GS';
method.plif_bins   = 20 ;   % number of plif bins

%%%

%%Signals
use.signals.trans = 0 ;
use.signals.transacc = 0 ;
use.signals.tss = 1;
use.signals.polya = 1;
use.signals.cleave =  1;

%%States
use.states.SL1 = 0 ;
use.states.SL2 = 0;

%%transitions
use.transitions.allow_missing_tss = 0;
use.transitions.one_gene = 0 ; % force exactly one gene
use.transitions.use_multi = 1 ; % allow zero or more than one gene 

%%loss
use.loss.orf_only = 0;
use.model.one_gene = 0;
use.model.allow_missing_tss = 0;
use.model.use_multi = 1;

%%Segments & lengths
use.segments.transexon = 0 ;
use.segments.intercistronic = 0 ;
use.segments.intergenictrans = 0;
use.segments.polya_tail = 1;

%%Content 

use.contents.frame = 1;
use.contents.pre_comp = 1;

%%additional features
use.addfeatures.conservation = 0 ;
use.addfeatures.ests = 0 ;
use.addfeatures.proteins = 0 ;
use.addfeatures.tiling = 0 ;
use.addfeatures.rna_seq = 0 ;

% keyboard
method.use = use;



%----- REGULARIZATION
%C = 1 ;
C = 1e-1;
%%% linear regularization
method.par_ms.C_regul.lengths = C ;        % linear smoothness for len_hist
method.par_ms.C_regul.signals = C ;        % for sig_hist
method.par_ms.C_regul.contents = C ;       % for content_hist
method.par_ms.C_regul.transitions = C ;    % for

method.par_ms.C_regul.tiling = C ;  
method.par_ms.C_regul.rna_seq = C ;  


%%% quadratic regularization
C_sq = 1e-1;

method.par_ms.C_regul.transitions_sq = C_sq;
method.par_ms.C_regul.plif_ys_sq = C_sq*0.1 ;  % small_plif_ys_penalty
method.par_ms.C_regul.smoothness_sq = C_sq ;


%----------- for predictions only
method.par_ms.cleave_offset = 6;
%-----------

method.solver = 'dual';
method.ignore_init_file = 1;

method.opt_step = 1000 ;
method.max_neg = 2 ;
method.margin = 1 ;

method.INF = 1e20;
method.max_num_iterations = 100;

method.exm_per_solve = 400 ;
% method.exm_per_batch = 5 ;

%method.cleave_offset = 6 ;
% -------
% -------
LSL.method = method;


return

function diff_fields = compare_PAR(PAR_1,PAR_2,silent)

if nargin<3
  silent=0;
end

diff_fields ={};
if isequal(PAR_1,PAR_2)
  fprintf(1,'\t parameters identical\n')
else
  fields_1 = fieldnames(PAR_1);
  fields_2 = fieldnames(PAR_2);
  common_fields = intersect(fields_1,fields_2);
  for i=1:length(common_fields)
    par_1 = getfield(PAR_1,common_fields{i});
    par_2 = getfield(PAR_2,common_fields{i});
    if ~isequal(par_1,par_2)
      diff_fields{end+1} = common_fields{i};
      if ~silent
        fprintf(1,'\t field "%s" not identical\n',common_fields{i})
        fprintf(1,'\t %s, 1:\n',common_fields{i});disp(par_1)
        fprintf(1,'\t %s, 2:\n',common_fields{i});disp(par_2)
      end
    end
  end
  fields_x1= setdiff(fields_1,fields_2);
  if ~isempty(fields_x1)
    if ~silent
      fprintf(1,'\t additional fields in PAR_1: \n')
    end
    diff_fields = {diff_fields{:},fields_x1{:}};
  end
  fields_x2 = setdiff(fields_2,fields_1);
  if ~isempty(fields_x2)
    if ~silent
      fprintf(1,'\t additional fields in PAR_2: \n')
    end
    diff_fields = {diff_fields{:},fields_x2{:}};
  end
end
   
return

function frames=find_open_frames(s, terminal)

s=lower(s) ;

if terminal,
  s=s(1:end-1) ;
end ;

s=lower(s) ;

frames = ones(1,3) ;
if length(s)>2,
  idx=[findstr(s,'taa') findstr(s,'tag') findstr(s,'tga')] ;
  frames(mod(idx-1,3)+1)=0 ;
end ;


return

function [regions,N]=get_connected_regions(bw);
  
 % similar to bw_label just for 1xm matrix 

reg_starts = find(bw(1:end)-[0 bw(1:end-1)]==1);
reg_ends = find(bw(1:end)-[bw(2:end) 0]==1);
assert(length(reg_starts)==length(reg_ends))

N = length(reg_starts);

regions = zeros(size(bw));
for i=1:N
  assert(reg_starts(i)<=reg_ends(i))
  regions(reg_starts(i):reg_ends(i))=i;
end
return

function ret=my_prctile(data, v)

  if all(size(data)>1)%apply percentile on columns
    assert(length(size(data))==2)
    ret = [];
    for k=1:size(data,2)
      x = sort(data(:,k));
      vec = [];
      for j=1:length(v)
        vec = [vec percentile(x, v(j))];
      end
      ret = [ret vec'];
    end
  else
    x = sort(data);
    vec = [];
    for j=1:length(v)
      vec = [vec percentile(x, v(j))];
    end
    ret = vec;
  end
return


function perc = percentile(x, k_percentile)

  %x = sort(data);
  num_x = length(x);
  k = k_percentile/100 * (num_x-1) + 1;
  
  [k_int, D] = rat(k);
  if isequal(D,1),  % k is an integer, take percentile element
    perc = x(k_int);
  else              % take midpoint between two elements
    perc = mean(x(floor(k):ceil(k)));
  end
return

return

function print_struct(fid,s)
% print_struct(fid,s)

fields = fieldnames(s);
max_len = 0;
for f=1:length(fields)
  max_len = max(max_len, length(fields{f}));
end

for f=1:length(fields),
  value = getfield(s,fields{f});
  if isempty(value),
    fprintf(fid, '%*s: %s\n',max_len+3,fields{f},'[]');
  elseif iscell(value),
    if size(value,1)==1 & size(value,2)<10,
      if isscalar(value{1}),
        vstr = ['{[' num2str(value{1}) ']'];
        for i=2:size(value,2),
          vstr = [vstr ' [' num2str(value{i}) ']'];
        end
        vstr = [vstr '}'];
      else
        if isempty(value{1}),
          vstr = ['{[]'];
        else
          vstr = ['{[' class(value{1}) ']'];
        end
        for i=2:size(value,2),
          if isempty(value{i}),
            vstr = [vstr ' []'];
          else
            vstr = [vstr ' [' class(value{i}) ']'];
          end
        end
        vstr = [vstr '}'];        
      end
      fprintf(fid, '%*s: %s\n',max_len+3,fields{f},vstr);  
    else
      fprintf(fid, '%*s: %s\n',max_len+3,fields{f},...
              sprintf('{%ix%i cell}',size(value,1),size(value,2)));  
    end
  elseif isstruct(value),
    fprintf(fid, '%*s: %s\n',max_len+3,fields{f},...
            sprintf('[%ix%i struct]',size(value,1),size(value,2)));
  elseif isscalar(value),
    if ischar(value),
      fprintf(fid, '%*s: "%s"\n',max_len+3,fields{f}, value);
    elseif isinteger(value) | mod(value,1)==0,
      fprintf(fid, '%*s: %i\n',max_len+3,fields{f}, value);
    elseif isnumeric(value),
      fprintf(fid, '%*s: %f\n',max_len+3,fields{f}, value);
    else 
      fprintf(fid, '%*s: ...\n',max_len+3,fields{f});     
    end    
  elseif isvector(value),
    if size(value,1)==1 & size(value,2)<10,
      if ischar(value),
        fprintf(fid, '%*s: "%s"\n',max_len+3,fields{f}, value);
      elseif isscalar(value(1)),
        vstr = ['[' num2str(value(1))];
        for i=2:size(value,2),
          vstr = [vstr ' ' num2str(value(i))];
        end
        vstr = [vstr ']'];
      else
        vstr = ['[' class(value(1))];
        for i=2:size(value,2),
          vstr = [vstr ' ' class(value(i))];
        end
        vstr = [vstr ']'];        
      end
      fprintf(fid, '%*s: %s\n',max_len+3,fields{f},vstr);      
    else
      fprintf(fid, '%*s: %s\n',max_len+3,fields{f},...
              sprintf('[%ix%i %s]',size(value,1),size(value,2),class(value)));
    end
  else
    fprintf(fid, '%*s: %s\n',max_len+3,fields{f},...
              sprintf('[%ix%i %s]',size(value,1),size(value,2),class(value)));      
  end
end
return

function new_field = set_default(old_struct,field,defaults,silent)  ;

% new_struct = set_default(old_struct,defaults,overwrite)  ;
% sets all values of all fields in old_struct to the default values specified in
% defaults IF field is empty in old_struct. If overwrite is set all entries
% will be overwritten.
% defaults must be a struct.
%
% e.g. 
% default.use_cons = 0;
% default.plif.bins = 50;
% method.use_cons = 0;
% method = set_default(method,default,1) 

if ~isfield(old_struct,field)
  new_field = defaults;
  return
end  

old_struct = old_struct.(field);

if nargin<4
  silent = 1 ;
end

names = fieldnames(defaults);
for j=1:length(names)
  if ~isfield(old_struct,names{j}) || isempty(old_struct.(names{j}))||...
        (iscell(old_struct.(names{j}))&&length(old_struct.(names{j}))==1&&isempty(old_struct.(names{j}){1}))
    if ~isfield(old_struct,names{j}) && ~silent
      warning('new field added')
      names{j}
    end
    
    old_struct.(names{j}) = defaults.(names{j});
  elseif isstruct(defaults.(names{j}))
    old_struct.(names{j}) = set_default(old_struct,names{j},defaults.(names{j}));
    %names2 =  fieldnames(defaults.(names{j}));
    %for k= 1:length(names2)
    %  if ~isfield(old_struct.(names{j}),names2{k})||isempty(old_struct.(names{j}).(names2{k}))||overwrite
    %    old_struct.(names{j}).(names2{k}) = defaults.(names{j}).(names2{k});
    %  end
    %end
  end
end
% if length(fieldnames(PAR))>length(names)
%   keyboard
% end
% PAR = orderfields(PAR,defaults);

new_field = orderfields(old_struct,defaults);

return

function genes = update_splicegraph(genes, run_locally)

%addpath ~/svn/projects/genefinding
%paths
if nargin==1
  run_locally=1;
end

if run_locally
  genes = rmfield(genes, 'splicegraph');
  genes = build_splice_graph_caller(genes);
  genes = infer_splice_graph_caller(genes);
  genes = alt_const(genes);
  %genes = correct_splicegraph(genes);
else
  for j=1:length(genes)
    dir_ = which('update_splicegraph');
    dir_(find(dir_=='/',1,'last')+1:end)=[];
    options.start_dir = dir_;
    jobinfo(j)=rproc_create('update_splicegraph',genes(j),500,options,60);
  end
  jobinfo = rproc_submit_batch(jobinfo,1000);
  rproc_wait(jobinfo, 60,1,0);
  for j = 1:length(genes)
    l = load(jobinfo(j).result_fname)
    genes(j)=l.retval1;
    clear l
  end
end


return

genes = build_splice_graph_caller(genes) ;

return

function genes = build_splice_graph_caller(genes) ;
% genes = build_splice_graph_caller(genes) ;
  
for gene_idx = 1:length(genes)
  % construct a new empty splice graph, based on
  % http://proline.bic.nus.edu.sg/dedb/
  vertices =  [];
  edges = [];
%  fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,100)==0)
    fprintf(1,'.');
  end

  for transcript_idx = 1:length(genes(gene_idx).transcripts)
    exon_start_end = genes(gene_idx).exons{transcript_idx};
    
    if (size(exon_start_end,1) == 1)
      exon1_start = exon_start_end(1,1);
      exon1_end = exon_start_end(1,2);

      
      if isempty(vertices)
	vertices(1,1) = exon1_start;
	vertices(2,1) = exon1_end;
	edges = 0;
	num_exons = 1;
      else
	vertices(1,num_exons+1) = exon1_start;
	vertices(2,num_exons+1) = exon1_end;
	edges(1,num_exons+1) = 0;
	edges(num_exons+1,1) = 0;
	num_exons = num_exons + 1;
      end

      
    else
      

      for exon_idx = 1:(size(exon_start_end,1)-1)
	exon1_start = exon_start_end(exon_idx,1);
	exon1_end = exon_start_end(exon_idx,2);
	exon2_start = exon_start_end(exon_idx+1,1);
	exon2_end = exon_start_end(exon_idx+1,2);


	
	if isempty(vertices)
	  vertices(1,1) = exon1_start;
	  vertices(2,1) = exon1_end;
	  vertices(1,2) = exon2_start;
	  vertices(2,2) = exon2_end;
	  edges = zeros(2);
	  edges(1,2) = 1;
	  edges(2,1) = 1;
	  num_exons = 2;
	else
	  exon1_idx = 0;
	  exon2_idx = 0;
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon1_start)||...
%		(vertices(1,idx)+1==exon1_start)||...
%		(vertices(1,idx)-1==exon1_start))...
%		  && ((vertices(2,idx)==exon1_end)||...
%		      (vertices(2,idx)+1==exon1_end)||...
%		      (vertices(2,idx)-1==exon1_end))
	    if ((vertices(1,idx)==exon1_start) &&...
		(vertices(2,idx)==exon1_end))
	      exon1_idx = idx;
	    end
	    idx = idx+1;
	  end
	  idx = 1;
	  while idx <= num_exons
%	    if ((vertices(1,idx)==exon2_start)||...
%		(vertices(1,idx)+1==exon2_start)||...
%		(vertices(1,idx)-1==exon2_start))...
%		  && ((vertices(2,idx)==exon2_end)||...
%		      (vertices(2,idx)+1==exon2_end)||...
%		      (vertices(2,idx)-1==exon2_end))
	    if ((vertices(1,idx)==exon2_start) &&... 
		(vertices(2,idx)==exon2_end))
	      exon2_idx = idx;
	    end
	    idx = idx+1;
	  end
	  if (exon1_idx~=0) && (exon2_idx~=0)
	    edges(exon1_idx,exon2_idx) = 1;
	    edges(exon2_idx,exon1_idx) = 1;
	  else
	    if ((exon1_idx==0) && (exon2_idx~=0))
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      edges(exon2_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon2_idx) = 1;
	      num_exons = num_exons + 1;
	    elseif ((exon2_idx==0) && (exon1_idx~=0))
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      edges(exon1_idx,num_exons+1) = 1;
	      edges(num_exons+1,exon1_idx) = 1;
	      num_exons = num_exons + 1;
	    else
	      assert((exon1_idx==0)&&(exon2_idx==0));
	      vertices(1,num_exons+1) = exon1_start;
	      vertices(2,num_exons+1) = exon1_end;
	      num_exons = num_exons + 1;
	      vertices(1,num_exons+1) = exon2_start;
	      vertices(2,num_exons+1) = exon2_end;
	      num_exons = num_exons + 1;	  
	      
	      edges(num_exons-1,num_exons) = 1;
	      edges(num_exons,num_exons-1) = 1;
	    end
	  end
	end
      end
    end
  end




  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');


return






return

function genes= alt_const(genes) ;
%function genes = alt_const(genes) ;

tot_exons = 0;
for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  tot_exons = tot_exons + num_exons ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;

  if isempty(edges)
    genes(ix).is_alt_spliced=0 ;
    genes(ix).is_alt=0 ;
    continue ;
  end 
  
  init_alt_idx = [] ;
  term_alt_idx = [] ;
  for i=1:num_exons,
    if ~any(edges(1:i-1,i)) % initial
      idx=setdiff(find(vertices(2,i)==vertices(2,:)),i) ;
      if ~isempty(idx) % other exon with same end
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(j,j+1:end))+j,find(edges(i,i+1:end))+i)
            is_simple = 0 ;
            break ;
          end ;
          idx_prev = find(edges(1:j-1,j)) ;
          for k=idx_prev,
            if vertices(2,k)>vertices(1,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        init_alt_idx(end+1) = i ;
      end
    end ;
    if ~any(edges(i,i+1:num_exons)) % terminal
      idx=setdiff(find(vertices(1,i)==vertices(1,:)),i) ;
      if ~isempty(idx) % other exon with same start
        is_simple=1 ; 
        for j=idx,
          if ~isequal(find(edges(1:j-1,j)),find(edges(1:i-1,i)))
            is_simple = 0 ;
            break ;
          end ;
          idx_next = find(edges(j,j+1:end))+j ;
          for k=idx_next,
            if vertices(1,k)<vertices(2,i)
              is_simple = 0 ;
              break ;
            end ;
          end ;
        end ;
        if ~is_simple, continue ; end ;
        term_alt_idx(end+1) = i ;
      end ;
    end ;
  end ;
  take_idx = setdiff(1:num_exons, [init_alt_idx,term_alt_idx]) ;
  vertices = genes(ix).splicegraph{1}(:,take_idx) ;
  edges = genes(ix).splicegraph{2}(take_idx,take_idx) ;

  start=min(vertices(1,:)-50) ;
  stop=max(vertices(2,:)+50) ;
  
  exon_loc = zeros(1,stop-start);

  for i=1:size(edges,1),
    for j=i+1:size(edges,1),
      if edges(i,j)==1,
        cur_edge = [vertices(2,i)+1-start, vertices(1,j)-1-start] ;
        exon_loc(cur_edge(1):cur_edge(2)) = ...
            exon_loc(cur_edge(1):cur_edge(2)) + 1;
      end ;
    end ;
  end ;

  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)) + 1;
  end ;

  if max(exon_loc)>1
    genes(ix).is_alt_spliced=1 ;
    genes(ix).is_alt=1 ;
  else
    genes(ix).is_alt_spliced=0 ;
    if isempty([init_alt_idx,term_alt_idx])
      genes(ix).is_alt=0 ;
    else
      genes(ix).is_alt=1 ;
    end ;
  end
end

fprintf(1,'\n\nTotal genes:\t\t\t\t\t\t\t%d\n',...
        length(genes));
fprintf(1,'Total exons:\t\t\t\t\t\t\t%d\n',...
        tot_exons);
fprintf(1,'Total genes with alternative isoforms:\t\t\t\t%d\n',...
        sum([genes(:).is_alt]));
fprintf(1,'Total genes alternatively spliced:\t\t\t\t%d\n',...
        sum([genes(:).is_alt_spliced]));
fprintf(1,'Total constitutively spliced:\t\t\t\t\t%d\n',...
        sum(~[genes(:).is_alt_spliced]));


return

function genes = annotate_splicegraph(genes)
% function annotate_splicegraph(info,origpath)
%
% annotate the genes structure with counts for:
% - exon skipping
% - intron retention
% - alternative donor (5 prime)
% - alternative acceptor (3 prime)
% - alternative transcription start
% - alternative transcription end
%
% Written by: Cheng Soon Ong, 14 May 2007
% Modified on: 31 May 2007 to include transcription starts and stops


%addpath(sprintf('%s/utils', origpath)) ;
%addpath(sprintf('%s/splicegraphs', origpath)) ;
%addpath(sprintf('%s/splicegraphs/detect_altsplice', origpath)) ;
%addpath(sprintf('%s/gff', origpath)) ;
%addpath(sprintf('%s/gff/read_sequences', origpath)) ;
%addpath /fml/ag-raetsch/share/software/matlab_tools/utils

%genome_info = init_genome(info) ;
%fprintf('directory: %s\n', genome_info.basedir) ;
%load(sprintf('%s/confirmed_sequences.mat', genome_info.basedir), 'genes') ;
%fprintf('loaded confirmed_sequences.mat\n\n') ;

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

if isfield(genes,'alt_3prime_details') || isfield(genes,'alt_5prime_details')
  warning('annotate_splicegraph adds to alt_3prime_details and alt_5prime_details,\nrunning more than one creates inconsistencies');
  return;
end

idx_alt=[] ;
for i=1:length(genes)
  if genes(i).is_alt_spliced
    idx_alt=[idx_alt, i] ;
  end
  if ~isfield(genes(i),'strands') || isempty(genes(i).strands)
    genes(i).strands(1) = genes(i).strand;
  end
end

[idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt);
[idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] = ...
    detect_altprime_pair(genes, idx_alt);
[idx_intron_reten,intron_intron_reten] = detect_intronreten(genes, idx_alt) ;
[idx_xor_exons, exon_xor_exons] = detect_xorexons(genes, idx_alt) ;
[idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;
idx_alt_tstart = [idx_alt_tstart{:}];
[idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;
idx_alt_tend = [idx_alt_tend{:}];

%keyboard
for ix = 1:length(genes)
    
  if mod(ix,1000)==0
    fprintf('.');
  end

  exons_is_alt = zeros(1,size(genes(ix).splicegraph{1},2));

  % exon skips
  if ismember(ix,idx_exon_skips)
    tix = find(idx_exon_skips==ix);
    genes(ix).alt_exon = length(tix);
    genes(ix).alt_exon_details.exons = exon_exon_skips(:,tix);
    exons_is_alt(exon_exon_skips(:,tix)) = exons_is_alt(exon_exon_skips(:,tix)) + 1;
  else
    genes(ix).alt_exon = 0;
  end

  % intron retentions
  if ismember(ix,idx_intron_reten)
    tix = find(idx_intron_reten==ix);
    genes(ix).alt_intron = length(tix);
    genes(ix).alt_intron_details.exons = intron_intron_reten(:,tix);
    exons_is_alt(intron_intron_reten(:,tix)) = exons_is_alt(intron_intron_reten(:,tix)) + 1;
  else
    genes(ix).alt_intron = 0;
  end

  % alternative 5prime
  if ismember(ix,idx_alt_5prime)
    genes(ix).alt_5prime_details.exons = zeros(3,0);
    tix = find(idx_alt_5prime==ix);
    genes(ix).alt_5prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_r = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      else
	idx_exon_r = exon_alt_5prime(ixt).fiveprimesites(1);
	idx_exon_l = exon_alt_5prime(ixt).threeprimesite;
	idx_exon_alt = exon_alt_5prime(ixt).fiveprimesites(2);
      end
      genes(ix).alt_5prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_5prime = 0;
  end

  % alternative 3prime
  if ismember(ix,idx_alt_3prime)
    genes(ix).alt_3prime_details.exons = zeros(3,0);
    tix = find(idx_alt_3prime==ix);
    genes(ix).alt_3prime = length(tix);
    for ixt = tix
      if genes(ix).strands(1) == '+'
	idx_exon_l = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_r = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      else
	idx_exon_r = exon_alt_3prime(ixt).fiveprimesite;
	idx_exon_l = exon_alt_3prime(ixt).threeprimesites(1);
	idx_exon_alt = exon_alt_3prime(ixt).threeprimesites(2);
      end
      genes(ix).alt_3prime_details.exons(:,end+1) = [idx_exon_l;idx_exon_r;idx_exon_alt];
      exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) = exons_is_alt([idx_exon_l;idx_exon_r;idx_exon_alt]) + 1;
    end
  else
    genes(ix).alt_3prime = 0;
  end

  % xor exons
  if ismember(ix,idx_xor_exons)
    tix = find(idx_xor_exons==ix);
    genes(ix).alt_xor_exon = length(tix);
    for ixt = tix
      exons_is_alt(exon_xor_exons(:,tix)) = exons_is_alt(exon_xor_exons(:,tix)) + 1;
    end
  else
    genes(ix).alt_xor_exon = 0;
  end

  % alternative transcriptions
  if ismember(ix,idx_alt_tstart)
    tix = find([idx_alt_tstart == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tstart = length(exon_alt_tstart{tix}.exons);
    genes(ix).alt_tstart_details.exons = exon_alt_tstart{tix}.exons;
  else
    genes(ix).alt_tstart = 0;
  end

  if ismember(ix,idx_alt_tend)
    tix = find([idx_alt_tend == ix]);
    % there is only one entry per gene.
    assert(all(size(tix) == [1,1]))
    genes(ix).alt_tend = length(exon_alt_tend{tix}.exons);
    genes(ix).alt_tend_details.exons = exon_alt_tend{tix}.exons;
  else
    genes(ix).alt_tend = 0;
  end

  genes(ix).exons_is_alt = exons_is_alt;
end


if 0,
  fid = fopen(sprintf('%s/alt_splice_type.tsv', genome_info.basedir),'w');
  fprintf(fid,'Clust\tExon Skip\tIntron Reten\tAlt 5prime\tAlt 3prime\n');
  for ix = idx_alt  
    fprintf(fid,'%d\t%d\t%d\t%d\t%d\n',ix,...
	    genes(ix).alt_exon,genes(ix).alt_intron,genes(ix).alt_5prime,genes(ix).alt_3prime);
  end
  fclose(fid);
end

return

function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
    = detect_altprime_pair(genes, idx_alt);
% function [idx_alt_5prime,exon_alt_5prime, idx_alt_3prime,exon_alt_3prime] ...
%    = detect_altprime_pair(genes, idx_alt);
%
% Detect the alternative 5 and 3 prime ends of the intron. Note that 5 prime refers to the left
% and 3 prime to the right for a positive strand
% The difference between this function and detect_altprime.m is that the other function collects
% all the alternative sites corresponding to an acceptor or donor.
% note that the 'overlap' relationship is not transitive

MIN_OVERLAP = 11; % two exons that are alternative should overlap by at least the BLAT seed

idx_alt_5prime = [];
idx_alt_3prime = [];
exon_alt_5prime = {};
exon_alt_3prime = {};

if ~isfield(genes, 'strands'),
  for i=1:length(genes),
    genes(i).strands = repmat(genes(i).strand, 1, length(genes(i).transcripts)) ;
  end ;
end ;

for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = genes(ix).splicegraph{2} ;
  
  % Find alternative sites on the right of the intron,
  % same site on the left of the intron.
  for exon_idx = 1:num_exons-2
    nr_exons=sum(edges(exon_idx,exon_idx+1:num_exons));
    if nr_exons>=2,
      which_exons = find([zeros(1,exon_idx),edges(exon_idx,exon_idx+1:num_exons)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the left splice site of the exons are different
	  % make sure exons overlap - either:
	  % - left splice site of exon(i) is in exon(j)
	  % - left splice site of exon(j) is in exon(i)
	  
	  if ((exons(1,i) ~= exons(1,j)) && ...
	       (( (exons(1,i) >= exons(1,j)) && (exons(1,i) <= exons(2,j)) ) ...
		|| ( (exons(1,j) >= exons(1,i)) && (exons(1,j) <= exons(2,i)) )) && ...
	       (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(1,i) ~= exons(1,j));
	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(1,i) > exons(1,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % i=1:nr_exons-1
    end
  end % for exon_idx

  
  % Find alternative sites on the left of the intron,
  % same site on the right of the intron.
  for exon_idx = 3:num_exons
    nr_exons=sum(edges(1:exon_idx-1,exon_idx)) ;
    if nr_exons>=2,
      which_exons = find([edges(1:exon_idx-1,exon_idx)', zeros(1,num_exons-exon_idx+1)]);
      exons=[vertices(:, which_exons)] ;
      for i=1:nr_exons-1
	for j=i+1:nr_exons
	  % check that the 5prime sites are different
	  % make sure exons overlap - either:
	  % - right splice site of exon(i) is in exon(j)
	  % - right splice site of exon(j) is in exon(i)
	  if ((exons(2,i) ~= exons(2,j)) &&...
	      (( (exons(2,i) <= exons(2,j)) && (exons(2,i) >= exons(1,j)) )...
	      || ( (exons(2,j) <= exons(2,i)) && (exons(2,j) >= exons(1,i)) )) && ...
	      (min(exons(2,i),exons(2,j))-max(exons(1,i),exons(1,j)) >= MIN_OVERLAP ) )
	    assert(~((exons(1,i) == exons(1,j)) && (exons(2,i) == exons(2,j))));
	    assert(exons(2,i) ~= exons(2,j));

	    % construct the output
	    if (genes(ix).strands(1)=='+'),
	      exon_alt_5prime(end+1).threeprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_5prime(end).fiveprimesites=which_exons([i j]);
	      else
		exon_alt_5prime(end).fiveprimesites=which_exons([j i]);
	      end
	      idx_alt_5prime(end+1) = ix;
	    end
	    if (genes(ix).strands(1)=='-'),
	      exon_alt_3prime(end+1).fiveprimesite=exon_idx ;
	      if exons(2,i) < exons(2,j)
		exon_alt_3prime(end).threeprimesites=which_exons([i j]);
	      else
		exon_alt_3prime(end).threeprimesites=which_exons([j i]);
	      end
	      idx_alt_3prime(end+1) = ix;
	    end
	  end % if altsplice
	  
	end % for j=i+1:nr_exons
      end % for i=1:nr_exons-1
    end

  end % construct output
end % for exon_idx
  



fprintf(1,'\n\nNumber of alternative 5 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_5prime));

fprintf(1,'Number of alternative 3 prime sites:\t\t\t\t%d\n',...
	length(idx_alt_3prime));






return

function [idx_alt_tend, exon_alt_tend] = detect_alttend(genes) ;

idx_alt_tend = {};
exon_alt_tend = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  term_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(i,:),2)==0,
      term_alt_idx(end+1) = i ;
    end ;
  end ;
  if length(term_alt_idx)>1
    idx_alt_tend{end+1} = ix;
    exon_alt_tend{end+1}.exons = term_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription ends:\t\t\t\t%d\n',...
        length(idx_alt_tend));

return

function [idx_alt_tstart, exon_alt_tstart] = detect_alttstart(genes) ;

idx_alt_tstart = {};
exon_alt_tstart = {};

for ix=1:length(genes)
  if (mod(ix,1000)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2) ;
  vertices = genes(ix).splicegraph{1} ;
  edges = triu(genes(ix).splicegraph{2}) ;
  
  init_alt_idx = [] ;
  for i=1:num_exons,
    if sum(edges(:,i),1)==0, % initial
      init_alt_idx(end+1) = i ;
    end ;
  end ;

  if length(init_alt_idx)>1,
    idx_alt_tstart{end+1} = ix;
    exon_alt_tstart{end+1}.exons = init_alt_idx;
  end
end

fprintf(1,'\nTotal alternative transcription starts:\t\t\t\t%d\n',...
        length(idx_alt_tstart));

return

function [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;
% [idx_exon_skips, exon_exon_skips] = detect_exonskips(genes, idx_alt) ;

idx_exon_skips = [];
exon_exon_skips = [];
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  for exon_idx = 1:num_exons-2
    for exon_idx1 = (exon_idx+1):(num_exons-1)
      for exon_idx2 = (exon_idx1+1):num_exons
        if (edges(exon_idx,exon_idx1))&&...
              (edges(exon_idx,exon_idx2))&&...
              (edges(exon_idx1,exon_idx2))
          idx_exon_skips = [idx_exon_skips,ix];
          exon_exon_skips = [exon_exon_skips,[exon_idx; exon_idx1; exon_idx2]];
        end
      end
    end
  end
end
sum_skips=length(idx_exon_skips) ;
fprintf(1,'\n\nNumber of single exon skips:\t\t\t\t\t%d\n', sum_skips);



return

function [idx_intron_reten,intron_intron_reten] = detect_intronreten(genes,idx_alt) ;

idx_intron_reten = [];
intron_intron_reten = [];
for ix=idx_alt
  %viewsplicegraph(genes(ix)) ;
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  vertices = genes(ix).splicegraph{1};
  edges = genes(ix).splicegraph{2};
  introns=[] ;
  for exon_idx = 1:num_exons-1 %start of intron
    %exon_idx
    %edges(exon_idx,exon_idx+1:length(edges))
    idx=find(edges(exon_idx,exon_idx+1:length(edges))==1) ;
    if isempty(idx), continue; end
    idx=idx+exon_idx ;
    for exon_idx2=idx, %end of intron
      is_intron_reten=0 ;
      for exon_idx1=1:num_exons %exon
	% check that the exon covers the intron
        if (vertices(2,exon_idx) >= vertices(1,exon_idx1)) && (vertices(1,exon_idx2) <= vertices(2,exon_idx1))
	
	% check that the exon covers the intron, but doesn't extend beyond
        %if (vertices(2,exon_idx) > vertices(1,exon_idx1)) && (vertices(1,exon_idx2) < vertices(2,exon_idx1))...
	%      && (vertices(1,exon_idx) <= vertices(1,exon_idx1)) && (vertices(2,exon_idx2) >= vertices(2,exon_idx1))
          is_intron_reten=1 ;
          long_exon=exon_idx1 ;
          for len=1:size(introns,2),
            if (vertices(2,exon_idx)==introns(1,len)) && (vertices(1,exon_idx2)==introns(2,len))
              is_intron_reten=0 ;
            end  
          end
        end
      end
      if is_intron_reten==1,
        idx_intron_reten = [idx_intron_reten, ix];
        intron_intron_reten = [intron_intron_reten, [exon_idx ; exon_idx2 ...
                   ; long_exon]] ;
        introns=[introns, [vertices(2,exon_idx) ; vertices(1,exon_idx2)]] ;
      end
    end
    %pause ;
  end
  
  %pause ;
end
fprintf(1,'\n\nNumber of intron retentions:\t\t\t\t\t%d\n', ...
	length(idx_intron_reten));


return

function [idx_xor_exons, exon_xor_exons,id_xor_exons] = detect_xorexons(genes, idx_alt);

idx_xor_exons = zeros(1,0); 
id_xor_exons = zeros(1,0); 
exon_xor_exons = zeros(4,0); %%%% 5primesite of first exon, the 2 skipped
                     %exons, 3primesite of last exon %%%
id = 0 ;
for ix=idx_alt
  if (mod(ix,50)==0)
    fprintf(1,'.');
  end
  num_exons = size(genes(ix).splicegraph{1},2);
  edges = genes(ix).splicegraph{2};
  vertices = genes(ix).splicegraph{1};
  for exon_idx1 = 1:num_exons-3
    id=id+1 ;
    for exon_idx2 = (exon_idx1+1):(num_exons-2)
      if edges(exon_idx1, exon_idx2)
        for exon_idx3=(exon_idx2+1):(num_exons-1)
          if edges(exon_idx1, exon_idx3) & edges(exon_idx2, exon_idx3)==0 & ...
             (vertices(1,exon_idx3) > vertices(2,exon_idx2)),
            for exon_idx4=(exon_idx3+1):num_exons
              if edges(exon_idx2, exon_idx4) & edges(exon_idx3, exon_idx4) & ...
                    edges(exon_idx1, exon_idx4)==0
                idx_xor_exons = [idx_xor_exons, ix];
                id_xor_exons = [id_xor_exons, id];
                exon_xor_exons = [exon_xor_exons,[exon_idx1; exon_idx2; exon_idx3; exon_idx4]] ;
              end
            end
          end
        end
      end
    end
  end
end
fprintf(1,'\n\nNumber of XOR exons:\t\t\t\t\t\t%d\n',...
	length(idx_xor_exons));



return

function [regions,level] = detectaltregions(gene)
% regions = detectaltregions(gene)
  
[max_exon_level,max_intron_level,exon_level,intron_level,level] = ...
    detectsplicegraph(gene) ;

if all(level(3,:)<=1), 
  regions=[] ;
  return ;
else
  regions = [] ;
  idx=find(level(3,:)>1) ;
  idx=unique([idx(1) find(level(3,2:end)>1 & level(3,1:end-1)<=1)+1]);

  
  for i=1:length(idx)
    %level(4,idx(i))
    %if level(4,idx(i))==0, keyboard; end ;

  %    if idx(i)==1
      start = level(1,idx(i)) ;
%    else
%      start = floor(mean([level(1,idx(i)-1) level(1,idx(i))])) ;
%    end ;
%    if idx(i)==size(level,2)
      stop = level(2,idx(i)) ;
%    else
%      stop = floor(mean([level(2,idx(i)) level(2,idx(i)+1)]))-1 ;
%    end ;
    depth = level(3,idx(i)) ;
    for j=idx(i):size(level,2),
      if level(3,j)>1 
%        if j==size(level,2)
          stop = level(2,j) ;
%        else
%          stop = floor(mean([level(2,j) level(2,j+1)]))-1 ;
%        end ;
        depth = max(depth, level(3,j)) ;
      else
        %if level(4,j)==0, keyboard; end ;
        break ;
      end ;
    end ;
    regions=[regions, [start; stop; depth]] ;
  end ;
end ;
return

function [max_exon_level,max_intron_level,exon_level,intron_level,level] = detectsplicegraph(gene)

  [max_exon_level,max_intron_level,exon_level,intron_level,level] = countlevel(gene.exons,gene.name,...
                                                    gene.transcripts,[], ...
                                                    gene.splicegraph{1},...
                                                    gene.splicegraph{2});
  return


function [max_level,max_leveli,exon_level,intron_level,level] = countlevel(truepos,name,labels,colors,...
                                                    vertices,edges)
  
  
  start=inf ;
  stop=-inf ;
  for k=1:length(truepos)
    start=min(start,truepos{k}(1,1)-50) ;
    stop=max(stop,truepos{k}(end,2)+50) ;
  end ;
  
  
  max_level = 0;
  max_leveli = 0;

  splice = unique(vertices(:)) ;
  
  exon_loc = zeros(1,stop-start);
  exon_level = zeros(3,size(vertices,2));
  for i=1:size(vertices,2)
    cur_vertex = vertices(:,i) - [start;start];
    exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
        exon_loc(cur_vertex(1):cur_vertex(2)-1) + 1;
    %exon_loc(cur_vertex(1):cur_vertex(2)-1) = ...
    %    max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    
    level = max(exon_loc(cur_vertex(1):cur_vertex(2)-1));
    if isempty(level), level=0 ; end ;
    max_level = max(level,max_level);
    exon_level(1,i) = vertices(1,i) ;  
    exon_level(2,i) = vertices(2,i) ;  
    exon_level(3,i) = level;  
  end
  
  intron_loc = zeros(1,stop-start);
  intron_level = zeros(3,0) ;
  if (size(edges,1) > 1)
    for i=1:size(vertices,2)
      for j=i+1:size(vertices,2)
        if edges(i,j)
          if (vertices(1,i) < vertices(1,j))
            istart = vertices(2,i);
            istop =  vertices(1,j);
            %level1 = exon_level(i);
            %level2 = exon_level(j);
          else
            istart = vertices(2,j);
            istop =  vertices(1,i);
            %level1 = exon_level(j);
            %level2 = exon_level(i);
          end
          
          cur_intron = [istart-start,istop-start];
          intron_loc(cur_intron(1):cur_intron(2)-1) = ...
              intron_loc(cur_intron(1):cur_intron(2)-1) + 1;
          intron_level(3,end+1) = max(intron_loc(cur_intron(1):cur_intron(2)-1));
          intron_level(1,end) = istart ;
          intron_level(2,end) = istop ;

          max_leveli = max(intron_level(3,end),max_leveli);
        end
      end
    end
  end
  
  splice_loc = exon_loc+intron_loc ;
  level=zeros(5,length(splice)-1) ;
  for i=1:length(splice)-1,
    level(:,i)=[splice(i);
                splice(i+1);
                max(splice_loc([splice(i)-start:splice(i+1)-start-1]));
                max(exon_loc([splice(i)-start:splice(i+1)-start-1]));
                max(intron_loc([splice(i)-start:splice(i+1)-start-1]));
               ] ;
  end ;

return


return

genes = infer_splice_graph_caller(genes) ;

return

function genes = infer_splice_graph_caller(genes) ;
% genes = infer_splice_graph_caller(genes) ;
  
%infer_splice_graph

MISALIGN = 5;

fprintf(1,'Number of genes:%d\n',length(genes));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some simple merging to reduce exons
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf(1,'Merging some genes\n');
genes = reduce_splice_graph_caller(genes) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% infer_splice_graph infers the relevant splice graph given a set of ESTs. 
%
% It is based on two assumptions, neither which are true in reality, but
% are necessary for any reasonable processing.
% 1. All splicing events are independent. That is a splice site
%    at location A in ESTA does not depend on splice site B in ESTA, and
%    neither does it depend on splice site C in ESTB.
% 2. There exists a splicing event at a particular location if and only if
%    there exists an EST that is spliced at this point.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


fprintf(1,'performing inference\n');

for gene_idx = 1:length(genes)

  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % For each gene, we consider each pair of exons in the splicegraph.
  % After sorting the exons by the location of the left side, we make
  % a local copy of the splicegraph.
  %
  % Note that 'left' means the 5 prime end of a positive strand, but
  % the 3 prime end of a negative strand.
  %
  % The two exons considered are indexed by exon_idx, and test_exon_idx.
  % The general idea of the algorithm is as follows:
  % For each pair:
  %   check which case it is in (more on this later)
  %   if it is possible to merge:
  %     form a new exon
  %   end merge.
  %   if new exon is not in current set
  %     add to splicegraph
  %   end new exon insert
  % end for
  % 
  % In words:
  % Each time two exons are considered, and they can be merged to form
  % a new exon, they are added to the list of exons. Then, each of the
  % original parent exons are considered for deletion (variable to_keep).
  % All the exons are kept till the very end, when all exons labelled
  % to_keep == 0 are removed.
  %
  % Hence the number of exons could potentially grow quadratically.
  % Since we have to also consider merging the pairs of exons which are
  % newly created, the number of pairs considered can be larger than
  % m^2 where m is the number of original exons. However, I believe that
  % the algorithm should terminate because there are only finitely many
  % splice sites and hence at worst, we have all possible left and right
  % exon ends used in our splicegraph.
  %
  % I am not sure what the best termination condition for the loops are.
  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if isempty(genes(gene_idx).splicegraph{1}),
    continue ; 
  end ;
  [dummy,exon_order] = sort(genes(gene_idx).splicegraph{1}(1,:),2,'ascend');
  vertices = genes(gene_idx).splicegraph{1}(:,exon_order);
  edges = genes(gene_idx).splicegraph{2}(exon_order,exon_order);
  to_keep = ones(1,size(vertices,2));

  changed = 0;
  exon_idx = 1;
  first_merge_exon = 1;
  test_exon_idx = 0;
  while exon_idx <= size(vertices,2)

    if changed
      changed = 0;
      exon_idx = first_merge_exon;
      continue;
    end
    

    num_exons = size(vertices,2);
    [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
    if ~isequal(vertices(1,:),dummy)
      vertices = vertices(:,exon_order);
      edges = edges(exon_order,exon_order);
      to_keep = to_keep(exon_order);
    end ;
    
    
    exon_take_idx = find(((vertices(1,exon_idx)-2*MISALIGN)<vertices(2,:))&...
			 ((vertices(2,exon_idx)+2*MISALIGN)>vertices(1,:)));
    if ~isempty(exon_take_idx)
      first_merge_exon = exon_take_idx(1);
    end
    exon_take_idx = exon_take_idx(exon_take_idx>exon_idx);
    internal_idx = 1;


    while ((internal_idx <= length(exon_take_idx)) && ~changed)
      test_exon_idx = exon_take_idx(internal_idx);
      if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard;end;
      cur_edge_left = sum(edges(1:exon_idx,exon_idx));
      test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
      cur_edge_right = sum(edges(exon_idx:end,exon_idx));
      test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  

      new_vertex = zeros(2,1);
	
      
      
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % In the following, the cases are labelled with a binary string
      % [cur_edge_left, cur_edge_right, test_edge_left, test_edge_right]
      %
      % cur refers to exon_idx
      % test refers to test_exon_idx
      % edge means introns.
      % e.g. cur_edge_left == 1 means that there exists an intron on
      % the left of exon_idx
      %
      % Hence there are 16 cases, and we treat each one individually.
      % For each case, the two logical tests are:
      % 1. should we merge?
      % 2. is the new exon known?
      %
      % Note that a splice site is defined by an intron-exon boundary.
      % Furthermore, we allow misalignment by MISALIGN nucleotides,
      % that is we shorten an exonAendA if it doesn't have a splice site
      % and there exists a splice site exonBendB within MISALIGN nucleotides.
      % 
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
      
      
      % 0000
      if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)

	if ((~isequal(vertices(1,exon_idx),vertices(1,test_exon_idx)))||...
	    (~isequal(vertices(2,exon_idx),vertices(2,test_exon_idx))))&&...
	      ((to_keep(test_exon_idx)~=0)||(to_keep(exon_idx)~=0))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	  
	  
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = zeros(1,num_exons);
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix))
	      known = ix;
	      break;
	    end
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  else
	    if (known ~= exon_idx) && to_keep(exon_idx)
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (known ~= test_exon_idx) && to_keep(test_exon_idx)
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	      
	      

	% 0001
      elseif (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:)) 
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 0100
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	     (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	
	  new_vertex(1) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  new_vertex(2) = vertices(2,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 0010
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))
	    
	    
	  new_vertex(1) = vertices(1,test_exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1000
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))
	
	    
	  new_vertex(1) = vertices(1,exon_idx);
	  new_vertex(2) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0011
      elseif (~cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	     (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	
	  new_vertex = vertices(:,test_exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=exon_idx);
	  
	  if to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= test_exon_idx) && to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1100
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)
	
	  new_vertex = vertices(:,exon_idx);
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  
	  known = 0;
	  for ix = 1:size(vertices,2)
	    if isequal(new_vertex,vertices(:,ix)) && issubset(new_edges,edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert(known~=test_exon_idx);
	  
	  if to_keep(test_exon_idx)
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif (known ~= exon_idx) && to_keep(exon_idx)
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 0101
      elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))

	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(1,exon_idx) > vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(1,exon_idx) <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    

	    
	    
	% 1010
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    edges(test_exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	    
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx))
	      to_keep(exon_idx) = 0;
	    else
	      to_keep(test_exon_idx) = 0;
	    end
	  
	  
	    changed = 1;
	  else
	    if (vertices(2,exon_idx) < vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	      to_keep(exon_idx) = 0;
	      changed = 1;
	    end
	    if (vertices(2,exon_idx) >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	      to_keep(test_exon_idx) = 0;
	      changed = 1;
	    end
	  end
	end
	    
	    
	    
	% 0110  
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(2,test_exon_idx)-MISALIGN)&&...
	      (vertices(1,exon_idx)<=vertices(2,test_exon_idx))

	  new_vertex = [vertices(1,test_exon_idx);vertices(2,exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:)))) == ~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end

	% 1001
      elseif (cur_edge_left&&~cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)+MISALIGN)&&...
	      (vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))&&...
	      (vertices(2,exon_idx)>=vertices(1,test_exon_idx))

	
	  new_vertex = [vertices(1,exon_idx);vertices(2,test_exon_idx)];
	  new_edges = or(edges(exon_idx,:),edges(test_exon_idx,:));
	  known = 0;
	  for ix = 1:size(vertices,2)
	    % assert(isequal(find(new_edges),intersect(find(new_edges),find(edges(ix,:))))==~any(new_edges & ~edges(ix,:)));
	    if isequal(new_vertex,vertices(:,ix)) && ~any(new_edges & ~edges(ix,:))
	      known = ix;
	      break;
	    end
	  end
	  assert((known~=exon_idx)&&(known~=test_exon_idx));
	  
	  if ~known
	    vertices = [vertices,new_vertex];
	    edges = [edges;new_edges];
	    edges = [edges,[new_edges';0]];
	    to_keep(end+1) = 1;
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  elseif to_keep(exon_idx) || to_keep(test_exon_idx)
	    to_keep(exon_idx) = 0;
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	  

	    
	  
	    
	% 0111 
      elseif (~cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)+MISALIGN>=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge, 1, 'first');
	  
	  
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)+MISALIGN >= vertices(1,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	    
	% 1101
      elseif (cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)
	if (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  
	  %    (vertices(1,exon_idx)-MISALIGN<=vertices(1,test_exon_idx))
	  
	  % index to the right
	  right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  idx = find(vertices(1,:)>right_edge,1,'first');
	
	
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	    
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  if (vertices(1,exon_idx)-MISALIGN <= vertices(1,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1011  
      elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %(vertices(2,exon_idx)-MISALIGN<=vertices(2,test_exon_idx))
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last');
	  
	  
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    
	    
	    %to_keep(exon_idx) = 0;
	      
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)-MISALIGN <= vertices(2,test_exon_idx)) && (to_keep(exon_idx))
	    to_keep(exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	
	% 1110
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&~test_edge_right)
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))
	  
	  %    (vertices(2,exon_idx)+MISALIGN>=vertices(2,test_exon_idx))
	  
	  % index to the left
	  left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  idx = find(vertices(2,:)<left_edge,1,'last') ;

	
	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	  
	    
	    %to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  end
	  
	  if (vertices(2,exon_idx)+MISALIGN >= vertices(2,test_exon_idx)) && (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	end
	
	% 1111
      elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	
	if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	  if (~isequal(edges(exon_idx,:),edges(test_exon_idx,:)))
	    edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(test_exon_idx,:) = edges(exon_idx,:);
	    edges(:,exon_idx) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	    edges(:,test_exon_idx) = edges(:,exon_idx);
	  
	    
	    to_keep(test_exon_idx) = 0;
	    
	    changed = 1;
	  elseif (to_keep(test_exon_idx))
	    to_keep(test_exon_idx) = 0;
	    changed = 1;
	  end
	elseif (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)~=vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the left
	  %left_edge = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	  %idx = find(vertices(2,:)<left_edge, 1, 'last');
	  idx = find(vertices(2,:)<vertices(1,exon_idx), 1, 'last');

	  if (~isequal(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx)))
	    edges(exon_idx,1:idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(test_exon_idx,1:idx) = edges(exon_idx,1:idx);
	    edges(1:idx,exon_idx) = or(edges(exon_idx,1:idx),edges(test_exon_idx,1:idx));
	    edges(1:idx,test_exon_idx) = edges(1:idx,exon_idx);
	    	    
	    changed = 1;
	  end

	elseif (vertices(1,exon_idx)~=vertices(1,test_exon_idx))&&...
	      (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
	      (to_keep(exon_idx)||to_keep(test_exon_idx))
	  % index to the right
	  %right_edge = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	  %idx = find(vertices(1,:)>right_edge,1,'first');
	  idx = find(vertices(1,:)>vertices(2,exon_idx),1,'first');
	
          
	  if (~isequal(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end)))
	  
	    edges(exon_idx,idx:end) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(test_exon_idx,idx:end) = edges(exon_idx,idx:end);
	    edges(idx:end,exon_idx) = or(edges(exon_idx,idx:end),edges(test_exon_idx,idx:end));
	    edges(idx:end,test_exon_idx) = edges(idx:end,exon_idx);
	    
	    changed = 1;
	  end
	  
	end
	
      else
	fprintf(1,'Unknown case!!\n');
        if keyboard_allowed(), keyboard; end ;
	
      end  % cases
      

      %test_exon_idx = test_exon_idx+1;
      internal_idx = internal_idx + 1;

    end
    exon_idx = exon_idx + 1;
  end


  
  
  

    
  genes(gene_idx).splicegraph = {};
  to_keep = find(to_keep);
  genes(gene_idx).splicegraph = {vertices(:,to_keep),edges(to_keep,to_keep)};
end



fprintf(1,'\n');












return

function found = issubset(A,B)
% function found = issubset(A,B)
%
% returns true if A is a subset of B, where 
% both A and B are vectors with 1 where elements exists and 0 otherwise.
found = isequal(find(A),intersect(find(A),find(B)));
return

function genes=reduce_splice_graph_caller(genes) ;
% genes=reduce_splice_graph_caller(genes) ;

for gene_idx = 1:length(genes)
%gene_idx
%genes(gene_idx)
  vertices = genes(gene_idx).splicegraph{1};
  edges = genes(gene_idx).splicegraph{2};
  %fprintf(1,'gene index:\t%d\n',gene_idx);
  if (mod(gene_idx,1000)==0)
    fprintf(1,'%d\n',gene_idx);
  end
  if isempty(vertices), continue ; end ;

  % find all the intron locations
  [dummy,exon_order] = sort(vertices(1,:),2,'ascend');
  vertices = vertices(:,exon_order);
  edges = edges(exon_order,exon_order);
  intron_loc = [];
  for ix1 = 1:size(vertices,2)-1
    for ix2 = 2:size(vertices,2)
      if edges(ix1,ix2)
	intron_loc = union(intron_loc,[vertices(2,ix1)+1:vertices(1,ix2)-1]);
      end
    end
  end
  
  
  
  if (size(edges,1) < 2)
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)
	
	test_exon_idx = exon_idx+1;
	while test_exon_idx <= size(vertices,2)
	  if (vertices(1,exon_idx)<=vertices(1,test_exon_idx)) &&...
		(vertices(2,exon_idx)>=vertices(2,test_exon_idx))
	    vertices(1,exon_idx) = min(vertices(1,exon_idx),...
				       vertices(1,test_exon_idx));
	    vertices(2,exon_idx) = max(vertices(2,exon_idx),...
				       vertices(2,test_exon_idx));
	    new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	    vertices = vertices(:,new_index);
	  
	    %disp(vertices)
	    
	    %test_exon_idx = exon_idx+1;
	    changed = 1;
	    
	  end
	  test_exon_idx = test_exon_idx+1;
	end
	exon_idx = exon_idx + 1;
      end
    end
    
  else
    changed = 1;
    while changed
      changed = 0;
      exon_idx = 1;
      while exon_idx <= size(vertices,2)

	[dummy,exon_order] = sort(vertices(1,:),2,'ascend');
	if ~isequal(vertices(1,:),dummy)
	  vertices = vertices(:,exon_order);
	  edges = edges(exon_order,exon_order);
	end
	
	test_exon_idx = exon_idx + 1;
	while test_exon_idx <= size(vertices,2)
	  reduce_now = 0;
	  
          if (test_exon_idx < exon_idx) && keyboard_allowed(), keyboard; end;
	  
	  cur_edge_left = sum(edges(1:exon_idx,exon_idx));
	  test_edge_left = sum(edges(1:test_exon_idx,test_exon_idx));
	  cur_edge_right = sum(edges(exon_idx:end,exon_idx));
	  test_edge_right = sum(edges(test_exon_idx:end,test_exon_idx));
	  
	  

	  
	  % 0000
	  if (~cur_edge_left&&~cur_edge_right&&~test_edge_left&&~test_edge_right)
	    
	    if ((vertices(2,exon_idx)>=vertices(1,test_exon_idx))&&...
		(vertices(1,exon_idx)<=vertices(1,test_exon_idx)))||...
		  ((vertices(2,test_exon_idx)>=vertices(1,exon_idx))&&...
		   (vertices(1,test_exon_idx)<=vertices(1,exon_idx)))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end
	    
	    
	  % 0101
	  elseif (~cur_edge_left&&cur_edge_right&&~test_edge_left&&test_edge_right)

	    if (vertices(2,exon_idx)==vertices(2,test_exon_idx))&&...
		  (sum(ismember([min(vertices(1,exon_idx),vertices(1,test_exon_idx)):...
				 vertices(2,exon_idx)],intron_loc))==0)
	      
	      vertices(1,exon_idx) = min(vertices(1,exon_idx),vertices(1,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	    
	  % 1010
	  elseif (cur_edge_left&&~cur_edge_right&&test_edge_left&&~test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		  (sum(ismember([vertices(1,exon_idx):...
				 max(vertices(2,exon_idx),vertices(2,test_exon_idx))],intron_loc))==0)
	      
	      vertices(2,exon_idx) = max(vertices(2,exon_idx),vertices(2,test_exon_idx));
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	    
	  
	    
	      reduce_now = 1;
	      changed = 1;
	    end

	  
	  % 1111
	  elseif (cur_edge_left&&cur_edge_right&&test_edge_left&&test_edge_right)
	    if (vertices(1,exon_idx)==vertices(1,test_exon_idx))&&...
		 (vertices(2,exon_idx)==vertices(2,test_exon_idx))
	    
	      new_index = [1:test_exon_idx-1,test_exon_idx+1:size(vertices,2)];
	    
	    
	      vertices = vertices(:,new_index);
	      edges(exon_idx,:) = or(edges(exon_idx,:),edges(test_exon_idx,:));
	      edges(:,exon_idx) = or(edges(:,exon_idx),edges(:,test_exon_idx));
	      edges = edges(new_index,new_index);
	      
	      reduce_now = 1;
	      changed = 1;
	  
	    end
	    
	  end % cases
	  

          if ~reduce_now
	    test_exon_idx = test_exon_idx+1;
	  end
	  

	end
	exon_idx = exon_idx + 1;
      end %while exon_idx <= size(vertices,2)
    end %while changed

  
  
  
  end 
    
  genes(gene_idx).splicegraph = {};
  genes(gene_idx).splicegraph = {vertices,edges};
end


fprintf(1,'\n');

return





return

function f=bigunique(s) ;
% f=bigunique(s) ;

f=[] ; step=ceil(length(s)/10) ;
for i=1:10,
  idx=1+(i-1)*step:min(length(s), i*step) ;
  f=unique([f unique(s(idx))]) ;
end ;


return

function len = contig_len(contig, genome_info)
% len = contig_len(contig, genome_info)

if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;

d=dir(fname) ; 
len=d.bytes;

return

function checksum = crc16(String)  
% checksum = crc16(String)  
 
checksum = sg('crc', String) ;
checksum = uint32(checksum) ;
checksum = bitand(checksum, 65535) + floor(checksum/65535) ;
checksum = bitand(checksum, 65535) ;


return

function [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
% [tmp_fname,STATS] = gff3_statistics(gff_fname,thesources, contig)
  

if nargin<2
  thesources = [];
end

                                                                                                                                                       
if nargin<3                                                                                     
  contig = [];                                                                                    
end 
                                                                                    

%%% SOFA mapping
SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};

%%% GFF file summary

[fd msg] = fopen(gff_fname, 'r');
disp(msg);

tmp_fname = tempname;
if ~isempty(thesources) 
  fprintf('filtering for source :\n');
  unix(sprintf('echo > %s', tmp_fname)) ;
  for i=1:length(thesources),
    fprintf('%s\n',thesources{i});
    unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s |grep -v "#" >> %s', thesources{i}, gff_fname, tmp_fname));
  end 
else
  unix(sprintf('grep -v "#" %s > %s', gff_fname, tmp_fname));
end

fprintf('\n------FILE SUMMARY-----\n');
[tmp,seqids] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u', tmp_fname));
if isempty(contig) 
   fprintf('gff file containes following %i seqids in column 1 :\n',length(seqids));
   if length(seqids)<100
      fprintf('%s\n',seqids)
   end
end

[tmp, sources]=unix(sprintf('sed -e "1d" %s | cut -f 2 |sort -u', tmp_fname));
if isempty(sources)
  error('specified sources not contained in file')
end
fprintf('gff file containes following sources in column 2 :\n');
sources = separate(sources,sprintf('\n'));
sources(end)=[];
for t=1:length(sources)
  [tmp,num_sources]=unix(sprintf('cut -f 2 %s |grep -e "^%s$" | wc -l',tmp_fname,sources{t}));
  fprintf('%s\t%i\n',sources{t},str2num(num_sources));
end
  
if ~isempty(contig) 
  fprintf('filtering for contig :\n');
  fprintf('^%s[[:space:]]\n',contig);

  seqids_array = separate(upper(seqids), sprintf('\n')) ;
  if ~isempty(seqids_array) && isempty(seqids_array{end})
    seqids_array(end)=[] ;
  end ;

  cnamepos = strmatch(upper(contig), seqids_array, 'exact');
  % contig = seqids(cnamepos:cnamepos+length(contig)-1); 
  if isempty(cnamepos)%all(isempty(strfind(upper(seqids),upper(contig))))
    fprintf('contig not found in file\n')
    return
  end
  tmp_fname1 = tempname;
  unix(sprintf('grep -e "^%s[[:space:]]" %s > %s', contig, tmp_fname, tmp_fname1));
  [tmp,num_seqs] = unix(sprintf('sed -e "1d" %s | cut -f 1 |sort -u | wc -l',tmp_fname1));
  if (str2num(num_seqs)~=1)
    error('grep for contig doesnt work, repeat with by_contig=0')
  end
else
  tmp_fname1 = tmp_fname;
end
[tmp,types_]=unix(sprintf('sed -e "1d" %s | cut -f 3 |sort -u',tmp_fname1));
fprintf('\nfiltered gff file containes following types in column 3 :\n');
types = separate(types_,sprintf('\n')) ;
idx = find(cellfun('isempty',types));
types(idx)=[];


for t=1:length(types)
  [tmp,num_types]=unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',tmp_fname1,types{t}));
  fprintf('%s\t%i\n',types{t},str2num(num_types));
  STATS.(types{t})=str2num(num_types);
end
% assert(all(ismember(lower(types),lower(SOFA_short_names)))|all(ismember(types,SOFA_IDs)))

tmp_fname= tmp_fname1;

return

function gtf2gff(gtf_fname, gff3_fname) ;
% gtf2gff(gtf_fname, gff3_fname) ;

tmpname = tempname ;
[res] = unix(sprintf('cut -f 9 %s | sort -u> %s', gtf_fname, tmpname));
assert(res==0) ;

gene_ids = {} ;
gene_info = struct ;
gene_info.chr='' ;
gene_info.start=inf ;
gene_info.stop=0 ;
gene_info.strand='' ;
gene_info.info='' ;
gene_info=gene_info([]) ;

transcript_ids = {} ;
fd=fopen(gtf_fname, 'r') ;
fd2=fopen(gff3_fname, 'w') ;
while (~feof(fd)),
  line=fgetl(fd) ;
  if ~ischar(line),
    break ;
  end ;
  if line(1)=='#', continue; end ;
  items1=separate(line) ;
  chr=items1{1} ;
  info=items1{2} ;
  start=str2num(items1{4}) ;
  stop=str2num(items1{5}) ;
  strand=items1{7} ;
  
  items=separate(items1{9}, ';') ;
  gene_id='' ;
  for i=1:length(items),
    items{i}=deblank(items{i}) ;
    gene_id_pos = strfind(items{i}, 'gene_id') ;
    if ~isempty(gene_id_pos),
      str=items{i}(gene_id_pos+7:end) ;
      str(str=='"') = [] ;
      while(str(1)==' ')
        str=str(2:end) ;
      end ;
      gene_id=deblank(str) ;
      if length(gene_ids)>1 && isequal(gene_ids{end}, gene_id),
        idx=length(gene_ids) ;
      else
        idx=strmatch(gene_id, gene_ids, 'exact') ;
      end ;
      if isempty(idx)
        gene_ids{end+1}=str ;
        gene_info(length(gene_ids)).chr=chr ;
        gene_info(length(gene_ids)).start=start ;
        gene_info(length(gene_ids)).stop=stop ;
        gene_info(length(gene_ids)).strand=strand ;
        gene_info(length(gene_ids)).info=info ;
        transcript_ids{end+1}={} ;
        idx=length(gene_ids) ;
        if mod(idx,1000)==0, 
          fprintf('reading gene %i\n', idx) ;
        end ;
      else
        gene_info(idx).start=min(start, gene_info(idx).start) ;
        gene_info(idx).stop=max(stop, gene_info(idx).stop) ;
      end ;
    end ;
  end ;
  if isempty(gene_id),
    fprintf('failed parsing line:\n%s\n',line) ;
    %save ~/tmp/gtf2gff_bug1.mat 
    assert(false);
  end ;
  transcript_id='' ;
  for i=1:length(items),
    items{i}=deblank(items{i}) ;
    transcript_id_pos = strfind(items{i}, 'transcript_id') ;
    if ~isempty(transcript_id_pos),
      str=items{i}(transcript_id_pos+13:end) ;
      str(str=='"') = [] ;
      while(str(1)==' ')
        str=str(2:end) ;
      end ;
      transcript_id = deblank(str) ;
      assert(length(idx)==1)
      if isempty(strmatch(transcript_id, transcript_ids{idx}))
        transcript_ids{idx}{end+1}=transcript_id ;
      end ;
    end ;
  end ;
  if isempty(transcript_id),
    fprintf('failed parsing line:\n%s\n', line) ;
    %save ~/tmp/gtf2gff_bug2.mat 
    assert(false);
  end ;
  
  fprintf(fd2, '%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\tParent=transcript:%s\n', items1{1}, items1{2}, items1{3}, items1{4}, items1{5}, items1{6}, items1{7}, items1{8}, transcript_id);
end ;
fclose(fd) ;

for i=1:length(gene_ids)
  fprintf(fd2, '%s\t%s\tgene\t%i\t%i\t.\t%c\t.\tID=gene:%s\n', gene_info(i).chr, gene_info(i).info, gene_info(i).start, gene_info(i).stop, gene_info(i).strand, gene_ids{i}) ;
  for j=1:length(transcript_ids{i}),
    fprintf(fd2, '%s\t%s\tmRNA\t%i\t%i\t.\t%c\t.\tID=transcript:%s;Parent=gene:%s\n', gene_info(i).chr, gene_info(i).info, gene_info(i).start, gene_info(i).stop, gene_info(i).strand, transcript_ids{i}{j}, gene_ids{i}) ;
  end ;
end
fclose(fd2) ;

% cleanup
res=unix(sprintf('rm %s', tmpname)) ;
assert(res==0) ;

return

function idx = hash_search(transcript_hash_names, transcript_hash_idx, transcript_name) ;
% idx = hash_search(transcript_hash_names, transcript_hash_idx, transcript_name) ;

checksum=crc16(transcript_name)+1 ;
idx_ = strmatch(transcript_name, transcript_hash_names{checksum}, 'exact') ;
idx = transcript_hash_idx{checksum}(idx_) ;

return

function genes = init_genes(num_genes,PAR,genes)
% genes = init_genes(num_genes,PAR,genes)
  
if nargin<1
  num_genes=0;
end
if nargin<2 || isempty(PAR),
  PAR.organism.name =[];
end
    
genes.id = [];
genes.anno_id = [];
genes.confgenes_id = [];
genes.name = '' ;
genes.alias = [];
genes.name2 = []; 
genes.strand = '';
genes.chr = '';
genes.chr_num = [];
genes.paralogs = [];

genes.start = [];
genes.stop = [];

genes.transcripts = {};
genes.transcript_info = [];
genes.transcript_status = [];
genes.transcript_valid = [];
genes.exons = {}; 
genes.exons_confirmed = {};
genes.cds_exons = {}; 
genes.utr5_exons = {}; 
genes.utr3_exons = {}; 
genes.tis = {};
genes.tis_conf = [];
genes.tis_info = [];

genes.cdsStop = {};
genes.cdsStop_conf = [];
genes.cdsStop_info = [];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.transacc = {};
  genes.transacc_info = [];
  genes.transacc_conf = [];
end
genes.tss = {};
genes.tss_info = [];
genes.tss_conf = [];
genes.cleave = {};
genes.cleave_info = [];
genes.cleave_conf = [];
genes.polya = {};
genes.polya_info = [];
genes.polya_conf = [];

genes.is_alt = [];
genes.is_alt_spliced = [];
genes.is_valid = [];
genes.transcript_complete = [];
genes.is_complete = [];
genes.is_correctly_gff3_referenced = [];

genes.splicegraph =[];

if isequal(PAR.organism.name,'C_elegans')||(isfield(PAR.organism,'clade')&&isequal(PAR.organism.clade,'nematode'))
  genes.in_operon=[];
end

if num_genes==0
  genes=genes([]);
elseif num_genes>1,
  genes(num_genes) = genes ;
end

return

function genome_info=init_genome(config_fname)
% genome_info=init_genome(config_fname)  
%
% global g_genome_info ;

global g_genome_info genome_strings;

genome_strings = [];
g_est_fnames={} ;
g_prot_fnames={} ;
g_cdna_fnames={} ;
g_flcdna_fnames={} ;
g_contig_names={} ;
g_flat_fnames={} ;
g_fasta_fnames={} ;
g_annotation_fnames={} ;
g_alphabet='acgt' ;
g_basedir='' ;
g_max_intron_len=1000000 ;
g_min_intron_len=10 ;
g_merge_est_transcripts=1 ;

fd=fopen(config_fname, 'r') ;
while ~feof(fd),
  line = fgetl(fd) ;
  if ~ischar(line), break ; end ;
  if isempty(line), continue ; end ;
  if line(1)=='#', continue ; end ;
  if length(line)>8 && isequal(line(1:7),'BASEDIR')
    g_basedir=deblank(line(9:end)) ;
  elseif length(line)>21 && isequal(line(1:21), 'MERGE_EST_TRANSCRIPTS')
    g_merge_est_transcripts = str2num(line(22:end)) ; 
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_QGAP')
  elseif length(line)>15 && isequal(line(1:15),'MAX_DOUBLE_TGAP')
  elseif length(line)>17 && isequal(line(1:17),'MAX_CDNA_DELETION')
  elseif length(line)>18 && isequal(line(1:18),'MAX_CDNA_INSERTION')
  elseif length(line)>21 && isequal(line(1:21),'TERMINAL_EXON_END_TOL')
  elseif length(line)>14 && isequal(line(1:14),'MIN_INTRON_LEN')
    g_min_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_min_intron_len) & g_min_intron_len<1000) ;
  elseif length(line)>14 && isequal(line(1:14),'MAX_INTRON_LEN')
    g_max_intron_len = str2num(line(15:end)) ; 
    assert(~isempty(g_max_intron_len) & g_max_intron_len>1000) ;
  elseif length(line)>18 && isequal(line(1:18),'MIN_EST_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_PROT_COVER_FRAC')

  elseif length(line)>19 && isequal(line(1:19),'MIN_CDNA_COVER_FRAC')
  
  elseif length(line)>18 && isequal(line(1:18),'BLAT_BEST_HIT_ONLY')
  
  elseif length(line)>20 && isequal(line(1:20),'BLAT_BEST_HIT_MARGIN')

  elseif length(line)>8 && isequal(line(1:8),'ALPHABET')
    g_alphabet=deblank(line(10:end)) ;
  elseif length(line)>7 && isequal(line(1:7),'CONTIGS'),
    for i=1:str2num(line(8:end))
      l2=fgetl(fd) ; 
      idx=strfind(l2,'  ') ;
      while ~isempty(idx), 
        l2(idx)=[] ;
        idx=strfind(l2,'  ') ;
      end ;
      l2(l2==' ')=sprintf('\t') ;
      elems=separate(l2) ; 
      assert(length(elems)==3) ;
      g_contig_names{i}=elems{1} ;
      g_flat_fnames{i}=path_with_basedir(elems{2}, g_basedir) ;
      g_fasta_fnames{i}=path_with_basedir(elems{3}, g_basedir) ; 
    end ;
  elseif length(line)>8 && isequal(line(1:8),'ESTFILES'),
    for i=1:str2num(line(9:end))
      g_est_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'PROTFILES'),
    for i=1:str2num(line(10:end))
      g_prot_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>9 && isequal(line(1:9),'CDNAFILES'),
    for i=1:str2num(line(10:end))
      g_cdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>11 && isequal(line(1:11),'FLCDNAFILES'),
    for i=1:str2num(line(12:end))
      g_flcdna_fnames{end+1}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif length(line)>15 && isequal(line(1:15),'ANNOTATIONFILES'),
    for i=1:str2num(line(16:end))
      g_annotation_fnames{i}=path_with_basedir(deblank(fgetl(fd)), g_basedir) ; 
    end ;
  elseif ~isempty(line),
    error(sprintf('cannot understand "%s" in config file', line)) ;
  end ;
end ;
fclose(fd) ;

genome_info=[] ;
genome_info.est_fnames=g_est_fnames ;
genome_info.prot_fnames=g_prot_fnames ;
genome_info.cdna_fnames=g_cdna_fnames ;
genome_info.flcdna_fnames=g_flcdna_fnames ;
genome_info.contig_names=g_contig_names ;
genome_info.flat_fnames=g_flat_fnames ;
genome_info.fasta_fnames=g_fasta_fnames ;
genome_info.annotation_fnames=g_annotation_fnames ;
genome_info.alphabet=g_alphabet ;
genome_info.basedir=g_basedir ;
genome_info.max_intron_len = g_max_intron_len ;
genome_info.min_intron_len = g_min_intron_len ;
genome_info.merge_est_transcripts = g_merge_est_transcripts ;
g_genome_info=genome_info ;

return ;


function p=path_with_basedir(p1,basedir)
  
  if p1(1)=='/',
    p=p1; 
    return ;
  else
    if basedir(end)=='/',
      p=[basedir p1] ;
    else
      p=[basedir '/' p1] ;
    end ;
  end ;
  
  

return

function [transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names, new_start, transcript_hash_names, transcript_hash_idx) ;
% [transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names, new_start, transcript_hash_names, transcript_hash_idx) ;

if nargin<=1,
  transcript_hash_names = {} ;
  transcript_hash_names{65536}={} ;
  transcript_hash_idx = {} ;
  transcript_hash_idx{65536} = [] ;
  new_start = 1 ;
end ;
for i=new_start:length(transcript_names),
        if mod(i,1000)==0, fprintf('generated %i hash keys\n', i) ; end ;
	checksum=crc16(transcript_names{i})+1 ;
        transcript_hash_names{checksum}{end+1} = transcript_names{i} ;
        transcript_hash_idx{checksum}(end+1) = i ;
end ;

return

function [str,valid_splice,ok] = load_genomic(contig,strand,start,stop, genome_info,check_splice)
% function [str,valid_splice,ok]=load_genomic(contig,strand,start,stop, genome_info,check_splice)

if nargin<6
  check_splice=0;
end ;
ok=1 ;

if ~check_splice 
  valid_splice=[];
  % fprintf(1,'no splice site consensus check!')
end ;


if length(start)>1 | length(stop)>1,
  if nargout<3
    assert(length(start)==length(stop)) ;
    assert(all([start(2:end)-stop(1:end-1)]>=0));
  else
    if ~(length(start)==length(stop)) || ~(all([start(2:end)-stop(1:end-1)]>=0)),
      ok=0 ;
      str='' ;
      valid_splice = [] ;
      return ;
    end ;
  end ;
  valid_splice = zeros(length(start),2); 
  if strand == '+', 
    idx = 1:length(start) ; 
  else 
    idx = length(start):-1:1 ; 
  end ;
  str = '' ;
  for i=idx  %1:length(start)
    seq = load_genomic(contig, strand, start(i)-5, stop(i)+5, genome_info) ;
    str = [str seq(6:end-5)] ;

    % check internal splicing sites
    if check_splice
      % if (i~=1 && strand == '+') || (i~=length(start) && strand == '-')
      if (strand == '+') || (i~=length(start) && strand == '-')
        valid_splice(i,1) = isequal(seq(4:5),'ag') ;
        %if ~isequal(seq(4:5),'ag'), fprintf('acc: %s %c\n', seq(3:6), strand); end ;
        % assert(isequal(seq(4:5),'ag'))
      end ;
      %if i~=idx(end)
      % if (i~=1 && strand == '-') || (i~=length(start) && strand == '+')
      if (strand == '-') || (i~=length(start) && strand == '+')        
        valid_splice(i,2) = ismember(seq(end-5:end-4), {'gt','gc'}) ;
        %if ~isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'), fprintf('don: %s %c\n', seq(end-5:end-2), strand); end ;
        % assert(isequal(seq(end-4:end-3),'gt')||isequal(seq(end-4:end-3),'gc'))
      end ; 
    end ;
  end
  return ;
else
  valid_splice=[0 0];
end ;


if ischar(contig),
  %contig_idx = find(ismember(upper(genome_info.contig_names),upper(contig))) ;
  contig_idx = strmatch(upper(contig), upper(genome_info.contig_names), 'exact') ;
  assert(all(size(contig_idx)==1) & ~isempty(contig_idx)) ;
else
  contig_idx = contig ;
end ;
  
fname=genome_info.flat_fnames{contig_idx} ;
if genome_info.alphabet(1)=='a',
  NN='n' ;
else
  NN='N' ;
end ;

d=dir(fname) ; left_n = 0 ; right_n=0 ;
if isinf(stop), stop=d.bytes; end ;

if start<1 | stop>d.bytes,
  warning('load_genomic:contig_boundary', 'boundary of contig reached (start: %i, stop: %i, d.bytes: %i), padding with "n"', start, stop,d.bytes) ;

  if start<1,
    left_n = -(start-1) ;
    start = 1 ;
  end ;
  if stop>d.bytes,
    right_n = stop-d.bytes ;
  end ;
  if start>d.bytes && stop>d.bytes,
    str = char(NN*ones(1,stop-start)) ;
    return ;
  end ;
end ;

fd=fopen(fname,'r') ;
fseek(fd, start-1, -1) ;
str=char(fread(fd,stop-start+1, 'char=>char'))' ;
fclose(fd) ;
if left_n>0,
  str=char([NN*ones(1,left_n) str]) ;
end ;
if right_n>0,
  str=char([str NN*ones(1,right_n)]) ;
end ;

assert(length(str)==stop-start+left_n+1)
%let = bigunique(str) ;
let = unique(str) ;
if genome_info.alphabet(1)=='a' && ~isempty(setdiff(let,'acgtn')),
  % warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'acgtn')));
  let = setdiff(let, 'acgtn') ;
  for c=let,
    str(str==c) = 'n';
  end ;
elseif genome_info.alphabet(1)=='A' && ~isempty(setdiff(let,'ACGTN')),
  warning(sprintf('invalid letter in genome! ("%s")', setdiff(let,'ACGTN')));
  let = setdiff(let, 'ACGTN') ;
  for c=let,
    str(str==c) = 'N';
  end ;
end

if strand=='-',
  str=reverse_complement(str) ;
end ;

return ;

return

function dna_pos=map_rna_pos(exons, strand, rna_pos) ;
% dna_pos=map_rna_pos(exons, strand, rna_pos) ;

if strand=='+',
  l=exons(:,2)-exons(:,1) ;
  L=[0; cumsum(l)] ; 
  L(end)=[] ;
  idx=find(rna_pos>L, 1, 'last') ;
  rp=rna_pos-L(idx) ;
  assert(rp>=0) ;

  dna_pos = exons(idx,1)+rp-1 ;
else
  l=exons(end:-1:1,2)-exons(end:-1:1,1) ;
  L=[0; cumsum(l)] ;
  L(end)=[] ;
  idx = find(rna_pos>L, 1, 'last') ;
  rp = rna_pos-L(idx) ;
  assert(rp>=0) ;
  
  dna_pos = exons(size(exons,1)+1-idx,2) - rp + 1;
end ;
return

function [genes, FAULTS] = parse_gff3(gff_fname, save_fname, thesources, contig, TYPE,log_fname)
%        genes = parse_gff3(gff_fname, save_fname, thesources, contig, TYPE.log_fname)
% can also be used for gff2 format:
% TYPE.short_names = {'CDS','exon','coding_exon'};
% TYPE.IDs = {'1' '2' '3'};
% TYPE.hierachies = [2 3 3];
  
if nargin<3
  thesources = [];
end
  
  
if nargin<4
  contig = [];
end

if nargin<5||isempty(TYPE)
  %%% SOFA mapping
  SOFA_short_names = {'gene','mRNA','exon','CDS','intron','polyA_sequence','polyA_site','five_prime_UTR','three_prime_UTR'};
  SOFA_IDs = {'SO:0000704','SO:0000234','SO:0000147','SO:0000316','SO:0000188','SO:0000610','SO:0000553','SO:0000204','SO:0000205'};
  hierachies = [1 2 3 3 3 3 3 3 3];
else 
  SOFA_short_names = TYPE.short_names;
  SOFA_IDs = TYPE.IDs;
  hierachies = TYPE.hierachies;
end
 
status={'Predicted','Partially_confirmed','Confirmed'};

%%% GFF file summary

filtered_fname = gff3_statistics(gff_fname, thesources, contig);


if ~isempty(save_fname) && fexist(sprintf('%s.mat', save_fname))
  warning(sprintf('file %s.mat already exists\n', save_fname));
  %fprintf('\nwill be overwritten %s\n', save_fname);
  %load(save_fname,'genes');
  %assert(length(genes)==num_genes) 
end

FAULTS.Gene.no_strand = {};
FAULTS.Gene.no_id = {};
FAULTS.Gene.id_not_unique = {};
FAULTS.Transcript.unknown_parent = {};
FAULTS.Transcript.no_gene = {};
FAULTS.Transcript.diff_strand_contig = {};
FAULTS.Transcript.id_not_unique = {};
FAULTS.Transcript.outside_boundaries = {};
FAULTS.Exon.no_transcript = {};
FAULTS.Exon.diff_strand_contig = {};


% first parse: genes only are extracted 

types = SOFA_short_names(hierachies==1);
types_IDs = SOFA_IDs(hierachies==1);

fprintf('\n------starting first parse: only look for the following types\n');

tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_genes(i) = str2double(num);
  fprintf('%s\t%i\n',types{i},num_genes(i));
end

num_genes = sum(num_genes);
if num_genes ==0
  warning('no genes found; looking for hierachy 2 (mRNAs instead)\n')
  types = SOFA_short_names(hierachies==2);
  types_IDs = SOFA_IDs(hierachies==2);
  for i=1:length(types)
    [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
    unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
    num_genes(i) = str2double(num);
    fprintf('\t%s\t%i\n',types{i},num_genes(i));
  end
  num_genes
  num_genes = sum(num_genes);
end
if num_genes ==0
   warning('no genes found')
  return
end

genes = init_genes(num_genes);
[fd msg] = fopen(tmp_fname1, 'r');

genes_cnt = 0;
gene_names = {} ;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      FAULTS.Gene.no_strand{end+1} = elems{9};
      continue
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  %if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
  if isempty(strmatch(elems{3}, {types{:} types_IDs{:}}))
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end
  

  identifier = '';    alias = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
      break;
      % keyboard
    end
  end
  if isempty(identifier)
    FAULTS.Gene.no_id{end+1} = elems{9};
    % warning('no ID found 1')
    identifier = attributes{1};
  end
      
  for i=1:length(attributes) 
    id=strfind(attributes{i},'Alias=');
    if ~isempty(id),
      alias = attributes{i}(id+6:end);
      break;
    end 
  end
  if length(identifier)>=3 
    assert(~isequal(identifier(1:3),'ID=')) ;
  end
  assert(~isempty(identifier));

  other_idx=strmatch(identifier,gene_names,'exact');
  if ~isempty(other_idx)
    % warning('gene appears more than once\n')
    FAULTS.Gene.id_not_unique{end+1} = elems{9}; 
    assert(isequal(genes(other_idx).alias,alias));
    assert(isequal(genes(other_idx).chr,elems{1}));
    assert(isequal(genes(other_idx).strand,elems{7})) ;
    if ~isequal(genes(other_idx).start,str2double(elems{4}))
      genes(other_idx).start = min(genes(other_idx).start,str2double(elems{4}));
    end
    if ~isequal(genes(other_idx).stop,str2double(elems{5}))
      genes(other_idx).stop = max(genes(other_idx).stop,str2double(elems{5}));
    end 
    % assert(isequal(genes(other_idx).start,str2double(elems{4})));
    % assert(isequal(genes(other_idx).stop,str2double(elems{5})));
    continue
  end
  genes_cnt =  genes_cnt +1;
  genes(genes_cnt).name = identifier;
  genes(genes_cnt).alias = alias;
  genes(genes_cnt).chr = elems{1};
  genes(genes_cnt).strand = elems{7} ;
  genes(genes_cnt).start(end+1) = str2double(elems{4});
  genes(genes_cnt).stop(end+1) = str2double(elems{5});
  genes(genes_cnt).is_correctly_gff3_referenced = 1 ;

  gene_names{end+1} = identifier ;
  if mod(genes_cnt,1000)==0||genes_cnt==num_genes
    fprintf('  parsed %i genes\r', genes_cnt);  
  end
end % end first parse
% assert(genes_cnt==num_genes)
genes=genes(1:genes_cnt);
fclose(fd);

fprintf('\n\n');
if length(FAULTS.Gene.no_strand)>0 || length(FAULTS.Gene.no_id)>0 || length(FAULTS.Gene.id_not_unique)>0,
  fprintf('Following problems were found:\n') ;
else
  fprintf('No problems found.\n') ;
end ;
if length(FAULTS.Gene.no_strand)>0,
  fprintf('number of genes without annotated strand (column 7):\t\t %i\n', length(FAULTS.Gene.no_strand)) ;
  fprintf('\t=> genes are ignored\n') ;
end 
if length(FAULTS.Gene.no_id)>0,
  fprintf('number of genes without annotated ID (column 9, ID=... ):\t %i\n', length(FAULTS.Gene.no_id)) ;
  fprintf('\t=> first part (to the first semicolon) of attribute field (column 9) is used as ID\n') ;
end
if length(FAULTS.Gene.id_not_unique)>0,
  fprintf('number of genes with non-unique ID (column 9, ID=... ):\t\t %i\n', length(FAULTS.Gene.id_not_unique)) ;
  fprintf('\t=> all on the same seqid and strand, min of start (column 4) and max of end (column 5) used\n ') ;
end ;

unix(sprintf('rm %s',tmp_fname1));




% second parse: mRNAs only are extracted 

types = SOFA_short_names(hierachies==2);
types_IDs = SOFA_IDs(hierachies==2);

fprintf('\n------starting second parse: only look for the following types\n');
tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));
num_transcripts = zeros(1,length(types)) ;
for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep -e "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_transcripts(i) = str2double(num);
  fprintf('%s\t%i\n',types{i},num_transcripts(i));
end

num_transcripts = sum(num_transcripts);
transcript_gene_map = zeros(1,num_transcripts);
transcript_names = cell(1,num_transcripts);
transcript_ids = cell(1,num_transcripts);
gene_names = {genes.name} ;

[fd msg] = fopen(tmp_fname1, 'r');
transcript_cnt = 0;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end

  identifier = ''; parent = ''; pred_status = '';
  for i=1:length(attributes),
    id=strfind(attributes{i},'ID=');
    if ~isempty(id),
      identifier = attributes{i}(id+3:end);
    end 
    id=strfind(upper(attributes{i}),'PARENT=');
    if ~isempty(id),
      parents=separate(attributes{i}(id+7:end), ',') ;
      parent =parents{1} ;
    end
    id=strfind(attributes{i},'prediction_status=');
    if ~isempty(id),
      pred_status = attributes{i}(id+18:end);
      pred_status = strmatch(pred_status,status)-2;
    end
  end
  if isempty(identifier)&&isempty(parent)% &length(attributes)==1
    identifier = attributes{1};
    parent = attributes{1};
  end    
  parent_idx=strmatch(parent,gene_names,'exact');
  if isempty(parent_idx)
    parent_idx=strmatch(identifier,gene_names,'exact');
  end
  if isempty(parent_idx),
    if isempty(parent),
      FAULTS.Transcript.unknown_parent{end+1} = elems{9};
      % warning(sprintf('Empty parent: using transcript name as parent (%s)', identifier)) ;
      parent = identifier ;
    end 
    %keyboard
    % warning(sprintf('Could not find parent gene ("%s"): Created a new gene descriptor', parent)) ;
    FAULTS.Transcript.no_gene{end+1} = elems{9};
    genes(end+1).name = parent ;
    genes(end).alias = parent ;
    genes(end).chr = elems{1};
    genes(end).strand = elems{7} ;
    genes(end).start(end+1) = 0 ;
    genes(end).stop(end+1) = inf ;
    genes(end).is_correctly_gff3_referenced = 0 ;
    gene_names = {genes.name} ;
    parent_idx=length(genes) ;
  end ;
  if ~isequal(elems{1}, genes(parent_idx).chr) ;
    FAULTS.Transcript.diff_strand_contig{end+1} = elems{9};
  end
  if ~isequal(elems{7}, genes(parent_idx).strand) ;
    FAULTS.Transcript.diff_strand_contig{end+1} = elems{9};
    % warning(sprintf('strand of parent gene (%s) different',genes(parent_idx).name))
    continue
  end
  assert(~isempty(parent_idx))
  if ~isempty(genes(parent_idx).transcripts)
    other_idx = strmatch(identifier,genes(parent_idx).transcripts,'exact');
    if ~isempty(other_idx);
      FAULTS.Transcript.id_not_unique{end+1} = elems{9};
      % warning('transcript appears more than once!')
      if ~isequal(genes(parent_idx).start(other_idx), str2double(elems{4})) 
        genes(parent_idx).start = min(genes(parent_idx).start(other_idx),str2double(elems{4}));             
      end 
      if ~isequal(genes(parent_idx).stop(other_idx), str2double(elems{5}))            
        genes(parent_idx).stop(other_idx) = max(genes(parent_idx).stop(other_idx),str2double(elems{5}));             
      end
      continue
    end
  end
  genes(parent_idx).transcripts{end+1} = identifier;
  if ~isempty(pred_status)
    genes(parent_idx).transcript_status(end+1) = pred_status;
  end
  if isempty(genes(parent_idx).exons) 
    genes(parent_idx).exons = cell(1,1) ;
  else
    genes(parent_idx).exons{end+1} = [] ;
  end
  %transcript must lie within gene boundaries
  if ~(str2double(elems{4})>=genes(parent_idx).start(1)&&str2double(elems{4})<=genes(parent_idx).stop(1))
    FAULTS.Transcript.outside_boundaries{end+1} = elems{9};
    % warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  if ~(str2double(elems{5})>=genes(parent_idx).start(1)&&str2double(elems{5})<=genes(parent_idx).stop(1))
    FAULTS.Transcript.outside_boundaries{end+1} = elems{9};
    % warning(sprintf('transcript outside gene boundaries (%s; %s)', identifier, parent)) ;
  end
  genes(parent_idx).start(end+1) = str2double(elems{4});
  genes(parent_idx).stop(end+1) = str2double(elems{5}); 
  transcript_cnt = transcript_cnt +1;
  transcript_gene_map(transcript_cnt) = parent_idx;
  transcript_names{transcript_cnt} = identifier;
  tmp = separate(identifier,':');
  if length(tmp)>1
    transcript_ids{transcript_cnt} = tmp{2}; 
  else
    transcript_ids{transcript_cnt} = tmp{1}; 
  end
  if mod(transcript_cnt,1000)==0||transcript_cnt == num_transcripts
    fprintf('  parsed %i transcripts\r', transcript_cnt);  
  end
end %end second parse
transcript_names = transcript_names(1:transcript_cnt); 
transcript_ids = transcript_ids(1:transcript_cnt); 

% assert(transcript_cnt==num_transcripts)
fclose(fd);
unix(sprintf('rm %s',tmp_fname1));

fprintf('\n\n');
if length(FAULTS.Transcript.unknown_parent)>0 || length(FAULTS.Transcript.no_gene)>0 || length(FAULTS.Transcript.diff_strand_contig)>0 || length(FAULTS.Transcript.id_not_unique)>0 || length(FAULTS.Transcript.outside_boundaries)
  fprintf('Following problems were found:\n')
else
  fprintf('No problems found.\n')
end ;
if length(FAULTS.Transcript.unknown_parent)>0 
  fprintf('Number of transcripts without annotated parent (keyword ''Parent='' missing in column 9):\t %i\n',length(FAULTS.Transcript.unknown_parent))
  fprintf('\t=> using ID as Parent\n')
end
if length(FAULTS.Transcript.no_gene)>0
  fprintf('Number of transcripts without annotated parent gene:\t\t\t %i\n',length(FAULTS.Transcript.no_gene))
  fprintf('\t=> created a new gene descriptor\n')
end
if length(FAULTS.Transcript.diff_strand_contig)>0
  fprintf('Number of transcripts with different strand or contig than parent gene:\t %i\n',length(FAULTS.Transcript.diff_strand_contig))
  fprintf('\t=> transcript ignored\n')
end
if length(FAULTS.Transcript.id_not_unique)>0
  fprintf('Number of transcripts with non-unique ID (column 9, ID=... ):\t\t %i\n',length(FAULTS.Transcript.id_not_unique))
  fprintf('\t=> only one transcript added, min of start (column 4) and max of end (column 5) used\n')
end
if length(FAULTS.Transcript.outside_boundaries)
  fprintf('Number of transcripts outside boundaries of gene:\t\t\t %i\n',length(FAULTS.Transcript.outside_boundaries))
  fprintf('\t=> gene boundaries extended\n')
end ;

% correcting gene boundaries for orphan transcripts
for i=genes_cnt+1:length(genes),
  genes(i).start(1) = min(genes(i).start(2:end)) ;
  genes(i).stop(1) = max(genes(i).stop(2:end)) ;
end ;

% third parse: genes are filled with exons

% keyboard
types = SOFA_short_names(hierachies==3);
types_IDs = SOFA_IDs(hierachies==3);
fprintf('\n------starting third parse: only look for the following type\n');
tmp_fname1 = tempname;
unix(sprintf('touch %s',tmp_fname1));

for i=1:length(types)
  [tmp,num] = unix(sprintf('cut -f 3 %s | grep "^%s$" |wc -l',filtered_fname,types{i}));
  unix(sprintf('grep -e "[[:space:]]%s[[:space:]]" %s >> %s',types{i},filtered_fname,tmp_fname1));
  num_segments(i) = str2num(num);
  fprintf('%s\t%i\n',types{i},num_segments(i));
end
num_segments = sum(num_segments);

%num_segments = zeros(1,length(types)) ;
%for i=1:length(types)
%  [tmp,num] = unix(sprintf('cut -f 3 %s | grep %s |wc -l',tmp_fname,types{i}));
%  num_segments(i) = str2double(num) ;
%  fprintf('%s\t%i\n',types{i},num_segments(i));
%end
%num_segments = sum(num_segments);

%disp('creating hash table for transcript names') ;
%[transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names) ;

[fd msg] = fopen(tmp_fname1, 'r');
disp(msg);
segment_cnt = 0;
segment_skipped=0;
while ~feof(fd),

  attributes = {} ;
  type_name = '' ;
  elems = {} ;
  
  Line = fgetl(fd);
  elems = separate(Line);
  assert(~isempty(elems{1}));
  
  if Line(1)=='#',
    continue ;
  end
  if ~isempty(contig)
    if ~isequal(upper(elems{1}),upper(contig)),
      continue ;
    end
  end
  if ~ischar(Line),
    break ;
  end
  assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)
  if~(elems{7}=='+' || elems{7}=='-') ;
    if elems{7}~='.',
      % warning('strand missing')
    end ;
  end
  
  if ~isempty(thesources)
    if isempty(strmatch(elems{2}, thesources))
      error('something wrong with source')
    end  
  end
  
  if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
    continue ;
  end
  attributes = separate(elems{9}, ';');
  
  if ismember(elems{3},types);
    type_name = elems{3};
  elseif ismember(elems{3},types_IDs);
    type_name =types(isequal(elems{3},types_IDs));
  else
    error('wrong type')
  end

  switch type_name ,
   case {'CDS','coding_exon'}
    type = 3;
   case 'three_prime_UTR'
    type = 4;
   case 'five_prime_UTR'
    type = 2; 
   case 'exon'
    type = 0;
   otherwise
    segment_skipped = segment_skipped+1;
    % disp(Line);
    continue
  end
  
  parent = '';
  for i=1:length(attributes), 
    id=strfind(upper(attributes{i}),'PARENT=');
    if ~isempty(id),
      parent=attributes{i}(id+7:end) ;
      %parent = parents{1} ;
    end
  end
  if isempty(parent) %&  length(attributes)==1
    parent = attributes{1};
  end
  assert(~isempty(parent));
  parents = separate(parent, ',') ;
  for parent=parents,

    if ~ischar(parent),
      assert(length(parent)==1) ;
      parent=parent{1} ;
    end ;

    parent_idx = strmatch(parent, transcript_names, 'exact');
    %parent_idx = hash_search(transcript_hash_names, transcript_hash_idx, parent) ;
    %assert(isequal(parent_idx, parent_idx_)) ;

    id_seg = separate(parent, ':') ;
    if length(id_seg)>1
      id_seg = id_seg{2} ;
    end
    if isempty(parent_idx)
      id_seg = separate(parent, ':') ;
      if length(id_seg)>1
        id_seg = id_seg{2} ;
      end
      parent_idx=strmatch(id_seg, transcript_ids, 'exact');
    end
    gene_idx = [];
    if isempty(parent_idx),
      gene_idx =strmatch(parent,gene_names,'exact');
    end

    if isempty(parent_idx)&&isempty(gene_idx),
      % make sure that we only create a parent, if none of the parents exists
      assert(~isempty(parent)) ;
      % warning(sprintf('Could not find parent transcript ("%s"): Created a new gene and transcript descriptor', parent)) ;
      FAULTS.Exon.no_transcript{end+1} = elems{9};
      genes(end+1).name = [parent '-gene'] ;
      genes(end).alias = parent ;
      genes(end).chr = elems{1};
      genes(end).strand = elems{7} ;
      genes(end).start(end+1) = 0 ;
      genes(end).stop(end+1) = inf ;
      genes(end).is_correctly_gff3_referenced = 0 ;
      genes(end).transcripts{end+1} = parent ;
      genes(end).exons{end+1} = [] ;
      
      transcript_cnt = transcript_cnt +1;
      transcript_gene_map(transcript_cnt) = length(genes) ;
      transcript_names{transcript_cnt} = parent ;

      % add to hash table
      %[transcript_hash_names, transcript_hash_idx]= init_hash_search(transcript_names, transcript_cnt, transcript_hash_names, transcript_hash_idx) ;
      break ;
    else
      break ;
    end ;
  end ;

  for parent = parents,
    segment_invalid = 0 ;
    if ~ischar(parent),
      parent=parent{1} ;
    end ;

    parent_idx = strmatch(parent, transcript_names, 'exact') ;
    %parent_idx = hash_search(transcript_hash_names, transcript_hash_idx, parent) ;
    %assert(parent_idx==parent_idx_) ;

    if isempty(parent_idx),
      fprintf('parent %s not found\n', parent) ;
      gene_idx = transcript_gene_map(parent_idx);
      if isempty(gene_idx),
        continue ;
      end
      genes(gene_idx).is_correctly_gff3_referenced = 0 ;
      transcript_idx = 1;
    else
      assert(~isempty(parent_idx) && length(parent_idx)==1);
      gene_idx = transcript_gene_map(parent_idx);
      transcript_idx = strmatch(parent, genes(gene_idx).transcripts, 'exact');
    end ;
    
    if isempty(transcript_idx)
      id_seg = separate(parent, ':') ;
      if length(id_seg)>1
        id_seg = id_seg{2} ;
      end
      for j = 1:length(genes(gene_idx).transcripts)
        tmp =  separate(genes(gene_idx).transcripts{j}, ':') ;
        id_trans{j} = tmp{2};
      end
      transcript_idx=strmatch(id_seg,id_trans,'exact');
    end
    assert(~isempty(transcript_idx) & length(transcript_idx)==1);

    if ~isequal(elems{1}, genes(gene_idx).chr)
      segment_invalid = 1 ;
      FAULTS.Exon.diff_strand_contig{end+1} = elems{9};
      % fprintf('Segment''s contig name does not match parent (is %s, should be %s). Ignoring segment.\nLine:\n %s\n', elems{1}, genes(gene_idx).chr, Line) ;
    end ;
    if ~isequal(elems{7}, genes(gene_idx).strand),
      segment_invalid = 1 ;
      FAULTS.Exon.diff_strand_contig{end+1} = elems{9};
      % fprintf('Segment''s strand name does not match parent (is %s, should be %s). Ignoring segment.\nLine:\n %s\n', elems{7}, genes(gene_idx).strand, Line) ;
    end ;

    if ~segment_invalid,
      if elems{7}=='+'
        genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2double(elems{4}), str2double(elems{5})+1, type];
      else
        genes(gene_idx).exons{transcript_idx}(end+1,:) = [str2double(elems{4})-1, str2double(elems{5}), type];
      end
    end ;
%    assert(str2double(elems{4})>=genes(gene_idx).start(1+transcript_idx)&str2double(elems{4})<=genes(gene_idx).stop(1+transcript_idx))
%    assert(str2double(elems{5})>=genes(gene_idx).start(1+transcript_idx)&str2double(elems{5})<=genes(gene_idx).stop(1+transcript_idx))
  end ;
  segment_cnt = segment_cnt+1;
  if mod(segment_cnt,5000)==0
    fprintf('  parsed %i gene segments\r', segment_cnt);
  end
end % end third parse
fclose(fd);


fprintf('\n\n');
if length(FAULTS.Exon.no_transcript)>0 || length(FAULTS.Exon.diff_strand_contig)>0,
  fprintf('Following problems were found:\n');
else
  fprintf('No problems found.\n');
end ;
if length(FAULTS.Exon.no_transcript)>0, 
  fprintf('number of gene segments without parent transcript:\t %i\n', length(FAULTS.Exon.no_transcript)) ;
  fprintf('\t=> created a new gene and transcript descriptor\n') ;
end ;
if length(FAULTS.Exon.diff_strand_contig)>0,
  fprintf('number of gene segments with different strand or contig than parent transcript:\t %i\n', length(FAULTS.Exon.diff_strand_contig)) ;
  fprintf('\t=> Segment ignored\n') ;
end ;

for i=1:length(genes)
  genes(i).start = min(genes(i).start);
  genes(i).stop = max(genes(i).stop);
end 

if ~isequal(gff_fname, tmp_fname1),
  unix(['rm ' tmp_fname1]);
end ;

if ~isempty(save_fname),
   save(save_fname, 'genes', 'FAULTS');
end ;

return

function [attributes, type_name, elems, action] = parse_starter(fd, types, types_IDs, contig, source)
% parse_starter(fd)

attributes = {} ;
type_name = '' ;
elems = {} ;
action = '' ;


Line = fgetl(fd);
elems = separate(Line);
assert(~isempty(elems{1}));

if Line(1)=='#',
  action = 'continue' ;
  return ;
end
if ~isempty(contig)
  if ~isequal(upper(elems{1}),upper(contig)),
    action='continue' ;
    return ;
  end
end
if ~ischar(Line),
  action='break' ;
  return ;
end

if isempty(elems{4}) | isempty(elems{4})
  action = 'continue' ;
  return ;
end


assert(str2double(elems{4})<=str2double(elems{5}) || str2double(elems{5})<0)

if~(elems{7}=='+' || elems{7}=='-') ;
  if elems{7}~='.',
    warning('strand missing')
  end ;
end

if ~isempty(source)
  if ~isequal(elems{2},source)
    error('something wrong with source')
  end  
end

if ~ismember(elems{3},types) && ~ismember(elems{3},types_IDs)
  action = 'continue' ;
  return ;
end
attributes = separate(elems{9}, ';');

if ismember(elems{3},types);
  type_name = elems{3};
elseif ismember(elems{3},types_IDs);
  type_name =types(isequal(elems{3},types_IDs));
else
  error('wrong type')
end

action='' ;

return

function [seqs,names]=read_fasta_file(fname, name_only)
% [seqs,names]=read_fasta_file(fname)
% 
% reads all entries in a fasta file
  
if nargin<2, name_only=0; end ; 

fd=fopen(fname,'r') ;
str=fread(fd) ;

% fix new lines
num13=sum(str==13) ;
num10=sum(str==10) ;
if num10==0 && num13>0,
  str(str==13)=10 ;
end ;
if num10>0 && num13>0,
  assert(num10==num13) ;
  str(str==13)=[] ;
end ;

newseq=find(str=='>') ;
newseq(end+1) = length(str)+1 ;

seqs={} ;
names={} ;

for i=1:length(newseq)-1,
  seq=char(str(newseq(i):newseq(i+1)-1)) ;
  idx=find(seq==10, 1, 'first') ;
  assert(length(idx)==1) ;
  name = seq(2:idx-1)' ;
  names{end+1}=name ;
  if ~name_only,
    seq  = seq(idx+1:end)' ;
    seq(seq==10) = [] ;
    seqs{end+1} = seq ;
  end ;
end ;

fclose(fd) ;

return

function x=reverse_complement(y)

if length(y)>200*1024*1024
  x=uint8(y) ;
  for i=1:length(y),
    x(i)=y(end-i+1) ;
  end ;
  y=x ;
else
  y=y(end:-1:1) ;
  x=y ;
end ;

x(y=='a')='t' ;
x(y=='c')='g' ;
x(y=='g')='c' ;
x(y=='t')='a' ;
x(y=='n')='n' ;
x(y=='A')='T' ;
x(y=='C')='G' ;
x(y=='G')='C' ;
x(y=='T')='A' ;
x(y=='N')='N' ;

x=char(x) ;

return

function  [Conf, Conf_cum] = Out2Conf(Out,prob, prob_cum,limits);

% [Conf, Conf_cum] = Out2Conf(Output ,prob, prob_cum,limits);
%  
% function transforms outputs of an SVM to probability and cumulative
% probability values by means of piecewise linear functions (PLIF). 
% Use learn_plifs to determine the function (it will return supporting
% points 'limits' and corresponding y-values, 'prob' and 'prob_cum')
%
% INPUT   Out       vector of SVM Output to be transformed
%         prob
%         prob_cum
%         limits    
%
% OUTPUT  Conf      vector (same length as Out) with estimated
%                   probability for true positive
%         Conf_cum  vector (same length as Out) with estimated 
%                   cumulative probability true positive
% 
% Functions called: -> pwl_add
%
% see also: learn_plifs

bins = length(prob); 
Conf = zeros(1, length(Out)) ;
Conf_cum = zeros(1, length(Out)) ;
for i=1:length(Out)
  x(:,1) = pwl_add(zeros(bins, 1), Out(i), limits) ;
  Conf_cum(i) = x'*prob_cum;
  Conf(i) = x'*prob;
end ;

return

function  [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs(Out,L,bins,do_balancing,min_inc);
% [prob, prob_cum,limits,prob_unsort,prob_cum_unsort] = learn_plifs_qp(Output,Labels,numberofbins,do_balancing,min_inc);
% 
% learn piecewise linear functions (PLIF), with which outputs of an SVM
% can be transformed to probability and cumulative
% probability values
% 
% INPUT    Out                vector of SVM Outputs on an evaluation set
%
%          L                  vector of corresponding labels (-1,1)  
%
%          bins               number of bins used for interpolation (default 50)
%                             (limits are choosen such that the number of
%                             examples in every bin is similar)  
%
%          do_balancing       should be set to 1 if negative class is
%                             much bigger than positive class,
%                             (negatives are downsampled for
%                             determining the bins) 
%
%          min_inc            probabilities have to be monotonically
%                             increasing (higher SVM output value
%                             corresponds to higher probability of true positive) 
%                             (default 1e-5)
%
%
% OUTPUT   prob               vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output = limits(i) to be true positive
% 
%          prob_cum           vector of length 'bins'. prob_cum(i) is the
%                             estimated probability of examples with 
%                             Output> limits(i) to be true positive
%
%          limits             vector of length 'bins' with supporting
%                             points (SVM Output values)
%
% see also: Out2Conf  
  
if nargin<5, min_inc = 1e-5 ; end ;
if nargin<4, do_balancing = 1; end ;
if nargin<3, bins = 50; end ;

if do_balancing
  s1 = [min(Out) max(Out)];
  s1 = [s1 Out(L==1)];
  idx = find(L==-1);
  num = min(sum(L==-1),sum(L==1));
  % ii = randperm(length(idx));
  idx = idx(randperm(length(idx)));
  s1 = sort([s1 Out(idx(1:num))]);
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
else
  s1 = sort(Out) ;
  limits = [s1(round(linspace(1,length(s1),bins-1))) inf] ;
end
  
prob = zeros(bins,1);
prob_cum = zeros(bins,1);
num = zeros(bins,1) ;
num_cum = zeros(bins,1) ;
for i=1:bins-1, 
  num_cum(i) = sum(Out>=limits(i)) ;
  prob_cum(i) = (sum(Out>=limits(i) & L==1 )+1e-10)/(sum(Out>=limits(i))+1e-10) ;
  num(i) = sum(Out>=limits(i) & Out<limits(i+1)) ;
  prob(i) = (sum(Out>=limits(i) & Out<limits(i+1) & L==1 )+1e-10)/(sum(Out>=limits(i) & Out<limits(i+1))+1e-10) ;
end 
prob_cum(bins) = 1;
prob(bins) = 1;

prob_unsort=prob ;
prob_cum_unsort=prob_cum ;

prob_cum = sort(prob_cum) ;
prob = sort(prob) ;

return ;

return

function vec = pwl_add(vec, value, limits)
% vec = pwl_add(vec, value, limits)
  
idx = sum(limits<=value) ;

if idx==0,
  vec(1)=vec(1)+1 ;
elseif idx==length(limits)-1,
  vec(end)=vec(end)+1 ;
else
  vec(idx+1) = vec(idx+1) + (value-limits(idx))/(limits(idx+1)-limits(idx)) ;  
  vec(idx)   = vec(idx)   + (limits(idx+1)-value)/(limits(idx+1)-limits(idx)) ;
end ;
return

function [score,tp,tdr]=calcrfcscore(output,LT)
% score=calcrfcscore(output,LT)
map=(LT==1) ;
[s,idx]=sort(-output) ; s=-s ;
map=map(idx) ;

tp=cumsum(map)/sum(LT==1) ;
tdr=cumsum(map)./[1:length(map)] ;

t=tp(2:end)-tp(1:end-1) ;
score = sum(0.5*[tdr(1:end-1)+tdr(2:end)].*t) ;



return

function [false_alarms,hits] = calcroc(output,LTE)

assert(all(size(output)==size(LTE))) ;

[ld,idx] = sort(output); 

i=0 ;

hits=1-cumsum(LTE(idx)>0)/sum(LTE > 0) ;
false_alarms=1-cumsum(LTE(idx)<0)/sum(LTE < 0) ;

% Da die Kurve nie alle Negativen als positiven zaehlt, hat
% Falsealarms nie 
% den Wert 1. Cumsum zaehlt das erste Element als negatives, diese
% fehlt bei
%  den False_alarms.
% Daher wird vorne an den Vektor noch eine 1 gehaengt

hits = [1 hits ];
false_alarms = [1 false_alarms];

if nargout<2,
  false_alarms = [false_alarms;hits];
end ;

return

function s = calcrocscore(oute,LTE)
% rocscore = calcrocscore(output,LT)
%
% computes the area under the ROC curve

[a,b]=calcroc(oute,LTE) ;

s=0 ;
for i=1:length(a)-1,
  s = s + (a(i)-a(i+1))*(b(i)+b(i+1))/2 ;
end ;


return

function [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% [engine, environment, basedir, mccdir, license_type] = determine_engine() ;
% 
% returns either 'matlab' or 'octave' in variable engine
% and 'internal' or 'galaxy' in variable environment
% basedir is the directory of the matlab or octave instance
% mccdir is the directory of the matlab compiler (does not yet exist for octave)

global g_license_type ;

if isempty(g_license_type),
  g_license_type = 'academic' ;
end ;
license_type = g_license_type ;

lic=license ;
if ~isequal(lic, 'GNU General Public License'),
  engine='matlab' ;
else
  engine='octave' ;
end ;

environment='internal' ;
if isequal(whoami, 'galaxy'),
  environment='galaxy' ;
end ;

if isequal(engine, 'matlab') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(engine, 'matlab') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/matlab-7.6/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'internal'),
  basedir = '/fml/ag-raetsch/share/software/octave-3.0.3/' ;
elseif isequal(engine, 'octave') && isequal(environment,  'galaxy'),
  basedir = '/home/galaxy/octave-3.0.4' ;
end ;

if isequal(environment,  'internal'),
  mccdir = '/fml/ag-raetsch/share/software/matlab/' ;
elseif isequal(environment,  'galaxy'),
  mccdir = '/home/galaxy/matlab-7.6/' ;
end ;

return

return

function x = equal(M1, M2)
% EQUAL(M1, M2) tests M1 and M2 for equality
% Function to test whether two matrices are identical
%
% returns 1 if true and 0 if false

x=isequal(M1, M2) 

return

function x = fexist(filename,gz)
% FEXIST Check if file exists
%        FEXIST('filename') returns:
%        0 if filename does not exist
%        1 if filename exists in the given path

if nargin<2,
  gz=0;
end ;

fd = fopen(filename);
if fd ~= -1
  x = 1;
  fclose(fd);
elseif gz,
  fd = fopen([filename '.gz']);
  if fd~=-1,
    x=1 ;
    fclose(fd) ;
  else
    x=0 ;
  end ;
else
  x = 0;
end

return

function ret=keyboard_allowed()
% ret=keyboard_allowed()
%
% returns 1, if a keyboard command would be allowed

global g_ignore_keyboard
global THIS_IS_A_RPROC_PROCESS  

if isequal(g_ignore_keyboard, 1),
  ret=0 ;
  return ;
end ;

if isequal(THIS_IS_A_RPROC_PROCESS, 1),
  ret=0 ;
  return ;
end ;

ret=1 ;


return

function var=load_matrix(filename, varname)
% var=load_matrix(filename, varname)

load(filename, 'meta')
if ~exist('meta')
  load(filename, varname);
  eval(sprintf('var = %s;',varname))
  return
end

var = [] ;
col = 1;
for j=1:meta.num_splits
  %if col>1
  %  assert(~all(var(:,col-1)==-1* ones(meta.size(1),1)))
  %end
  %assert(all(var(:,col)==-1*ones(meta.size(1),1)))
   
  load(filename, sprintf('%s_split_%i',varname,j));
  eval(sprintf('temp = %s_split_%i;',varname,j));

  if j==1,
    if isnumeric(temp)
      var = zeros(meta.size(1), meta.size(2), class(temp));
    else
      var = zeros(meta.size(1), meta.size(2),'uint8');
      var = char(var);
    end
  end ;

  var(:,col:col + size(temp, 2)-1)=temp;
  col = col + size(temp, 2);
end
%assert(~all(var(:,end)==-1* zeros(meta.size(1),1)))

return

function array = load_struct(infile_name, arrayname)
% array = load_struct(infile_name, arrayname)
  
if nargin<2,
  arrayname='array' ;
end ;

L_=load(infile_name) ;
if isfield(L_,'numvars')
  fprintf('arrayload2 loads %i chunks:', L_.numvars) ;
  ix=1;
  L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
  eval(sprintf('array = L.%s1;', arrayname)) ;
  fprintf('.') ;
  for ix = 2:L_.numvars
    L=load(sprintf('%s_%i.mat', infile_name, ix), sprintf('%s%d', arrayname, ix)) ;
    if size(array,1) == 1
      array = [array, eval(sprintf('L.%s%d', arrayname, ix))];
    elseif size(array,2) == 1
      array = [array; eval(sprintf('L.%s%d', arrayname, ix))];
    end
    fprintf('.') ;
  end
  fprintf('\n') ;
  if isfield(L_, 'dimensions'),
    array = reshape(array, L_.dimensions) ;
  end ;
else
  eval(sprintf('array=L_.%s;', arrayname)) ;
end

return

function save_append(fname__, append_flag, varargin)
% save_append(fname__, append_flag, varargin)
%
% octave compatible version of save -append

try,
  if append_flag
    L__=load(fname__) ;
  else
    L__=struct ;
  end ;
catch 
  L__=struct ;
end 

if length(varargin)>1,
  
  % assume varargin to have the following structure: varname1, var1, ...
  
  assert(mod(length(varargin),2)==0) ;
  for i=1:2:length(varargin),
    L__.(varargin{i}) = varargin{i+1} ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
else
  
  % assume the second argument is a struct
  assert(length(varargin)==1) ;
  
  f = fieldnames(varargin{1}) ;
  for i=1:2:length(f),
    L__.(f{i}) = varargin{1}.(f{i}) ;
  end ;
  
  f=fieldnames(L__) ;
  for i=1:length(f),
    eval(sprintf('%s=L__.%s;', f{i}, f{i})) ;
  end ;
  
  eval(sprintf('save -V7 %s %s', fname__, sprintf('%s ', f{:}))) ;
  
end ;




return

function save_matrix(filen, M, varname, split_size)
%save_matrix(filen, M, varname, split_size)
%split matrix into parts with size of about split_size 
%bytes (1e+8 default) and save them in one file

if nargin<4
  split_size=1e+8;
end

info = whos('M');
num_splits = ceil(info.bytes/split_size);
num_cols = ceil(size(M, 2)/num_splits);

  
  meta.size=size(M);
  meta.num_splits = num_splits;
  mata.name = varname;
  if fexist(filen)
    save_append(filen, 1, 'meta', meta);
  else
    save(filen, 'meta');
  end
  for j=1:num_splits-1
    %eval(sprintf('%s_split_%i = M(:, 1:num_cols);',varname, j));
    var_split__ = M(:, 1:num_cols);
    M(:, 1:num_cols)=[];
    %save(filen, '-append', sprintf('%s_split_%i',varname,j));
    save_append(filen, 1, sprintf('%s_split_%i',varname,j), var_split__);
    clear var_split__
  end
  if isempty(j), j=0; end;
  var_split__ = M; 
  save_append(filen, 1, sprintf('%s_split_%i',varname,j+1), var_split__);

return

function save_struct(outfile_name, array, arrayname, MAXSIZE)
% save_struct(outfile_name, array[, arrayname])

if nargin<3
  arrayname='array' ;
end ;

if nargin<4, 
  MAXSIZE = 1e9; % about 1Gb
end ;

whosinfo = whos('array');
if whosinfo.bytes > MAXSIZE
  % linearize first, store original size
  dimensions = size(array) ;
  array = reshape(array, 1, prod(dimensions)) ;

  elementsize = whosinfo.bytes/length(array);
  chunklength = round(MAXSIZE/elementsize);
  numvars = ceil(length(array)/chunklength);
  %fprintf('arraysave2 processes %i chunks:', numvars) ;
  for ix = 1:numvars-1
    eval(sprintf('%s%d = array(%d:%d);',arrayname, ix,(ix-1)*chunklength+1, ...
                 ix*chunklength));
    %fprintf('.') ;
  end
  %fprintf('\n') ;
  eval(sprintf('%s%d = array(%d:end);',arrayname, numvars,(numvars-1)*chunklength+1));
  clear array ix ;
  fprintf('save_struct saves %i chunks:', numvars) ;
  save(outfile_name, '-V7', 'chunklength', 'elementsize', 'numvars', 'dimensions') ;
  for ix = 1:numvars
    save(sprintf('%s_%i.mat', outfile_name, ix), '-V7', sprintf('%s%d', arrayname, ix)) ;
    fprintf('.') ;
  end ;
  fprintf('\n') ;
else
  %eval(sprintf('%s=array;', arrayname)) ;
  save_append(outfile_name, 0, arrayname, array) ;
end

return

function f=separate(str, delim, merge_multiple) ;
% f=separate(str, delimiter, merge_multiple_delimiters) ;
  
if nargin<2, delim=sprintf('\t') ; end ;
if nargin<3, merge_multiple=0 ; end ;
f={} ;
idx=[0 find(str==delim) length(str)+1] ;
if merge_multiple,
  idx2=find(idx(1:end-1)~=idx(2:end)-1) ;
  idx=[idx(idx2) idx(end)] ;
end 
for i=1:length(idx)-1
  f{i}=deblank(str(idx(i)+1:idx(i+1)-1)) ;
end ;

return

function A=spdiag(diagonal) ;
% A=spdiag(diagonal) ;

if size(diagonal,1)>1
  A=spdiags(diagonal, 0, length(diagonal),length(diagonal)) ;
else
  A=spdiags(diagonal', 0, length(diagonal),length(diagonal)) ;
end ;

return

function A=spzeros(m,n,szmax) ;
% A=spzeros(m,n) ;

if nargin<=1
  A=sparse([],[],[],m,m) ;
  return ;
end 
if nargin<=2
  A=sparse([],[],[],m,n) ;
  return ;
end 
A=sparse([],[],[],m,n,szmax) ;

return

function list=who_file(fname) ;
% list=who_file(fname) ;
  
engine = determine_engine() ;
if isequal(engine, 'matlab')
  list=who('-file', fname) ;
else
  L=load(fname) ;
  list=fieldnames(L) ;
end ;

return

function user_name = whoami
% user_name = whoami
  
global whoami_user_name  
  
if isempty(whoami_user_name)
  
  [ret, whoami_user_name]=unix('whoami') ; 
  if ret~=0,
    if keyboard_allowed(),
      keyboard ;
    end ;
    whoami_user_name='unknown' ;
  end ;
end ;

user_name= whoami_user_name ;
idx=find(user_name<30, 1, 'first') ;
user_name = user_name(1:idx-1) ;

return

